@@ -6,8 +6,6 @@
 #include <QtNetwork>
 #include <QtCore>
 
-QT_BEGIN_NAMESPACE
-
 //! [0]
 class DtlsAssociation : public QObject
 {
@@ -45,6 +43,4 @@ private:
 };
 //! [0]
 
-QT_END_NAMESPACE
-
 #endif // ASSOCIATION_H
