@@ -104,7 +104,7 @@ QString XmlOutput::doConversion(const QString &text)
                 output += QLatin1String("&gt;");
             } else {
                 if (c.unicode() < 0x20) {
-                    output += QString("&#x%1;").arg(c.unicode(), 2, 16, QLatin1Char('0'));
+                    output += QString("&#x%1;").arg(ushort{c.unicode()}, 2, 16, QLatin1Char('0'));
                 } else {
                     output += c;
                 }
