@@ -1,11 +1,13 @@
 // Copyright (C) 2021 The Qt Company Ltd.
 // Copyright (C) 2021 Intel Corporation.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:critical reason:execute-external-code
 
 #include "qdir.h"
 #include "qstringlist.h"
 #include "qfile.h"
 #if QT_CONFIG(settings)
+#include "qresource.h"
 #include "qsettings.h"
 #endif
 #include "qlibraryinfo.h"
@@ -13,7 +15,7 @@
 
 #include "qcoreapplication.h"
 
-#include "private/qglobal_p.h"
+#include "private/qfilesystementry_p.h"
 #include "archdetect.cpp"
 #include "qconfig.cpp"
 
@@ -103,7 +105,7 @@ static std::unique_ptr<QSettings> findConfiguration()
         return std::make_unique<QSettings>(*qtconfManualPath, QSettings::IniFormat);
 
     QString qtconfig = QStringLiteral(":/qt/etc/qt.conf");
-    if (QFile::exists(qtconfig))
+    if (QResource(qtconfig, QLocale::c()).isValid())
         return std::make_unique<QSettings>(qtconfig, QSettings::IniFormat);
 #ifdef Q_OS_DARWIN
     CFBundleRef bundleRef = CFBundleGetMainBundle();
@@ -121,11 +123,11 @@ static std::unique_ptr<QSettings> findConfiguration()
     }
 #endif
     if (QCoreApplication::instance()) {
-        QDir pwd(QCoreApplication::applicationDirPath());
-        qtconfig = pwd.filePath(u"qt" QT_STRINGIFY(QT_VERSION_MAJOR) ".conf"_s);
+        QString pwd = QCoreApplication::applicationDirPath();
+        qtconfig = pwd + u"/qt" QT_STRINGIFY(QT_VERSION_MAJOR) ".conf"_s;
         if (QFile::exists(qtconfig))
             return std::make_unique<QSettings>(qtconfig, QSettings::IniFormat);
-        qtconfig = pwd.filePath("qt.conf"_L1);
+        qtconfig = pwd + u"/qt.conf";
         if (QFile::exists(qtconfig))
             return std::make_unique<QSettings>(qtconfig, QSettings::IniFormat);
     }
@@ -600,25 +602,42 @@ static QVariant libraryPathToValue(QLibraryInfo::LibraryPath loc)
 }
 #endif // settings
 
+// TODO: There apparently are paths that are both absolute and relative for QFileSystemEntry.
+//       In particular on windows.
+
+static bool pathIsRelative(const QString &path)
+{
+    using FromInternalPath = QFileSystemEntry::FromInternalPath;
+    return !path.startsWith(':'_L1) && QFileSystemEntry(path, FromInternalPath{}).isRelative();
+}
+
+static bool pathIsAbsolute(const QString &path)
+{
+    using FromInternalPath = QFileSystemEntry::FromInternalPath;
+    return path.startsWith(':'_L1) || QFileSystemEntry(path, FromInternalPath{}).isAbsolute();
+}
+
 QStringList QLibraryInfoPrivate::paths(QLibraryInfo::LibraryPath p,
                                        UsageMode usageMode)
 {
     const QLibraryInfo::LibraryPath loc = p;
     QList<QString> ret;
     bool fromConf = false;
+    bool pathsAreAbsolute = true;
 #if QT_CONFIG(settings)
     if (havePaths()) {
         fromConf = true;
 
         QVariant value = libraryPathToValue(loc);
         if (value.isValid()) {
-
             if (auto *asList = get_if<QList<QString>>(&value))
                 ret = std::move(*asList);
             else
                 ret = QList<QString>({ std::move(value).toString()});
-            for (qsizetype i = 0, end = ret.size(); i < end; ++i)
+            for (qsizetype i = 0, end = ret.size(); i < end; ++i) {
                 ret[i] = normalizePath(ret[i]);
+                pathsAreAbsolute = pathsAreAbsolute && pathIsAbsolute(ret[i]);
+            }
         }
     }
 #endif // settings
@@ -639,10 +658,12 @@ QStringList QLibraryInfoPrivate::paths(QLibraryInfo::LibraryPath p,
             noConfResult = QString::fromLocal8Bit(path);
 #endif
         }
-        if (!noConfResult.isEmpty())
+        if (!noConfResult.isEmpty()) {
+            pathsAreAbsolute = pathsAreAbsolute && pathIsAbsolute(noConfResult);
             ret.push_back(std::move(noConfResult));
+        }
     }
-    if (ret.isEmpty())
+    if (ret.isEmpty() || pathsAreAbsolute)
         return ret;
 
     QString baseDir;
@@ -653,7 +674,7 @@ QStringList QLibraryInfoPrivate::paths(QLibraryInfo::LibraryPath p,
         baseDir = QLibraryInfoPrivate::path(QLibraryInfo::PrefixPath, usageMode);
     }
     for (qsizetype i = 0, end = ret.size(); i < end; ++i)
-        if (QDir::isRelativePath(ret[i]))
+        if (pathIsRelative(ret[i]))
             ret[i] = QDir::cleanPath(baseDir + u'/' +  std::move(ret[i]));
     return ret;
 }
