@@ -1,10 +1,13 @@
 // Copyright (C) 2020 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #ifndef QDIR_H
 #define QDIR_H
 
+#include <QtCore/qcompare.h>
 #include <QtCore/qstring.h>
+#include <QtCore/qdirlisting.h>
 #include <QtCore/qfile.h>
 #include <QtCore/qfileinfo.h>
 #include <QtCore/qstringlist.h>
@@ -66,7 +69,7 @@ public:
     QDir(const QString &path = QString());
     QDir(const QString &path, const QString &nameFilter,
          SortFlags sort = SortFlags(Name | IgnoreCase), Filters filter = AllEntries);
-#ifdef Q_CLANG_QDOC
+#ifdef Q_QDOC
     QDir(const std::filesystem::path &path);
     QDir(const std::filesystem::path &path, const QString &nameFilter,
          SortFlags sort = SortFlags(Name | IgnoreCase), Filters filter = AllEntries);
@@ -91,7 +94,7 @@ public:
     { d_ptr.swap(other.d_ptr); }
 
     void setPath(const QString &path);
-#ifdef Q_CLANG_QDOC
+#ifdef Q_QDOC
     void setPath(const std::filesystem::path &path);
 #elif QT_CONFIG(cxx17_filesystem)
     template<typename T, QtPrivate::ForceFilesystemPath<T> = 0>
@@ -103,7 +106,7 @@ public:
     QString path() const;
     QString absolutePath() const;
     QString canonicalPath() const;
-#if QT_CONFIG(cxx17_filesystem) || defined(Q_CLANG_QDOC)
+#if QT_CONFIG(cxx17_filesystem) || defined(Q_QDOC)
     std::filesystem::path filesystemPath() const
     { return QtPrivate::toFilesystemPath(path()); }
     std::filesystem::path filesystemAbsolutePath() const
@@ -112,9 +115,10 @@ public:
     { return QtPrivate::toFilesystemPath(canonicalPath()); }
 #endif // QT_CONFIG(cxx17_filesystem)
 
+#ifndef QT_BOOTSTRAPPED
     static void setSearchPaths(const QString &prefix, const QStringList &searchPaths);
     static void addSearchPath(const QString &prefix, const QString &path);
-#ifdef Q_CLANG_QDOC
+#ifdef Q_QDOC
     static void addSearchPath(const QString &prefix, const std::filesystem::path &path);
 #elif QT_CONFIG(cxx17_filesystem)
     template<typename T, QtPrivate::ForceFilesystemPath<T> = 0>
@@ -124,6 +128,7 @@ public:
     }
 #endif // QT_CONFIG(cxx17_filesystem)
     static QStringList searchPaths(const QString &prefix);
+#endif // QT_BOOTSTRAPPED
 
     QString dirName() const;
     QString filePath(const QString &fileName) const;
@@ -144,10 +149,16 @@ public:
     SortFlags sorting() const;
     void setSorting(SortFlags sort);
 
+#if QT_CORE_REMOVED_SINCE(6, 5)
     uint count() const;
+#endif
+    qsizetype count(QT6_DECL_NEW_OVERLOAD) const;
     bool isEmpty(Filters filters = Filters(AllEntries | NoDotAndDotDot)) const;
 
+#if QT_CORE_REMOVED_SINCE(6, 5) && QT_POINTER_SIZE != 4
     QString operator[](int) const;
+#endif
+    QString operator[](qsizetype) const;
 
     static QStringList nameFiltersFromString(const QString &nameFilter);
 
@@ -177,8 +188,10 @@ public:
     inline bool isAbsolute() const { return !isRelative(); }
     bool makeAbsolute();
 
+#if QT_CORE_REMOVED_SINCE(6, 8)
     bool operator==(const QDir &dir) const;
     inline bool operator!=(const QDir &dir) const { return !operator==(dir); }
+#endif
 
     bool remove(const QString &fileName);
     bool rename(const QString &oldName, const QString &newName);
@@ -229,7 +242,11 @@ protected:
     QSharedDataPointer<QDirPrivate> d_ptr;
 
 private:
+    friend Q_CORE_EXPORT bool comparesEqual(const QDir &lhs, const QDir &rhs);
+    Q_DECLARE_EQUALITY_COMPARABLE_NON_NOEXCEPT(QDir)
     friend class QDirIterator;
+    friend class QDirListing;
+    friend class QDirListingPrivate;
     // Q_DECLARE_PRIVATE equivalent for shared data pointers
     QDirPrivate *d_func();
     const QDirPrivate *d_func() const { return d_ptr.constData(); }
