@@ -1,5 +1,6 @@
 // Copyright (C) 2011 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com, author Stephen Kelly <stephen.kelly@kdab.com>
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qidentityproxymodel.h"
 #include "qidentityproxymodel_p.h"
@@ -197,19 +198,7 @@ QModelIndex QIdentityProxyModel::mapToSource(const QModelIndex& proxyIndex) cons
  */
 QModelIndexList QIdentityProxyModel::match(const QModelIndex& start, int role, const QVariant& value, int hits, Qt::MatchFlags flags) const
 {
-    Q_D(const QIdentityProxyModel);
-    Q_ASSERT(start.isValid() ? start.model() == this : true);
-    if (!d->model)
-        return QModelIndexList();
-
-    const QModelIndexList sourceList = d->model->match(mapToSource(start), role, value, hits, flags);
-    QModelIndexList::const_iterator it = sourceList.constBegin();
-    const QModelIndexList::const_iterator end = sourceList.constEnd();
-    QModelIndexList proxyList;
-    proxyList.reserve(sourceList.size());
-    for ( ; it != end; ++it)
-        proxyList.append(mapFromSource(*it));
-    return proxyList;
+    return QAbstractProxyModel::match(start, role, value, hits, flags);
 }
 
 /*!
