@@ -51,6 +51,8 @@ static qint64 add_saturate(qint64 t1, Duration1 dur, Durations... extra)
     \reentrant
     \ingroup tools
 
+    \compares strong
+
     The QDeadlineTimer class is usually used to calculate future deadlines and
     verify whether the deadline has expired. QDeadlineTimer can also be used
     for deadlines without expiration ("forever"). It forms a counterpart to
@@ -84,11 +86,12 @@ static qint64 add_saturate(qint64 t1, Duration1 dur, Durations... extra)
 
     \section1 Timer types
 
-    Like QTimer, QDeadlineTimer can select among different levels of coarseness
-    on the timers. You can select precise timing by passing Qt::PreciseTimer to
-    the functions that set of change the timer, or you can select coarse timing
-    by passing Qt::CoarseTimer. Qt::VeryCoarseTimer is currently interpreted
-    the same way as Qt::CoarseTimer.
+    Like QTimer and QChronoTimer, QDeadlineTimer can select among
+    different levels of coarseness on the timers. You can select
+    precise timing by passing Qt::PreciseTimer to the functions that
+    set of change the timer, or you can select coarse timing by passing
+    Qt::CoarseTimer. Qt::VeryCoarseTimer is currently interpreted the same
+    way as Qt::CoarseTimer.
 
     This feature is dependent on support from the operating system: if the OS
     does not support a coarse timer functionality, then QDeadlineTimer will
@@ -104,7 +107,8 @@ static qint64 add_saturate(qint64 t1, Duration1 dur, Durations... extra)
     QDeadlineTimer is compatible with the \c{std::chrono} API from C++11 and
     can be constructed from or compared to both \c{std::chrono::duration} and
     \c{std::chrono::time_point} objects. In addition, it is fully compatible
-    with the time literals from C++14, which allow one to write code as:
+    with the \l{std::literals::chrono_literals Symbol Index}{time literals
+    from C++14}, which allow one to write code such as:
 
     \snippet code/src_corelib_kernel_qdeadlinetimer.cpp 1
 
@@ -120,7 +124,7 @@ static qint64 add_saturate(qint64 t1, Duration1 dur, Durations... extra)
 
     \snippet code/src_corelib_kernel_qdeadlinetimer.cpp 2
 
-    \sa QTime, QTimer, QDeadlineTimer, Qt::TimerType
+    \sa QTime, QChronoTimer, QDeadlineTimer, Qt::TimerType
 */
 
 /*!
@@ -215,7 +219,8 @@ QDeadlineTimer::QDeadlineTimer(qint64 msecs, Qt::TimerType type) noexcept
 
     The QDeadlineTimer object will be constructed with the specified timer \a type.
 
-    This constructor can be used with C++14's user-defined literals for time, such as in:
+    This constructor can be used with \l{std::literals::chrono_literals Symbol Index}
+    {C++14's user-defined literals for time}, such as in:
 
     \snippet code/src_corelib_kernel_qdeadlinetimer.cpp 3
 
@@ -325,15 +330,11 @@ void QDeadlineTimer::setPreciseRemainingTime(qint64 secs, qint64 nsecs, Qt::Time
 
     The timer type for this QDeadlineTimer object will be set to the specified \a type.
 
-    This function can be used with C++14's user-defined literals for time, such as in:
+    This function can be used with \l{std::literals::chrono_literals Symbol Index}
+    {C++14's user-defined literals for time}, such as in:
 
     \snippet code/src_corelib_kernel_qdeadlinetimer.cpp 4
 
-    \note Qt detects the necessary C++14 compiler support by way of the feature
-    test recommendations from
-    \l{https://isocpp.org/std/standing-documents/sd-6-sg10-feature-test-recommendations}
-    {C++ Committee's Standing Document 6}.
-
     \sa setDeadline(), remainingTime(), hasExpired(), isForever()
 */
 
@@ -590,9 +591,9 @@ QDeadlineTimer QDeadlineTimer::current(Qt::TimerType timerType) noexcept
 }
 
 /*!
-    \fn bool QDeadlineTimer::operator==(QDeadlineTimer d1, QDeadlineTimer d2)
+    \fn bool QDeadlineTimer::operator==(const QDeadlineTimer &lhs, const QDeadlineTimer &rhs)
 
-    Returns true if the deadline on \a d1 and the deadline in \a d2 are the
+    Returns true if the deadline on \a lhs and the deadline in \a rhs are the
     same, false otherwise. The timer type used to create the two deadlines is
     ignored. This function is equivalent to:
 
@@ -603,9 +604,9 @@ QDeadlineTimer QDeadlineTimer::current(Qt::TimerType timerType) noexcept
 */
 
 /*!
-    \fn bool QDeadlineTimer::operator!=(QDeadlineTimer d1, QDeadlineTimer d2)
+    \fn bool QDeadlineTimer::operator!=(const QDeadlineTimer &lhs, const QDeadlineTimer &rhs)
 
-    Returns true if the deadline on \a d1 and the deadline in \a d2 are
+    Returns true if the deadline on \a lhs and the deadline in \a rhs are
     different, false otherwise. The timer type used to create the two deadlines
     is ignored. This function is equivalent to:
 
@@ -616,10 +617,10 @@ QDeadlineTimer QDeadlineTimer::current(Qt::TimerType timerType) noexcept
 */
 
 /*!
-    \fn bool QDeadlineTimer::operator<(QDeadlineTimer d1, QDeadlineTimer d2)
+    \fn bool QDeadlineTimer::operator<(const QDeadlineTimer &lhs, const QDeadlineTimer &rhs)
 
-    Returns true if the deadline on \a d1 is earlier than the deadline in \a
-    d2, false otherwise. The timer type used to create the two deadlines is
+    Returns true if the deadline on \a lhs is earlier than the deadline in \a
+    rhs, false otherwise. The timer type used to create the two deadlines is
     ignored. This function is equivalent to:
 
     \snippet code/src_corelib_kernel_qdeadlinetimer.cpp 10
@@ -629,10 +630,10 @@ QDeadlineTimer QDeadlineTimer::current(Qt::TimerType timerType) noexcept
 */
 
 /*!
-    \fn bool QDeadlineTimer::operator<=(QDeadlineTimer d1, QDeadlineTimer d2)
+    \fn bool QDeadlineTimer::operator<=(const QDeadlineTimer &lhs, const QDeadlineTimer &rhs)
 
-    Returns true if the deadline on \a d1 is earlier than or the same as the
-    deadline in \a d2, false otherwise. The timer type used to create the two
+    Returns true if the deadline on \a lhs is earlier than or the same as the
+    deadline in \a rhs, false otherwise. The timer type used to create the two
     deadlines is ignored. This function is equivalent to:
 
     \snippet code/src_corelib_kernel_qdeadlinetimer.cpp 11
@@ -642,10 +643,10 @@ QDeadlineTimer QDeadlineTimer::current(Qt::TimerType timerType) noexcept
 */
 
 /*!
-    \fn bool QDeadlineTimer::operator>(QDeadlineTimer d1, QDeadlineTimer d2)
+    \fn bool QDeadlineTimer::operator>(const QDeadlineTimer &lhs, const QDeadlineTimer &rhs)
 
-    Returns true if the deadline on \a d1 is later than the deadline in \a
-    d2, false otherwise. The timer type used to create the two deadlines is
+    Returns true if the deadline on \a lhs is later than the deadline in \a
+    rhs, false otherwise. The timer type used to create the two deadlines is
     ignored. This function is equivalent to:
 
     \snippet code/src_corelib_kernel_qdeadlinetimer.cpp 12
@@ -655,10 +656,10 @@ QDeadlineTimer QDeadlineTimer::current(Qt::TimerType timerType) noexcept
 */
 
 /*!
-    \fn bool QDeadlineTimer::operator>=(QDeadlineTimer d1, QDeadlineTimer d2)
+    \fn bool QDeadlineTimer::operator>=(const QDeadlineTimer &lhs, const QDeadlineTimer &rhs)
 
-    Returns true if the deadline on \a d1 is later than or the same as the
-    deadline in \a d2, false otherwise. The timer type used to create the two
+    Returns true if the deadline on \a lhs is later than or the same as the
+    deadline in \a rhs, false otherwise. The timer type used to create the two
     deadlines is ignored. This function is equivalent to:
 
     \snippet code/src_corelib_kernel_qdeadlinetimer.cpp 13
@@ -726,8 +727,7 @@ QDeadlineTimer operator+(QDeadlineTimer dt, qint64 msecs)
 
 /*!
   \fn void QDeadlineTimer::swap(QDeadlineTimer &other)
-
-  Swaps this deadline timer with the \a other deadline timer.
+    \memberswap{deadline timer}
  */
 
 /*!
