@@ -2,7 +2,6 @@
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qjnienvironment.h"
-#include "qjniobject.h"
 #include "qjnihelpers_p.h"
 
 #include <QtCore/QThread>
@@ -30,8 +29,6 @@ QT_BEGIN_NAMESPACE
     It has not been tested for other platforms.
 */
 
-static const char qJniThreadName[] = "QtThread";
-
 class QJniEnvironmentPrivate
 {
 public:
@@ -47,47 +44,42 @@ public:
     }
 };
 
-struct QJniLocalRefDeleterPrivate
-{
-    static void cleanup(jobject obj)
-    {
-        if (!obj)
-            return;
-
-        QJniEnvironment env;
-        env->DeleteLocalRef(obj);
-    }
-};
-
-// To simplify this we only define it for jobjects.
-typedef QScopedPointer<_jobject, QJniLocalRefDeleterPrivate> QJniScopedLocalRefPrivate;
-
-
 Q_GLOBAL_STATIC(QThreadStorage<QJniEnvironmentPrivateTLS *>, jniEnvTLS)
 
 
-
 /*!
-    \fn QJniEnvironment::QJniEnvironment()
-
     Constructs a new JNI Environment object and attaches the current thread to the Java VM.
 */
 QJniEnvironment::QJniEnvironment()
     : d(new QJniEnvironmentPrivate{})
 {
+    d->jniEnv = getJniEnv();
+}
+
+/*!
+    Returns the JNIEnv pointer for the current thread.
+
+    The current thread will be attached to the Java VM.
+*/
+JNIEnv *QJniEnvironment::getJniEnv()
+{
+    JNIEnv *jniEnv = nullptr;
+
     JavaVM *vm = QtAndroidPrivate::javaVM();
-    const jint ret = vm->GetEnv((void**)&d->jniEnv, JNI_VERSION_1_6);
-    if (ret == JNI_OK) // Already attached
-        return;
+    const jint ret = vm->GetEnv((void**)&jniEnv, JNI_VERSION_1_6);
 
     if (ret == JNI_EDETACHED) { // We need to (re-)attach
-        JavaVMAttachArgs args = { JNI_VERSION_1_6, qJniThreadName, nullptr };
-        if (vm->AttachCurrentThread(&d->jniEnv, &args) != JNI_OK)
-            return;
-
-        if (!jniEnvTLS->hasLocalData()) // If we attached the thread we own it.
-            jniEnvTLS->setLocalData(new QJniEnvironmentPrivateTLS);
+        const QByteArray threadName = QThread::currentThread()->objectName().toUtf8();
+        JavaVMAttachArgs args = { JNI_VERSION_1_6,
+                                  threadName.isEmpty() ? "QtThread" : threadName.constData(),
+                                  nullptr
+                                };
+        if (vm->AttachCurrentThread(&jniEnv, &args) == JNI_OK) {
+            if (!jniEnvTLS->hasLocalData()) // If we attached the thread we own it.
+                jniEnvTLS->setLocalData(new QJniEnvironmentPrivateTLS);
+        }
     }
+    return jniEnv;
 }
 
 /*!
@@ -112,8 +104,6 @@ bool QJniEnvironment::isValid() const
 }
 
 /*!
-    \fn JNIEnv *QJniEnvironment::operator->() const
-
     Provides access to the JNI Environment's \c JNIEnv pointer.
 */
 JNIEnv *QJniEnvironment::operator->() const
@@ -122,8 +112,6 @@ JNIEnv *QJniEnvironment::operator->() const
 }
 
 /*!
-    \fn JNIEnv &QJniEnvironment::operator*() const
-
     Returns the JNI Environment's \c JNIEnv object.
 */
 JNIEnv &QJniEnvironment::operator*() const
@@ -132,8 +120,6 @@ JNIEnv &QJniEnvironment::operator*() const
 }
 
 /*!
-    \fn JNIEnv *QJniEnvironment::jniEnv() const
-
     Returns the JNI Environment's \c JNIEnv pointer.
 */
 JNIEnv *QJniEnvironment::jniEnv() const
@@ -324,8 +310,6 @@ jfieldID QJniEnvironment::findStaticField(jclass clazz, const char *fieldName, c
 */
 
 /*!
-    \fn JavaVM *QJniEnvironment::javaVM()
-
     Returns the Java VM interface for the current process. Although it might
     be possible to have multiple Java VMs per process, Android allows only one.
 
@@ -336,6 +320,30 @@ JavaVM *QJniEnvironment::javaVM()
 }
 
 /*!
+    \fn template <typename Class> bool QJniEnvironment::registerNativeMethods(std::initializer_list<JNINativeMethod> methods)
+    \overload
+
+    Registers the Java methods in \a methods with the Java class represented by
+    \c Class, and returns whether the registration was successful.
+
+    The \c Class type has to be declared within the QtJniTypes namespace using
+    the Q_DECLARE_JNI_CLASS macro. Functions that are implemented as free C or
+    C++ functions have to be declared using one of the
+    Q_DECLARE_JNI_NATIVE_METHOD macros, and passed into the registration using
+    the Q_JNI_NATIVE_METHOD macro.
+
+    \include jni.qdoc register-free-function
+
+    For functions that are implemented as static class member functions, use
+    the \l{Q_DECLARE_JNI_NATIVE_METHOD_IN_CURRENT_SCOPE}{macros for scoped
+    functions} instead.
+
+    \include jni.qdoc register-scoped-function
+*/
+
+/*!
+    \overload
+
     Registers the Java methods in the array \a methods of size \a size, each of
     which can call native C++ functions from class \a className. These methods
     must be registered before any attempt to call them.
@@ -367,6 +375,15 @@ bool QJniEnvironment::registerNativeMethods(const char *className, const JNINati
 
     return registerNativeMethods(clazz, methods, size);
 }
+
+/*!
+    \fn bool QJniEnvironment::registerNativeMethods(const char *className, std::initializer_list<JNINativeMethod> methods)
+    \overload
+
+    Registers the native functions methods in \a methods for the Java class \a className.
+    Returns \c true if the registration is successful, otherwise \c false.
+*/
+
 #if QT_DEPRECATED_SINCE(6, 2)
 /*!
     \overload
@@ -421,6 +438,14 @@ bool QJniEnvironment::registerNativeMethods(jclass clazz, const JNINativeMethod
     return true;
 }
 
+/*!
+    \fn bool QJniEnvironment::registerNativeMethods(jclass clazz, std::initializer_list<JNINativeMethod> methods)
+    \overload
+
+    Registers the native functions methods in \a methods for the Java class \a clazz.
+    Returns \c true if the registration is successful, otherwise \c false.
+*/
+
 /*!
     \enum QJniEnvironment::OutputMode
 
@@ -444,20 +469,59 @@ bool QJniEnvironment::registerNativeMethods(jclass clazz, const JNINativeMethod
 */
 bool QJniEnvironment::checkAndClearExceptions(QJniEnvironment::OutputMode outputMode)
 {
-    if (Q_UNLIKELY(d->jniEnv->ExceptionCheck())) {
-        if (outputMode != OutputMode::Silent)
-            d->jniEnv->ExceptionDescribe();
-        d->jniEnv->ExceptionClear();
-
-        return true;
-    }
-
-    return false;
+    return checkAndClearExceptions(d->jniEnv, outputMode);
 }
 
-/*!
-    \fn QJniEnvironment::checkAndClearExceptions(JNIEnv *env, OutputMode outputMode = OutputMode::Verbose)
+namespace {
+    // Any pending exception need to be cleared before calling this
+    QString exceptionMessage(JNIEnv *env, jthrowable exception)
+    {
+        if (!exception)
+            return {};
 
+        auto logError = []() {
+            qWarning() << "QJniEnvironment: a null object returned or an exception occurred while "
+                          "fetching a prior exception message";
+        };
+
+        auto checkAndClear = [env]() {
+            if (Q_UNLIKELY(env->ExceptionCheck())) {
+                env->ExceptionClear();
+                return true;
+            }
+            return false;
+        };
+
+        const jclass logClazz = env->FindClass("android/util/Log");
+        if (checkAndClear() || !logClazz) {
+            logError();
+            return {};
+        }
+
+        const jmethodID methodId = env->GetStaticMethodID(logClazz, "getStackTraceString",
+                                            "(Ljava/lang/Throwable;)Ljava/lang/String;");
+        if (checkAndClear() || !methodId) {
+            logError();
+            return {};
+        }
+
+        jvalue value;
+        value.l = static_cast<jobject>(exception);
+        const jobject messageObj = env->CallStaticObjectMethodA(logClazz, methodId, &value);
+        const jstring jmessage = static_cast<jstring>(messageObj);
+        if (checkAndClear())
+            return {};
+
+        char const *utfMessage = env->GetStringUTFChars(jmessage, 0);
+        const QString message = QString::fromUtf8(utfMessage);
+
+        env->ReleaseStringUTFChars(jmessage, utfMessage);
+
+        return message;
+    }
+} // end namespace
+
+/*!
     Cleans any pending exceptions for \a env, either silently or reporting
     stack backtrace, depending on the \a outputMode. This is useful when you
     already have a \c JNIEnv pointer such as in a native function implementation.
@@ -472,9 +536,22 @@ bool QJniEnvironment::checkAndClearExceptions(QJniEnvironment::OutputMode output
 bool QJniEnvironment::checkAndClearExceptions(JNIEnv *env, QJniEnvironment::OutputMode outputMode)
 {
     if (Q_UNLIKELY(env->ExceptionCheck())) {
-        if (outputMode != OutputMode::Silent)
-            env->ExceptionDescribe();
-        env->ExceptionClear();
+        if (outputMode == OutputMode::Verbose) {
+            if (jthrowable exception = env->ExceptionOccurred()) {
+                env->ExceptionClear();
+                const QString message = exceptionMessage(env, exception);
+                // Print to QWARN since env->ExceptionDescribe() does the same
+                if (!message.isEmpty())
+                    qWarning().noquote() << message;
+                env->DeleteLocalRef(exception);
+            } else {
+                // if the exception object is null for some reason just
+                env->ExceptionDescribe();
+                env->ExceptionClear();
+            }
+        } else {
+            env->ExceptionClear();
+        }
 
         return true;
     }
