@@ -328,10 +328,18 @@ public:
         }
     }
 
+    bool isJString(JNIEnv *env) const;
+
     QByteArray m_className;
     jobject m_jobject = nullptr;
     jclass m_jclass = nullptr;
     bool m_own_jclass = true;
+    enum class IsJString : quint8 {
+        Unknown,
+        Yes,
+        No,
+    };
+    mutable IsJString m_is_jstring = IsJString::Unknown;
 };
 
 template <typename ...Args>
@@ -351,6 +359,29 @@ static jclass getCachedClass(const QByteArray &className)
     return it != cachedClasses->constEnd() ? it.value() : nullptr;
 }
 
+
+bool QJniObjectPrivate::isJString(JNIEnv *env) const
+{
+    Q_ASSERT(m_jobject);
+    if (m_is_jstring != IsJString::Unknown)
+        return m_is_jstring == IsJString::Yes;
+
+    static constexpr auto stringSignature = QtJniTypes::Traits<jstring>::className();
+    if (!m_className.isEmpty()) {
+        m_is_jstring = m_className == stringSignature
+                     ? IsJString::Yes
+                     : IsJString::No;
+    } else {
+        const jclass strClass = getCachedClass(stringSignature.data());
+        const bool isStringInstance = strClass
+                                    ? env->IsInstanceOf(m_jobject, strClass)
+                                    : false;
+        m_is_jstring = isStringInstance ? IsJString::Yes : IsJString::No;
+    }
+
+    return m_is_jstring == IsJString::Yes;
+}
+
 /*!
     \internal
 
@@ -706,7 +737,7 @@ QJniObject::QJniObject(jobject object)
     if (!object)
         return;
 
-    JNIEnv *env = QJniEnvironment::getJniEnv();
+    JNIEnv *env = jniEnv();
     d->m_jobject = env->NewGlobalRef(object);
     jclass cls = env->GetObjectClass(object);
     d->m_jclass = static_cast<jclass>(env->NewGlobalRef(cls));
@@ -975,11 +1006,12 @@ QByteArray QJniObject::className() const
 */
 QJniObject QJniObject::callObjectMethod(const char *methodName, const char *signature, ...) const
 {
-    jmethodID id = getCachedMethodID(jniEnv(), methodName, signature);
+    JNIEnv *env = jniEnv();
+    jmethodID id = getCachedMethodID(env, methodName, signature);
     if (id) {
         va_list args;
         va_start(args, signature);
-        QJniObject res = getCleanJniObject(jniEnv()->CallObjectMethodV(d->m_jobject, id, args), jniEnv());
+        QJniObject res = getCleanJniObject(env->CallObjectMethodV(d->m_jobject, id, args), env);
         va_end(args);
         return res;
     }
@@ -1063,10 +1095,10 @@ QJniObject QJniObject::callStaticObjectMethod(jclass clazz, const char *methodNa
 QJniObject QJniObject::callStaticObjectMethod(jclass clazz, jmethodID methodId, ...)
 {
     if (clazz && methodId) {
-        QJniEnvironment env;
+        JNIEnv *env = QJniEnvironment::getJniEnv();
         va_list args;
         va_start(args, methodId);
-        QJniObject res = getCleanJniObject(env->CallStaticObjectMethodV(clazz, methodId, args), env.jniEnv());
+        QJniObject res = getCleanJniObject(env->CallStaticObjectMethodV(clazz, methodId, args), env);
         va_end(args);
         return res;
     }
@@ -1270,11 +1302,12 @@ QJniObject QJniObject::getStaticObjectField(jclass clazz, const char *fieldName,
 */
 QJniObject QJniObject::getObjectField(const char *fieldName, const char *signature) const
 {
-    jfieldID id = getCachedFieldID(jniEnv(), fieldName, signature);
+    JNIEnv *env = jniEnv();
+    jfieldID id = getCachedFieldID(env, fieldName, signature);
     if (!id)
         return QJniObject();
 
-    return getCleanJniObject(jniEnv()->GetObjectField(d->m_jobject, id), jniEnv());
+    return getCleanJniObject(env->GetObjectField(d->m_jobject, id), env);
 }
 
 /*!
@@ -1324,11 +1357,11 @@ QJniObject QJniObject::getObjectField(const char *fieldName, const char *signatu
 */
 QJniObject QJniObject::fromString(const QString &string)
 {
-    QJniEnvironment env;
-    jstring stringRef = env->NewString(reinterpret_cast<const jchar*>(string.constData()),
-                                                                      string.length());
-    QJniObject stringObject = getCleanJniObject(stringRef, env.jniEnv());
-    stringObject.d->m_className = "java/lang/String";
+    JNIEnv *env = QJniEnvironment::getJniEnv();
+    jstring stringRef = QtJniTypes::Detail::fromQString(string, env);
+    QJniObject stringObject = getCleanJniObject(stringRef, env);
+    stringObject.d->m_className = QtJniTypes::Traits<jstring>::className();
+    stringObject.d->m_is_jstring = QJniObjectPrivate::IsJString::Yes;
     return stringObject;
 }
 
@@ -1351,11 +1384,12 @@ QString QJniObject::toString() const
     if (!isValid())
         return QString();
 
-    QJniObject string = callObjectMethod<jstring>("toString");
-    const int strLength = string.jniEnv()->GetStringLength(string.object<jstring>());
-    QString res(strLength, Qt::Uninitialized);
-    string.jniEnv()->GetStringRegion(string.object<jstring>(), 0, strLength, reinterpret_cast<jchar *>(res.data()));
-    return res;
+    JNIEnv *env = jniEnv();
+    if (d->isJString(env))
+        return QtJniTypes::Detail::toQString(object<jstring>(), env);
+
+    const QJniObject string = callMethod<jstring>("toString");
+    return QtJniTypes::Detail::toQString(string.object<jstring>(), env);
 }
 
 /*!
@@ -1371,12 +1405,12 @@ QString QJniObject::toString() const
 */
 bool QJniObject::isClassAvailable(const char *className)
 {
-    QJniEnvironment env;
+    JNIEnv *env = QJniEnvironment::getJniEnv();
 
-    if (!env.jniEnv())
+    if (!env)
         return false;
 
-    return loadClass(className, env.jniEnv());
+    return loadClass(className, env);
 }
 
 /*!
@@ -1441,7 +1475,7 @@ void QJniObject::assign(jobject obj)
 
     d = QSharedPointer<QJniObjectPrivate>::create();
     if (obj) {
-        JNIEnv *env = QJniEnvironment::getJniEnv();
+        JNIEnv *env = jniEnv();
         d->m_jobject = env->NewGlobalRef(obj);
         jclass objectClass = env->GetObjectClass(obj);
         d->m_jclass = static_cast<jclass>(env->NewGlobalRef(objectClass));
