@@ -18,18 +18,27 @@
 
 #include <QtCore/private/qglobal_p.h>
 #include "QtCore/qcoreevent.h"
+#include <QtCore/qfunctionaltools_impl.h>
 #include "QtCore/qlist.h"
 #include "QtCore/qobject.h"
 #include "QtCore/qpointer.h"
-#include "QtCore/qsharedpointer.h"
 #include "QtCore/qvariant.h"
 #include "QtCore/qproperty.h"
+#include <QtCore/qshareddata.h>
 #include "QtCore/private/qproperty_p.h"
 
 #include <string>
 
 QT_BEGIN_NAMESPACE
 
+#ifdef Q_MOC_RUN
+#define QT_ANONYMOUS_PROPERTY(text) QT_ANONYMOUS_PROPERTY(text)
+#define QT_ANONYMOUS_PRIVATE_PROPERTY(d, text) QT_ANONYMOUS_PRIVATE_PROPERTY(d, text)
+#elif !defined QT_NO_META_MACROS
+#define QT_ANONYMOUS_PROPERTY(...) QT_ANNOTATE_CLASS(qt_anonymous_property, __VA_ARGS__)
+#define QT_ANONYMOUS_PRIVATE_PROPERTY(d, text) QT_ANNOTATE_CLASS2(qt_anonymous_private_property, d, text)
+#endif
+
 class QVariant;
 class QThreadData;
 class QObjectConnectionListVector;
@@ -82,7 +91,7 @@ public:
 
         QList<QByteArray> propertyNames;
         QList<QVariant> propertyValues;
-        QList<int> runningTimers;
+        QList<Qt::TimerId> runningTimers;
         QList<QPointer<QObject>> eventFilters;
         Q_OBJECT_COMPAT_PROPERTY(QObjectPrivate::ExtraData, QString, objectName,
                                  &QObjectPrivate::ExtraData::setObjectNameForwarder,
@@ -95,6 +104,7 @@ public:
         if (!extraData)
             extraData = new ExtraData(this);
     }
+    void setObjectNameWithoutBindings(const QString &name);
 
     typedef void (*StaticMetaCallFunction)(QObject *, QMetaObject::Call, int, void **);
     struct Connection;
@@ -103,6 +113,7 @@ public:
     struct ConnectionOrSignalVector;
     struct SignalVector;
     struct Sender;
+    struct TaggedSignalVector;
 
     /*
         This contains the all connections from and to an object.
@@ -130,11 +141,8 @@ public:
     void setParent_helper(QObject *);
     void moveToThread_helper();
     void setThreadData_helper(QThreadData *currentData, QThreadData *targetData, QBindingStatus *status);
-    void _q_reregisterTimers(void *pointer);
 
-    bool isSender(const QObject *receiver, const char *signal) const;
     QObjectList receiverList(const char *signal) const;
-    QObjectList senderList() const;
 
     inline void ensureConnectionData();
     inline void addConnection(int signal, Connection *c);
@@ -179,6 +187,13 @@ public:
 
     virtual std::string flagsForDumping() const;
 
+#ifndef QT_NO_DEBUG_STREAM
+    virtual void writeToDebugStream(QDebug &) const;
+#endif
+
+    QtPrivate::QPropertyAdaptorSlotObject *
+    getPropertyAdaptorSlotObject(const QMetaProperty &property);
+
 public:
     mutable ExtraData *extraData; // extra data set by the user
     // This atomic requires acquire/release semantics in a few places,
@@ -240,30 +255,10 @@ inline void QObjectPrivate::disconnectNotify(const QMetaMethod &signal)
 }
 
 namespace QtPrivate {
+inline const QObject *getQObject(const QObjectPrivate *d) { return d->q_func(); }
 
 template <typename Func>
-struct FunctionStorageByValue
-{
-    Func f;
-    Func &func() noexcept { return f; }
-};
-
-template <typename Func>
-struct FunctionStorageEmptyBaseClassOptimization : Func
-{
-    Func &func() noexcept { return *this; }
-    using Func::Func;
-};
-
-template <typename Func>
-using FunctionStorage = typename std::conditional_t<
-        std::conjunction_v<
-            std::is_empty<Func>,
-            std::negation<std::is_final<Func>>
-        >,
-        FunctionStorageEmptyBaseClassOptimization<Func>,
-        FunctionStorageByValue<Func>
-    >;
+using FunctionStorage = QtPrivate::CompactStorage<Func>;
 
 template <typename ObjPrivate> inline void assertObjectType(QObjectPrivate *d)
 {
@@ -275,18 +270,23 @@ template<typename Func, typename Args, typename R>
 class QPrivateSlotObject : public QSlotObjectBase, private FunctionStorage<Func>
 {
     typedef QtPrivate::FunctionPointer<Func> FuncType;
+#if QT_VERSION < QT_VERSION_CHECK(7, 0, 0)
     static void impl(int which, QSlotObjectBase *this_, QObject *r, void **a, bool *ret)
+#else
+    static void impl(QSlotObjectBase *this_, QObject *r, void **a, int which, bool *ret)
+#endif
     {
+        const auto that = static_cast<QPrivateSlotObject*>(this_);
         switch (which) {
             case Destroy:
-                delete static_cast<QPrivateSlotObject*>(this_);
+                delete that;
                 break;
             case Call:
-                FuncType::template call<Args, R>(static_cast<QPrivateSlotObject*>(this_)->func(),
+                FuncType::template call<Args, R>(that->object(),
                                                  static_cast<typename FuncType::Object *>(QObjectPrivate::get(r)), a);
                 break;
             case Compare:
-                *ret = *reinterpret_cast<Func *>(a) == static_cast<QPrivateSlotObject*>(this_)->func();
+                *ret = *reinterpret_cast<Func *>(a) == that->object();
                 break;
             case NumOperations: ;
         }
@@ -319,7 +319,7 @@ inline QMetaObject::Connection QObjectPrivate::connect(const typename QtPrivate:
         types = QtPrivate::ConnectionTypes<typename SignalType::Arguments>::types();
 
     return QObject::connectImpl(sender, reinterpret_cast<void **>(&signal),
-        receiverPrivate->q_ptr, reinterpret_cast<void **>(&slot),
+        QtPrivate::getQObject(receiverPrivate), reinterpret_cast<void **>(&slot),
         new QtPrivate::QPrivateSlotObject<Func2, typename QtPrivate::List_Left<typename SignalType::Arguments, SlotType::ArgumentCount>::Value,
                                         typename SignalType::ReturnType>(slot),
         type, types, &SignalType::Object::staticMetaObject);
@@ -377,6 +377,9 @@ public:
     QMetaCallEvent(QtPrivate::QSlotObjectBase *slotObj,
                    const QObject *sender, int signalId,
                    void **args, QSemaphore *semaphore);
+    QMetaCallEvent(QtPrivate::SlotObjUniquePtr slotObj,
+                   const QObject *sender, int signalId,
+                   void **args, QSemaphore *semaphore);
 
     // queued - args allocated by event, copied by caller
     QMetaCallEvent(ushort method_offset, ushort method_relative,
@@ -386,9 +389,31 @@ public:
     QMetaCallEvent(QtPrivate::QSlotObjectBase *slotObj,
                    const QObject *sender, int signalId,
                    int nargs);
+    QMetaCallEvent(QtPrivate::SlotObjUniquePtr slotObj,
+                   const QObject *sender, int signalId,
+                   int nargs);
 
     ~QMetaCallEvent() override;
 
+    template<typename ...Args>
+    static QMetaCallEvent *create(QtPrivate::QSlotObjectBase *slotObj, const QObject *sender,
+                                  int signal_index, const Args &...argv)
+    {
+        const void* const argp[] = { nullptr, std::addressof(argv)... };
+        const QMetaType metaTypes[] = { QMetaType::fromType<void>(), QMetaType::fromType<Args>()... };
+        constexpr auto argc = sizeof...(Args) + 1;
+        return create_impl(slotObj, sender, signal_index, argc, argp, metaTypes);
+    }
+    template<typename ...Args>
+    static QMetaCallEvent *create(QtPrivate::SlotObjUniquePtr slotObj, const QObject *sender,
+                                  int signal_index, const Args &...argv)
+    {
+        const void* const argp[] = { nullptr, std::addressof(argv)... };
+        const QMetaType metaTypes[] = { QMetaType::fromType<void>(), QMetaType::fromType<Args>()... };
+        constexpr auto argc = sizeof...(Args) + 1;
+        return create_impl(std::move(slotObj), sender, signal_index, argc, argp, metaTypes);
+    }
+
     inline int id() const { return d.method_offset_ + d.method_relative_; }
     inline const void * const* args() const { return d.args_; }
     inline void ** args() { return d.args_; }
@@ -398,10 +423,22 @@ public:
     virtual void placeMetaCall(QObject *object) override;
 
 private:
+    static QMetaCallEvent *create_impl(QtPrivate::QSlotObjectBase *slotObj, const QObject *sender,
+                                       int signal_index, size_t argc, const void * const argp[],
+                                       const QMetaType metaTypes[])
+    {
+        if (slotObj)
+            slotObj->ref();
+        return create_impl(QtPrivate::SlotObjUniquePtr{slotObj}, sender,
+                           signal_index, argc, argp, metaTypes);
+    }
+    static QMetaCallEvent *create_impl(QtPrivate::SlotObjUniquePtr slotObj, const QObject *sender,
+                                       int signal_index, size_t argc, const void * const argp[],
+                                       const QMetaType metaTypes[]);
     inline void allocArgs();
 
     struct Data {
-        QtPrivate::QSlotObjectBase *slotObj_;
+        QtPrivate::SlotObjUniquePtr slotObj_;
         void **args_;
         QObjectPrivate::StaticMetaCallFunction callFunction_;
         int nargs_;
@@ -416,7 +453,9 @@ class QBoolBlocker
 {
     Q_DISABLE_COPY_MOVE(QBoolBlocker)
 public:
-    explicit inline QBoolBlocker(bool &b, bool value = true) : block(b), reset(b) { block = value; }
+    Q_NODISCARD_CTOR explicit QBoolBlocker(bool &b, bool value = true)
+        : block(b), reset(b)
+    { block = value; }
     inline ~QBoolBlocker() { block = reset; }
 
 private:
@@ -424,15 +463,17 @@ private:
     bool reset;
 };
 
-void Q_CORE_EXPORT qDeleteInEventHandler(QObject *o);
-
 struct QAbstractDynamicMetaObject;
 struct Q_CORE_EXPORT QDynamicMetaObjectData
 {
     virtual ~QDynamicMetaObjectData();
     virtual void objectDestroyed(QObject *) { delete this; }
 
+#if QT_VERSION >= QT_VERSION_CHECK(7, 0, 0)
+    virtual const QMetaObject *toDynamicMetaObject(QObject *) const = 0;
+#else
     virtual QMetaObject *toDynamicMetaObject(QObject *) = 0;
+#endif
     virtual int metaCall(QObject *, QMetaObject::Call, int _id, void **) = 0;
 };
 
@@ -440,7 +481,11 @@ struct Q_CORE_EXPORT QAbstractDynamicMetaObject : public QDynamicMetaObjectData,
 {
     ~QAbstractDynamicMetaObject();
 
+#if QT_VERSION >= QT_VERSION_CHECK(7, 0, 0)
+    const QMetaObject *toDynamicMetaObject(QObject *) const override { return this; }
+#else
     QMetaObject *toDynamicMetaObject(QObject *) override { return this; }
+#endif
     virtual int createProperty(const char *, const char *) { return -1; }
     int metaCall(QObject *, QMetaObject::Call c, int _id, void **a) override
     { return metaCall(c, _id, a); }
