@@ -40,10 +40,19 @@ public:
     constexpr ErrorScope scope() const { return errorScope; }
     constexpr int error() const { return errorCode; }
 
+    constexpr bool ok() const noexcept { return errorScope == NoError; }
+    static constexpr QSystemError stdError(int error)
+    { return QSystemError(error, StandardLibraryError); }
+
     static Q_CORE_EXPORT QString string(ErrorScope errorScope, int errorCode);
     static Q_CORE_EXPORT QString stdString(int errorCode = -1);
 #ifdef Q_OS_WIN
     static Q_CORE_EXPORT QString windowsString(int errorCode = -1);
+    using HRESULT = long;
+    static Q_CORE_EXPORT QString windowsComString(HRESULT hr);
+
+    static constexpr QSystemError nativeError(int error)
+    { return QSystemError(error, NativeError); }
 #endif
 
     // data members
