@@ -333,11 +333,6 @@ static QVariant::Private clonePrivate(const QVariant::Private &other)
 
     \compares equality
 
-    Because C++ forbids unions from including types that have
-    non-default constructors or destructors, most interesting Qt
-    classes cannot be used in unions. Without QVariant, this would be
-    a problem for QObject::property() and for database work, etc.
-
     A QVariant object holds a single value of a single typeId() at a
     time. (Some types are multi-valued, for example a string list.)
     You can find out what type, T, the variant holds, convert it to a
@@ -556,7 +551,7 @@ QVariant::QVariant(const QVariant &p)
 
     \since 6.6
     Constructs a new variant containing a value of type \c T. The contained
-    value is is initialized with the arguments
+    value is initialized with the arguments
     \c{std::forward<Args>(args)...}.
 
     This constructor is provided for STL/std::any compatibility.
