@@ -1,6 +1,7 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // Copyright (C) 2015 Klaralvdalens Datakonsult AB, a KDAB Group company, info@kdab.com, author David Faure <david.faure@kdab.com>
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:critical reason:data-parser
 
 #include <qplatformdefs.h> // always first
 
@@ -13,7 +14,6 @@
 #include <private/qduplicatetracker_p.h>
 #include <private/qfilesystementry_p.h>
 
-#include <QtCore/QMap>
 #include <QtCore/QFile>
 #include <QtCore/QFileInfo>
 #include <QtCore/QStandardPaths>
@@ -72,7 +72,11 @@ bool QMimeDatabasePrivate::shouldCheck()
 
 static QStringList locateMimeDirectories()
 {
-    return QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, QStringLiteral("mime"), QStandardPaths::LocateDirectory);
+    QStringList dirs =
+            QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, QStringLiteral("mime"),
+                                      QStandardPaths::LocateDirectory);
+    dirs.append(u":/qt-project.org/qmime"_s);
+    return dirs;
 }
 
 #if defined(Q_OS_UNIX) && !defined(Q_OS_INTEGRITY)
@@ -338,13 +342,14 @@ QMimeType QMimeDatabasePrivate::findByData(const QByteArray &data, int *accuracy
         return mimeTypeForName(QStringLiteral("application/x-zerosize"));
     }
 
-    *accuracyPtr = 0;
-    QString candidate;
+    QMimeMagicResult result;
     for (const auto &provider : providers())
-        provider->findByMagic(data, accuracyPtr, &candidate);
+        provider->findByMagic(data, result);
 
-    if (!candidate.isEmpty())
-        return QMimeType(QMimeTypePrivate(candidate));
+    if (result.isValid()) {
+        *accuracyPtr = result.accuracy;
+        return QMimeType(QMimeTypePrivate(result.candidate));
+    }
 
     if (isTextFile(data)) {
         *accuracyPtr = 5;
