@@ -272,7 +272,7 @@ public:
 
 #if QT_CONFIG(library)
 
-static Q_LOGGING_CATEGORY_WITH_ENV_OVERRIDE(lcFactoryLoader, "QT_DEBUG_PLUGINS",
+Q_STATIC_LOGGING_CATEGORY_WITH_ENV_OVERRIDE(lcFactoryLoader, "QT_DEBUG_PLUGINS",
                                             "qt.core.plugin.factoryloader")
 
 namespace {
@@ -374,8 +374,12 @@ inline void QFactoryLoaderPrivate::updateSinglePath(const QString &path)
                 // whereas the one we're considering has a Qt version that fits
                 // better, we prioritize the better match.
                 int existingVersion = existingLibrary->metaData.value(QtPluginMetaDataKeys::QtVersion).toInteger();
-                if (!(existingVersion > QtVersionNoPatch && thisVersion <= QtVersionNoPatch))
-                    continue; // Existing version is a better match
+                if (existingVersion == QtVersionNoPatch)
+                    continue; // Prefer exact Qt version match
+                if (existingVersion < QtVersionNoPatch && thisVersion > QtVersionNoPatch)
+                    continue; // Better too old than too new
+                if (existingVersion < QtVersionNoPatch && thisVersion < existingVersion)
+                    continue; // Otherwise prefer newest
             }
 
             keyMapEntry = library.get();
@@ -501,8 +505,10 @@ QFactoryLoader::MetaDataList QFactoryLoader::metaData() const
     QList<QPluginParsedMetaData> metaData;
 #if QT_CONFIG(library)
     QMutexLocker locker(&d->mutex);
+    metaData.reserve(qsizetype(d->libraries.size()));
     for (const auto &library : d->libraries)
         metaData.append(library->metaData);
+    locker.unlock();
 #endif
 
     QLatin1StringView iid(d->iid.constData(), d->iid.size());
@@ -526,10 +532,12 @@ QList<QCborArray> QFactoryLoader::metaDataKeys() const
     QList<QCborArray> metaData;
 #if QT_CONFIG(library)
     QMutexLocker locker(&d->mutex);
+    metaData.reserve(qsizetype(d->libraries.size()));
     for (const auto &library : d->libraries) {
         const QCborValue md = library->metaData.value(QtPluginMetaDataKeys::MetaData);
         metaData.append(md["Keys"_L1].toArray());
     }
+    locker.unlock();
 #endif
 
     QLatin1StringView iid(d->iid.constData(), d->iid.size());
