@@ -1,6 +1,7 @@
 // Copyright (C) 2020 The Qt Company Ltd.
 // Copyright (C) 2021 Intel Corporation.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:critical reason:data-parser
 
 #ifndef QT_BOOTSTRAPPED
 #include <qcoreapplication.h>
@@ -11,23 +12,14 @@
 #include "private/qstringconverter_p.h"
 #include "private/qcborvalue_p.h"
 #include "private/qnumeric_p.h"
-
-//#define PARSER_DEBUG
-#ifdef PARSER_DEBUG
-Q_CONSTINIT static int indent = 0;
-#define BEGIN qDebug() << QByteArray(4*indent++, ' ').constData() << "pos=" << current
-#define END --indent
-#define DEBUG qDebug() << QByteArray(4*indent, ' ').constData()
-#else
-#define BEGIN if (1) ; else qDebug()
-#define END do {} while (0)
-#define DEBUG if (1) ; else qDebug()
-#endif
+#include <private/qtools_p.h>
 
 static const int nestingLimit = 1024;
 
 QT_BEGIN_NAMESPACE
 
+using namespace QtMiscUtils;
+
 // error strings for the JSON parser
 #define JSONERR_OK          QT_TRANSLATE_NOOP("QJsonParseError", "no error occurred")
 #define JSONERR_UNTERM_OBJ  QT_TRANSLATE_NOOP("QJsonParseError", "unterminated object")
@@ -50,12 +42,13 @@ QT_BEGIN_NAMESPACE
     \inmodule QtCore
     \ingroup json
     \ingroup shared
+    \ingroup qtserialization
     \reentrant
     \since 5.0
 
     \brief The QJsonParseError class is used to report errors during JSON parsing.
 
-    \sa {JSON Support in Qt}, {JSON Save Game Example}
+    \sa {JSON Support in Qt}, {Saving and Loading a Game}
 */
 
 /*!
@@ -69,7 +62,7 @@ QT_BEGIN_NAMESPACE
     \value UnterminatedArray        The array is not correctly terminated with a closing square bracket
     \value MissingValueSeparator    A colon separating keys from values inside objects is missing
     \value IllegalValue             The value is illegal
-    \value TerminationByNumber      The input stream ended while parsing a number
+    \value TerminationByNumber      The input stream ended while parsing a number (as of 6.9, this is no longer returned)
     \value IllegalNumber            The number is not well formed
     \value IllegalEscapeSequence    An illegal escape sequence occurred in the input
     \value IllegalUTF8String        An illegal UTF8 sequence occurred in the input
@@ -94,9 +87,9 @@ QT_BEGIN_NAMESPACE
 /*!
     \variable QJsonParseError::offset
 
-    Contains the offset in the input string where the parse error occurred.
+    Contains the byte offset in the UTF-8 byte array where the parse error occurred.
 
-    \sa error, errorString()
+    \sa error, errorString(), QJsonDocument::fromJson()
 */
 
 /*!
@@ -169,21 +162,20 @@ class StashedContainer
 public:
     StashedContainer(QExplicitlySharedDataPointer<QCborContainerPrivate> *container,
                      QCborValue::Type type)
-        : type(type), stashed(std::move(*container)), current(container)
+        : type(type), stashed(std::move(*container))
     {
     }
 
-    ~StashedContainer()
+    QCborValue intoValue(QExplicitlySharedDataPointer<QCborContainerPrivate> *parent)
     {
-        stashed->append(QCborContainerPrivate::makeValue(type, -1, current->take(),
-                                                         QCborContainerPrivate::MoveContainer));
-        *current = std::move(stashed);
+        std::swap(stashed, *parent);
+        return QCborContainerPrivate::makeValue(type, -1, stashed.take(),
+                                                QCborContainerPrivate::MoveContainer);
     }
 
 private:
     QCborValue::Type type;
     QExplicitlySharedDataPointer<QCborContainerPrivate> stashed;
-    QExplicitlySharedDataPointer<QCborContainerPrivate> *current;
 };
 
 Parser::Parser(const char *json, int length)
@@ -288,53 +280,46 @@ char Parser::nextToken()
 */
 QCborValue Parser::parse(QJsonParseError *error)
 {
-#ifdef PARSER_DEBUG
-    indent = 0;
-    qDebug(">>>>> parser begin");
-#endif
     eatBOM();
-    char token = nextToken();
 
-    QCborValue data;
+    char token;
+    QCborValue value;
 
-    DEBUG << Qt::hex << (uint)token;
-    if (token == BeginArray) {
-        container = new QCborContainerPrivate;
-        if (!parseArray())
-            goto error;
-        data = QCborContainerPrivate::makeValue(QCborValue::Array, -1, container.take(),
-                                                QCborContainerPrivate::MoveContainer);
-    } else if (token == BeginObject) {
-        container = new QCborContainerPrivate;
-        if (!parseObject())
-            goto error;
-        data = QCborContainerPrivate::makeValue(QCborValue::Map, -1, container.take(),
-                                                QCborContainerPrivate::MoveContainer);
-    } else {
+    if (!eatSpace()) {
         lastError = QJsonParseError::IllegalValue;
         goto error;
     }
 
+    token = *json;
+    if (token == Quote) {
+        container = new QCborContainerPrivate;
+        json++;
+        if (!parseString())
+            goto error;
+        value = QCborContainerPrivate::makeValue(QCborValue::String, 0, container.take(),
+                                                 QCborContainerPrivate::MoveContainer);
+    } else {
+        value = parseValue();
+        if (value.isUndefined())
+            goto error;
+    }
+
     eatSpace();
     if (json < end) {
         lastError = QJsonParseError::GarbageAtEnd;
         goto error;
     }
 
-    END;
     {
         if (error) {
             error->offset = 0;
             error->error = QJsonParseError::NoError;
         }
 
-        return data;
+        return value;
     }
 
 error:
-#ifdef PARSER_DEBUG
-    qDebug(">>>>> parser error");
-#endif
     container.reset();
     if (error) {
         error->offset = json - head;
@@ -440,6 +425,20 @@ static void sortContainer(QCborContainerPrivate *container)
     container->elements.erase(result.elementsIterator(), container->elements.end());
 }
 
+bool Parser::parseValueIntoContainer()
+{
+    QCborValue value = parseValue();
+    switch (value.type()) {
+    case QCborValue::Undefined:
+        return false; // error while parsing
+    case QCborValue::String:
+        break; // strings were already added
+    default:
+        container->append(std::move(value));
+    }
+
+    return true;
+}
 
 /*
     object = begin-object [ member *( value-separator member ) ]
@@ -453,8 +452,6 @@ bool Parser::parseObject()
         return false;
     }
 
-    BEGIN << "parseObject" << json;
-
     char token = nextToken();
     while (token == Quote) {
         if (!container)
@@ -471,14 +468,11 @@ bool Parser::parseObject()
         }
     }
 
-    DEBUG << "end token=" << token;
     if (token != EndObject) {
         lastError = QJsonParseError::UnterminatedObject;
         return false;
     }
 
-    END;
-
     --nestingLevel;
 
     if (container)
@@ -491,8 +485,6 @@ bool Parser::parseObject()
 */
 bool Parser::parseMember()
 {
-    BEGIN << "parseMember";
-
     if (!parseString())
         return false;
     char token = nextToken();
@@ -504,11 +496,8 @@ bool Parser::parseMember()
         lastError = QJsonParseError::UnterminatedObject;
         return false;
     }
-    if (!parseValue())
-        return false;
 
-    END;
-    return true;
+    return parseValueIntoContainer();
 }
 
 /*
@@ -516,8 +505,6 @@ bool Parser::parseMember()
 */
 bool Parser::parseArray()
 {
-    BEGIN << "parseArray";
-
     if (++nestingLevel > nestingLimit) {
         lastError = QJsonParseError::DeepNesting;
         return false;
@@ -537,8 +524,10 @@ bool Parser::parseArray()
             }
             if (!container)
                 container = new QCborContainerPrivate;
-            if (!parseValue())
+
+            if (!parseValueIntoContainer())
                 return false;
+
             char token = nextToken();
             if (token == EndArray)
                 break;
@@ -552,9 +541,6 @@ bool Parser::parseArray()
         }
     }
 
-    DEBUG << "size =" << (container ? container->elements.size() : 0);
-    END;
-
     --nestingLevel;
 
     return true;
@@ -565,98 +551,81 @@ value = false / null / true / object / array / number / string
 
 */
 
-bool Parser::parseValue()
+QCborValue Parser::parseValue()
 {
-    BEGIN << "parse Value" << json;
-
     switch (*json++) {
     case 'n':
-        if (end - json < 4) {
+        if (end - json < 3) {
             lastError = QJsonParseError::IllegalValue;
-            return false;
+            return QCborValue();
         }
         if (*json++ == 'u' &&
             *json++ == 'l' &&
             *json++ == 'l') {
-            container->append(QCborValue(QCborValue::Null));
-            DEBUG << "value: null";
-            END;
-            return true;
+            return QCborValue(QCborValue::Null);
         }
         lastError = QJsonParseError::IllegalValue;
-        return false;
+        return QCborValue();
     case 't':
-        if (end - json < 4) {
+        if (end - json < 3) {
             lastError = QJsonParseError::IllegalValue;
-            return false;
+            return QCborValue();
         }
         if (*json++ == 'r' &&
             *json++ == 'u' &&
             *json++ == 'e') {
-            container->append(QCborValue(true));
-            DEBUG << "value: true";
-            END;
-            return true;
+            return QCborValue(true);
         }
         lastError = QJsonParseError::IllegalValue;
-        return false;
+        return QCborValue();
     case 'f':
-        if (end - json < 5) {
+        if (end - json < 4) {
             lastError = QJsonParseError::IllegalValue;
-            return false;
+            return QCborValue();
         }
         if (*json++ == 'a' &&
             *json++ == 'l' &&
             *json++ == 's' &&
             *json++ == 'e') {
-            container->append(QCborValue(false));
-            DEBUG << "value: false";
-            END;
-            return true;
+            return QCborValue(false);
         }
         lastError = QJsonParseError::IllegalValue;
-        return false;
+        return QCborValue();
     case Quote: {
-        if (!parseString())
-            return false;
-        DEBUG << "value: string";
-        END;
-        return true;
+        if (parseString())
+            // strings are already added to the container
+            // callers must check for this type
+            return QCborValue(QCborValue::String);
+
+        return QCborValue();
     }
     case BeginArray: {
         StashedContainer stashedContainer(&container, QCborValue::Array);
-        if (!parseArray())
-            return false;
-        DEBUG << "value: array";
-        END;
-        return true;
+        if (parseArray())
+            return stashedContainer.intoValue(&container);
+
+        return QCborValue();
     }
     case BeginObject: {
         StashedContainer stashedContainer(&container, QCborValue::Map);
-        if (!parseObject())
-            return false;
-        DEBUG << "value: object";
-        END;
-        return true;
+        if (parseObject())
+            return stashedContainer.intoValue(&container);
+
+        return QCborValue();
     }
     case ValueSeparator:
         // Essentially missing value, but after a colon, not after a comma
         // like the other MissingObject errors.
         lastError = QJsonParseError::IllegalValue;
-        return false;
+        return QCborValue();
     case EndObject:
     case EndArray:
         lastError = QJsonParseError::MissingObject;
-        return false;
+        return QCborValue();
     default:
         --json;
-        if (!parseNumber())
-            return false;
-        DEBUG << "value: number";
-        END;
+        return parseNumber();
     }
-
-    return true;
 }
 
 
@@ -677,10 +646,8 @@ bool Parser::parseValue()
 
 */
 
-bool Parser::parseNumber()
+QCborValue Parser::parseNumber()
 {
-    BEGIN << "parseNumber" << json;
-
     const char *start = json;
     bool isInt = true;
 
@@ -692,14 +659,14 @@ bool Parser::parseNumber()
     if (json < end && *json == '0') {
         ++json;
     } else {
-        while (json < end && *json >= '0' && *json <= '9')
+        while (json < end && isAsciiDigit(*json))
             ++json;
     }
 
     // frac = decimal-point 1*DIGIT
     if (json < end && *json == '.') {
         ++json;
-        while (json < end && *json >= '0' && *json <= '9') {
+        while (json < end && isAsciiDigit(*json)) {
             isInt = isInt && *json == '0';
             ++json;
         }
@@ -711,25 +678,17 @@ bool Parser::parseNumber()
         ++json;
         if (json < end && (*json == '-' || *json == '+'))
             ++json;
-        while (json < end && *json >= '0' && *json <= '9')
+        while (json < end && isAsciiDigit(*json))
             ++json;
     }
 
-    if (json >= end) {
-        lastError = QJsonParseError::TerminationByNumber;
-        return false;
-    }
-
     const QByteArray number = QByteArray::fromRawData(start, json - start);
-    DEBUG << "numberstring" << number;
 
     if (isInt) {
         bool ok;
         qlonglong n = number.toLongLong(&ok);
         if (ok) {
-            container->append(QCborValue(n));
-            END;
-            return true;
+            return QCborValue(n);
         }
     }
 
@@ -738,17 +697,13 @@ bool Parser::parseNumber()
 
     if (!ok) {
         lastError = QJsonParseError::IllegalNumber;
-        return false;
+        return QCborValue();
     }
 
     qint64 n;
     if (convertDoubleTo(d, &n))
-        container->append(QCborValue(n));
-    else
-        container->append(QCborValue(d));
-
-    END;
-    return true;
+        return QCborValue(n);
+    return QCborValue(d);
 }
 
 /*
@@ -776,15 +731,13 @@ bool Parser::parseNumber()
 static inline bool addHexDigit(char digit, char32_t *result)
 {
     *result <<= 4;
-    if (digit >= '0' && digit <= '9')
-        *result |= (digit - '0');
-    else if (digit >= 'a' && digit <= 'f')
-        *result |= (digit - 'a') + 10;
-    else if (digit >= 'A' && digit <= 'F')
-        *result |= (digit - 'A') + 10;
-    else
-        return false;
-    return true;
+    const int h = fromHex(digit);
+    if (h != -1) {
+        *result |= h;
+        return true;
+    }
+
+    return false;
 }
 
 static inline bool scanEscapeSequence(const char *&json, const char *end, char32_t *ch)
@@ -793,7 +746,6 @@ static inline bool scanEscapeSequence(const char *&json, const char *end, char32
     if (json >= end)
         return false;
 
-    DEBUG << "scan escape" << (char)*json;
     uchar escaped = *json++;
     switch (escaped) {
     case '"':
@@ -837,7 +789,7 @@ static inline bool scanUtf8Char(const char *&json, const char *end, char32_t *re
     const auto *usrc = reinterpret_cast<const uchar *>(json);
     const auto *uend = reinterpret_cast<const uchar *>(end);
     const uchar b = *usrc++;
-    int res = QUtf8Functions::fromUtf8<QUtf8BaseTraits>(b, result, usrc, uend);
+    qsizetype res = QUtf8Functions::fromUtf8<QUtf8BaseTraits>(b, result, usrc, uend);
     if (res < 0)
         return false;
 
@@ -851,7 +803,6 @@ bool Parser::parseString()
 
     // try to parse a utf-8 string without escape sequences, and note whether it's 7bit ASCII.
 
-    BEGIN << "parse string" << json;
     bool isUtf8 = true;
     bool isAscii = true;
     while (json < end) {
@@ -872,11 +823,9 @@ bool Parser::parseString()
         }
         if (ch > 0x7f)
             isAscii = false;
-        DEBUG << "  " << ch << char(ch);
     }
     ++json;
-    DEBUG << "end of string";
-    if (json >= end) {
+    if (json > end) {
         lastError = QJsonParseError::UnterminatedString;
         return false;
     }
@@ -887,12 +836,9 @@ bool Parser::parseString()
             container->appendAsciiString(start, json - start - 1);
         else
             container->appendUtf8String(start, json - start - 1);
-        END;
         return true;
     }
 
-    DEBUG << "has escape sequences";
-
     json = start;
 
     QString ucs4;
@@ -915,14 +861,13 @@ bool Parser::parseString()
     }
     ++json;
 
-    if (json >= end) {
+    if (json > end) {
         lastError = QJsonParseError::UnterminatedString;
         return false;
     }
 
     container->appendByteData(reinterpret_cast<const char *>(ucs4.constData()), ucs4.size() * 2,
                               QCborValue::String, QtCbor::Element::StringIsUtf16);
-    END;
     return true;
 }
 
