@@ -72,10 +72,6 @@ public:
         iterator& operator++() { advance(); return *this; }
         iterator  operator++(int) { auto tmp = *this; advance(); return tmp; }
 
-        friend constexpr bool operator==(const iterator &lhs, const iterator &rhs) noexcept
-        { return lhs.current.ok == rhs.current.ok && (!lhs.current.ok || (Q_ASSERT(lhs.tokenizer == rhs.tokenizer), lhs.current.state == rhs.current.state)); }
-        friend constexpr bool operator!=(const iterator &lhs, const iterator &rhs) noexcept
-        { return !operator==(lhs, rhs); }
         friend constexpr bool operator==(const iterator &lhs, sentinel) noexcept
         { return !lhs.current.ok; }
         friend constexpr bool operator!=(const iterator &lhs, sentinel) noexcept
@@ -117,9 +113,9 @@ QT_END_INCLUDE_NAMESPACE
 namespace QtPrivate {
 namespace Tok {
 
-    constexpr qsizetype size(QChar) noexcept { return 1; }
+    constexpr qsizetype tokenSize(QChar) noexcept { return 1; }
     template <typename String>
-    constexpr qsizetype size(const String &s) noexcept { return static_cast<qsizetype>(s.size()); }
+    constexpr qsizetype tokenSize(const String &s) noexcept { return static_cast<qsizetype>(s.size()); }
 
     template <typename String> struct ViewForImpl {};
     template <> struct ViewForImpl<QStringView>   { using type = QStringView; };
@@ -390,7 +386,7 @@ auto QStringTokenizerBase<Haystack, Needle>::next(tokenizer_state state) const n
         if (state.end >= 0) {
             // token separator found => return intermediate element:
             result = m_haystack.sliced(state.start, state.end - state.start);
-            const auto ns = QtPrivate::Tok::size(m_needle);
+            const auto ns = QtPrivate::Tok::tokenSize(m_needle);
             state.start = state.end + ns;
             state.extra = (ns == 0 ? 1 : 0);
         } else {
