@@ -3,21 +3,18 @@
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qthread.h"
-#include "qthreadstorage.h"
-#include "qmutex.h"
-#include "qreadwritelock.h"
+#include "qthread_p.h"
+
 #include "qabstracteventdispatcher.h"
 #include "qbindingstorage.h"
-
-#include <qeventloop.h>
-
-#include "qthread_p.h"
 #include "private/qcoreapplication_p.h"
-
-#include <limits>
+#include "qeventloop.h"
+#include "qmutex.h"
 
 QT_BEGIN_NAMESPACE
 
+using namespace Qt::StringLiterals;
+
 /*
     QPostEventList
 */
@@ -45,29 +42,16 @@ void QPostEventList::addEvent(const QPostEvent &ev)
   QThreadData
 */
 
-QThreadData::QThreadData(int initialRefCount)
-    : _ref(initialRefCount), loopLevel(0), scopeLevel(0),
-      eventDispatcher(nullptr),
-      quitNow(false), canWait(true), isAdopted(false), requiresCoreApplication(true)
-{
-    // fprintf(stderr, "QThreadData %p created\n", this);
-}
-
 QThreadData::~QThreadData()
 {
 #if QT_CONFIG(thread)
     Q_ASSERT(_ref.loadRelaxed() == 0);
 #endif
 
-    // In the odd case that Qt is running on a secondary thread, the main
-    // thread instance will have been dereffed asunder because of the deref in
-    // QThreadData::current() and the deref in the pthread_destroy. To avoid
-    // crashing during QCoreApplicationData's global static cleanup we need to
-    // safeguard the main thread here.. This fix is a bit crude, but it solves
-    // the problem...
-    if (this->thread.loadAcquire() == QCoreApplicationPrivate::theMainThread.loadAcquire()) {
-       QCoreApplicationPrivate::theMainThread.storeRelease(nullptr);
-       QThreadData::clearCurrentThreadData();
+    if (threadId.loadAcquire() == QCoreApplicationPrivate::theMainThreadId.loadAcquire()) {
+        QCoreApplicationPrivate::theMainThread.storeRelease(nullptr);
+        QCoreApplicationPrivate::theMainThreadId.storeRelaxed(nullptr);
+        QThreadData::clearCurrentThreadData();
     }
 
     // ~QThread() sets thread to nullptr, so if it isn't null here, it's
@@ -81,10 +65,10 @@ QThreadData::~QThreadData()
     thread.storeRelease(nullptr);
     delete t;
 
-    for (int i = 0; i < postEventList.size(); ++i) {
+    for (qsizetype i = 0; i < postEventList.size(); ++i) {
         const QPostEvent &pe = postEventList.at(i);
         if (pe.event) {
-            --pe.receiver->d_func()->postedEvents;
+            pe.receiver->d_func()->postedEvents.fetchAndSubRelaxed(1);
             pe.event->m_posted = false;
             delete pe.event;
         }
@@ -93,22 +77,6 @@ QThreadData::~QThreadData()
     // fprintf(stderr, "QThreadData %p destroyed\n", this);
 }
 
-void QThreadData::ref()
-{
-#if QT_CONFIG(thread)
-    (void) _ref.ref();
-    Q_ASSERT(_ref.loadRelaxed() != 0);
-#endif
-}
-
-void QThreadData::deref()
-{
-#if QT_CONFIG(thread)
-    if (!_ref.deref())
-        delete this;
-#endif
-}
-
 QAbstractEventDispatcher *QThreadData::createEventDispatcher()
 {
     QAbstractEventDispatcher *ed = QThreadPrivate::createEventDispatcher(this);
@@ -123,11 +91,26 @@ QAbstractEventDispatcher *QThreadData::createEventDispatcher()
 QAdoptedThread::QAdoptedThread(QThreadData *data)
     : QThread(*new QThreadPrivate(data))
 {
+    // avoid a cyclic reference count: QThreadData owns this QAdoptedThread
+    // object but QObject's constructor increased the count
+    data->deref();
+
+    data->isAdopted = true;
+    Qt::HANDLE id = QThread::currentThreadId();
+    data->threadId.storeRelaxed(id);
+    if (!QCoreApplicationPrivate::theMainThreadId.loadAcquire()) {
+        // we are the main thread
+        QCoreApplicationPrivate::theMainThread.storeRelease(this);
+        QCoreApplicationPrivate::theMainThreadId.storeRelaxed(id);
+
+        // bypass the bindings because nothing can be listening yet
+        d_func()->setObjectNameWithoutBindings(u"Qt mainThread"_s);
+    }
+
     // thread should be running and not finished for the lifetime
     // of the application (even if QCoreApplication goes away)
 #if QT_CONFIG(thread)
-    d_func()->running = true;
-    d_func()->finished = false;
+    d_func()->threadState = QThreadPrivate::Running;
     init();
     d_func()->m_statusOrPendingObjects.setStatusAndClearList(
                 QtPrivate::getBindingStatus({}));
@@ -146,16 +129,15 @@ void QAdoptedThread::run()
     // this function should never be called
     qFatal("QAdoptedThread::run(): Internal error, this implementation should never be called.");
 }
+#endif
 
+#if QT_CONFIG(thread)
 /*
   QThreadPrivate
 */
 
 QThreadPrivate::QThreadPrivate(QThreadData *d)
-    : QObjectPrivate(), running(false), finished(false),
-      isInFinish(false), interruptionRequested(false),
-      exited(false), returnCode(-1),
-      stackSize(0), priority(QThread::InheritPriority), data(d)
+    : QObjectPrivate(), data(d)
 {
 
 // INTEGRITY doesn't support self-extending stack. The default stack size for
@@ -172,8 +154,6 @@ QThreadPrivate::QThreadPrivate(QThreadData *d)
 
 #if defined (Q_OS_WIN)
     handle = 0;
-    id = 0;
-    waiters = 0;
     terminationEnabled = true;
     terminatePending = false;
 #endif
@@ -256,22 +236,21 @@ QThreadPrivate::~QThreadPrivate()
     documentation for terminate() and setTerminationEnabled() for
     detailed information.
 
-    From Qt 4.8 onwards, it is possible to deallocate objects that
-    live in a thread that has just ended, by connecting the
-    finished() signal to QObject::deleteLater().
+    You often want to deallocate objects that live in a thread when
+    a thread ends. To do this, connect the finished() signal to
+    QObject::deleteLater().
 
     Use wait() to block the calling thread, until the other thread
     has finished execution (or until a specified time has passed).
 
     QThread also provides static, platform independent sleep
     functions: sleep(), msleep(), and usleep() allow full second,
-    millisecond, and microsecond resolution respectively. These
-    functions were made public in Qt 5.0.
+    millisecond, and microsecond resolution respectively.
 
     \note wait() and the sleep() functions should be unnecessary in
     general, since Qt is an event-driven framework. Instead of
     wait(), consider listening for the finished() signal. Instead of
-    the sleep() functions, consider using QTimer.
+    the sleep() functions, consider using QChronoTimer.
 
     The static functions currentThreadId() and currentThread() return
     identifiers for the currently executing thread. The former
@@ -284,11 +263,12 @@ QThreadPrivate::~QThreadPrivate()
     If you don't call \l{QObject::setObjectName()}{setObjectName()},
     the name given to your thread will be the class name of the runtime
     type of your thread object (for example, \c "RenderThread" in the case of the
-    \l{Mandelbrot Example}, as that is the name of the QThread subclass).
+    \l{Mandelbrot} example, as that is the name of the QThread subclass).
     Note that this is currently not available with release builds on Windows.
 
     \sa {Thread Support in Qt}, QThreadStorage, {Synchronizing Threads},
-        {Mandelbrot Example}, {Semaphores Example}, {Wait Conditions Example}
+        Mandelbrot, {Producer and Consumer using Semaphores},
+        {Producer and Consumer using Wait Conditions}
 */
 
 /*!
@@ -353,9 +333,12 @@ QThreadPrivate::~QThreadPrivate()
     \fn void QThread::started()
 
     This signal is emitted from the associated thread when it starts executing,
-    before the run() function is called.
+    so any slots connected to it may be called via queued invocation. Whilst
+    the event may have been posted before run() is called, any
+    \l {Signals and Slots Across Threads} {cross-thread delivery} of the signal
+    may still be pending.
 
-    \sa finished()
+    \sa run(), finished()
 */
 
 /*!
@@ -408,6 +391,23 @@ QThread *QThread::currentThread()
     return data->thread.loadAcquire();
 }
 
+/*!
+    \since 6.8
+
+    Returns whether the currently executing thread is the main thread.
+
+    The main thread is the thread in which QCoreApplication was created.
+    This is usually the thread that called the \c{main()} function, but not necessarily so.
+    It is the thread that is processing the GUI events and in which graphical objects
+    (QWindow, QWidget) can be created.
+
+    \sa currentThread(), QCoreApplication::instance()
+*/
+bool QThread::isMainThread() noexcept
+{
+    return currentThreadId() == QCoreApplicationPrivate::theMainThreadId.loadRelaxed();
+}
+
 /*!
     Constructs a new QThread to manage a new thread. The \a parent
     takes ownership of the QThread. The thread does not begin
@@ -457,13 +457,10 @@ QThread::~QThread()
     Q_D(QThread);
     {
         QMutexLocker locker(&d->mutex);
-        if (d->isInFinish) {
-            locker.unlock();
-            wait();
-            locker.relock();
-        }
-        if (d->running && !d->finished && !d->data->isAdopted)
-            qFatal("QThread: Destroyed while thread is still running");
+        if (d->threadState == QThreadPrivate::Finishing)
+            d->wait(locker, QDeadlineTimer::Forever);
+        if (d->threadState == QThreadPrivate::Running && !d->data->isAdopted)
+            qFatal("QThread: Destroyed while thread '%ls' is still running", qUtf16Printable(objectName()));
 
         d->data->thread.storeRelease(nullptr);
     }
@@ -473,33 +470,56 @@ QThread::~QThread()
     \threadsafe
     Returns \c true if the thread is finished; otherwise returns \c false.
 
+    A thread is considered finished if it has returned from the run() function
+    and the finished() signal has been emitted.
+
+//! [execution-after-finished]
+    Note the thread may still run for arbitrary amount of time after the
+    finished() signal is emitted, running clean-up operations such as executing
+    the destructors to \c{thread_local} variables. To synchronize with all
+    effects from the thread, call wait() and verify it returned true.
+//! [execution-after-finished]
+
     \sa isRunning()
 */
 bool QThread::isFinished() const
 {
     Q_D(const QThread);
     QMutexLocker locker(&d->mutex);
-    return d->finished || d->isInFinish;
+    return d->threadState >= QThreadPrivate::Finishing;
 }
 
 /*!
     \threadsafe
     Returns \c true if the thread is running; otherwise returns \c false.
 
+    A thread is considered to be running if QThread has been started with
+    start() but is not yet finished.
+
+    \include qthread.cpp execution-after-finished
+
     \sa isFinished()
 */
 bool QThread::isRunning() const
 {
     Q_D(const QThread);
     QMutexLocker locker(&d->mutex);
-    return d->running && !d->isInFinish;
+    return d->threadState == QThreadPrivate::Running;
 }
 
 /*!
-    Sets the maximum stack size for the thread to \a stackSize. If \a
-    stackSize is greater than zero, the maximum stack size is set to
-    \a stackSize bytes, otherwise the maximum stack size is
-    automatically determined by the operating system.
+    Sets the stack size for the thread to \a stackSize. If \a stackSize is
+    zero, the operating system or runtime will choose a default value.
+    Otherwise, the thread's stack size will be the value provided (which may be
+    rounded up or down).
+
+    On most operating systems, the amount of memory allocated to serve the
+    stack will initially be smaller than \a stackSize and will grow as the
+    thread uses the stack. This parameter sets the maximum size it will be
+    allowed to grow to (that is, it sets the size of the virtual memory space
+    the stack is allowed to occupy).
+
+    This function can only be called before the thread is started.
 
     \warning Most operating systems place minimum and maximum limits
     on thread stack sizes. The thread will fail to start if the stack
@@ -510,9 +530,9 @@ bool QThread::isRunning() const
 void QThread::setStackSize(uint stackSize)
 {
     Q_D(QThread);
-    QMutexLocker locker(&d->mutex);
-    Q_ASSERT_X(!d->running, "QThread::setStackSize",
+    Q_ASSERT_X(!isRunning(), "QThread::setStackSize",
                "cannot change stack size while the thread is running");
+    QMutexLocker locker(&d->mutex);
     d->stackSize = stackSize;
 }
 
@@ -529,6 +549,67 @@ uint QThread::stackSize() const
     return d->stackSize;
 }
 
+/*!
+    \enum QThread::QualityOfService
+    \since 6.9
+
+    This enum describes the quality of service level of a thread, and provides
+    the scheduler with information about the kind of work that the thread
+    performs. On platforms with different CPU profiles, or with the ability to
+    clock certain cores of a CPU down, this allows the scheduler to select or
+    configure a CPU core with suitable performance and energy characteristics
+    for the thread.
+
+    \value Auto The default value, leaving it to the scheduler to decide which
+                CPU core to run the thread on.
+    \value High The scheduler should run this thread to a high-performance CPU
+                core.
+    \value Eco  The scheduler should run this thread to an energy-efficient CPU
+                core.
+
+    \sa Priority, serviceLevel(), QThreadPool::serviceLevel()
+*/
+
+/*!
+    \since 6.9
+
+    Set the Quality of Service level of the thread object to \a serviceLevel.
+    This can only be called from the thread itself or before the thread is
+    started!
+
+    This is currently only implemented on Apple platforms, and Windows.
+    The function call will complete successfully on other platforms but will
+    not currently have any effect.
+
+    \sa serviceLevel(), QThreadPool::setServiceLevel()
+*/
+void QThread::setServiceLevel(QualityOfService serviceLevel)
+{
+    Q_D(QThread);
+    QMutexLocker locker(&d->mutex);
+    if (d->threadState != QThreadPrivate::Running) {
+        d->serviceLevel = serviceLevel;
+    } else {
+        Q_ASSERT_X(isCurrentThread(), "QThread::setServiceLevel",
+                "cannot change quality of service level of a separate, running, thread");
+        d->setQualityOfServiceLevel(serviceLevel);
+    }
+}
+
+/*!
+    \since 6.9
+
+    Return the current Quality of Service level of the thread.
+
+    \sa setServiceLevel(), QThreadPool::serviceLevel()
+*/
+QThread::QualityOfService QThread::serviceLevel() const
+{
+    Q_D(const QThread);
+    QMutexLocker locker(&d->mutex);
+    return d->serviceLevel;
+}
+
 /*!
     \internal
     Transitions BindingStatusOrList to the binding status state. If we had a list of
@@ -726,7 +807,7 @@ void QThread::setPriority(Priority priority)
     }
     Q_D(QThread);
     QMutexLocker locker(&d->mutex);
-    if (!d->running) {
+    if (d->threadState != QThreadPrivate::Running) {
         qWarning("QThread::setPriority: Cannot set priority, thread is not running");
         return;
     }
@@ -751,16 +832,28 @@ QThread::Priority QThread::priority() const
 }
 
 /*!
-    \fn void QThread::sleep(unsigned long secs)
+    \fn void QThread::sleep(std::chrono::nanoseconds nsecs)
+    \since 6.6
 
-    Forces the current thread to sleep for \a secs seconds.
+    Forces the current thread to sleep for \a nsecs.
 
     Avoid using this function if you need to wait for a given condition to
     change. Instead, connect a slot to the signal that indicates the change or
     use an event handler (see \l QObject::event()).
 
     \note This function does not guarantee accuracy. The application may sleep
-    longer than \a secs under heavy load conditions.
+    longer than \a nsecs under heavy load conditions.
+*/
+
+/*!
+    \fn void QThread::sleep(unsigned long secs)
+
+    Forces the current thread to sleep for \a secs seconds.
+
+    This is an overloaded function, equivalent to calling:
+    \code
+    QThread::sleep(std::chrono::seconds{secs});
+    \endcode
 
     \sa msleep(), usleep()
 */
@@ -768,11 +861,10 @@ QThread::Priority QThread::priority() const
 /*!
     \fn void QThread::msleep(unsigned long msecs)
 
-    Forces the current thread to sleep for \a msecs milliseconds.
-
-    Avoid using this function if you need to wait for a given condition to
-    change. Instead, connect a slot to the signal that indicates the change or
-    use an event handler (see \l QObject::event()).
+    This is an overloaded function, equivalent to calling:
+    \code
+    QThread::sleep(std::chrono::milliseconds{msecs});
+    \endcode
 
     \note This function does not guarantee accuracy. The application may sleep
     longer than \a msecs under heavy load conditions. Some OSes might round \a
@@ -784,11 +876,10 @@ QThread::Priority QThread::priority() const
 /*!
     \fn void QThread::usleep(unsigned long usecs)
 
-    Forces the current thread to sleep for \a usecs microseconds.
-
-    Avoid using this function if you need to wait for a given condition to
-    change. Instead, connect a slot to the signal that indicates the change or
-    use an event handler (see \l QObject::event()).
+    This is an overloaded function, equivalent to calling:
+    \code
+    QThread::sleep(std::chrono::microseconds{secs});
+    \endcode
 
     \note This function does not guarantee accuracy. The application may sleep
     longer than \a usecs under heavy load conditions. Some OSes might round \a
@@ -848,8 +939,27 @@ QThread::Priority QThread::priority() const
     This provides similar functionality to the POSIX \c
     pthread_join() function.
 
+    \note On some operating systems, this function may return true while the
+    operating system thread is still running and may be executing clean-up code
+    such as C++11 \c{thread_local} destructors. Operating systems where this
+    function only returns true after the OS thread has fully exited include
+    Linux, Windows, and Apple operating systems.
+
     \sa sleep(), terminate()
 */
+bool QThread::wait(QDeadlineTimer deadline)
+{
+    Q_D(QThread);
+    QMutexLocker locker(&d->mutex);
+
+    if (d->threadState == QThreadPrivate::NotStarted || d->threadState == QThreadPrivate::Finished)
+        return true;
+    if (isCurrentThread()) {
+        qWarning("QThread::wait: Thread tried to wait on itself");
+        return false;
+    }
+    return d->wait(locker, deadline);
+}
 
 /*!
     \fn void QThread::setTerminationEnabled(bool enabled)
@@ -886,6 +996,36 @@ int QThread::loopLevel() const
     return d->data->eventLoops.size();
 }
 
+/*!
+    \internal
+    Returns the thread handle of this thread.
+    It can be compared with the return value of currentThreadId().
+
+    This is used to implement isCurrentThread, and might be useful
+    for debugging (e.g. by comparing the value in gdb with info threads).
+
+    \note Thread handles of destroyed threads might be reused by the
+    operating system. Storing the return value of this function can
+    therefore give surprising results if it outlives the QThread object
+    (threads claimed to be the same even if they aren't).
+*/
+Qt::HANDLE QThreadPrivate::threadId() const noexcept
+{
+    return data->threadId.loadRelaxed();
+}
+
+/*!
+    \since 6.8
+    Returns true if this thread is QThread::currentThread.
+
+    \sa currentThreadId()
+*/
+bool QThread::isCurrentThread() const noexcept
+{
+    Q_D(const QThread);
+    return QThread::currentThreadId() == d->threadId();
+}
+
 #else // QT_CONFIG(thread)
 
 QThread::QThread(QObject *parent)
@@ -958,6 +1098,16 @@ QThread *QThread::currentThread()
     return QThreadData::current()->thread.loadAcquire();
 }
 
+bool QThread::isMainThread() noexcept
+{
+    return true;
+}
+
+bool QThread::isCurrentThread() const noexcept
+{
+    return true;
+}
+
 int QThread::idealThreadCount() noexcept
 {
     return 1;
@@ -989,20 +1139,23 @@ bool QThread::isInterruptionRequested() const
     return false;
 }
 
+void QThread::setTerminationEnabled(bool)
+{
+}
+
 // No threads: so we can just use static variables
 Q_CONSTINIT static QThreadData *data = nullptr;
 
-QThreadData *QThreadData::current(bool createIfNecessary)
+QThreadData *QThreadData::currentThreadData() noexcept
 {
-    if (!data && createIfNecessary) {
-        data = new QThreadData;
-        data->thread = new QAdoptedThread(data);
-        data->threadId.storeRelaxed(Qt::HANDLE(data->thread.loadAcquire()));
-        data->deref();
-        data->isAdopted = true;
-        if (!QCoreApplicationPrivate::theMainThread.loadAcquire())
-            QCoreApplicationPrivate::theMainThread.storeRelease(data->thread.loadRelaxed());
-    }
+    return data;
+}
+
+QThreadData *QThreadData::createCurrentThreadData()
+{
+    Q_ASSERT(!currentThreadData());
+    data = new QThreadData;
+    data->thread = new QAdoptedThread(data);
     return data;
 }
 
@@ -1077,7 +1230,7 @@ void QThread::setEventDispatcher(QAbstractEventDispatcher *eventDispatcher)
     } else {
         eventDispatcher->moveToThread(this);
         if (eventDispatcher->thread() == this) // was the move successful?
-            d->data->eventDispatcher = eventDispatcher;
+            d->data->eventDispatcher.storeRelaxed(eventDispatcher);
         else
             qWarning("QThread::setEventDispatcher: Could not move event dispatcher to target thread");
     }
@@ -1121,13 +1274,13 @@ bool QThread::event(QEvent *event)
 
 void QThread::requestInterruption()
 {
-    if (this == QCoreApplicationPrivate::theMainThread.loadAcquire()) {
+    Q_D(QThread);
+    if (d->threadId() == QCoreApplicationPrivate::theMainThreadId.loadAcquire()) {
         qWarning("QThread::requestInterruption has no effect on the main thread");
         return;
     }
-    Q_D(QThread);
     QMutexLocker locker(&d->mutex);
-    if (!d->running || d->finished || d->isInFinish)
+    if (d->threadState != QThreadPrivate::Running)
         return;
     d->interruptionRequested.store(true, std::memory_order_relaxed);
 }
@@ -1166,7 +1319,7 @@ bool QThread::isInterruptionRequested() const
         return false;
     // slow path: if the flag is set, take into account run status:
     QMutexLocker locker(&d->mutex);
-    return d->running && !d->finished && !d->isInFinish;
+    return d->threadState == QThreadPrivate::Running;
 }
 
 /*!
@@ -1191,7 +1344,6 @@ bool QThread::isInterruptionRequested() const
     \sa start()
 */
 
-#if QT_CONFIG(cxx11_future)
 class QThreadCreateThread : public QThread
 {
 public:
@@ -1220,7 +1372,6 @@ QThread *QThread::createThreadImpl(std::future<void> &&future)
 {
     return new QThreadCreateThread(std::move(future));
 }
-#endif // QT_CONFIG(cxx11_future)
 
 /*!
     \class QDaemonThread
@@ -1235,7 +1386,9 @@ QDaemonThread::QDaemonThread(QObject *parent)
 {
     // QThread::started() is emitted from the thread we start
     connect(this, &QThread::started,
-            [](){ QThreadData::current()->requiresCoreApplication = false; });
+            this,
+            [](){ QThreadData::current()->requiresCoreApplication = false; },
+            Qt::DirectConnection);
 }
 
 QDaemonThread::~QDaemonThread()
