@@ -2,6 +2,7 @@
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qthreadstorage.h"
+#include "qthreadstorage_p.h"
 
 #include "private/qcoreapplication_p.h"
 #include "qthread.h"
@@ -133,6 +134,13 @@ void **QThreadStorageData::set(void *p)
     return &value;
 }
 
+void QThreadStoragePrivate::init()
+{
+    // Make sure the Q_GLOBAL_STATIC is initialized, ensuring consistent
+    // destruction order.
+    destructors();
+}
+
 void QThreadStorageData::finish(void **p)
 {
     QList<void *> *tls = reinterpret_cast<QList<void *> *>(p);
