@@ -15,11 +15,10 @@ QT_BEGIN_NAMESPACE
     \brief The QRomanCalendar class is a shared base for calendars based on the
     ancient Roman calendar.
 
-    \section1
-
-    Calendars based on the ancient Roman calendar share the names of months,
-    whose lengths depend in a common way on whether the year is a leap
-    year. They differ in how they determine which years are leap years.
+    Calendars based on the ancient Roman calendar have several common properties:
+    they have the same names for months, the month lengths depend in a common
+    way on whether the year is a leap year. They differ in how they determine
+    which years are leap years.
 
     \sa QGregorianCalendar, QJulianCalendar, QMilankovicCalendar
 */
@@ -58,12 +57,12 @@ bool QRomanCalendar::isSolar() const
 
 const QCalendarLocale *QRomanCalendar::localeMonthIndexData() const
 {
-    return locale_data;
+    return QtPrivate::Roman::locale_data;
 }
 
 const char16_t *QRomanCalendar::localeMonthData() const
 {
-    return months_data;
+    return QtPrivate::Roman::months_data;
 }
 
 QT_END_NAMESPACE
