@@ -43,13 +43,6 @@ QAndroidTimeZonePrivate::QAndroidTimeZonePrivate(const QByteArray &ianaId)
     init(ianaId);
 }
 
-QAndroidTimeZonePrivate::QAndroidTimeZonePrivate(const QAndroidTimeZonePrivate &other)
-    : QTimeZonePrivate(other)
-{
-    androidTimeZone = other.androidTimeZone;
-    m_id = other.id();
-}
-
 QAndroidTimeZonePrivate::~QAndroidTimeZonePrivate()
 {
 }
@@ -210,14 +203,19 @@ QList<QByteArray> QAndroidTimeZonePrivate::availableTimeZoneIds() const
 {
     using namespace QtJniTypes;
 
-    const QJniArray androidAvailableIdList = TimeZone::callStaticMethod<String[]>("getAvailableIDs");
+    const QJniArray androidAvailableIdList
+        = TimeZone::callStaticMethod<String[]>("getAvailableIDs");
+    // Does not document order of entries.
 
-    QList<QByteArray> availableTimeZoneIdList;
-    availableTimeZoneIdList.reserve(androidAvailableIdList.size());
+    QList<QByteArray> result;
+    result.reserve(androidAvailableIdList.size());
     for (const auto &id : androidAvailableIdList)
-        availableTimeZoneIdList.append(id.toString().toUtf8());
+        result.append(id.toString().toUtf8());
 
-    return availableTimeZoneIdList;
+    // Sort & uniquify (just to be sure; it appears to not need this, but we can't rely on that).
+    std::sort(result.begin(), result.end());
+    result.erase(std::unique(result.begin(), result.end()), result.end());
+    return result;
 }
 
 QT_END_NAMESPACE
