@@ -89,12 +89,12 @@ static QTzTimeZoneHash loadTzTimeZones()
         return QTzTimeZoneHash();
 
     QTzTimeZoneHash zonesHash;
-    while (!tzif.atEnd()) {
-        const QByteArray line = tzif.readLine().trimmed();
-        if (line.isEmpty() || line.at(0) == '#') // Ignore empty or comment
+    QByteArray line;
+    while (tzif.readLineInto(&line)) {
+        QByteArrayView text = QByteArrayView(line).trimmed();
+        if (text.isEmpty() || text.at(0) == '#') // Ignore empty or comment
             continue;
         // Data rows are tab-separated columns Region, Coordinates, ID, Optional Comments
-        QByteArrayView text(line);
         int cut = text.indexOf('\t');
         if (Q_LIKELY(cut > 0)) {
             QTzTimeZone zone;
@@ -514,7 +514,7 @@ static inline bool asciiIsLetter(char ch)
 
 namespace {
 
-struct PosixZone
+struct PosixZone // TODO: QTBUG-112006 - make this cross-platform.
 {
     enum {
         InvalidOffset = INT_MIN,
@@ -600,7 +600,7 @@ static auto validatePosixRule(const QByteArray &posixRule, bool requireOffset =
     // See also calculatePosixTransition()'s reference.
     const auto parts = posixRule.split(',');
     const struct { bool isValid, hasDst; } fail{false, false}, good{true, parts.size() > 1};
-    const QByteArray &zoneinfo = parts.at(0);
+    const QByteArray &zoneinfo = parts.at(0).trimmed();
     if (zoneinfo.isEmpty())
         return fail;
 
@@ -656,7 +656,7 @@ static QList<QTimeZonePrivate::Data> calculatePosixTransitions(const QByteArray
 
     PosixZone stdZone, dstZone;
     {
-        const QByteArray &zoneinfo = parts.at(0);
+        const QByteArray &zoneinfo = parts.at(0).trimmed();
         const char *begin = zoneinfo.constBegin();
 
         stdZone = PosixZone::parse(begin, zoneinfo.constEnd());
@@ -1025,12 +1025,12 @@ QString QTzTimeZonePrivate::displayName(QTimeZone::TimeType timeType,
 {
     // TZ only provides C-locale abbreviations and offset:
     if (nameType != QTimeZone::LongName && isDataLocale(locale)) {
-        QTimeZonePrivate::Data tran = data(timeType);
+        Data tran = data(timeType);
         if (tran.atMSecsSinceEpoch != invalidMSecs()) {
             if (nameType == QTimeZone::ShortName)
                 return tran.abbreviation;
             // Save base class repeating the data(timeType) query:
-            if (locale.language() == QLocale::C)
+            if (isAnglicLocale(locale))
                 return isoOffsetFormat(tran.offsetFromUtc);
         }
     }
@@ -1045,7 +1045,7 @@ QString QTzTimeZonePrivate::abbreviation(qint64 atMSecsSinceEpoch) const
 
 int QTzTimeZonePrivate::offsetFromUtc(qint64 atMSecsSinceEpoch) const
 {
-    const QTimeZonePrivate::Data tran = data(atMSecsSinceEpoch);
+    const Data tran = data(atMSecsSinceEpoch);
     return tran.offsetFromUtc; // == tran.standardTimeOffset + tran.daylightTimeOffset
 }
 
@@ -1095,14 +1095,14 @@ QTimeZonePrivate::Data QTzTimeZonePrivate::data(qint64 forMSecsSinceEpoch) const
     // and we have a POSIX rule, then use it:
     if (!cached_data.m_posixRule.isEmpty()
         && (tranCache().isEmpty() || tranCache().last().atMSecsSinceEpoch < forMSecsSinceEpoch)) {
-        QList<QTimeZonePrivate::Data> posixTrans = getPosixTransitions(forMSecsSinceEpoch);
+        QList<Data> posixTrans = getPosixTransitions(forMSecsSinceEpoch);
         auto it = std::partition_point(posixTrans.cbegin(), posixTrans.cend(),
-                                       [forMSecsSinceEpoch] (const QTimeZonePrivate::Data &at) {
+                                       [forMSecsSinceEpoch] (const Data &at) {
                                            return at.atMSecsSinceEpoch <= forMSecsSinceEpoch;
                                        });
         // Use most recent, if any in the past; or the first if we have no other rules:
         if (it > posixTrans.cbegin() || (tranCache().isEmpty() && it < posixTrans.cend())) {
-            QTimeZonePrivate::Data data = *(it > posixTrans.cbegin() ? it - 1 : it);
+            Data data = *(it > posixTrans.cbegin() ? it - 1 : it);
             data.atMSecsSinceEpoch = forMSecsSinceEpoch;
             return data;
         }
@@ -1112,7 +1112,7 @@ QTimeZonePrivate::Data QTzTimeZonePrivate::data(qint64 forMSecsSinceEpoch) const
 
     // Otherwise, use the rule for the most recent or first transition:
     auto last = std::partition_point(tranCache().cbegin(), tranCache().cend(),
-                                     [forMSecsSinceEpoch] (const QTzTransitionTime &at) {
+                                     [forMSecsSinceEpoch] (QTzTransitionTime at) {
                                          return at.atMSecsSinceEpoch <= forMSecsSinceEpoch;
                                      });
     if (last == tranCache().cbegin())
@@ -1130,14 +1130,14 @@ QTimeZonePrivate::Data QTzTimeZonePrivate::data(qint64 forMSecsSinceEpoch) const
 QTimeZonePrivate::Data QTzTimeZonePrivate::data(QTimeZone::TimeType timeType) const
 {
     // True if tran is valid and has the DST-ness to match timeType:
-    const auto validMatch = [timeType](const QTimeZonePrivate::Data &tran) {
+    const auto validMatch = [timeType](const Data &tran) {
         return tran.atMSecsSinceEpoch != invalidMSecs()
             && ((timeType == QTimeZone::DaylightTime) != (tran.daylightTimeOffset == 0));
     };
 
     // Get current tran, use if suitable:
     const qint64 currentMSecs = QDateTime::currentMSecsSinceEpoch();
-    QTimeZonePrivate::Data tran = data(currentMSecs);
+    Data tran = data(currentMSecs);
     if (validMatch(tran))
         return tran;
 
@@ -1158,7 +1158,7 @@ QTimeZonePrivate::Data QTzTimeZonePrivate::data(QTimeZone::TimeType timeType) co
     // Otherwise, we can look backwards through transitions for a match; if we
     // have a POSIX rule, it clearly doesn't do DST (or we'd have hit it by
     // now), so we only need to look in the tranCache() up to now.
-    const auto untilNow = [currentMSecs](const QTzTransitionTime &at) {
+    const auto untilNow = [currentMSecs](QTzTransitionTime at) {
         return at.atMSecsSinceEpoch <= currentMSecs;
     };
     auto it = std::partition_point(tranCache().cbegin(), tranCache().cend(), untilNow);
@@ -1176,8 +1176,8 @@ QTimeZonePrivate::Data QTzTimeZonePrivate::data(QTimeZone::TimeType timeType) co
 
 bool QTzTimeZonePrivate::isDataLocale(const QLocale &locale) const
 {
-    // TZ data uses English / C locale names:
-    return locale.language() == QLocale::C || locale.language() == QLocale::English;
+    // TZ data uses en-Latn-* / C locale names:
+    return isAnglicLocale(locale);
 }
 
 bool QTzTimeZonePrivate::hasTransitions() const
@@ -1191,9 +1191,9 @@ QTimeZonePrivate::Data QTzTimeZonePrivate::nextTransition(qint64 afterMSecsSince
     // and we have a POSIX rule, then use it:
     if (!cached_data.m_posixRule.isEmpty()
         && (tranCache().isEmpty() || tranCache().last().atMSecsSinceEpoch < afterMSecsSinceEpoch)) {
-        QList<QTimeZonePrivate::Data> posixTrans = getPosixTransitions(afterMSecsSinceEpoch);
+        QList<Data> posixTrans = getPosixTransitions(afterMSecsSinceEpoch);
         auto it = std::partition_point(posixTrans.cbegin(), posixTrans.cend(),
-                                       [afterMSecsSinceEpoch] (const QTimeZonePrivate::Data &at) {
+                                       [afterMSecsSinceEpoch] (const Data &at) {
                                            return at.atMSecsSinceEpoch <= afterMSecsSinceEpoch;
                                        });
 
@@ -1202,7 +1202,7 @@ QTimeZonePrivate::Data QTzTimeZonePrivate::nextTransition(qint64 afterMSecsSince
 
     // Otherwise, if we can find a valid tran, use its rule:
     auto last = std::partition_point(tranCache().cbegin(), tranCache().cend(),
-                                     [afterMSecsSinceEpoch] (const QTzTransitionTime &at) {
+                                     [afterMSecsSinceEpoch] (QTzTransitionTime at) {
                                          return at.atMSecsSinceEpoch <= afterMSecsSinceEpoch;
                                      });
     return last != tranCache().cend() ? dataForTzTransition(*last) : Data{};
@@ -1214,9 +1214,9 @@ QTimeZonePrivate::Data QTzTimeZonePrivate::previousTransition(qint64 beforeMSecs
     // and we have a POSIX rule, then use it:
     if (!cached_data.m_posixRule.isEmpty()
         && (tranCache().isEmpty() || tranCache().last().atMSecsSinceEpoch < beforeMSecsSinceEpoch)) {
-        QList<QTimeZonePrivate::Data> posixTrans = getPosixTransitions(beforeMSecsSinceEpoch);
+        QList<Data> posixTrans = getPosixTransitions(beforeMSecsSinceEpoch);
         auto it = std::partition_point(posixTrans.cbegin(), posixTrans.cend(),
-                                       [beforeMSecsSinceEpoch] (const QTimeZonePrivate::Data &at) {
+                                       [beforeMSecsSinceEpoch] (const Data &at) {
                                            return at.atMSecsSinceEpoch < beforeMSecsSinceEpoch;
                                        });
         if (it > posixTrans.cbegin())
@@ -1227,7 +1227,7 @@ QTimeZonePrivate::Data QTzTimeZonePrivate::previousTransition(qint64 beforeMSecs
 
     // Otherwise if we can find a valid tran then use its rule
     auto last = std::partition_point(tranCache().cbegin(), tranCache().cend(),
-                                     [beforeMSecsSinceEpoch] (const QTzTransitionTime &at) {
+                                     [beforeMSecsSinceEpoch] (QTzTransitionTime at) {
                                          return at.atMSecsSinceEpoch < beforeMSecsSinceEpoch;
                                      });
     return last > tranCache().cbegin() ? dataForTzTransition(*--last) : Data{};
@@ -1345,8 +1345,8 @@ private:
         constexpr StatIdent() : m_dev(bad), m_ino(bad) {}
         StatIdent(const QT_STATBUF &data) : m_dev(data.st_dev), m_ino(data.st_ino) {}
         bool isValid() { return m_dev != bad || m_ino != bad; }
-        bool operator==(const StatIdent &other)
-        { return other.m_dev == m_dev && other.m_ino == m_ino; }
+        friend constexpr bool operator==(StatIdent lhs, StatIdent rhs)
+        { return lhs.m_dev == rhs.m_dev && lhs.m_ino == rhs.m_ino; }
     };
     StatIdent m_last;
 
