@@ -8,11 +8,14 @@
 #include "qdatetime.h"
 #include "qdebug.h"
 #include <private/qnumeric_p.h>
+#include <private/wcharhelpers_win_p.h>
 
 #include <algorithm>
 
 #include <private/qwinregistry_p.h>
 
+QT_REQUIRE_CONFIG(timezone_locale);
+
 QT_BEGIN_NAMESPACE
 
 using namespace Qt::StringLiterals;
@@ -488,13 +491,6 @@ QWinTimeZonePrivate::QWinTimeZonePrivate(const QByteArray &ianaId)
     init(ianaId);
 }
 
-QWinTimeZonePrivate::QWinTimeZonePrivate(const QWinTimeZonePrivate &other)
-                   : QTimeZonePrivate(other), m_windowsId(other.m_windowsId),
-                     m_displayName(other.m_displayName), m_standardName(other.m_standardName),
-                     m_daylightName(other.m_daylightName), m_tranRules(other.m_tranRules)
-{
-}
-
 QWinTimeZonePrivate::~QWinTimeZonePrivate()
 {
 }
@@ -542,10 +538,9 @@ void QWinTimeZonePrivate::init(const QByteArray &ianaId)
                 const int endYear = dynamicKey.value<int>(L"LastEntry").value_or(0);
                 for (int year = startYear; year <= endYear; ++year) {
                     bool ruleOk;
-                    QWinTransitionRule rule =
-                        readRegistryRule(dynamicKey,
-                                         reinterpret_cast<LPCWSTR>(QString::number(year).utf16()),
-                                         &ruleOk);
+                    QWinTransitionRule rule = readRegistryRule(dynamicKey,
+                                                               qt_castToWchar(QString::number(year)),
+                                                               &ruleOk);
                     if (ruleOk
                         // Don't repeat a recurrent rule:
                         && (m_tranRules.isEmpty()
@@ -596,27 +591,19 @@ QString QWinTimeZonePrivate::displayName(QTimeZone::TimeType timeType,
                                          QTimeZone::NameType nameType,
                                          const QLocale &locale) const
 {
-    // TODO Registry holds MUI keys, should be able to look up translations?
-    Q_UNUSED(locale);
-
-    if (nameType == QTimeZone::OffsetName) {
-        const QWinTransitionRule &rule =
-            m_tranRules.at(ruleIndexForYear(m_tranRules, QDate::currentDate().year()));
-        int offset = rule.standardTimeBias;
-        if (timeType == QTimeZone::DaylightTime)
-            offset += rule.daylightTimeBias;
-        return isoOffsetFormat(offset * -60);
+    // Registry gave us long names for the system locale:
+    if (nameType == QTimeZone::LongName && locale == QLocale::system()) {
+        switch (timeType) {
+        case  QTimeZone::DaylightTime :
+            return m_daylightName;
+        case  QTimeZone::GenericTime :
+            return m_displayName;
+        case  QTimeZone::StandardTime :
+            return m_standardName;
+        }
     }
-
-    switch (timeType) {
-    case  QTimeZone::DaylightTime :
-        return m_daylightName;
-    case  QTimeZone::GenericTime :
-        return m_displayName;
-    case  QTimeZone::StandardTime :
-        return m_standardName;
-    }
-    return m_standardName;
+    // Fall back to base class for everything else.
+    return QTimeZonePrivate::displayName(timeType, nameType, locale);;
 }
 
 QString QWinTimeZonePrivate::abbreviation(qint64 atMSecsSinceEpoch) const
@@ -871,18 +858,17 @@ QTimeZonePrivate::Data QWinTimeZonePrivate::ruleToData(const QWinTransitionRule
     tran.standardTimeOffset = rule.standardTimeBias * -60;
     if (fakeDst) {
         tran.daylightTimeOffset = 0;
-        tran.abbreviation = m_standardName;
         // Rule may claim we're in DST when it's actually a standard time change:
         if (type == QTimeZone::DaylightTime)
             tran.standardTimeOffset += rule.daylightTimeBias * -60;
     } else if (type == QTimeZone::DaylightTime) {
         tran.daylightTimeOffset = rule.daylightTimeBias * -60;
-        tran.abbreviation = m_daylightName;
     } else {
         tran.daylightTimeOffset = 0;
-        tran.abbreviation = m_standardName;
     }
     tran.offsetFromUtc = tran.standardTimeOffset + tran.daylightTimeOffset;
+    tran.abbreviation = localeName(atMSecsSinceEpoch, tran.offsetFromUtc,
+                                   type, QTimeZone::ShortName, QLocale::system());
     return tran;
 }
 
