@@ -1,8 +1,10 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // Copyright (C) 2016 Intel Corporation.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:load-external-library-via-qlibrary
 
-#include <QtCore/qglobal.h>
+#include "qdbus_symbols_p.h"
+#include <QtCore/qlatin1stringview.h>
 #if QT_CONFIG(library)
 #include <QtCore/qlibrary.h>
 #include <QtCore/private/qlocking_p.h>
@@ -11,14 +13,10 @@
 
 #ifndef QT_NO_DBUS
 
-extern "C" void dbus_shutdown();
-
 QT_BEGIN_NAMESPACE
 
 using namespace Qt::StringLiterals;
 
-void (*qdbus_resolve_me(const char *name))();
-
 #if !defined QT_LINKED_LIBDBUS
 
 #if QT_CONFIG(library)
@@ -57,7 +55,7 @@ bool qdbus_loadLibDBus()
     lib->setLoadHints(QLibrary::ExportExternalSymbolsHint); // make libdbus symbols available for apps that need more advanced control over the dbus
     triedToLoadLibrary = true;
 
-    static int majorversions[] = { 3, 2, -1 };
+    static constexpr int majorversions[] = { 3, 2, -1 };
     const QString baseNames[] = {
 #ifdef Q_OS_WIN
         "dbus-1"_L1,
@@ -91,7 +89,7 @@ bool qdbus_loadLibDBus()
 #endif
 }
 
-void (*qdbus_resolve_conditionally(const char *name))()
+QFunctionPointer qdbus_resolve_conditionally(const char *name)
 {
 #if QT_CONFIG(library)
     if (qdbus_loadLibDBus())
@@ -102,7 +100,7 @@ void (*qdbus_resolve_conditionally(const char *name))()
     return nullptr;
 }
 
-void (*qdbus_resolve_me(const char *name))()
+QFunctionPointer qdbus_resolve_me(const char *name)
 {
 #if QT_CONFIG(library)
     if (Q_UNLIKELY(!qdbus_loadLibDBus()))
