@@ -1,6 +1,7 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // Copyright (C) 2016 Intel Corporation.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qdbusmetaobject_p.h"
 
@@ -382,7 +383,7 @@ void QDBusMetaObjectGenerator::write(QDBusMetaObject *obj)
             - methods.size(); // ditto
 
     QDBusMetaObjectPrivate *header = reinterpret_cast<QDBusMetaObjectPrivate *>(idata.data());
-    static_assert(QMetaObjectPrivate::OutputRevision == 12, "QtDBus meta-object generator should generate the same version as moc");
+    static_assert(QMetaObjectPrivate::OutputRevision == 13, "QtDBus meta-object generator should generate the same version as moc");
     header->revision = QMetaObjectPrivate::OutputRevision;
     header->className = 0;
     header->classInfoCount = 0;
@@ -396,7 +397,7 @@ void QDBusMetaObjectGenerator::write(QDBusMetaObject *obj)
     header->enumeratorData = 0;
     header->constructorCount = 0;
     header->constructorData = 0;
-    header->flags = RequiresVariantMetaObject;
+    header->flags = RequiresVariantMetaObject | AllocatedMetaObject;
     header->signalCount = signals_.size();
     // These are specific to QDBusMetaObject:
     header->propertyDBusData = int(header->propertyData + header->propertyCount
