@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 
 #include "qdbusunixfiledescriptor.h"
@@ -89,9 +90,7 @@ public:
     QAtomicInt fd;
 };
 
-template<> inline
-QExplicitlySharedDataPointer<QDBusUnixFileDescriptorPrivate>::~QExplicitlySharedDataPointer()
-{ if (d && !d->ref.deref()) delete d; }
+QT_DEFINE_QESDP_SPECIALIZATION_DTOR(QDBusUnixFileDescriptorPrivate)
 
 /*!
     Constructs a QDBusUnixFileDescriptor without a wrapped file descriptor.
