@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:critical reason:data-parser
 
 //#define QIMAGEREADER_DEBUG
 
@@ -11,7 +12,6 @@
     \inmodule QtGui
     \reentrant
     \ingroup painting
-    \ingroup io
 
     The most common way to read images is through QImage and QPixmap's
     constructors, or by calling QImage::load() and
@@ -138,6 +138,9 @@ QT_BEGIN_NAMESPACE
 using namespace QImageReaderWriterHelpers;
 using namespace Qt::StringLiterals;
 
+Q_TRACE_POINT(qtgui, QImageReader_read_before_reading, QImageReader *reader, const QString &filename);
+Q_TRACE_POINT(qtgui, QImageReader_read_after_reading, QImageReader *reader, bool result);
+
 static QImageIOHandler *createReadHandlerHelper(QIODevice *device,
                                                 const QByteArray &format,
                                                 bool autoDetectImageFormat,
@@ -150,7 +153,7 @@ static QImageIOHandler *createReadHandlerHelper(QIODevice *device,
     QImageIOHandler *handler = nullptr;
     QByteArray suffix;
 
-#ifndef QT_NO_IMAGEFORMATPLUGIN
+#if QT_CONFIG(imageformatplugin)
     Q_CONSTINIT static QBasicMutex mutex;
     const auto locker = qt_scoped_lock(mutex);
 
@@ -165,93 +168,44 @@ static QImageIOHandler *createReadHandlerHelper(QIODevice *device,
              << keyMap.uniqueKeys().size() << "plugins available: " << keyMap;
 #endif
 
-    int suffixPluginIndex = -1;
-#endif // QT_NO_IMAGEFORMATPLUGIN
+    int testFormatPluginIndex = -1;
+#endif // QT_CONFIG(imageformatplugin)
 
     if (device && format.isEmpty() && autoDetectImageFormat && !ignoresFormatAndExtension) {
-        // if there's no format, see if \a device is a file, and if so, find
-        // the file suffix and find support for that format among our plugins.
-        // this allows plugins to override our built-in handlers.
+        // if there's no format, see if \a device is a file, and if so, find the file suffix
         if (QFile *file = qobject_cast<QFile *>(device)) {
+            suffix = QFileInfo(file->fileName()).suffix().toLower().toLatin1();
 #ifdef QIMAGEREADER_DEBUG
-            qDebug() << "QImageReader::createReadHandler: device is a file:" << file->fileName();
+            qDebug() << "QImageReader::createReadHandler: device file suffix:" << suffix;
 #endif
-            if (!(suffix = QFileInfo(file->fileName()).suffix().toLower().toLatin1()).isEmpty()) {
-#ifndef QT_NO_IMAGEFORMATPLUGIN
-                const int index = keyMap.key(QString::fromLatin1(suffix), -1);
-                if (index != -1) {
-#ifdef QIMAGEREADER_DEBUG
-                    qDebug() << "QImageReader::createReadHandler: suffix recognized; the"
-                             << suffix << "plugin might be able to read this";
-#endif
-                    suffixPluginIndex = index;
-                }
-#endif // QT_NO_IMAGEFORMATPLUGIN
-            }
         }
     }
 
     QByteArray testFormat = !form.isEmpty() ? form : suffix;
-
     if (ignoresFormatAndExtension)
         testFormat = QByteArray();
 
-#ifndef QT_NO_IMAGEFORMATPLUGIN
-    if (suffixPluginIndex != -1) {
-        // check if the plugin that claims support for this format can load
-        // from this device with this format.
+#if QT_CONFIG(imageformatplugin)
+    if (!testFormat.isEmpty()) {
+        // Check first support for the given format name or suffix among our plugins' registered
+        // formats. This allows plugins to override our built-in handlers.
         const qint64 pos = device ? device->pos() : 0;
-        const int index = keyMap.key(QString::fromLatin1(suffix), -1);
-        if (index != -1) {
-            QImageIOPlugin *plugin = qobject_cast<QImageIOPlugin *>(l->instance(index));
+        for (int testIndex : keyMap.keys(QLatin1StringView(testFormat))) {
+            QImageIOPlugin *plugin = qobject_cast<QImageIOPlugin *>(l->instance(testIndex));
             if (plugin && plugin->capabilities(device, testFormat) & QImageIOPlugin::CanRead) {
+#ifdef QIMAGEREADER_DEBUG
+                qDebug() << "QImageReader::createReadHandler: format" << testFormat
+                         << "recognized, the" << keyMap.values(testIndex) << "plugin can read this format.";
+#endif
                 handler = plugin->create(device, testFormat);
-#ifdef QIMAGEREADER_DEBUG
-                qDebug() << "QImageReader::createReadHandler: using the" << suffix
-                         << "plugin";
-#endif
+                testFormatPluginIndex = testIndex;
+                break;
             }
         }
         if (device && !device->isSequential())
-            device->seek(pos);
+            device->seek(pos); // Should not have moved, but guard against buggy plugins
     }
-
-    if (!handler && !testFormat.isEmpty() && !ignoresFormatAndExtension) {
-        // check if any plugin supports the format (they are not allowed to
-        // read from the device yet).
-        const qint64 pos = device ? device->pos() : 0;
-
-        if (autoDetectImageFormat) {
-            const int keyCount = keyMap.size();
-            for (int i = 0; i < keyCount; ++i) {
-                if (i != suffixPluginIndex) {
-                    QImageIOPlugin *plugin = qobject_cast<QImageIOPlugin *>(l->instance(i));
-                    if (plugin && plugin->capabilities(device, testFormat) & QImageIOPlugin::CanRead) {
-#ifdef QIMAGEREADER_DEBUG
-                        qDebug() << "QImageReader::createReadHandler: the" << keyMap.keys().at(i) << "plugin can read this format";
-#endif
-                        handler = plugin->create(device, testFormat);
-                        break;
-                    }
-                }
-            }
-        } else {
-            const int testIndex = keyMap.key(QLatin1StringView(testFormat), -1);
-            if (testIndex != -1) {
-                QImageIOPlugin *plugin = qobject_cast<QImageIOPlugin *>(l->instance(testIndex));
-                if (plugin && plugin->capabilities(device, testFormat) & QImageIOPlugin::CanRead) {
-#ifdef QIMAGEREADER_DEBUG
-                    qDebug() << "QImageReader::createReadHandler: the" << testFormat << "plugin can read this format";
-#endif
-                    handler = plugin->create(device, testFormat);
-                }
-            }
-        }
-        if (device && !device->isSequential())
-            device->seek(pos);
-    }
-
-#endif // QT_NO_IMAGEFORMATPLUGIN
+#endif // QT_CONFIG(imageformatplugin)
 
     // if we don't have a handler yet, check if we have built-in support for
     // the format
@@ -290,6 +244,30 @@ static QImageIOHandler *createReadHandlerHelper(QIODevice *device,
 #endif
     }
 
+#if QT_CONFIG(imageformatplugin)
+    if (!handler && !testFormat.isEmpty() && autoDetectImageFormat) {
+        // check if any other plugin supports the format name (they are not allowed to
+        // read from the device yet).
+        const qint64 pos = device ? device->pos() : 0;
+
+        const int keyCount = keyMap.size();
+        for (int i = 0; i < keyCount; ++i) {
+            if (i != testFormatPluginIndex) {
+                QImageIOPlugin *plugin = qobject_cast<QImageIOPlugin *>(l->instance(i));
+                if (plugin && plugin->capabilities(device, testFormat) & QImageIOPlugin::CanRead) {
+#ifdef QIMAGEREADER_DEBUG
+                    qDebug() << "QImageReader::createReadHandler: the" << keyMap.values(i) << "plugin can read this format";
+#endif
+                    handler = plugin->create(device, testFormat);
+                    break;
+                }
+            }
+        }
+        if (device && !device->isSequential())
+            device->seek(pos);  // Should not have moved, but guard against buggy plugins
+    }
+#endif // QT_CONFIG(imageformatplugin)
+
     if (handler && device && !suffix.isEmpty()) {
         Q_ASSERT(qobject_cast<QFile *>(device));
         // We have a file claiming to be of a recognized format. Now confirm that
@@ -313,18 +291,18 @@ static QImageIOHandler *createReadHandlerHelper(QIODevice *device,
         handler = nullptr;
     }
 
-#ifndef QT_NO_IMAGEFORMATPLUGIN
+#if QT_CONFIG(imageformatplugin)
     if (!handler && (autoDetectImageFormat || ignoresFormatAndExtension)) {
         // check if any of our plugins recognize the file from its contents.
         const qint64 pos = device ? device->pos() : 0;
         const int keyCount = keyMap.size();
         for (int i = 0; i < keyCount; ++i) {
-            if (i != suffixPluginIndex) {
+            if (i != testFormatPluginIndex) {
                 QImageIOPlugin *plugin = qobject_cast<QImageIOPlugin *>(l->instance(i));
                 if (plugin && plugin->capabilities(device, QByteArray()) & QImageIOPlugin::CanRead) {
                     handler = plugin->create(device, testFormat);
 #ifdef QIMAGEREADER_DEBUG
-                    qDebug() << "QImageReader::createReadHandler: the" << keyMap.value(i) << "plugin can read this data";
+                    qDebug() << "QImageReader::createReadHandler: the" << keyMap.values(i) << "plugin can read this data";
 #endif
                     break;
                 }
@@ -333,7 +311,7 @@ static QImageIOHandler *createReadHandlerHelper(QIODevice *device,
         if (device && !device->isSequential())
             device->seek(pos);
     }
-#endif // QT_NO_IMAGEFORMATPLUGIN
+#endif // QT_CONFIG(imageformatplugin)
 
     if (!handler && (autoDetectImageFormat || ignoresFormatAndExtension)) {
         // check if any of our built-in handlers recognize the file from its
@@ -461,7 +439,7 @@ public:
     static int maxAlloc;
 };
 
-int QImageReaderPrivate::maxAlloc = 128; // 128 MB is enough for an 8K 32bpp image
+int QImageReaderPrivate::maxAlloc = 256; // 256 MB is enough for an 8K 64bpp image
 
 /*!
     \internal
@@ -484,9 +462,9 @@ QImageReaderPrivate::QImageReaderPrivate(QImageReader *qq)
 */
 QImageReaderPrivate::~QImageReaderPrivate()
 {
+    delete handler;
     if (deleteDevice)
         delete device;
-    delete handler;
 }
 
 /*!
@@ -527,14 +505,15 @@ bool QImageReaderPrivate::initHandler()
         int currentExtension = 0;
 
         QString fileName = file->fileName();
+        bool fileIsOpen;
 
         do {
             file->setFileName(fileName + u'.'
                     + QLatin1StringView(extensions.at(currentExtension++).constData()));
-            file->open(QIODevice::ReadOnly);
-        } while (!file->isOpen() && currentExtension < extensions.size());
+            fileIsOpen = file->open(QIODevice::ReadOnly);
+        } while (!fileIsOpen && currentExtension < extensions.size());
 
-        if (!device->isOpen()) {
+        if (!fileIsOpen) {
             imageReaderError = QImageReader::FileNotFoundError;
             errorString = QImageReader::tr("File not found");
             file->setFileName(fileName); // restore the old file name
@@ -556,7 +535,7 @@ bool QImageReaderPrivate::initHandler()
 */
 void QImageReaderPrivate::getText()
 {
-    if (text.isEmpty() && initHandler() && handler->supportsOption(QImageIOHandler::Description))
+    if (text.isEmpty() && q->supportsOption(QImageIOHandler::Description))
         text = qt_getImageTextFromDescription(handler->option(QImageIOHandler::Description).toString());
 }
 
@@ -746,12 +725,12 @@ bool QImageReader::decideFormatFromContent() const
 */
 void QImageReader::setDevice(QIODevice *device)
 {
+    delete d->handler;
+    d->handler = nullptr;
     if (d->device && d->deleteDevice)
         delete d->device;
     d->device = device;
     d->deleteDevice = false;
-    delete d->handler;
-    d->handler = nullptr;
     d->text.clear();
 }
 
@@ -797,8 +776,6 @@ QString QImageReader::fileName() const
 }
 
 /*!
-    \since 4.2
-
     Sets the quality setting of the image format to \a quality.
 
     Some image formats, in particular lossy ones, entail a tradeoff between a)
@@ -822,8 +799,6 @@ void QImageReader::setQuality(int quality)
 }
 
 /*!
-    \since 4.2
-
     Returns the quality setting of the image format.
 
     \sa setQuality()
@@ -846,18 +821,13 @@ int QImageReader::quality() const
 */
 QSize QImageReader::size() const
 {
-    if (!d->initHandler())
-        return QSize();
-
-    if (d->handler->supportsOption(QImageIOHandler::Size))
+    if (supportsOption(QImageIOHandler::Size))
         return d->handler->option(QImageIOHandler::Size).toSize();
 
     return QSize();
 }
 
 /*!
-    \since 4.5
-
     Returns the format of the image, without actually reading the image
     contents. The format describes the image format \l QImageReader::read()
     returns, not the format of the actual image.
@@ -869,18 +839,13 @@ QSize QImageReader::size() const
 */
 QImage::Format QImageReader::imageFormat() const
 {
-    if (!d->initHandler())
-        return QImage::Format_Invalid;
-
-    if (d->handler->supportsOption(QImageIOHandler::ImageFormat))
+    if (supportsOption(QImageIOHandler::ImageFormat))
         return (QImage::Format)d->handler->option(QImageIOHandler::ImageFormat).toInt();
 
     return QImage::Format_Invalid;
 }
 
 /*!
-    \since 4.1
-
     Returns the text keys for this image. You can use
     these keys with text() to list the image text for
     a certain key.
@@ -897,8 +862,6 @@ QStringList QImageReader::textKeys() const
 }
 
 /*!
-    \since 4.1
-
     Returns the image text associated with \a key.
 
     Support for this option is implemented through
@@ -944,6 +907,10 @@ QRect QImageReader::clipRect() const
     support scaling), QImageReader will use QImage::scale() with
     Qt::SmoothScaling.
 
+    If only one dimension is set in \a size, the other one will be
+    computed from the image's \l {size()} {natural size} so as to
+    maintain the aspect ratio.
+
     \sa scaledSize(), setClipRect(), setScaledClipRect()
 */
 void QImageReader::setScaledSize(const QSize &size)
@@ -984,8 +951,6 @@ QRect QImageReader::scaledClipRect() const
 }
 
 /*!
-    \since 4.1
-
     Sets the background color to \a color.
     Image formats that support this operation are expected to
     initialize the background to \a color before reading an image.
@@ -994,15 +959,11 @@ QRect QImageReader::scaledClipRect() const
 */
 void QImageReader::setBackgroundColor(const QColor &color)
 {
-    if (!d->initHandler())
-        return;
-    if (d->handler->supportsOption(QImageIOHandler::BackgroundColor))
+    if (supportsOption(QImageIOHandler::BackgroundColor))
         d->handler->setOption(QImageIOHandler::BackgroundColor, color);
 }
 
 /*!
-    \since 4.1
-
     Returns the background color that's used when reading an image.
     If the image format does not support setting the background color
     an invalid color is returned.
@@ -1011,16 +972,12 @@ void QImageReader::setBackgroundColor(const QColor &color)
 */
 QColor QImageReader::backgroundColor() const
 {
-    if (!d->initHandler())
-        return QColor();
-    if (d->handler->supportsOption(QImageIOHandler::BackgroundColor))
+    if (supportsOption(QImageIOHandler::BackgroundColor))
         return qvariant_cast<QColor>(d->handler->option(QImageIOHandler::BackgroundColor));
     return QColor();
 }
 
 /*!
-    \since 4.1
-
     Returns \c true if the image format supports animation;
     otherwise, false is returned.
 
@@ -1028,9 +985,7 @@ QColor QImageReader::backgroundColor() const
 */
 bool QImageReader::supportsAnimation() const
 {
-    if (!d->initHandler())
-        return false;
-    if (d->handler->supportsOption(QImageIOHandler::Animation))
+    if (supportsOption(QImageIOHandler::Animation))
         return d->handler->option(QImageIOHandler::Animation).toBool();
     return false;
 }
@@ -1042,10 +997,7 @@ bool QImageReader::supportsAnimation() const
 */
 QByteArray QImageReader::subType() const
 {
-    if (!d->initHandler())
-        return QByteArray();
-
-    if (d->handler->supportsOption(QImageIOHandler::SubType))
+    if (supportsOption(QImageIOHandler::SubType))
         return d->handler->option(QImageIOHandler::SubType).toByteArray();
     return QByteArray();
 }
@@ -1057,10 +1009,7 @@ QByteArray QImageReader::subType() const
 */
 QList<QByteArray> QImageReader::supportedSubTypes() const
 {
-    if (!d->initHandler())
-        return QList<QByteArray>();
-
-    if (d->handler->supportsOption(QImageIOHandler::SupportedSubTypes))
+    if (supportsOption(QImageIOHandler::SupportedSubTypes))
         return qvariant_cast<QList<QByteArray> >(d->handler->option(QImageIOHandler::SupportedSubTypes));
     return QList<QByteArray>();
 }
@@ -1076,7 +1025,7 @@ QList<QByteArray> QImageReader::supportedSubTypes() const
 QImageIOHandler::Transformations QImageReader::transformation() const
 {
     int option = QImageIOHandler::TransformationNone;
-    if (d->initHandler() && d->handler->supportsOption(QImageIOHandler::ImageTransformation))
+    if (supportsOption(QImageIOHandler::ImageTransformation))
         option = d->handler->option(QImageIOHandler::ImageTransformation).toInt();
     return QImageIOHandler::Transformations(option);
 }
@@ -1194,20 +1143,39 @@ bool QImageReader::read(QImage *image)
     if (!d->initHandler())
         return false;
 
-    // set the handler specific options.
-    if (d->handler->supportsOption(QImageIOHandler::ScaledSize) && d->scaledSize.isValid()) {
-        if ((d->handler->supportsOption(QImageIOHandler::ClipRect) && !d->clipRect.isNull())
-            || d->clipRect.isNull()) {
-            // Only enable the ScaledSize option if there is no clip rect, or
-            // if the handler also supports ClipRect.
-            d->handler->setOption(QImageIOHandler::ScaledSize, d->scaledSize);
+    QSize scaledSize = d->scaledSize;
+    if ((scaledSize.width() <= 0 && scaledSize.height() > 0) ||
+        (scaledSize.height() <= 0 && scaledSize.width() > 0)) {
+        // if only one dimension is given, let's try to calculate the second one
+        // based on the original image size and maintaining the aspect ratio
+        if (const QSize originalSize = size(); !originalSize.isEmpty()) {
+            if (scaledSize.width() <= 0) {
+                const auto ratio = qreal(scaledSize.height()) / originalSize.height();
+                scaledSize.setWidth(qRound(originalSize.width() * ratio));
+            } else {
+                const auto ratio = qreal(scaledSize.width()) / originalSize.width();
+                scaledSize.setHeight(qRound(originalSize.height() * ratio));
+            }
         }
     }
-    if (d->handler->supportsOption(QImageIOHandler::ClipRect) && !d->clipRect.isNull())
+
+    const bool supportScaledSize = supportsOption(QImageIOHandler::ScaledSize) && scaledSize.isValid();
+    const bool supportClipRect = supportsOption(QImageIOHandler::ClipRect) && !d->clipRect.isNull();
+    const bool supportScaledClipRect = supportsOption(QImageIOHandler::ScaledClipRect) && !d->scaledClipRect.isNull();
+
+    // set the handler specific options.
+    if (supportScaledSize) {
+        if (supportClipRect || d->clipRect.isNull()) {
+            // Only enable the ScaledSize option if there is no clip rect, or
+            // if the handler also supports ClipRect.
+            d->handler->setOption(QImageIOHandler::ScaledSize, scaledSize);
+        }
+    }
+    if (supportClipRect)
         d->handler->setOption(QImageIOHandler::ClipRect, d->clipRect);
-    if (d->handler->supportsOption(QImageIOHandler::ScaledClipRect) && !d->scaledClipRect.isNull())
+    if (supportScaledClipRect)
         d->handler->setOption(QImageIOHandler::ScaledClipRect, d->scaledClipRect);
-    if (d->handler->supportsOption(QImageIOHandler::Quality))
+    if (supportsOption(QImageIOHandler::Quality))
         d->handler->setOption(QImageIOHandler::Quality, d->quality);
 
     // read the image
@@ -1228,9 +1196,9 @@ bool QImageReader::read(QImage *image)
 
     // provide default implementations for any unsupported image
     // options
-    if (d->handler->supportsOption(QImageIOHandler::ClipRect) && !d->clipRect.isNull()) {
-        if (d->handler->supportsOption(QImageIOHandler::ScaledSize) && d->scaledSize.isValid()) {
-            if (d->handler->supportsOption(QImageIOHandler::ScaledClipRect) && !d->scaledClipRect.isNull()) {
+    if (supportClipRect) {
+        if (supportScaledSize) {
+            if (supportScaledClipRect) {
                 // all features are supported by the handler; nothing to do.
             } else {
                 // the image is already scaled, so apply scaled clipping.
@@ -1238,12 +1206,12 @@ bool QImageReader::read(QImage *image)
                     *image = image->copy(d->scaledClipRect);
             }
         } else {
-            if (d->handler->supportsOption(QImageIOHandler::ScaledClipRect) && !d->scaledClipRect.isNull()) {
+            if (supportScaledClipRect) {
                 // supports scaled clipping but not scaling, most
                 // likely a broken handler.
             } else {
-                if (d->scaledSize.isValid()) {
-                    *image = image->scaled(d->scaledSize, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
+                if (scaledSize.isValid()) {
+                    *image = image->scaled(scaledSize, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
                 }
                 if (d->scaledClipRect.isValid()) {
                     *image = image->copy(d->scaledClipRect);
@@ -1251,8 +1219,8 @@ bool QImageReader::read(QImage *image)
             }
         }
     } else {
-        if (d->handler->supportsOption(QImageIOHandler::ScaledSize) && d->scaledSize.isValid() && d->clipRect.isNull()) {
-            if (d->handler->supportsOption(QImageIOHandler::ScaledClipRect) && !d->scaledClipRect.isNull()) {
+        if (supportScaledSize && d->clipRect.isNull()) {
+            if (supportScaledClipRect) {
                 // nothing to do (ClipRect is ignored!)
             } else {
                 // provide all workarounds.
@@ -1261,7 +1229,7 @@ bool QImageReader::read(QImage *image)
                 }
             }
         } else {
-            if (d->handler->supportsOption(QImageIOHandler::ScaledClipRect) && !d->scaledClipRect.isNull()) {
+            if (supportScaledClipRect) {
                 // this makes no sense; a handler that supports
                 // ScaledClipRect but not ScaledSize is broken, and we
                 // can't work around it.
@@ -1269,8 +1237,8 @@ bool QImageReader::read(QImage *image)
                 // provide all workarounds.
                 if (d->clipRect.isValid())
                     *image = image->copy(d->clipRect);
-                if (d->scaledSize.isValid())
-                    *image = image->scaled(d->scaledSize, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
+                if (scaledSize.isValid())
+                    *image = image->scaled(scaledSize, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
                 if (d->scaledClipRect.isValid())
                     *image = image->copy(d->scaledClipRect);
             }
@@ -1424,8 +1392,6 @@ QString QImageReader::errorString() const
 }
 
 /*!
-    \since 4.2
-
     Returns \c true if the reader supports \a option; otherwise returns
     false.
 
@@ -1499,7 +1465,7 @@ QByteArray QImageReader::imageFormat(QIODevice *device)
     Reading and writing SVG files is supported through the \l{Qt SVG} module.
     The \l{Qt Image Formats} module provides support for additional image formats.
 
-    Note that the QApplication instance must be created before this function is
+    Note that the QCoreApplication instance must be created before this function is
     called.
 
     \sa setFormat(), QImageWriter::supportedImageFormats(), QImageIOPlugin
@@ -1570,6 +1536,8 @@ int QImageReader::allocationLimit()
     loading corrupt image files. It is normally not needed to change it. The
     default limit is large enough for all commonly used image sizes.
 
+    At runtime, this value may be overridden by the environment variable \c QT_IMAGEIO_MAXALLOC.
+
     \note The memory requirements are calculated for a minimum of 32 bits per pixel, since Qt will
     typically convert an image to that depth when it is used in GUI. This means that the effective
     allocation limit is significantly smaller than \a mbLimit when reading 1 bpp and 8 bpp images.
