@@ -6,7 +6,6 @@
 #include <QtCore/qdatetime.h>
 #include <QtCore/qlist.h>
 #include <QtCore/qmap.h>
-#include <QtCore/qpair.h>
 #include <QtCore/qvariant.h>
 #include <QtCore/qstringlist.h>
 #include <QtCore/qbitarray.h>
@@ -19,6 +18,11 @@
 
 QT_BEGIN_NAMESPACE
 
+// Used internally to store the flags
+namespace {
+constexpr auto DataFlagsRole = Qt::ItemDataRole(Qt::UserRole - 1);
+}
+
 static inline QString qStandardItemModelDataListMimeType()
 {
     return QStringLiteral("application/x-qstandarditemmodeldatalist");
@@ -30,8 +34,8 @@ public:
     inline QStandardItemModelLessThan()
         { }
 
-    inline bool operator()(const QPair<QStandardItem*, int> &l,
-                           const QPair<QStandardItem*, int> &r) const
+    inline bool operator()(const std::pair<QStandardItem*, int> &l,
+                           const std::pair<QStandardItem*, int> &r) const
     {
         return *(l.first) < *(r.first);
     }
@@ -43,8 +47,8 @@ public:
     inline QStandardItemModelGreaterThan()
         { }
 
-    inline bool operator()(const QPair<QStandardItem*, int> &l,
-                           const QPair<QStandardItem*, int> &r) const
+    inline bool operator()(const std::pair<QStandardItem*, int> &l,
+                           const std::pair<QStandardItem*, int> &r) const
     {
         return *(r.first) < *(l.first);
     }
@@ -53,16 +57,16 @@ public:
 /*!
   \internal
 */
-QPair<int, int> QStandardItemPrivate::position() const
+std::pair<int, int> QStandardItemPrivate::position() const
 {
     if (QStandardItem *par = parent) {
         int idx = par->d_func()->childIndex(q_func());
         if (idx == -1)
-            return QPair<int, int>(-1, -1);
-        return QPair<int, int>(idx / par->columnCount(), idx % par->columnCount());
+            return std::pair<int, int>(-1, -1);
+        return std::pair<int, int>(idx / par->columnCount(), idx % par->columnCount());
     }
     // ### support header items?
-    return QPair<int, int>(-1, -1);
+    return std::pair<int, int>(-1, -1);
 }
 
 /*!
@@ -275,14 +279,12 @@ void QStandardItemPrivate::setItemData(const QMap<int, QVariant> &roles)
 /*!
   \internal
 */
-const QMap<int, QVariant> QStandardItemPrivate::itemData() const
+QMap<int, QVariant> QStandardItemPrivate::itemData() const
 {
     QMap<int, QVariant> result;
-    QList<QStandardItemData>::const_iterator it;
-    for (it = values.cbegin(); it != values.cend(); ++it){
-        // Qt::UserRole - 1 is used internally to store the flags
-        if (it->role != Qt::UserRole - 1)
-            result.insert(it->role, it->value);
+    for (const auto &data : values) {
+        if (data.role != DataFlagsRole)
+            result.insert(data.role, data.value);
     }
     return result;
 }
@@ -296,7 +298,7 @@ void QStandardItemPrivate::sortChildren(int column, Qt::SortOrder order)
     if (column >= columnCount())
         return;
 
-    QList<QPair<QStandardItem*, int> > sortable;
+    QList<std::pair<QStandardItem*, int> > sortable;
     QList<int> unsortable;
 
     sortable.reserve(rowCount());
@@ -305,7 +307,7 @@ void QStandardItemPrivate::sortChildren(int column, Qt::SortOrder order)
     for (int row = 0; row < rowCount(); ++row) {
         QStandardItem *itm = q->child(row, column);
         if (itm)
-            sortable.append(QPair<QStandardItem*,int>(itm, row));
+            sortable.emplace_back(itm, row);
         else
             unsortable.append(row);
     }
@@ -448,8 +450,7 @@ bool QStandardItemPrivate::insertRows(int row, const QList<QStandardItem*> &item
     }
     for (int i = 0; i < items.size(); ++i) {
         QStandardItem *item = items.at(i);
-        item->d_func()->model = model;
-        item->d_func()->parent = q;
+        item->d_func()->setParentAndModel(q, model);
         int index = childIndex(i + row, 0);
         children.replace(index, item);
         if (item)
@@ -869,9 +870,15 @@ QStandardItem *QStandardItem::parent() const
     Sets the item's data for the given \a role to the specified \a value.
 
     If you subclass QStandardItem and reimplement this function, your
-    reimplementation should call emitDataChanged() if you do not call
-    the base implementation of setData(). This will ensure that e.g.
-    views using the model are notified of the changes.
+    reimplementation should:
+    \list
+    \li call emitDataChanged() if you do not call the base implementation of
+        setData(). This will ensure that e.g. views using the model are notified
+        of the changes
+    \li call the base implementation for roles you don't handle, otherwise
+        setting flags, e.g. by calling setFlags(), setCheckable(), setEditable()
+        etc., will not work.
+    \endlist
 
     \note The default implementation treats Qt::EditRole and Qt::DisplayRole
     as referring to the same data.
@@ -925,6 +932,11 @@ void QStandardItem::clearData()
     Returns the item's data for the given \a role, or an invalid
     QVariant if there is no data for the role.
 
+    If you reimplement this function, your reimplementation should call
+    the base implementation for roles you don't handle, otherwise getting
+    flags, e.g. by calling flags(), isCheckable(), isEditable() etc.,
+    will not work.
+
     \note The default implementation treats Qt::EditRole and Qt::DisplayRole
     as referring to the same data.
 */
@@ -984,7 +996,7 @@ void QStandardItem::emitDataChanged()
 */
 void QStandardItem::setFlags(Qt::ItemFlags flags)
 {
-    setData((int)flags, Qt::UserRole - 1);
+    setData((int)flags, DataFlagsRole);
 }
 
 /*!
@@ -999,7 +1011,7 @@ void QStandardItem::setFlags(Qt::ItemFlags flags)
 */
 Qt::ItemFlags QStandardItem::flags() const
 {
-    QVariant v = data(Qt::UserRole - 1);
+    QVariant v = data(DataFlagsRole);
     if (!v.isValid())
         return (Qt::ItemIsSelectable|Qt::ItemIsEnabled|Qt::ItemIsEditable
                 |Qt::ItemIsDragEnabled|Qt::ItemIsDropEnabled);
@@ -1472,7 +1484,7 @@ void QStandardItem::setDropEnabled(bool dropEnabled)
 int QStandardItem::row() const
 {
     Q_D(const QStandardItem);
-    QPair<int, int> pos = d->position();
+    std::pair<int, int> pos = d->position();
     return pos.first;
 }
 
@@ -1485,7 +1497,7 @@ int QStandardItem::row() const
 int QStandardItem::column() const
 {
     Q_D(const QStandardItem);
-    QPair<int, int> pos = d->position();
+    std::pair<int, int> pos = d->position();
     return pos.second;
 }
 
@@ -1859,28 +1871,30 @@ QStandardItem *QStandardItem::takeChild(int row, int column)
     if (index != -1) {
         QModelIndex changedIdx;
         item = d->children.at(index);
-        if (item && d->model) {
+        if (item) {
             QStandardItemPrivate *const item_d = item->d_func();
-            const int savedRows = item_d->rows;
-            const int savedCols = item_d->columns;
-            const QVector<QStandardItem*> savedChildren = item_d->children;
-            if (savedRows > 0) {
-                d->model->d_func()->rowsAboutToBeRemoved(item, 0, savedRows - 1);
-                item_d->rows = 0;
-                item_d->children = QVector<QStandardItem*>(); //slightly faster than clear
-                d->model->d_func()->rowsRemoved(item, 0, savedRows);
-            }
-            if (savedCols > 0) {
-                d->model->d_func()->columnsAboutToBeRemoved(item, 0, savedCols - 1);
-                item_d->columns = 0;
-                if (!item_d->children.isEmpty())
+            if (d->model) {
+                QStandardItemModelPrivate *const model_d = d->model->d_func();
+                const int savedRows = item_d->rows;
+                const int savedCols = item_d->columns;
+                const QVector<QStandardItem*> savedChildren = item_d->children;
+                if (savedRows > 0) {
+                    model_d->rowsAboutToBeRemoved(item, 0, savedRows - 1);
+                    item_d->rows = 0;
                     item_d->children = QVector<QStandardItem*>(); //slightly faster than clear
-                d->model->d_func()->columnsRemoved(item, 0, savedCols);
+                    model_d->rowsRemoved(item, 0, savedRows);
+                }
+                if (savedCols > 0) {
+                    model_d->columnsAboutToBeRemoved(item, 0, savedCols - 1);
+                    item_d->columns = 0;
+                    item_d->children = QVector<QStandardItem*>(); //slightly faster than clear
+                    model_d->columnsRemoved(item, 0, savedCols);
+                }
+                item_d->rows = savedRows;
+                item_d->columns = savedCols;
+                item_d->children = savedChildren;
+                changedIdx = d->model->indexFromItem(item);
             }
-            item_d->rows = savedRows;
-            item_d->columns = savedCols;
-            item_d->children = savedChildren;
-            changedIdx = d->model->indexFromItem(item);
             item_d->setParentAndModel(nullptr, nullptr);
         }
         d->children.replace(index, nullptr);
@@ -2237,7 +2251,7 @@ void QStandardItemModel::setItemRoleNames(const QHash<int,QByteArray> &roleNames
 }
 
 /*!
-  reimp
+  \reimp
 */
 QHash<int, QByteArray> QStandardItemModel::roleNames() const
 {
@@ -2316,7 +2330,7 @@ QStandardItem *QStandardItemModel::itemFromIndex(const QModelIndex &index) const
 QModelIndex QStandardItemModel::indexFromItem(const QStandardItem *item) const
 {
     if (item && item->d_func()->parent) {
-        QPair<int, int> pos = item->d_func()->position();
+        std::pair<int, int> pos = item->d_func()->position();
         return createIndex(pos.first, pos.second, item->d_func()->parent);
     }
     return QModelIndex();
@@ -3100,13 +3114,13 @@ QStringList QStandardItemModel::mimeTypes() const
 */
 QMimeData *QStandardItemModel::mimeData(const QModelIndexList &indexes) const
 {
-    QMimeData *data = QAbstractItemModel::mimeData(indexes);
+    std::unique_ptr<QMimeData> data(QAbstractItemModel::mimeData(indexes));
     if (!data)
         return nullptr;
 
     const QString format = qStandardItemModelDataListMimeType();
     if (!mimeTypes().contains(format))
-        return data;
+        return data.release();
     QByteArray encoded;
     QDataStream stream(&encoded, QIODevice::WriteOnly);
 
@@ -3158,7 +3172,7 @@ QMimeData *QStandardItemModel::mimeData(const QModelIndexList &indexes) const
     }
 
     data->setData(format, encoded);
-    return data;
+    return data.release();
 }
 
 
