@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtGui module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qrasterizer_p.h"
 
@@ -1100,28 +1064,26 @@ void QRasterizer::rasterizeLine(const QPointF &a, const QPointF &b, qreal width,
                 while (x <= leftMax) {
                     QScFixed excluded = 0;
 
-                    if (yFP <= iLeftFP)
+                    if (yFP <= iLeftFP && rowBottomLeft > rowTop)
                         excluded += intersectPixelFP(x, rowTop, rowBottomLeft,
                                                      bottomLeftIntersectAf, topLeftIntersectAf,
                                                      topLeftSlopeFP, invTopLeftSlopeFP);
-                    if (yFP >= iLeftFP)
+                    if (yFP >= iLeftFP && rowBottom > rowTopLeft)
                         excluded += intersectPixelFP(x, rowTopLeft, rowBottom,
                                                      topLeftIntersectBf, bottomLeftIntersectBf,
                                                      bottomLeftSlopeFP, invBottomLeftSlopeFP);
-
                     if (x >= rightMin) {
-                        if (yFP <= iRightFP)
+                        if (yFP <= iRightFP && rowBottomRight > rowTop)
                             excluded += (rowBottomRight - rowTop) - intersectPixelFP(x, rowTop, rowBottomRight,
                                                                                      topRightIntersectAf, bottomRightIntersectAf,
                                                                                      topRightSlopeFP, invTopRightSlopeFP);
-                        if (yFP >= iRightFP)
+                        if (yFP >= iRightFP && rowBottom > rowTopRight)
                             excluded += (rowBottom - rowTopRight) - intersectPixelFP(x, rowTopRight, rowBottom,
                                                                                      bottomRightIntersectBf, topRightIntersectBf,
                                                                                      bottomRightSlopeFP, invBottomRightSlopeFP);
                     }
-                    if (excluded > QScFixedFactor)
-                        excluded = excluded % QScFixedFactor;
 
+                    Q_ASSERT(excluded >= 0 && excluded <= rowHeight);
                     QScFixed coverage = rowHeight - excluded;
                     buffer.addSpan(x, 1, QScFixedToInt(yFP),
                                    QScFixedToInt(255 * coverage));
@@ -1134,17 +1096,16 @@ void QRasterizer::rasterizeLine(const QPointF &a, const QPointF &b, qreal width,
                 }
                 while (x <= rightMax) {
                     QScFixed excluded = 0;
-                    if (yFP <= iRightFP)
+                    if (yFP <= iRightFP && rowBottomRight > rowTop)
                         excluded += (rowBottomRight - rowTop) - intersectPixelFP(x, rowTop, rowBottomRight,
                                                                                  topRightIntersectAf, bottomRightIntersectAf,
                                                                                  topRightSlopeFP, invTopRightSlopeFP);
-                    if (yFP >= iRightFP)
+                    if (yFP >= iRightFP && rowBottom > rowTopRight)
                         excluded += (rowBottom - rowTopRight) - intersectPixelFP(x, rowTopRight, rowBottom,
                                                                                  bottomRightIntersectBf, topRightIntersectBf,
                                                                                  bottomRightSlopeFP, invBottomRightSlopeFP);
-                    if (excluded > QScFixedFactor)
-                        excluded = excluded % QScFixedFactor;
 
+                    Q_ASSERT(excluded >= 0 && excluded <= rowHeight);
                     QScFixed coverage = rowHeight - excluded;
                     buffer.addSpan(x, 1, QScFixedToInt(yFP),
                                    QScFixedToInt(255 * coverage));
