@@ -39,6 +39,7 @@ class QRhiCommandBuffer;
 class QRhiResourceUpdateBatch;
 class QRhiResourceUpdateBatchPrivate;
 class QRhiSwapChain;
+class QRhiShadingRateMap;
 
 class Q_GUI_EXPORT QRhiDepthStencilClearValue
 {
@@ -250,7 +251,15 @@ public:
         Half4,
         Half3,
         Half2,
-        Half
+        Half,
+        UShort4,
+        UShort3,
+        UShort2,
+        UShort,
+        SShort4,
+        SShort3,
+        SShort2,
+        SShort,
     };
 
     QRhiVertexInputAttribute() = default;
@@ -592,6 +601,9 @@ public:
     int resolveLevel() const { return m_resolveLevel; }
     void setResolveLevel(int level) { m_resolveLevel = level; }
 
+    int multiViewCount() const { return m_multiViewCount; }
+    void setMultiViewCount(int count) { m_multiViewCount = count; }
+
 private:
     QRhiTexture *m_texture = nullptr;
     QRhiRenderBuffer *m_renderBuffer = nullptr;
@@ -600,6 +612,7 @@ private:
     QRhiTexture *m_resolveTexture = nullptr;
     int m_resolveLayer = 0;
     int m_resolveLevel = 0;
+    int m_multiViewCount = 0;
 };
 
 Q_DECLARE_TYPEINFO(QRhiColorAttachment, Q_RELOCATABLE_TYPE);
@@ -630,10 +643,18 @@ public:
     QRhiTexture *depthTexture() const { return m_depthTexture; }
     void setDepthTexture(QRhiTexture *texture) { m_depthTexture = texture; }
 
+    QRhiTexture *depthResolveTexture() const { return m_depthResolveTexture; }
+    void setDepthResolveTexture(QRhiTexture *tex) { m_depthResolveTexture = tex; }
+
+    QRhiShadingRateMap *shadingRateMap() const { return m_shadingRateMap; }
+    void setShadingRateMap(QRhiShadingRateMap *map) { m_shadingRateMap = map; }
+
 private:
     QVarLengthArray<QRhiColorAttachment, 8> m_colorAttachments;
     QRhiRenderBuffer *m_depthStencilBuffer = nullptr;
     QRhiTexture *m_depthTexture = nullptr;
+    QRhiTexture *m_depthResolveTexture = nullptr;
+    QRhiShadingRateMap *m_shadingRateMap = nullptr;
 };
 
 class Q_GUI_EXPORT QRhiTextureSubresourceUploadDescription
@@ -799,7 +820,8 @@ public:
         GraphicsPipeline,
         SwapChain,
         ComputePipeline,
-        CommandBuffer
+        CommandBuffer,
+        ShadingRateMap
     };
 
     virtual ~QRhiResource();
@@ -865,6 +887,7 @@ public:
 
     virtual char *beginFullDynamicBufferUpdateForCurrentFrame();
     virtual void endFullDynamicBufferUpdateForCurrentFrame();
+    virtual void fullDynamicBufferUpdateForCurrentFrame(const void *data, quint32 size = 0);
 
 protected:
     QRhiBuffer(QRhiImplementation *rhi, Type type_, UsageFlags usage_, quint32 size_);
@@ -891,7 +914,8 @@ public:
         ThreeDimensional = 1 << 10,
         TextureRectangleGL = 1 << 11,
         TextureArray = 1 << 12,
-        OneDimensional = 1 << 13
+        OneDimensional = 1 << 13,
+        UsedAsShadingRateMap = 1 << 14
     };
     Q_DECLARE_FLAGS(Flags, Flag)
 
@@ -913,10 +937,16 @@ public:
 
         RGB10A2,
 
+        R8UI,
+        R32UI,
+        RG32UI,
+        RGBA32UI,
+
         D16,
         D24,
         D24S8,
         D32F,
+        D32FS8,
 
         BC1,
         BC2,
@@ -979,6 +1009,15 @@ public:
     int sampleCount() const { return m_sampleCount; }
     void setSampleCount(int s) { m_sampleCount = s; }
 
+    struct ViewFormat {
+        QRhiTexture::Format format;
+        bool srgb;
+    };
+    ViewFormat readViewFormat() const { return m_readViewFormat; }
+    void setReadViewFormat(const ViewFormat &fmt) { m_readViewFormat = fmt; }
+    ViewFormat writeViewFormat() const { return m_writeViewFormat; }
+    void setWriteViewFormat(const ViewFormat &fmt) { m_writeViewFormat = fmt; }
+
     virtual bool create() = 0;
     virtual NativeTexture nativeTexture();
     virtual bool createFrom(NativeTexture src);
@@ -995,6 +1034,8 @@ protected:
     Flags m_flags;
     int m_arrayRangeStart = -1;
     int m_arrayRangeLength = -1;
+    ViewFormat m_readViewFormat = { UnknownFormat, false };
+    ViewFormat m_writeViewFormat = { UnknownFormat, false };
 };
 
 Q_DECLARE_OPERATORS_FOR_FLAGS(QRhiTexture::Flags)
@@ -1111,6 +1152,22 @@ protected:
 
 Q_DECLARE_OPERATORS_FOR_FLAGS(QRhiRenderBuffer::Flags)
 
+class Q_GUI_EXPORT QRhiShadingRateMap : public QRhiResource
+{
+public:
+    struct NativeShadingRateMap {
+        quint64 object;
+    };
+
+    QRhiResource::Type resourceType() const override;
+
+    virtual bool createFrom(NativeShadingRateMap src);
+    virtual bool createFrom(QRhiTexture *src);
+
+protected:
+    QRhiShadingRateMap(QRhiImplementation *rhi);
+};
+
 class Q_GUI_EXPORT QRhiRenderPassDescriptor : public QRhiResource
 {
 public:
@@ -1158,7 +1215,8 @@ class Q_GUI_EXPORT QRhiTextureRenderTarget : public QRhiRenderTarget
 public:
     enum Flag {
         PreserveColorContents = 1 << 0,
-        PreserveDepthStencilContents = 1 << 1
+        PreserveDepthStencilContents = 1 << 1,
+        DoNotStoreDepthStencilContents = 1 << 2
     };
     Q_DECLARE_FLAGS(Flags, Flag)
 
@@ -1233,6 +1291,11 @@ Q_DECLARE_OPERATORS_FOR_FLAGS(QRhiShaderResourceBindings::UpdateFlags)
 Q_GUI_EXPORT QDebug operator<<(QDebug, const QRhiShaderResourceBindings &);
 #endif
 
+// The proper name. Until it gets rolled out universally, have the better name
+// as a typedef. Eventually it should be reversed (the old name being a typedef
+// to the new one).
+using QRhiShaderResourceBindingSet = QRhiShaderResourceBindings;
+
 class Q_GUI_EXPORT QRhiGraphicsPipeline : public QRhiResource
 {
 public:
@@ -1240,7 +1303,8 @@ public:
         UsesBlendConstants = 1 << 0,
         UsesStencilRef = 1 << 1,
         UsesScissor = 1 << 2,
-        CompileShadersWithDebugInfo = 1 << 3
+        CompileShadersWithDebugInfo = 1 << 3,
+        UsesShadingRate = 1 << 4
     };
     Q_DECLARE_FLAGS(Flags, Flag)
 
@@ -1437,6 +1501,9 @@ public:
     PolygonMode polygonMode() const {return m_polygonMode; }
     void setPolygonMode(PolygonMode mode) {m_polygonMode = mode; }
 
+    int multiViewCount() const { return m_multiViewCount; }
+    void setMultiViewCount(int count) { m_multiViewCount = count; }
+
     virtual bool create() = 0;
 
 protected:
@@ -1460,6 +1527,7 @@ protected:
     float m_slopeScaledDepthBias = 0.0f;
     int m_patchControlPointCount = 3;
     PolygonMode m_polygonMode = Fill;
+    int m_multiViewCount = 0;
     QVarLengthArray<QRhiShaderStage, 4> m_shaderStages;
     QRhiVertexInputLayout m_vertexInputLayout;
     QRhiShaderResourceBindings *m_shaderResourceBindings = nullptr;
@@ -1472,11 +1540,16 @@ Q_DECLARE_TYPEINFO(QRhiGraphicsPipeline::TargetBlend, Q_RELOCATABLE_TYPE);
 
 struct QRhiSwapChainHdrInfo
 {
-    bool isHardCodedDefaults;
     enum LimitsType {
         LuminanceInNits,
         ColorComponentValue
     };
+
+    enum LuminanceBehavior {
+        SceneReferred,
+        DisplayReferred
+    };
+
     LimitsType limitsType;
     union {
         struct {
@@ -1488,6 +1561,8 @@ struct QRhiSwapChainHdrInfo
             float maxPotentialColorComponentValue;
         } colorComponentValue;
     } limits;
+    LuminanceBehavior luminanceBehavior;
+    float sdrWhiteLevel;
 };
 
 Q_DECLARE_TYPEINFO(QRhiSwapChainHdrInfo, Q_RELOCATABLE_TYPE);
@@ -1517,7 +1592,8 @@ public:
     enum Format {
         SDR,
         HDRExtendedSrgbLinear,
-        HDR10
+        HDR10,
+        HDRExtendedDisplayP3Linear
     };
 
     enum StereoTargetBuffer {
@@ -1548,6 +1624,9 @@ public:
     QRhiRenderPassDescriptor *renderPassDescriptor() const { return m_renderPassDesc; }
     void setRenderPassDescriptor(QRhiRenderPassDescriptor *desc) { m_renderPassDesc = desc; }
 
+    QRhiShadingRateMap *shadingRateMap() const { return m_shadingRateMap; }
+    void setShadingRateMap(QRhiShadingRateMap *map) { m_shadingRateMap = map; }
+
     QSize currentPixelSize() const { return m_currentPixelSize; }
 
     virtual QRhiCommandBuffer *currentFrameCommandBuffer() = 0;
@@ -1569,6 +1648,7 @@ protected:
     QRhiRenderPassDescriptor *m_renderPassDesc = nullptr;
     QSize m_currentPixelSize;
     QRhiSwapChainProxyData m_proxyData;
+    QRhiShadingRateMap *m_shadingRateMap = nullptr;
 };
 
 Q_DECLARE_OPERATORS_FOR_FLAGS(QRhiSwapChain::Flags)
@@ -1628,11 +1708,11 @@ public:
     void endPass(QRhiResourceUpdateBatch *resourceUpdates = nullptr);
 
     void setGraphicsPipeline(QRhiGraphicsPipeline *ps);
-    using DynamicOffset = QPair<int, quint32>; // binding, offset
+    using DynamicOffset = std::pair<int, quint32>; // binding, offset
     void setShaderResources(QRhiShaderResourceBindings *srb = nullptr,
                             int dynamicOffsetCount = 0,
                             const DynamicOffset *dynamicOffsets = nullptr);
-    using VertexInput = QPair<QRhiBuffer *, quint32>; // buffer, offset
+    using VertexInput = std::pair<QRhiBuffer *, quint32>; // buffer, offset
     void setVertexInput(int startBinding, int bindingCount, const VertexInput *bindings,
                         QRhiBuffer *indexBuf = nullptr, quint32 indexOffset = 0,
                         IndexFormat indexFormat = IndexUInt16);
@@ -1641,6 +1721,7 @@ public:
     void setScissor(const QRhiScissor &scissor);
     void setBlendConstants(const QColor &c);
     void setStencilRef(quint32 refValue);
+    void setShadingRate(const QSize &coarsePixelSize);
 
     void draw(quint32 vertexCount,
               quint32 instanceCount = 1,
@@ -1771,7 +1852,8 @@ public:
         EnableDebugMarkers = 1 << 0,
         PreferSoftwareRenderer = 1 << 1,
         EnablePipelineCacheDataSave = 1 << 2,
-        EnableTimestamps = 1 << 3
+        EnableTimestamps = 1 << 3,
+        SuppressSmokeTestWarnings = 1 << 4
     };
     Q_DECLARE_FLAGS(Flags, Flag)
 
@@ -1822,7 +1904,15 @@ public:
         OneDimensionalTextureMipmaps,
         HalfAttributes,
         RenderToOneDimensionalTexture,
-        ThreeDimensionalTextureMipmaps
+        ThreeDimensionalTextureMipmaps,
+        MultiView,
+        TextureViewFormat,
+        ResolveDepthStencil,
+        VariableRateShading,
+        VariableRateShadingMap,
+        VariableRateShadingMapWithTexture,
+        PerRenderTargetBlending,
+        SampleVariables
     };
 
     enum BeginFrameFlag {
@@ -1848,7 +1938,8 @@ public:
         TextureArraySizeMax,
         MaxUniformBufferRange,
         MaxVertexInputs,
-        MaxVertexOutputs
+        MaxVertexOutputs,
+        ShadingRateImageTileSize
     };
 
     ~QRhi();
@@ -1908,6 +1999,8 @@ public:
                             QRhiSampler::AddressMode addressV,
                             QRhiSampler::AddressMode addressW = QRhiSampler::Repeat);
 
+    QRhiShadingRateMap *newShadingRateMap();
+
     QRhiTextureRenderTarget *newTextureRenderTarget(const QRhiTextureRenderTargetDescription &desc,
                                                     QRhiTextureRenderTarget::Flags flags = {});
 
@@ -1944,6 +2037,7 @@ public:
 
     const QRhiNativeHandles *nativeHandles();
     bool makeThreadLocalNativeContextCurrent();
+    void setQueueSubmitParams(QRhiNativeHandles *params);
 
     static constexpr int MAX_MIP_LEVELS = 16; // -> max width or height is 65536
 
@@ -1958,6 +2052,8 @@ public:
 
     static QRhiSwapChainProxyData updateSwapChainProxyData(Implementation impl, QWindow *window);
 
+    QList<QSize> supportedShadingRates(int sampleCount) const;
+
 protected:
     QRhi();
 
