@@ -16,6 +16,7 @@
 //
 
 #include <QtGui/private/qtguiglobal_p.h>
+#include <QtGui/qfontvariableaxis.h>
 #include "QtCore/qatomic.h"
 #include <QtCore/qvarlengtharray.h>
 #include <QtCore/qhashfunctions.h>
@@ -29,13 +30,6 @@ class QFontEngineGlyphCache;
 
 struct QGlyphLayout;
 
-#define MAKE_TAG(ch1, ch2, ch3, ch4) (\
-    (((quint32)(ch1)) << 24) | \
-    (((quint32)(ch2)) << 16) | \
-    (((quint32)(ch3)) << 8) | \
-    ((quint32)(ch4)) \
-   )
-
 // ### this only used in getPointInOutline(), refactor it and then remove these magic numbers
 enum HB_Compat_Error {
     Err_Ok                           = 0x0000,
@@ -48,6 +42,8 @@ enum HB_Compat_Error {
 typedef void (*qt_destroy_func_t) (void *user_data);
 typedef bool (*qt_get_font_table_func_t) (void *user_data, uint tag, uchar *buffer, uint *length);
 
+Q_DECLARE_LOGGING_CATEGORY(lcColrv1)
+
 class Q_GUI_EXPORT QFontEngine
 {
 public:
@@ -83,7 +79,8 @@ public:
 
     enum ShaperFlag {
         DesignMetrics = 0x0002,
-        GlyphIndicesOnly = 0x0004
+        GlyphIndicesOnly = 0x0004,
+        FullStringFallback = 0x008
     };
     Q_DECLARE_FLAGS(ShaperFlags, ShaperFlag)
 
@@ -127,11 +124,13 @@ public:
     virtual bool getSfntTableData(uint tag, uchar *buffer, uint *length) const;
 
     struct FaceId {
-        FaceId() : index(0), encoding(0) {}
+        FaceId() : index(0), instanceIndex(-1), encoding(0) {}
         QByteArray filename;
         QByteArray uuid;
         int index;
+        int instanceIndex;
         int encoding;
+        QMap<QFont::Tag, float> variableAxes;
     };
     virtual FaceId faceId() const { return FaceId(); }
     enum SynthesizedFlags {
@@ -152,11 +151,23 @@ public:
         return subPixelPositionFor(QFixedPoint(x, 0)).x;
     }
 
-    virtual QFixed emSquareSize() const { return ascent(); }
+    bool preferTypoLineMetrics() const;
+    bool isColorFont() const { return glyphFormat == Format_ARGB; }
+    static bool isIgnorableChar(char32_t ucs4)
+    {
+        return ucs4 == QChar::LineSeparator
+               || ucs4 == QChar::LineFeed
+               || ucs4 == QChar::CarriageReturn
+               || ucs4 == QChar::ParagraphSeparator
+               || (!disableEmojiSegmenter() && (ucs4 & 0xFFF0) == 0xFE00)
+               || QChar::category(ucs4) == QChar::Other_Control;
+    }
+
+    virtual QFixed emSquareSize() const;
 
     /* returns 0 as glyph index for non existent glyphs */
     virtual glyph_t glyphIndex(uint ucs4) const = 0;
-    virtual bool stringToCMap(const QChar *str, int len, QGlyphLayout *glyphs, int *nglyphs, ShaperFlags flags) const = 0;
+    virtual int stringToCMap(const QChar *str, int len, QGlyphLayout *glyphs, int *nglyphs, ShaperFlags flags) const = 0;
     virtual void recalcAdvances(QGlyphLayout *, ShaperFlags) const {}
     virtual void doKerning(QGlyphLayout *, ShaperFlags) const;
 
@@ -179,6 +190,7 @@ public:
     virtual QImage alphaMapForGlyph(glyph_t, const QFixedPoint &subPixelPosition, const QTransform &t);
     virtual QImage alphaRGBMapForGlyph(glyph_t, const QFixedPoint &subPixelPosition, const QTransform &t);
     virtual QImage bitmapForGlyph(glyph_t, const QFixedPoint &subPixelPosition, const QTransform &t, const QColor &color = QColor());
+    QImage renderedPathForGlyph(glyph_t glyph, const QColor &color);
     virtual Glyph *glyphData(glyph_t glyph, const QFixedPoint &subPixelPosition, GlyphFormat neededFormat, const QTransform &t);
     virtual bool hasInternalCaching() const { return false; }
 
@@ -189,7 +201,7 @@ public:
 
     virtual void removeGlyphFromCache(glyph_t);
 
-    virtual glyph_metrics_t boundingBox(const QGlyphLayout &glyphs) = 0;
+    virtual glyph_metrics_t boundingBox(const QGlyphLayout &glyphs);
     virtual glyph_metrics_t boundingBox(glyph_t glyph) = 0;
     virtual glyph_metrics_t boundingBox(glyph_t glyph, const QTransform &matrix);
     glyph_metrics_t tightBoundingBox(const QGlyphLayout &glyphs);
@@ -222,6 +234,11 @@ public:
 
     virtual Qt::HANDLE handle() const;
 
+    virtual QList<QFontVariableAxis> variableAxes() const;
+
+    virtual QString glyphName(glyph_t index) const;
+    virtual glyph_t findGlyph(QLatin1StringView name) const;
+
     void *harfbuzzFont() const;
     void *harfbuzzFace() const;
     bool supportsScript(QChar::Script script) const;
@@ -246,6 +263,8 @@ public:
     virtual bool hasUnreliableGlyphOutline() const;
     virtual bool expectsGammaCorrectedBlending() const;
 
+    static bool disableEmojiSegmenter();
+
     enum HintStyle {
         HintNone,
         HintLight,
@@ -277,8 +296,8 @@ public:
         explicit Holder(void *p, qt_destroy_func_t d) : ptr(p), destroy_func(d) {}
         ~Holder() { if (ptr && destroy_func) destroy_func(ptr); }
         Holder(Holder &&other) noexcept
-            : ptr(qExchange(other.ptr, nullptr)),
-              destroy_func(qExchange(other.destroy_func, nullptr))
+            : ptr(std::exchange(other.ptr, nullptr)),
+              destroy_func(std::exchange(other.destroy_func, nullptr))
         {
         }
         QT_MOVE_ASSIGNMENT_OPERATOR_IMPL_VIA_PURE_SWAP(Holder)
@@ -332,6 +351,7 @@ public:
 protected:
     explicit QFontEngine(Type type);
 
+    QFixed firstLeftBearing(const QGlyphLayout &glyphs);
     QFixed lastRightBearing(const QGlyphLayout &glyphs);
 
     QFixed calculatedCapHeight() const;
@@ -369,13 +389,18 @@ Q_DECLARE_OPERATORS_FOR_FLAGS(QFontEngine::ShaperFlags)
 
 inline bool operator ==(const QFontEngine::FaceId &f1, const QFontEngine::FaceId &f2)
 {
-    return f1.index == f2.index && f1.encoding == f2.encoding && f1.filename == f2.filename && f1.uuid == f2.uuid;
+    return f1.index == f2.index
+            && f1.encoding == f2.encoding
+            && f1.filename == f2.filename
+            && f1.uuid == f2.uuid
+            && f1.instanceIndex == f2.instanceIndex
+            && f1.variableAxes == f2.variableAxes;
 }
 
 inline size_t qHash(const QFontEngine::FaceId &f, size_t seed = 0)
     noexcept(noexcept(qHash(f.filename)))
 {
-    return qHashMulti(seed, f.filename, f.uuid, f.index, f.encoding);
+    return qHashMulti(seed, f.filename, f.uuid, f.index, f.instanceIndex, f.encoding, f.variableAxes.keys(), f.variableAxes.values());
 }
 
 
@@ -390,7 +415,7 @@ public:
     ~QFontEngineBox();
 
     virtual glyph_t glyphIndex(uint ucs4) const override;
-    virtual bool stringToCMap(const QChar *str, int len, QGlyphLayout *glyphs, int *nglyphs, ShaperFlags flags) const override;
+    virtual int stringToCMap(const QChar *str, int len, QGlyphLayout *glyphs, int *nglyphs, ShaperFlags flags) const override;
     virtual void recalcAdvances(QGlyphLayout *, ShaperFlags) const override;
 
     void draw(QPaintEngine *p, qreal x, qreal y, const QTextItemInt &si);
@@ -400,6 +425,7 @@ public:
     virtual glyph_metrics_t boundingBox(glyph_t glyph) override;
     virtual QFontEngine *cloneWithSize(qreal pixelSize) const override;
 
+    virtual QFixed emSquareSize() const override { return _size; }
     virtual QFixed ascent() const override;
     virtual QFixed capHeight() const override;
     virtual QFixed descent() const override;
@@ -428,7 +454,7 @@ public:
     ~QFontEngineMulti();
 
     virtual glyph_t glyphIndex(uint ucs4) const override;
-    virtual bool stringToCMap(const QChar *str, int len, QGlyphLayout *glyphs, int *nglyphs, ShaperFlags flags) const override;
+    virtual int stringToCMap(const QChar *str, int len, QGlyphLayout *glyphs, int *nglyphs, ShaperFlags flags) const override;
 
     virtual glyph_metrics_t boundingBox(const QGlyphLayout &glyphs) override;
     virtual glyph_metrics_t boundingBox(glyph_t glyph) override;
@@ -438,6 +464,7 @@ public:
     virtual void addOutlineToPath(qreal, qreal, const QGlyphLayout &, QPainterPath *, QTextItem::RenderFlags flags) override;
     virtual void getGlyphBearings(glyph_t glyph, qreal *leftBearing = nullptr, qreal *rightBearing = nullptr) override;
 
+    virtual QFixed emSquareSize() const override;
     virtual QFixed ascent() const override;
     virtual QFixed capHeight() const override;
     virtual QFixed descent() const override;
@@ -456,7 +483,11 @@ public:
     virtual qreal minLeftBearing() const override;
     virtual qreal minRightBearing() const override;
 
+    virtual QList<QFontVariableAxis> variableAxes() const override;
+
     virtual bool canRender(const QChar *string, int len) const override;
+    QString glyphName(glyph_t glyph) const override;
+    glyph_t findGlyph(QLatin1StringView name) const override;
 
     inline int fallbackFamilyCount() const { return m_fallbackFamilies.size(); }
     inline QString fallbackFamilyAt(int at) const { return m_fallbackFamilies.at(at); }
