@@ -1,6 +1,7 @@
 // Copyright (C) 2017 The Qt Company Ltd.
 // Copyright (C) 2016 Intel Corporation.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:critical reason:data-parser
 
 #include "qhostaddress.h"
 #include "qhostaddress_p.h"
@@ -972,7 +973,7 @@ bool QHostAddress::isInSubnet(const QHostAddress &subnet, int netmask) const
     prefix and the int (second) member contains the netmask (prefix
     length).
 */
-bool QHostAddress::isInSubnet(const QPair<QHostAddress, int> &subnet) const
+bool QHostAddress::isInSubnet(const std::pair<QHostAddress, int> &subnet) const
 {
     return isInSubnet(subnet.first, subnet.second);
 }
@@ -1002,7 +1003,7 @@ bool QHostAddress::isInSubnet(const QPair<QHostAddress, int> &subnet) const
 
     \sa isInSubnet()
 */
-QPair<QHostAddress, int> QHostAddress::parseSubnet(const QString &subnet)
+std::pair<QHostAddress, int> QHostAddress::parseSubnet(const QString &subnet)
 {
     // We support subnets in the form:
     //   ddd.ddd.ddd.ddd/nn
@@ -1019,7 +1020,7 @@ QPair<QHostAddress, int> QHostAddress::parseSubnet(const QString &subnet)
     //
     //  where nn can be an IPv4-style netmask for the IPv4 forms
 
-    const QPair<QHostAddress, int> invalid = qMakePair(QHostAddress(), -1);
+    const std::pair<QHostAddress, int> invalid = std::pair(QHostAddress(), -1);
     if (subnet.isEmpty())
         return invalid;
 
@@ -1062,7 +1063,7 @@ QPair<QHostAddress, int> QHostAddress::parseSubnet(const QString &subnet)
             return invalid;     // failed to parse the IP
 
         clearBits(net.d->a6.c, netmask, 128);
-        return qMakePair(net, netmask);
+        return std::pair(net, netmask);
     }
 
     if (netmask > 32)
@@ -1100,7 +1101,7 @@ QPair<QHostAddress, int> QHostAddress::parseSubnet(const QString &subnet)
         addr &= mask;
     }
 
-    return qMakePair(QHostAddress(addr), netmask);
+    return std::pair(QHostAddress(addr), netmask);
 }
 
 /*!
@@ -1191,7 +1192,7 @@ bool QHostAddress::isSiteLocal() const
     4193 says that, in practice, "applications may treat these addresses like
     global scoped addresses." Only routers need care about the distinction.
 
-    \sa isLoopback(), isGlobal(), isMulticast(), isLinkLocal(), isUniqueLocalUnicast(), isPrivateUse()
+    \sa isLoopback(), isGlobal(), isMulticast(), isLinkLocal(), isPrivateUse()
 */
 bool QHostAddress::isUniqueLocalUnicast() const
 {
