@@ -1,13 +1,14 @@
 // Copyright (C) 2021 The Qt Company Ltd.
 // Copyright (C) 2016 Intel Corporation.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 //#define QNATIVESOCKETENGINE_DEBUG
 #include "qnativesocketengine_p_p.h"
 #include "private/qnet_unix_p.h"
+#include "qdeadlinetimer.h"
 #include "qiodevice.h"
 #include "qhostaddress.h"
-#include "qelapsedtimer.h"
 #include "qvarlengtharray.h"
 #include "qnetworkinterface.h"
 #include "qendian.h"
@@ -17,19 +18,6 @@
 #include <time.h>
 #include <errno.h>
 #include <fcntl.h>
-#ifndef QT_NO_IPV6IFNAME
-#ifdef Q_OS_LINUX
-#include <linux/if.h>
-#else // Q_OS_LINUX
-#include <net/if.h>
-#endif // !Q_OS_LINUX
-#endif
-#ifdef QT_LINUXBASE
-#include <arpa/inet.h>
-#endif
-#ifdef Q_OS_BSD4
-#include <net/if_dl.h>
-#endif
 #ifdef Q_OS_INTEGRITY
 #include <sys/uio.h>
 #endif
@@ -414,6 +402,7 @@ bool QNativeSocketEnginePrivate::nativeConnect(const QHostAddress &addr, quint16
             break;
         case ECONNREFUSED:
         case EINVAL:
+        case ENOENT:
             setError(QAbstractSocket::ConnectionRefusedError, ConnectionRefusedErrorString);
             socketState = QAbstractSocket::UnconnectedState;
             break;
@@ -449,6 +438,7 @@ bool QNativeSocketEnginePrivate::nativeConnect(const QHostAddress &addr, quint16
         case EFAULT:
         case ENOTSOCK:
             socketState = QAbstractSocket::UnconnectedState;
+            break;
         default:
             break;
         }
@@ -833,6 +823,12 @@ qint64 QNativeSocketEnginePrivate::nativePendingDatagramSize() const
     recvResult = getsockopt(socketDescriptor, SOL_SOCKET, SO_NREAD, &value, &valuelen);
     if (recvResult != -1)
         recvResult = value;
+#elif defined(Q_OS_VXWORKS)
+    // VxWorks: use ioctl(FIONREAD) to query the number of bytes available
+    int available = 0;
+    int ioctlResult = ::ioctl(socketDescriptor, FIONREAD, &available);
+    if (ioctlResult != -1)
+        recvResult = available;
 #else
     // We need to grow the buffer to fit the entire datagram.
     // We start at 1500 bytes (the MTU for Ethernet V2), which should catch
@@ -1288,6 +1284,9 @@ qint64 QNativeSocketEnginePrivate::nativeWrite(const char *data, qint64 len)
             setError(QAbstractSocket::RemoteHostClosedError, RemoteHostClosedErrorString);
             q->close();
             break;
+#if EWOULDBLOCK != EAGAIN
+        case EWOULDBLOCK:
+#endif
         case EAGAIN:
             writtenBytes = 0;
             break;
@@ -1357,16 +1356,17 @@ qint64 QNativeSocketEnginePrivate::nativeRead(char *data, qint64 maxSize)
     return qint64(r);
 }
 
-int QNativeSocketEnginePrivate::nativeSelect(int timeout, bool selectForRead) const
+int QNativeSocketEnginePrivate::nativeSelect(QDeadlineTimer deadline, bool selectForRead) const
 {
     bool dummy;
-    return nativeSelect(timeout, selectForRead, !selectForRead, &dummy, &dummy);
+    return nativeSelect(deadline, selectForRead, !selectForRead, &dummy, &dummy);
 }
 
 #ifndef Q_OS_WASM
 
-int QNativeSocketEnginePrivate::nativeSelect(int timeout, bool checkRead, bool checkWrite,
-                       bool *selectForRead, bool *selectForWrite) const
+int QNativeSocketEnginePrivate::nativeSelect(QDeadlineTimer deadline, bool checkRead,
+                                             bool checkWrite, bool *selectForRead,
+                                             bool *selectForWrite) const
 {
     pollfd pfd = qt_make_pollfd(socketDescriptor, 0);
 
@@ -1376,7 +1376,7 @@ int QNativeSocketEnginePrivate::nativeSelect(int timeout, bool checkRead, bool c
     if (checkWrite)
         pfd.events |= POLLOUT;
 
-    const int ret = qt_poll_msecs(&pfd, 1, timeout);
+    const int ret = qt_safe_poll(&pfd, 1, deadline);
 
     if (ret <= 0)
         return ret;
@@ -1397,13 +1397,16 @@ int QNativeSocketEnginePrivate::nativeSelect(int timeout, bool checkRead, bool c
 
 #else
 
-int QNativeSocketEnginePrivate::nativeSelect(int timeout, bool checkRead, bool checkWrite,
-                        bool *selectForRead, bool *selectForWrite) const
+int QNativeSocketEnginePrivate::nativeSelect(QDeadlineTimer deadline, bool checkRead,
+                                             bool checkWrite, bool *selectForRead,
+                                             bool *selectForWrite) const
 {
     *selectForRead = checkRead;
     *selectForWrite = checkWrite;
     bool socketDisconnect = false;
-    QEventDispatcherWasm::socketSelect(timeout, socketDescriptor, checkRead, checkWrite,selectForRead, selectForWrite, &socketDisconnect);
+    QEventDispatcherWasm::socketSelect(deadline.remainingTime(), socketDescriptor, checkRead,
+                                       checkWrite, selectForRead, selectForWrite,
+                                       &socketDisconnect);
 
     // The disconnect/close handling code in QAbstractsScket::canReadNotification()
     // does not detect remote disconnect properly; do that here as a workardound.
