@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2013 Klaralvdalens Datakonsult AB (KDAB).
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtOpenGL module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2013 Klaralvdalens Datakonsult AB (KDAB).
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qopengltimerquery.h"
 
@@ -143,11 +107,11 @@ bool QOpenGLTimerQueryPrivate::create()
 
     // Check to see if we also need to resolve the functions for EXT_timer_query
     QSurfaceFormat f = context->format();
-    if (f.version() <= qMakePair(3, 2)
+    if (f.version() <= std::pair(3, 2)
         && !context->hasExtension(QByteArrayLiteral("GL_ARB_timer_query"))
         && context->hasExtension(QByteArrayLiteral("GL_EXT_timer_query"))) {
         ext = new QExtTimerQueryHelper(context);
-    } else if (f.version() <= qMakePair(3, 2)
+    } else if (f.version() <= std::pair(3, 2)
                && !context->hasExtension(QByteArrayLiteral("GL_ARB_timer_query"))
                && !context->hasExtension(QByteArrayLiteral("GL_EXT_timer_query"))) {
         qWarning("QOpenGLTimerQuery requires one of:\n"
@@ -530,11 +494,11 @@ bool QOpenGLTimeMonitorPrivate::create()
 
     // Check to see if we also need to resolve the functions for EXT_timer_query
     QSurfaceFormat f = context->format();
-    if (f.version() <= qMakePair(3, 2)
+    if (f.version() <= std::pair(3, 2)
         && !context->hasExtension(QByteArrayLiteral("GL_ARB_timer_query"))
         && context->hasExtension(QByteArrayLiteral("GL_EXT_timer_query"))) {
         ext = new QExtTimerQueryHelper(context);
-    } else if (f.version() <= qMakePair(3, 2)
+    } else if (f.version() <= std::pair(3, 2)
                && !context->hasExtension(QByteArrayLiteral("GL_ARB_timer_query"))
                && !context->hasExtension(QByteArrayLiteral("GL_EXT_timer_query"))) {
         qWarning("QOpenGLTimeMonitor requires one of:\n"
@@ -878,3 +842,5 @@ void QOpenGLTimeMonitor::reset()
 }
 
 QT_END_NAMESPACE
+
+#include "moc_qopengltimerquery.cpp"
