@@ -1,5 +1,6 @@
 // Copyright (C) 2021 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #ifndef ANDROIDCONNECTIVITYMANAGER_H
 #define ANDROIDCONNECTIVITYMANAGER_H
@@ -33,7 +34,7 @@ public:
     static AndroidConnectivityManager *getInstance();
     ~AndroidConnectivityManager();
 
-    inline bool isValid() const { return m_connectivityManager.isValid(); }
+    inline bool isValid() const;
 
 Q_SIGNALS:
     void connectivityChanged(AndroidConnectivity connectivity);
@@ -44,8 +45,7 @@ Q_SIGNALS:
 private:
     friend struct AndroidConnectivityManagerInstance;
     AndroidConnectivityManager();
-    bool registerNatives();
-    QJniObject m_connectivityManager;
+    bool registerNatives() const;
 
     Q_DISABLE_COPY_MOVE(AndroidConnectivityManager);
 };
