@@ -1,6 +1,7 @@
 ﻿// Copyright (C) 2019 Volker Krause <vkrause@kde.org>
 // Copyright (C) 2022 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:critical reason:data-parser
 
 #include "androidcontentfileengine.h"
 
@@ -18,6 +19,7 @@ using namespace Qt::StringLiterals;
 
 Q_DECLARE_JNI_CLASS(ParcelFileDescriptorType, "android/os/ParcelFileDescriptor");
 Q_DECLARE_JNI_CLASS(CursorType, "android/database/Cursor");
+Q_DECLARE_JNI_CLASS(QtContentFileEngine, "org/qtproject/qt/android/QtContentFileEngine");
 
 static QJniObject &contentResolverInstance()
 {
@@ -75,11 +77,12 @@ bool AndroidContentFileEngine::open(QIODevice::OpenMode openMode,
         openModeStr += u'a';
     }
 
-    m_pfd = contentResolverInstance().callMethod<
-            QtJniTypes::ParcelFileDescriptorType, QtJniTypes::Uri, jstring>(
+    using namespace QtJniTypes;
+    m_pfd = QtContentFileEngine::callStaticMethod<ParcelFileDescriptorType>(
                 "openFileDescriptor",
-                m_documentFile->uri().object(),
-                QJniObject::fromString(openModeStr).object<jstring>());
+                contentResolverInstance().object<ContentResolver>(),
+                m_documentFile->uri().object<Uri>(),
+                openModeStr);
 
     if (!m_pfd.isValid())
         return false;
@@ -368,13 +371,14 @@ public:
                                             const QStringList &selectionArgs = {},
                                             const QString &sortOrder = {})
     {
-        auto cursor = contentResolverInstance().callMethod<QtJniTypes::CursorType>(
-            "query",
-            uri.object<QtJniTypes::Uri>(),
-            QJniArray(projection),
-            selection.isEmpty() ? nullptr : QJniObject::fromString(selection).object<jstring>(),
-            QJniArray(selectionArgs),
-            sortOrder.isEmpty() ? nullptr : QJniObject::fromString(sortOrder).object<jstring>());
+        using namespace QtJniTypes;
+        auto cursor = QtContentFileEngine::callStaticMethod<CursorType>("query",
+                contentResolverInstance().object<ContentResolver>(),
+                uri.object<Uri>(),
+                QJniArray(projection),
+                selection.isEmpty() ? nullptr : selection,
+                QJniArray(selectionArgs),
+                sortOrder.isEmpty() ? nullptr : sortOrder);
         if (!cursor.isValid())
             return {};
         return std::make_unique<Cursor>(cursor);
