@@ -1,6 +1,7 @@
 // Copyright (C) 2021 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
+#include "androiddeadlockprotector.h"
 #include "androidjniaccessibility.h"
 #include "androidjnimain.h"
 #include "qandroidplatformintegration.h"
@@ -14,11 +15,12 @@
 #include <QtCore/private/qjnihelpers_p.h>
 #include <QtCore/QJniObject>
 #include <QtGui/private/qhighdpiscaling_p.h>
+
 #include <QtCore/QObject>
+#include <QtCore/qpointer.h>
 #include <QtCore/qvarlengtharray.h>
 
 static const char m_qtTag[] = "Qt A11Y";
-static const char m_classErrorMsg[] = "Can't find class \"%s\"";
 
 QT_BEGIN_NAMESPACE
 
@@ -26,6 +28,7 @@ using namespace Qt::StringLiterals;
 
 namespace QtAndroidAccessibility
 {
+    static jmethodID m_setClassNameMethodID = 0;
     static jmethodID m_addActionMethodID = 0;
     static jmethodID m_setCheckableMethodID = 0;
     static jmethodID m_setCheckedMethodID = 0;
@@ -47,7 +50,7 @@ namespace QtAndroidAccessibility
     // Because of that almost every method here is split into two parts.
     // The _helper part is executed in the context of m_accessibilityContext
     // on the main thread. The other part is executed in Java thread.
-    static QPointer<QObject> m_accessibilityContext = nullptr;
+    Q_CONSTINIT static QPointer<QObject> m_accessibilityContext = {};
 
     // This method is called from the Qt main thread, and normally a
     // QGuiApplication instance will be used as a parent.
@@ -61,6 +64,14 @@ namespace QtAndroidAccessibility
     template <typename Func, typename Ret>
     void runInObjectContext(QObject *context, Func &&func, Ret *retVal)
     {
+        AndroidDeadlockProtector protector;
+        if (!protector.acquire()) {
+            __android_log_print(ANDROID_LOG_WARN, m_qtTag,
+                                "Could not run accessibility call in object context, accessing "
+                                "main thread could lead to deadlock");
+            return;
+        }
+
         if (!QtAndroid::blockEventLoopsWhenSuspended()
             || QGuiApplication::applicationState() != Qt::ApplicationSuspended) {
             QMetaObject::invokeMethod(context, func, Qt::BlockingQueuedConnection, retVal);
@@ -72,8 +83,7 @@ namespace QtAndroidAccessibility
 
     void initialize()
     {
-        QJniObject::callStaticMethod<void>(QtAndroid::applicationClass(),
-                                           "initializeAccessibility");
+        QtAndroid::initializeAccessibility();
     }
 
     bool isActive()
@@ -86,10 +96,12 @@ namespace QtAndroidAccessibility
         QMutexLocker lock(QtAndroid::platformInterfaceMutex());
         QAndroidPlatformIntegration *platformIntegration = QtAndroid::androidPlatformIntegration();
         m_accessibilityActivated = active;
-        if (platformIntegration)
+        if (platformIntegration) {
             platformIntegration->accessibility()->setActive(active);
-        else
-            __android_log_print(ANDROID_LOG_WARN, m_qtTag, "Could not (yet) activate platform accessibility.");
+        } else {
+            __android_log_print(ANDROID_LOG_DEBUG, m_qtTag,
+                "Android platform integration is not ready, accessibility activation deferred.");
+        }
     }
 
     QAccessibleInterface *interfaceFromId(jint objectId)
@@ -118,6 +130,12 @@ namespace QtAndroidAccessibility
         QtAndroid::notifyObjectHide(accessibilityObjectId, parentObjectId);
     }
 
+    void notifyObjectShow(uint accessibilityObjectId)
+    {
+        const auto parentObjectId = parentId_helper(accessibilityObjectId);
+        QtAndroid::notifyObjectShow(parentObjectId);
+    }
+
     void notifyObjectFocus(uint accessibilityObjectId)
     {
         QtAndroid::notifyObjectFocus(accessibilityObjectId);
@@ -136,6 +154,11 @@ namespace QtAndroidAccessibility
         QtAndroid::notifyScrolledEvent(accessiblityObjectId);
     }
 
+    void notifyAnnouncementEvent(uint accessibilityObjectId, const QString &message)
+    {
+        QtAndroid::notifyAnnouncementEvent(accessibilityObjectId, message);
+    }
+
     static QVarLengthArray<int, 8> childIdListForAccessibleObject_helper(int objectId)
     {
         QAccessibleInterface *iface = interfaceFromId(objectId);
@@ -342,16 +365,6 @@ namespace QtAndroidAccessibility
         return result && oldPosition != screenRect_helper(firstChildId, false);
     }
 
-
-#define FIND_AND_CHECK_CLASS(CLASS_NAME) \
-clazz = env->FindClass(CLASS_NAME); \
-if (!clazz) { \
-    __android_log_print(ANDROID_LOG_FATAL, m_qtTag, m_classErrorMsg, CLASS_NAME); \
-    return JNI_FALSE; \
-}
-
-        //__android_log_print(ANDROID_LOG_FATAL, m_qtTag, m_methodErrorMsg, METHOD_NAME, METHOD_SIGNATURE);
-
     static QString textFromValue(QAccessibleInterface *iface)
     {
         QString valueStr;
@@ -416,6 +429,130 @@ if (!clazz) { \
         return jstr;
     }
 
+    static QString classNameForRole(QAccessible::Role role, QAccessible::State state) {
+        switch (role) {
+        case QAccessible::Role::Button:
+        case QAccessible::Role::Link:
+        {
+            if (state.checkable)
+                // There is also a android.widget.Switch for which we have no match.
+                return QStringLiteral("android.widget.ToggleButton");
+            return QStringLiteral("android.widget.Button");
+        }
+        case QAccessible::Role::CheckBox:
+            // As of android/accessibility/utils/Role.java::getRole a CheckBox
+            // is NOT android.widget.CheckBox
+            return QStringLiteral("android.widget.CompoundButton");
+        case QAccessible::Role::Clock:
+            return QStringLiteral("android.widget.TextClock");
+        case QAccessible::Role::ComboBox:
+            return QStringLiteral("android.widget.Spinner");
+        case QAccessible::Role::Graphic:
+            // QQuickImage does not provide this role it inherits Client from QQuickItem
+            return QStringLiteral("android.widget.ImageView");
+        case QAccessible::Role::Grouping:
+            return QStringLiteral("android.view.ViewGroup");
+        case QAccessible::Role::List:
+            // As of android/accessibility/utils/Role.java::getRole a List
+            // is NOT android.widget.ListView
+            return QStringLiteral("android.widget.AbsListView");
+        case QAccessible::Role::MenuItem:
+            return QStringLiteral("android.view.MenuItem");
+        case QAccessible::Role::PopupMenu:
+            return QStringLiteral("android.widget.PopupMenu");
+        case QAccessible::Role::Separator:
+            return QStringLiteral("android.widget.Space");
+        case QAccessible::Role::ToolBar:
+            return QStringLiteral("android.view.Toolbar");
+        case QAccessible::Role::Heading: [[fallthrough]];
+        case QAccessible::Role::StaticText:
+            // Heading vs. regular Text is finally determined by AccessibilityNodeInfo.isHeading()
+            return QStringLiteral("android.widget.TextView");
+        case QAccessible::Role::EditableText:
+            return QStringLiteral("android.widget.EditText");
+        case QAccessible::Role::RadioButton:
+            return QStringLiteral("android.widget.RadioButton");
+        case QAccessible::Role::ProgressBar:
+            return QStringLiteral("android.widget.ProgressBar");
+            // Range information need to be filled to announce percentages
+        case QAccessible::Role::SpinBox:
+            return QStringLiteral("android.widget.NumberPicker");
+        case QAccessible::Role::WebDocument:
+            return QStringLiteral("android.webkit.WebView");
+        case QAccessible::Role::Dialog:
+            return QStringLiteral("android.app.AlertDialog");
+        case QAccessible::Role::PageTab:
+            return QStringLiteral("android.app.ActionBar.Tab");
+        case QAccessible::Role::PageTabList:
+            return QStringLiteral("android.widget.TabWidget");
+        case QAccessible::Role::ScrollBar: [[fallthrough]];
+        case QAccessible::Role::Slider:
+            return QStringLiteral("android.widget.SeekBar");
+        case QAccessible::Role::Table:
+            // #TODO Evaluate the usage of AccessibleNodeInfo.setCollectionItemInfo() to provide
+            // infos about colums, rows und items.
+            return QStringLiteral("android.widget.GridView");
+        case QAccessible::Role::Pane:
+            // #TODO QQuickScrollView, QQuickListView (see QTBUG-137806)
+            return QStringLiteral("android.view.ViewGroup");
+        case QAccessible::Role::AlertMessage:
+        case QAccessible::Role::Animation:
+        case QAccessible::Role::Application:
+        case QAccessible::Role::Assistant:
+        case QAccessible::Role::BlockQuote:
+        case QAccessible::Role::Border:
+        case QAccessible::Role::ButtonDropGrid:
+        case QAccessible::Role::ButtonDropDown:
+        case QAccessible::Role::ButtonMenu:
+        case QAccessible::Role::Canvas:
+        case QAccessible::Role::Caret:
+        case QAccessible::Role::Cell:
+        case QAccessible::Role::Chart:
+        case QAccessible::Role::Client:
+        case QAccessible::Role::ColorChooser:
+        case QAccessible::Role::Column:
+        case QAccessible::Role::ColumnHeader:
+        case QAccessible::Role::ComplementaryContent:
+        case QAccessible::Role::Cursor:
+        case QAccessible::Role::Desktop:
+        case QAccessible::Role::Dial:
+        case QAccessible::Role::Document:
+        case QAccessible::Role::Equation:
+        case QAccessible::Role::Footer:
+        case QAccessible::Role::Form:
+        case QAccessible::Role::Grip:
+        case QAccessible::Role::HelpBalloon:
+        case QAccessible::Role::HotkeyField:
+        case QAccessible::Role::Indicator:
+        case QAccessible::Role::LayeredPane:
+        case QAccessible::Role::ListItem:
+        case QAccessible::Role::MenuBar:
+        case QAccessible::Role::NoRole:
+        case QAccessible::Role::Note:
+        case QAccessible::Role::Notification:
+        case QAccessible::Role::Paragraph:
+        case QAccessible::Role::PropertyPage:
+        case QAccessible::Role::Row:
+        case QAccessible::Role::RowHeader:
+        case QAccessible::Role::Section:
+        case QAccessible::Role::Sound:
+        case QAccessible::Role::Splitter:
+        case QAccessible::Role::StatusBar:
+        case QAccessible::Role::Terminal:
+        case QAccessible::Role::TitleBar:
+        case QAccessible::Role::ToolTip:
+        case QAccessible::Role::Tree:
+        case QAccessible::Role::TreeItem:
+        case QAccessible::Role::UserRole:
+        case QAccessible::Role::Whitespace:
+        case QAccessible::Role::Window:
+            // If unsure, every visible or interactive element in Android
+            // inherits android.view.View and by many extends also TextView.
+            // Android itself does a similar thing e.g. in its Settings-App.
+            return QStringLiteral("android.view.TextView");
+        }
+    }
+
     static QString descriptionForInterface(QAccessibleInterface *iface)
     {
         QString desc;
@@ -465,6 +602,7 @@ if (!clazz) { \
         QAccessible::Role role;
         QStringList actions;
         QString description;
+        QString identifier;
         bool hasTextSelection = false;
         int selectionStart = 0;
         int selectionEnd = 0;
@@ -480,6 +618,7 @@ if (!clazz) { \
             info.role = iface->role();
             info.actions = QAccessibleBridgeUtils::effectiveActionNames(iface);
             info.description = descriptionForInterface(iface);
+            info.identifier = QAccessibleBridgeUtils::accessibleId(iface);
             QAccessibleTextInterface *textIface = iface->textInterface();
             if (textIface && (textIface->selectionCount() > 0)) {
                 info.hasTextSelection = true;
@@ -502,6 +641,10 @@ if (!clazz) { \
             return false;
         }
 
+        const QString role = classNameForRole(info.role, info.state);
+        jstring jrole = env->NewString((jchar*)role.constData(), (jsize)role.size());
+        env->CallVoidMethod(node, m_setClassNameMethodID, jrole);
+
         const bool hasClickableAction =
                 info.actions.contains(QAccessibleActionInterface::pressAction()) ||
                 info.actions.contains(QAccessibleActionInterface::toggleAction());
@@ -545,10 +688,12 @@ if (!clazz) { \
         //CALL_METHOD(node, "setText", "(Ljava/lang/CharSequence;)V", jdesc)
         env->CallVoidMethod(node, m_setContentDescriptionMethodID, jdesc);
 
+        QJniObject(node).callMethod<void>("setViewIdResourceName", info.identifier);
+
         return true;
     }
 
-    static JNINativeMethod methods[] = {
+    static const JNINativeMethod methods[] = {
         {"setActive","(Z)V",(void*)setActive},
         {"childIdListForAccessibleObject", "(I)[I", (jintArray)childIdListForAccessibleObject},
         {"parentId", "(I)I", (void*)parentId},
@@ -568,18 +713,16 @@ if (!clazz) { \
         return false; \
     }
 
-    bool registerNatives(JNIEnv *env)
+    bool registerNatives(QJniEnvironment &env)
     {
-        jclass clazz;
-        FIND_AND_CHECK_CLASS("org/qtproject/qt/android/accessibility/QtNativeAccessibility");
-        jclass appClass = static_cast<jclass>(env->NewGlobalRef(clazz));
-
-        if (env->RegisterNatives(appClass, methods, sizeof(methods) / sizeof(methods[0])) < 0) {
+        if (!env.registerNativeMethods("org/qtproject/qt/android/QtNativeAccessibility",
+                                      methods, sizeof(methods) / sizeof(methods[0]))) {
             __android_log_print(ANDROID_LOG_FATAL,"Qt A11y", "RegisterNatives failed");
             return false;
         }
 
         jclass nodeInfoClass = env->FindClass("android/view/accessibility/AccessibilityNodeInfo");
+        GET_AND_CHECK_STATIC_METHOD(m_setClassNameMethodID, nodeInfoClass, "setClassName", "(Ljava/lang/CharSequence;)V");
         GET_AND_CHECK_STATIC_METHOD(m_addActionMethodID, nodeInfoClass, "addAction", "(I)V");
         GET_AND_CHECK_STATIC_METHOD(m_setCheckableMethodID, nodeInfoClass, "setCheckable", "(Z)V");
         GET_AND_CHECK_STATIC_METHOD(m_setCheckedMethodID, nodeInfoClass, "setChecked", "(Z)V");
