@@ -15,9 +15,10 @@ QT_BEGIN_NAMESPACE
 
 class AndroidAssetsFileEngineHandler: public QAbstractFileEngineHandler
 {
+    Q_DISABLE_COPY_MOVE(AndroidAssetsFileEngineHandler)
 public:
     AndroidAssetsFileEngineHandler();
-    QAbstractFileEngine *create(const QString &fileName) const override;
+    std::unique_ptr<QAbstractFileEngine> create(const QString &fileName) const override;
 
 private:
     AAssetManager *m_assetManager;
