@@ -45,10 +45,15 @@ void QAndroidPlatformFontDatabase::populateFontDatabase()
 QStringList QAndroidPlatformFontDatabase::fallbacksForFamily(const QString &family,
                                                              QFont::Style style,
                                                              QFont::StyleHint styleHint,
-                                                             QChar::Script script) const
+                                                             QFontDatabasePrivate::ExtendedScript script) const
 {
     QStringList result;
 
+    if (script == QFontDatabasePrivate::Script_Emoji) {
+        result.append(QStringLiteral("Noto Color Emoji"));
+        result.append(QStringLiteral("Noto Color Emoji Flags"));
+    }
+
     // Prepend CJK fonts by the locale.
     QLocale locale = QLocale::system();
     switch (locale.language()) {
