@@ -13,7 +13,12 @@
 
 QT_DECLARE_NAMESPACED_OBJC_INTERFACE(QMacAccessibilityElement, NSObject <NSAccessibilityElement>
 - (instancetype)initWithId:(QAccessible::Id)anId;
+- (instancetype)initWithId:(QAccessible::Id)anId role:(NSAccessibilityRole)role;
 + (instancetype)elementWithId:(QAccessible::Id)anId;
++ (instancetype)elementWithInterface:(QAccessibleInterface *)iface;
++ (void)removeElementsFromCache:(NSArray *)array;
+- (void)updateTableModel;
+- (QAccessibleInterface *)qtInterface;
 )
 
 #endif // QT_CONFIG(accessibility)
