@@ -159,8 +159,6 @@ static void convertLineOffset(QAccessibleTextInterface *text, int *line, int *of
                                 qCWarning(lcAccessibilityTable)
                                         << "Table representation column count is out of sync:"
                                         << rowElement->columns.count << "!=" << tableInterface->columnCount();
-                            }
-                            if (rowElement->columns) {
                                 [rowElement->columns autorelease];
                                 rowElement->columns = nil;
                             }
@@ -184,6 +182,15 @@ static void convertLineOffset(QAccessibleTextInterface *text, int *line, int *of
     return self;
 }
 
+/*!
+    \internal
+
+    Constructs a new element with the ID \a anId and inserts it into the cache.
+
+    Elements representing table rows, columns, and cells are created directly
+    via initWithId (in populateTableArray and populateTableRow), as they don't
+    get added to the cache until later.
+*/
 + (instancetype)elementWithId:(QAccessible::Id)anId
 {
     Q_ASSERT(anId);
@@ -196,7 +203,8 @@ static void convertLineOffset(QAccessibleTextInterface *text, int *line, int *of
     if (!element) {
         Q_ASSERT(QAccessible::accessibleInterface(anId));
         element = [[self alloc] initWithId:anId];
-        cache->insertElement(anId, element);
+        if (cache->insertElement(anId, element))
+            [element release];
     }
     return element;
 }
@@ -211,28 +219,54 @@ static void convertLineOffset(QAccessibleTextInterface *text, int *line, int *of
     return [self elementWithId:anId];
 }
 
-// called by QAccessibleCache::removeAccessibleElement
++ (void)removeElementsFromCache:(NSArray *)array {
+    for (uint i = 0; i < array.count; ++i) {
+        QMacAccessibilityElement *cell = [array objectAtIndex:i];
+        if (cell->axid) { // it's a proper cell, remove from cache
+            QAccessibleCache::instance()->deleteInterface(cell->axid);
+        }
+    }
+}
+
+// called by QAccessibleCache::removeAccessibleElement, which also releases
 - (void)invalidate {
     axid = 0;
     if (rows) {
+        [QMacAccessibilityElement removeElementsFromCache:rows];
         [rows autorelease];
         rows = nil;
     }
     if (columns) {
+        [QMacAccessibilityElement removeElementsFromCache:columns];
         [columns autorelease];
         columns = nil;
     }
     synthesizedRole = nil;
 
     NSAccessibilityPostNotification(self, NSAccessibilityUIElementDestroyedNotification);
-    [self release];
 }
 
+/*!
+    \internal
+
+    If this element represents a table, then the rows and columns array are both
+    populated with elements representing the rows and columns. If this elements
+    represents a row, then the columns array is populated with elements
+    representing the cells. Not all of those synthesized elements might be in
+    the cache, but those that are need to be removed so that we don't end up
+    with stale representations of children when the higher-level element
+    expires.
+*/
 - (void)dealloc {
-    if (rows)
+    if (rows) {
+        [QMacAccessibilityElement removeElementsFromCache:rows];
         [rows release]; // will also release all entries first
-    if (columns)
+    }
+    if (columns) {
+        [QMacAccessibilityElement removeElementsFromCache:columns];
         [columns release]; // will also release all entries first
+    }
+    QAccessibleCache::instance()->deleteInterface(axid);
     [super dealloc];
 }
 
@@ -255,7 +289,7 @@ static void convertLineOffset(QAccessibleTextInterface *text, int *line, int *of
 
 - (NSMutableArray *)populateTableArray:(NSAccessibilityRole)role count:(int)count
 {
-    if (QAccessibleInterface *iface = self.qtInterface) {
+    if (self.qtInterface) {
         auto *array = [NSMutableArray<QMacAccessibilityElement *> arrayWithCapacity:count];
         Q_ASSERT(array);
         for (int n = 0; n < count; ++n) {
@@ -410,7 +444,7 @@ static void convertLineOffset(QAccessibleTextInterface *text, int *line, int *of
 }
 
 - (NSString *) accessibilityRoleDescription {
-    if (QAccessibleInterface *iface = self.qtInterface)
+    if (self.qtInterface)
         return NSAccessibilityRoleDescription(self.accessibilityRole, self.accessibilitySubRole);
     return NSAccessibilityUnknownRole;
 }
@@ -519,8 +553,11 @@ static void convertLineOffset(QAccessibleTextInterface *text, int *line, int *of
 }
 
 - (id) accessibilityWindow {
-    // We're in the same window as our parent.
-    return [self.accessibilityParent accessibilityWindow];
+    // Go up until we find a parent that is a window
+    NSAccessibilityElement *parent = self.accessibilityParent;
+    if (parent && parent.accessibilityRole == NSAccessibilityWindowRole)
+        return parent;
+    return [parent accessibilityWindow];
 }
 
 - (id) accessibilityTopLevelUIElementAttribute {
@@ -571,11 +608,6 @@ static void convertLineOffset(QAccessibleTextInterface *text, int *line, int *of
         return NSAccessibilityUnignoredAncestor([QMacAccessibilityElement elementWithId:axid]);
     }
 
-    // macOS expects that the hierarchy is:
-    // App -> Window -> Children
-    // We don't actually have the window reflected properly in QAccessibility.
-    // Check if the parent is the application and then instead return the native window.
-
     if (QAccessibleInterface *parent = iface->parent()) {
         if (parent->tableInterface()) {
             QMacAccessibilityElement *tableElement =
@@ -593,7 +625,12 @@ static void convertLineOffset(QAccessibleTextInterface *text, int *line, int *of
             QMacAccessibilityElement *rowElement = tableElement->rows[rowIndex];
             return NSAccessibilityUnignoredAncestor(rowElement);
         }
-        if (parent->role() != QAccessible::Application)
+        // macOS expects that the hierarchy is:
+        // App -> Window -> Children
+        // We don't actually have the window reflected properly in QAccessibility;
+        // the native framework does that for us. Check if the parent is the
+        // Application or a window, and if so return the native NSView instead.
+        if (parent->role() != QAccessible::Application && parent->role() != QAccessible::Window)
             return NSAccessibilityUnignoredAncestor([QMacAccessibilityElement elementWithInterface: parent]);
     }
 
@@ -1000,7 +1037,7 @@ static void convertLineOffset(QAccessibleTextInterface *text, int *line, int *of
         if (self.isManagedByParent) {
             // axid matches the parent table axid so that we can easily find the parent table
             // children of row are cell/any items
-            if (QAccessibleTableInterface *table = iface->tableInterface()) {
+            if (iface->tableInterface()) {
                 if (m_rowIndex >= 0)
                     index = NSInteger(m_rowIndex);
                 else if (m_columnIndex >= 0)
@@ -1043,6 +1080,19 @@ static void convertLineOffset(QAccessibleTextInterface *text, int *line, int *of
     return nil;
 }
 
+// tabs
+
+- (NSArray *) accessibilityTabs {
+    QAccessibleInterface *iface = self.qtInterface;
+    if (iface && iface->role() == QAccessible::PageTabList) {
+        return QCocoaAccessible::unignoredChildren(iface, [](QAccessibleInterface *child){
+            return QCocoaAccessible::defaultUnignored(child)
+                && child->role() == QAccessible::PageTab;
+        });
+    }
+    return nil;
+}
+
 @end
 
 #endif // QT_CONFIG(accessibility)
