@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the plugins of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include <AppKit/AppKit.h>
 
@@ -45,6 +9,7 @@
 #include "qcocoahelpers.h"
 
 #include <QtCore/qmath.h>
+#include <QtCore/private/qcore_mac_p.h>
 #include <QtGui/qpainter.h>
 
 #include <QuartzCore/CATransaction.h>
@@ -58,8 +23,9 @@ QCocoaBackingStore::QCocoaBackingStore(QWindow *window)
 
 QCFType<CGColorSpaceRef> QCocoaBackingStore::colorSpace() const
 {
-    NSView *view = static_cast<QCocoaWindow *>(window()->handle())->view();
-    return QCFType<CGColorSpaceRef>::constructFromGet(view.window.colorSpace.CGColorSpace);
+    const auto *platformWindow = static_cast<QCocoaWindow *>(window()->handle());
+    const QNSView *view = qnsview_cast(platformWindow->view());
+    return QCFType<CGColorSpaceRef>::constructFromGet(view.colorSpace.CGColorSpace);
 }
 
 // ----------------------------------------------------------------------------
@@ -107,12 +73,11 @@ bool QCALayerBackingStore::eventFilter(QObject *watched, QEvent *event)
 
 void QCALayerBackingStore::resize(const QSize &size, const QRegion &staticContents)
 {
-    qCDebug(lcQpaBackingStore) << "Resize requested to" << size;
-
-    if (!staticContents.isNull())
-        qCWarning(lcQpaBackingStore) << "QCALayerBackingStore does not support static contents";
+    qCDebug(lcQpaBackingStore) << "Resize requested to" << size
+                               << "with static contents" << staticContents;
 
     m_requestedSize = size;
+    m_staticContents = staticContents;
 }
 
 void QCALayerBackingStore::beginPaint(const QRegion &region)
@@ -123,6 +88,13 @@ void QCALayerBackingStore::beginPaint(const QRegion &region)
 
     qCInfo(lcQpaBackingStore) << "Beginning paint of" << region << "into backingstore of" << m_requestedSize;
 
+    if (m_requestedSize.isEmpty()) {
+        // We can't create IOSurfaces with and empty size, so instead reset our back buffer
+        qCDebug(lcQpaBackingStore) << "Size is empty, throwing away back buffer";
+        m_buffers.back().reset(nullptr);
+        return;
+    }
+
     ensureBackBuffer(); // Find an unused back buffer, or reserve space for a new one
 
     const bool bufferWasRecreated = recreateBackBufferIfNeeded();
@@ -224,11 +196,50 @@ bool QCALayerBackingStore::recreateBackBufferIfNeeded()
         }
 #endif
 
-        qCInfo(lcQpaBackingStore) << "Creating surface of" << requestedBufferSize
-            << "based on requested" << m_requestedSize << "and dpr =" << devicePixelRatio;
+        qCInfo(lcQpaBackingStore)<< "Creating surface of" << requestedBufferSize
+            << "for" << window() << "based on requested" << m_requestedSize
+            << "dpr =" << devicePixelRatio << "and color space" << colorSpace();
 
         static auto pixelFormat = QImage::toPixelFormat(QImage::Format_ARGB32_Premultiplied);
-        m_buffers.back().reset(new GraphicsBuffer(requestedBufferSize, devicePixelRatio, pixelFormat, colorSpace()));
+        auto *newBackBuffer = new GraphicsBuffer(requestedBufferSize, devicePixelRatio, pixelFormat, colorSpace());
+
+        if (!m_staticContents.isEmpty() && m_buffers.back()) {
+            // We implicitly support static backingstore content as a result of
+            // finalizing the back buffer on flush, where we copy any non-painted
+            // areas from the front buffer. But there is no guarantee that a resize
+            // will always come after a flush, where we have a pristine front buffer
+            // to copy from. It may come after a few begin/endPaints, where the back
+            // buffer then contains (part of) the latest state. We also have the case
+            // of single-buffered backingstore, where the front and back buffer is
+            // the same, which means we must do the copy from the old back buffer
+            // to the newly resized buffer now, before we replace it below.
+
+            // If the back buffer has been partially filled already, we need to
+            // copy parts of the static content from that. The rest we copy from
+            // the front buffer.
+            const QRegion backBufferRegion = m_staticContents - m_buffers.back()->dirtyRegion;
+            const QRegion frontBufferRegion = m_staticContents - backBufferRegion;
+
+            qCInfo(lcQpaBackingStore) << "Preserving static content" << backBufferRegion
+                << "from back buffer, and" << frontBufferRegion << "from front buffer";
+
+            newBackBuffer->lock(QPlatformGraphicsBuffer::SWWriteAccess);
+            blitBuffer(m_buffers.back().get(), backBufferRegion, newBackBuffer);
+            Q_ASSERT(frontBufferRegion.isEmpty() || m_buffers.front());
+            blitBuffer(m_buffers.front().get(), frontBufferRegion, newBackBuffer);
+            newBackBuffer->unlock();
+
+            // The new back buffer now is valid for the static contents region.
+            // We don't need to maintain the static contents region for resizes
+            // of any other buffers in the swap chain, as these will finalize
+            // their content on flush from the buffer we just filled, and we
+            // don't need to mark them dirty for the area we just filled, as
+            // new buffers are fully dirty when created.
+            newBackBuffer->dirtyRegion -= m_staticContents;
+            m_staticContents = {};
+        }
+
+        m_buffers.back().reset(newBackBuffer);
         return true;
     }
 
@@ -237,12 +248,19 @@ bool QCALayerBackingStore::recreateBackBufferIfNeeded()
 
 QPaintDevice *QCALayerBackingStore::paintDevice()
 {
-    Q_ASSERT(m_buffers.back());
-    return m_buffers.back()->asImage();
+    if (m_buffers.back()) {
+        return m_buffers.back()->asImage();
+    } else {
+        static QImage fallbackDevice;
+        return &fallbackDevice;
+    }
 }
 
 void QCALayerBackingStore::endPaint()
 {
+    if (!m_buffers.back())
+        return;
+
     qCInfo(lcQpaBackingStore) << "Paint ended. Back buffer valid region is now" << m_buffers.back()->validRegion();
     m_buffers.back()->unlock();
 
@@ -271,25 +289,27 @@ bool QCALayerBackingStore::scroll(const QRegion &region, int dx, int dy)
     m_buffers.back()->lock(QPlatformGraphicsBuffer::SWWriteAccess);
 
     if (!inPlaceRegion.isEmpty()) {
-        qCDebug(lcQpaBackingStore) << "Scrolling" << inPlaceRegion << "in place";
+        // We have to scroll everything in one go, instead of scrolling the
+        // individual rects of the region, as otherwise we may end up reading
+        // already overwritten (scrolled) pixels.
+        const QRect inPlaceBoundingRect = inPlaceRegion.boundingRect();
+
+        qCDebug(lcQpaBackingStore) << "Scrolling" << inPlaceBoundingRect << "in place";
         QImage *backBufferImage = m_buffers.back()->asImage();
         const qreal devicePixelRatio = backBufferImage->devicePixelRatio();
         const QPoint devicePixelDelta = scrollDelta * devicePixelRatio;
 
         extern void qt_scrollRectInImage(QImage &, const QRect &, const QPoint &);
 
-        for (const QRect &rect : inPlaceRegion) {
-            qt_scrollRectInImage(*backBufferImage,
-                QRect(rect.topLeft() * devicePixelRatio,
-                      rect.size() * devicePixelRatio),
-                      devicePixelDelta);
-        }
-
+        qt_scrollRectInImage(*backBufferImage,
+            QRect(inPlaceBoundingRect.topLeft() * devicePixelRatio,
+                  inPlaceBoundingRect.size() * devicePixelRatio),
+                  devicePixelDelta);
     }
 
     if (!frontBufferRegion.isEmpty()) {
         qCDebug(lcQpaBackingStore) << "Scrolling" << frontBufferRegion << "by copying from front buffer";
-        preserveFromFrontBuffer(frontBufferRegion, scrollDelta);
+        blitBuffer(m_buffers.front().get(), frontBufferRegion, m_buffers.back().get(), scrollDelta);
     }
 
     m_buffers.back()->unlock();
@@ -311,7 +331,7 @@ void QCALayerBackingStore::flush(QWindow *flushedWindow, const QRegion &region,
     Q_UNUSED(offset);
 
     if (!m_buffers.back()) {
-        qCWarning(lcQpaBackingStore) << "Tried to flush backingstore without painting to it first";
+        qCWarning(lcQpaBackingStore) << "Flush requested with no back buffer. Ignoring.";
         return;
     }
 
@@ -393,7 +413,9 @@ void QCALayerBackingStore::flushSubWindow(QWindow *subWindow)
         subWindowBackingStore->m_clearSurfaceOnPaint = false;
     }
 
-    auto subWindowSize = subWindow->size();
+    // Query platform window for subwindow size, so that we
+    // incorporate any effects of the QHighDpiScaling layer.
+    auto subWindowSize = subWindow->handle()->geometry().size();
     static const auto kNoStaticContents = QRegion();
     subWindowBackingStore->resize(subWindowSize, kNoStaticContents);
 
@@ -428,20 +450,24 @@ void QCALayerBackingStore::windowDestroyed(QObject *object)
     m_subWindowBackingstores.erase(window);
 }
 
-#ifndef QT_NO_OPENGL
-void QCALayerBackingStore::composeAndFlush(QWindow *window, const QRegion &region, const QPoint &offset,
-                                    QPlatformTextureList *textures, bool translucentBackground)
+QPlatformBackingStore::FlushResult QCALayerBackingStore::rhiFlush(QWindow *window,
+                                                                  qreal sourceDevicePixelRatio,
+                                                                  const QRegion &region,
+                                                                  const QPoint &offset,
+                                                                  QPlatformTextureList *textures,
+                                                                  bool translucentBackground,
+                                                                  qreal sourceTransformFactor)
 {
     if (!m_buffers.back()) {
-        qCWarning(lcQpaBackingStore) << "Tried to flush backingstore without painting to it first";
-        return;
+        qCWarning(lcQpaBackingStore) << "Flush requested with no back buffer. Ignoring.";
+        return FlushFailed;
     }
 
     finalizeBackBuffer();
 
-    QPlatformBackingStore::composeAndFlush(window, region, offset, textures, translucentBackground);
+    return QPlatformBackingStore::rhiFlush(window, sourceDevicePixelRatio,
+        region, offset, textures, translucentBackground, sourceTransformFactor);
 }
-#endif
 
 QImage QCALayerBackingStore::toImage() const
 {
@@ -471,10 +497,11 @@ void QCALayerBackingStore::backingPropertiesChanged()
 
     qCDebug(lcQpaBackingStore) << "Backing properties for" << window() << "did change";
 
-    qCDebug(lcQpaBackingStore) << "Updating color space of existing buffers";
+    const auto newColorSpace = colorSpace();
+    qCDebug(lcQpaBackingStore) << "Updating color space of existing buffers to" << newColorSpace;
     for (auto &buffer : m_buffers) {
         if (buffer)
-            buffer->setColorSpace(colorSpace());
+            buffer->setColorSpace(newColorSpace);
     }
 }
 
@@ -506,54 +533,76 @@ void QCALayerBackingStore::finalizeBackBuffer()
     if (!m_buffers.back()->isDirty())
         return;
 
-    m_buffers.back()->lock(QPlatformGraphicsBuffer::SWWriteAccess);
-    preserveFromFrontBuffer(m_buffers.back()->dirtyRegion);
-    m_buffers.back()->unlock();
+    qCDebug(lcQpaBackingStore) << "Finalizing back buffer with dirty region" << m_buffers.back()->dirtyRegion;
+
+    if (m_buffers.back() != m_buffers.front()) {
+        m_buffers.back()->lock(QPlatformGraphicsBuffer::SWWriteAccess);
+        blitBuffer(m_buffers.front().get(), m_buffers.back()->dirtyRegion, m_buffers.back().get());
+        m_buffers.back()->unlock();
+    } else {
+        qCDebug(lcQpaBackingStore) << "Front and back buffer is the same. Can not finalize back buffer.";
+    }
 
     // The back buffer is now completely in sync, ready to be presented
     m_buffers.back()->dirtyRegion = QRegion();
 }
 
-void QCALayerBackingStore::preserveFromFrontBuffer(const QRegion &region, const QPoint &offset)
+/*
+    \internal
+
+    Blits \a sourceRegion from \a sourceBuffer to \a destinationBuffer,
+    at offset \a destinationOffset.
+
+    The source buffer is automatically locked for read only access
+    during the blit.
+
+    The destination buffer has to be locked for write access by the
+    caller.
+*/
+
+void QCALayerBackingStore::blitBuffer(GraphicsBuffer *sourceBuffer, const QRegion &sourceRegion,
+                                      GraphicsBuffer *destinationBuffer, const QPoint &destinationOffset)
 {
+    Q_ASSERT(sourceBuffer && destinationBuffer);
+    Q_ASSERT(sourceBuffer != destinationBuffer);
 
-    if (m_buffers.front() == m_buffers.back())
-        return; // Nothing to preserve from
+    if (sourceRegion.isEmpty())
+        return;
 
-    qCDebug(lcQpaBackingStore) << "Preserving" << region << "of front buffer to"
-        << region.translated(offset) << "of back buffer";
+    qCDebug(lcQpaBackingStore) << "Blitting" << sourceRegion << "of" << sourceBuffer
+        << "to" << sourceRegion.translated(destinationOffset) << "of" << destinationBuffer;
 
-    Q_ASSERT(m_buffers.back()->isLocked() == QPlatformGraphicsBuffer::SWWriteAccess);
+    Q_ASSERT(destinationBuffer->isLocked() == QPlatformGraphicsBuffer::SWWriteAccess);
 
-    m_buffers.front()->lock(QPlatformGraphicsBuffer::SWReadAccess);
-    const QImage *frontBuffer = m_buffers.front()->asImage();
+    sourceBuffer->lock(QPlatformGraphicsBuffer::SWReadAccess);
+    const QImage *sourceImage = sourceBuffer->asImage();
 
-    const QRect frontSurfaceBounds(QPoint(0, 0), m_buffers.front()->size());
-    const qreal sourceDevicePixelRatio = frontBuffer->devicePixelRatio();
+    const QRect sourceBufferBounds(QPoint(0, 0), sourceBuffer->size());
+    const qreal sourceDevicePixelRatio = sourceImage->devicePixelRatio();
 
-    QPainter painter(m_buffers.back()->asImage());
+    QPainter painter(destinationBuffer->asImage());
     painter.setCompositionMode(QPainter::CompositionMode_Source);
 
     // Let painter operate in device pixels, to make it easier to compare coordinates
-    const qreal targetDevicePixelRatio = painter.device()->devicePixelRatio();
-    painter.scale(1.0 / targetDevicePixelRatio, 1.0 / targetDevicePixelRatio);
+    const qreal destinationDevicePixelRatio = painter.device()->devicePixelRatio();
+    painter.scale(1.0 / destinationDevicePixelRatio, 1.0 / destinationDevicePixelRatio);
 
-    for (const QRect &rect : region) {
+    for (const QRect &rect : sourceRegion) {
         QRect sourceRect(rect.topLeft() * sourceDevicePixelRatio,
                          rect.size() * sourceDevicePixelRatio);
-        QRect targetRect((rect.topLeft() + offset) * targetDevicePixelRatio,
-                          rect.size() * targetDevicePixelRatio);
+        QRect destinationRect((rect.topLeft() + destinationOffset) * destinationDevicePixelRatio,
+                               rect.size() * destinationDevicePixelRatio);
 
 #ifdef QT_DEBUG
-        if (Q_UNLIKELY(!frontSurfaceBounds.contains(sourceRect.bottomRight()))) {
-            qCWarning(lcQpaBackingStore) << "Front buffer too small to preserve"
-                << QRegion(sourceRect).subtracted(frontSurfaceBounds);
+        if (Q_UNLIKELY(!sourceBufferBounds.contains(sourceRect.bottomRight()))) {
+            qCWarning(lcQpaBackingStore) << "Source buffer of size" << sourceBuffer->size()
+                                         << "is too small to blit" << sourceRect;
         }
 #endif
-        painter.drawImage(targetRect, *frontBuffer, sourceRect);
+        painter.drawImage(destinationRect, *sourceImage, sourceRect);
     }
 
-    m_buffers.front()->unlock();
+    sourceBuffer->unlock();
 }
 
 // ----------------------------------------------------------------------------
@@ -591,6 +640,6 @@ QImage *QCALayerBackingStore::GraphicsBuffer::asImage()
     return &m_image;
 }
 
-#include "moc_qcocoabackingstore.cpp"
-
 QT_END_NAMESPACE
+
+#include "moc_qcocoabackingstore.cpp"
