@@ -41,15 +41,15 @@ public:
     QWindow *topLevelAt(const QPoint &point) const override;
     QList<QPlatformScreen *> virtualSiblings() const override;
     QPlatformScreen::SubpixelAntialiasingType subpixelAntialiasingTypeHint() const override;
+    Qt::ScreenOrientation orientation() const override;
 
     // ----------------------------------------------------
 
     static NSScreen *nativeScreenForDisplayId(CGDirectDisplayID displayId);
     NSScreen *nativeScreen() const;
 
-    void requestUpdate();
+    bool requestUpdate();
     void deliverUpdateRequests();
-    bool isRunningDisplayLink() const;
 
     static QCocoaScreen *primaryScreen();
     static QCocoaScreen *get(NSScreen *nsScreen);
@@ -90,10 +90,14 @@ private:
     QSizeF m_physicalSize;
     QCocoaCursor *m_cursor;
     qreal m_devicePixelRatio = 0;
+    qreal m_rotation = 0;
 
     CVDisplayLinkRef m_displayLink = nullptr;
     dispatch_source_t m_displayLinkSource = nullptr;
-    QAtomicInt m_pendingUpdates;
+    QAtomicInt m_pendingUpdateRequests;
+    QAtomicInt m_pendingDisplayLinkUpdates;
+
+    void maybeStopDisplayLink();
 
     friend class QCocoaIntegration;
     friend class QCocoaWindow;
