@@ -6,7 +6,9 @@
 #include "qeglfskmsgbmscreen_p.h"
 #include "qeglfskmsgbmdevice_p.h"
 #include "qeglfskmsgbmcursor_p.h"
+
 #include <private/qeglfsintegration_p.h>
+#include <private/qeglfskmsintegration_p.h>
 
 #include <QtCore/QLoggingCategory>
 
@@ -18,8 +20,6 @@
 
 QT_BEGIN_NAMESPACE
 
-Q_DECLARE_LOGGING_CATEGORY(qLcEglfsKmsDebug)
-
 QMutex QEglFSKmsGbmScreen::m_nonThreadedFlipMutex;
 
 static inline uint32_t drmFormatToGbmFormat(uint32_t drmFormat)
@@ -73,8 +73,9 @@ QEglFSKmsGbmScreen::FrameBuffer *QEglFSKmsGbmScreen::framebufferForBufferObject(
         return nullptr;
     }
 
-    gbm_bo_set_user_data(bo, fb.get(), bufferDestroyedHandler);
-    return fb.release();
+    auto res = fb.get();
+    gbm_bo_set_user_data(bo, fb.release(), bufferDestroyedHandler);
+    return res;
 }
 
 QEglFSKmsGbmScreen::QEglFSKmsGbmScreen(QEglFSKmsDevice *device, const QKmsOutput &output, bool headless)
@@ -142,11 +143,12 @@ gbm_surface *QEglFSKmsGbmScreen::createSurface(EGLConfig eglConfig)
             }
         }
 
+        const uint32_t gbmFormat = drmFormatToGbmFormat(m_output.drm_format);
+
         // Fallback for older drivers, and when "format" is explicitly specified
         // in the output config. (not guaranteed that the requested format works
         // of course, but do what we are told to)
         if (!m_gbm_surface) {
-            uint32_t gbmFormat = drmFormatToGbmFormat(m_output.drm_format);
             if (queryFromEgl)
                 qCDebug(qLcEglfsKmsDebug, "Could not create surface with EGL_NATIVE_VISUAL_ID, falling back to format %x", gbmFormat);
             m_gbm_surface = gbm_surface_create(gbmDevice,
@@ -155,13 +157,41 @@ gbm_surface *QEglFSKmsGbmScreen::createSurface(EGLConfig eglConfig)
                                            gbmFormat,
                                            gbmFlags());
         }
+
+#ifndef Q_OS_VXWORKS
+        // Fallback for some drivers, its required to request with modifiers
+        if (!m_gbm_surface) {
+            uint64_t modifier = DRM_FORMAT_MOD_LINEAR;
+
+            m_gbm_surface = gbm_surface_create_with_modifiers(gbmDevice,
+                                    rawGeometry().width(),
+                                    rawGeometry().height(),
+                                    gbmFormat,
+                                    &modifier, 1);
+        }
+#endif
+        // Fail here, as it would fail with the next usage of the GBM surface, which is very unexpected
+        if (!m_gbm_surface)
+            qFatal("Could not create GBM surface!");
     }
     return m_gbm_surface; // not owned, gets destroyed in QEglFSKmsGbmIntegration::destroyNativeWindow() via QEglFSKmsGbmWindow::invalidateSurface()
 }
 
 void QEglFSKmsGbmScreen::resetSurface()
 {
+    m_flipPending = false; // not necessarily true but enough to keep bo_next
+    m_gbm_bo_current = nullptr;
     m_gbm_surface = nullptr;
+
+    // Leave m_gbm_bo_next untouched. waitForFlip() should
+    // still do its work, when called. Otherwise we end up
+    // in device-is-busy errors if there is a new QWindow
+    // created afterwards. (QTBUG-122663)
+
+    // If not using atomic, will need a new drmModeSetCrtc if a new window
+    // gets created later on (and so there's a new fb).
+    if (!device()->hasAtomicSupport())
+        needsNewModeSetForNextFb = true;
 }
 
 void QEglFSKmsGbmScreen::initCloning(QPlatformScreen *screenThisScreenClones,
@@ -191,8 +221,9 @@ void QEglFSKmsGbmScreen::ensureModeSet(uint32_t fb)
     QKmsOutput &op(output());
     const int fd = device()->fd();
 
-    if (!op.mode_set) {
+    if (!op.mode_set || needsNewModeSetForNextFb) {
         op.mode_set = true;
+        needsNewModeSetForNextFb = false;
 
         bool doModeSet = true;
         drmModeCrtcPtr currentMode = drmModeGetCrtc(fd, op.crtc_id);
@@ -349,7 +380,17 @@ void QEglFSKmsGbmScreen::flip()
         return;
     }
 
+    auto gbmRelease = qScopeGuard([this]{
+        m_flipPending = false;
+        gbm_surface_release_buffer(m_gbm_surface, m_gbm_bo_next);
+        m_gbm_bo_next = nullptr;
+    });
+
     FrameBuffer *fb = framebufferForBufferObject(m_gbm_bo_next);
+    if (!fb) {
+        qWarning("FrameBuffer not available. Cannot flip");
+        return;
+    }
     ensureModeSet(fb->fb);
 
     const QKmsOutput &thisOutput(output());
@@ -381,9 +422,6 @@ void QEglFSKmsGbmScreen::flip()
                                   this);
         if (ret) {
             qErrnoWarning("Could not queue DRM page flip on screen %s", qPrintable(name()));
-            m_flipPending = false;
-            gbm_surface_release_buffer(m_gbm_surface, m_gbm_bo_next);
-            m_gbm_bo_next = nullptr;
             return;
         }
     }
@@ -423,9 +461,15 @@ void QEglFSKmsGbmScreen::flip()
         }
     }
 
+    if (device()->hasAtomicSupport()) {
 #if QT_CONFIG(drm_atomic)
-    device()->threadLocalAtomicCommit(this);
+        if (!device()->threadLocalAtomicCommit(this)) {
+            return;
+        }
 #endif
+    }
+
+    gbmRelease.dismiss();
 }
 
 void QEglFSKmsGbmScreen::flipFinished()
