@@ -12,19 +12,22 @@ QT_BEGIN_NAMESPACE
 
 class QIOSFileEngineFactory : public QAbstractFileEngineHandler
 {
+    Q_DISABLE_COPY_MOVE(QIOSFileEngineFactory)
 public:
-    QAbstractFileEngine* create(const QString &fileName) const
+    QIOSFileEngineFactory() = default;
+
+    std::unique_ptr<QAbstractFileEngine> create(const QString &fileName) const
     {
-        static QLatin1StringView assetsScheme("assets-library:");
+        Q_CONSTINIT static QLatin1StringView assetsScheme("assets-library:");
 
 #ifndef Q_OS_TVOS
         if (fileName.toLower().startsWith(assetsScheme))
-            return new QIOSFileEngineAssetsLibrary(fileName);
+            return std::make_unique<QIOSFileEngineAssetsLibrary>(fileName);
 #else
         Q_UNUSED(fileName);
 #endif
 
-        return 0;
+        return {};
     }
 };
 
