@@ -9,6 +9,7 @@
 #include <qpa/qplatformwindow.h>
 
 #include <QMap>
+#include <tuple>
 
 QT_BEGIN_NAMESPACE
 
@@ -35,9 +36,9 @@ public:
 
     void requestUpdate();
     enum UpdateRequestDeliveryType { ExposeEventDelivery, UpdateRequestDelivery };
-    void requestUpdateWindow(QWasmWindow *window, UpdateRequestDeliveryType updateType = ExposeEventDelivery);
+    void requestUpdateWindow(QWasmWindow *window, const QRect &updateRect, UpdateRequestDeliveryType updateType = ExposeEventDelivery);
 
-    void handleBackingStoreFlush(QWindow *window);
+    void handleBackingStoreFlush(QWindow *window, const QRect &updateRect);
     void onWindowTreeChanged(QWasmWindowTreeNodeChangeType changeType, QWasmWindow *window);
 
 private:
@@ -46,10 +47,10 @@ private:
     void deregisterEventHandlers();
 
     void deliverUpdateRequests();
-    void deliverUpdateRequest(QWasmWindow *window, UpdateRequestDeliveryType updateType);
+    void deliverUpdateRequest(QWasmWindow *window, const QRect &updateRect, UpdateRequestDeliveryType updateType);
 
     bool m_isEnabled = true;
-    QMap<QWasmWindow *, UpdateRequestDeliveryType> m_requestUpdateWindows;
+    QMap<QWasmWindow *, std::tuple<QRect, UpdateRequestDeliveryType>> m_requestUpdateWindows;
     int m_requestAnimationFrameId = -1;
     bool m_inDeliverUpdateRequest = false;
     static bool m_requestUpdateHoldEnabled;
