@@ -30,6 +30,8 @@ static bool initResources()
 
 QT_BEGIN_NAMESPACE
 
+using namespace Qt::Literals::StringLiterals;
+
 /*!
     \class QWindowsCursorCacheKey
     \brief Cache key for storing values in a QHash with a QCursor as key.
@@ -145,8 +147,8 @@ static HCURSOR createBitmapCursor(const QCursor &cursor, qreal scaleFactor = 1)
         bbits = bbits.scaled(scaledSize, Qt::KeepAspectRatio, Qt::SmoothTransformation);
         mbits = mbits.scaled(scaledSize, Qt::KeepAspectRatio, Qt::SmoothTransformation);
     }
-    bbits = bbits.convertToFormat(QImage::Format_Mono);
-    mbits = mbits.convertToFormat(QImage::Format_Mono);
+    bbits = std::move(bbits).convertToFormat(QImage::Format_Mono);
+    mbits = std::move(mbits).convertToFormat(QImage::Format_Mono);
     const bool invb = bbits.colorCount() > 1 && qGray(bbits.color(0)) < qGray(bbits.color(1));
     const bool invm = mbits.colorCount() > 1 && qGray(mbits.color(0)) < qGray(mbits.color(1));
     return createBitmapCursor(bbits, mbits, cursor.hotSpot(), invb, invm);
@@ -443,8 +445,8 @@ QWindowsCursor::PixmapCursor QWindowsCursor::customCursor(Qt::CursorShape cursor
     if (!bestFit)
         return PixmapCursor();
 
-    const QPixmap rawImage(QStringLiteral(":/qt-project.org/windows/cursors/images/") +
-                           QString::fromLatin1(bestFit->fileName));
+    const QPixmap rawImage(":/qt-project.org/windows/cursors/images/"_L1 +
+                           QLatin1StringView(bestFit->fileName));
     return PixmapCursor(rawImage, QPoint(bestFit->hotSpotX, bestFit->hotSpotY));
 }
 #endif // !QT_NO_IMAGEFORMAT_PNG
@@ -521,6 +523,7 @@ CursorHandlePtr QWindowsCursor::standardWindowCursor(Qt::CursorShape shape)
 
 HCURSOR QWindowsCursor::m_overriddenCursor = nullptr;
 HCURSOR QWindowsCursor::m_overrideCursor = nullptr;
+POINT QWindowsCursor::m_cursorPositionCache = {0,0};
 
 /*!
     \brief Return cached pixmap cursor or create new one.
@@ -631,8 +634,9 @@ void QWindowsCursor::clearOverrideCursor()
 QPoint QWindowsCursor::mousePosition()
 {
     POINT p;
-    GetCursorPos(&p);
-    return QPoint(p.x, p.y);
+    if (GetCursorPos(&p))
+        m_cursorPositionCache = p;
+    return QPoint(m_cursorPositionCache.x, m_cursorPositionCache.y);
 }
 
 QWindowsCursor::State QWindowsCursor::cursorState()
@@ -656,6 +660,7 @@ QPoint QWindowsCursor::pos() const
 
 void QWindowsCursor::setPos(const QPoint &pos)
 {
+    m_cursorPositionCache = {pos.x(), pos.y()};
     SetCursorPos(pos.x() , pos.y());
 }
 
@@ -666,16 +671,16 @@ void QWindowsCursor::setPos(const QPoint &pos)
 */
 QSize QWindowsCursor::size() const
 {
-    const QPair<DWORD,bool> cursorSizeSetting =
+    const auto cursorSizeSetting =
         QWinRegistryKey(HKEY_CURRENT_USER, LR"(Control Panel\Cursors)")
-                       .dwordValue(L"CursorBaseSize");
+                       .value<DWORD>(L"CursorBaseSize");
     const int baseSize = screenCursorSize(m_screen).width() / 2;
-    if (!cursorSizeSetting.second)
+    if (!cursorSizeSetting)
         return QSize(baseSize / 2, baseSize / 2);
 
     // The registry values are dpi-independent, so we need to scale the result.
-    int cursorSizeValue = cursorSizeSetting.first * m_screen->logicalDpi().first
-                                                  / m_screen->logicalBaseDpi().first;
+    int cursorSizeValue = *cursorSizeSetting * m_screen->logicalDpi().first
+                                             / m_screen->logicalBaseDpi().first;
 
     // map from registry value 32-256 to 0-14, and from there to pixels
     cursorSizeValue = (cursorSizeValue - 2 * baseSize) / baseSize;
@@ -703,6 +708,7 @@ QPixmap QWindowsCursor::dragDefaultCursor(Qt::DropAction action) const
         break;
     }
 
+#if QT_CONFIG(imageformat_xpm)
     static const char * const ignoreDragCursorXpmC[] = {
     "24 30 3 1",
     ".        c None",
@@ -738,6 +744,7 @@ QPixmap QWindowsCursor::dragDefaultCursor(Qt::DropAction action) const
     "............XaaaaaaaaX..",
     ".............XXaaaaXX...",
     "...............XXXX....."};
+#endif
 
     if (m_ignoreDragCursor.isNull()) {
         HCURSOR cursor = LoadCursor(nullptr, IDC_NO);
@@ -756,8 +763,10 @@ QPixmap QWindowsCursor::dragDefaultCursor(Qt::DropAction action) const
 
             m_ignoreDragCursor = QPixmap::fromImage(colorImage);
             delete [] colorBits;
+#if QT_CONFIG(imageformat_xpm)
         } else {
             m_ignoreDragCursor = QPixmap(ignoreDragCursorXpmC);
+#endif
         }
 
         DeleteObject(iconInfo.hbmMask);
