@@ -1,13 +1,11 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
-#ifndef QXCBINCLUDE_H
-#define QXCBINCLUDE_H
+#pragma once
 
 #include <QtGui/QPalette>
 #include <QtCore/QTextStream>
 #include <QtGui/private/qmath_p.h>
-#include <QtGui/private/qcssparser_p.h>
 #include <QtGui/private/qtextengine_p.h>
 
 #include <QtGui/private/qt_egl_p.h>
@@ -15,5 +13,3 @@
 QT_BEGIN_NAMESPACE
 
 QT_END_NAMESPACE
-
-#endif //QXCBINCLUDE_H
