@@ -3,9 +3,7 @@
 
 #include "qgtk3theme.h"
 #include "qgtk3dialoghelpers.h"
-#include "qgtk3menu.h"
 #include <QVariant>
-#include <QtCore/qregularexpression.h>
 #include <QGuiApplication>
 #include <qpa/qwindowsysteminterface.h>
 
@@ -90,6 +88,7 @@ QGtk3Theme::QGtk3Theme()
     };
 
     GtkSettings *settings = gtk_settings_get_default();
+    SETTING_CONNECT("gtk-cursor-blink");
     SETTING_CONNECT("gtk-cursor-blink-time");
     SETTING_CONNECT("gtk-double-click-distance");
     SETTING_CONNECT("gtk-double-click-time");
@@ -101,21 +100,11 @@ QGtk3Theme::QGtk3Theme()
     SETTING_CONNECT("gtk-font-name");
     SETTING_CONNECT("gtk-application-prefer-dark-theme");
     SETTING_CONNECT("gtk-theme-name");
+    SETTING_CONNECT("gtk-cursor-theme-name");
+    SETTING_CONNECT("gtk-cursor-theme-size");
 #undef SETTING_CONNECT
 
-    /* Set XCURSOR_SIZE and XCURSOR_THEME for Wayland sessions */
-    if (QGuiApplication::platformName().startsWith("wayland"_L1)) {
-        if (qEnvironmentVariableIsEmpty("XCURSOR_SIZE")) {
-            const int cursorSize = gtkSetting<gint>("gtk-cursor-theme-size");
-            if (cursorSize > 0)
-                qputenv("XCURSOR_SIZE", QString::number(cursorSize).toUtf8());
-        }
-        if (qEnvironmentVariableIsEmpty("XCURSOR_THEME")) {
-            const QString cursorTheme = gtkSetting("gtk-cursor-theme-name");
-            if (!cursorTheme.isEmpty())
-                qputenv("XCURSOR_THEME", cursorTheme.toUtf8());
-        }
-    }
+    m_storage.reset(new QGtk3Storage);
 }
 
 static inline QVariant gtkGetLongPressTime()
@@ -131,7 +120,10 @@ QVariant QGtk3Theme::themeHint(QPlatformTheme::ThemeHint hint) const
 {
     switch (hint) {
     case QPlatformTheme::CursorFlashTime:
-        return QVariant(gtkSetting<gint>("gtk-cursor-blink-time"));
+        if (gtkSetting<gboolean>("gtk-cursor-blink"))
+            return QVariant(gtkSetting<gint>("gtk-cursor-blink-time"));
+        else
+            return 0;
     case QPlatformTheme::MouseDoubleClickDistance:
         return QVariant(gtkSetting<gint>("gtk-double-click-distance"));
     case QPlatformTheme::MouseDoubleClickInterval:
@@ -150,6 +142,14 @@ QVariant QGtk3Theme::themeHint(QPlatformTheme::ThemeHint hint) const
         return QVariant(gtkSetting("gtk-icon-theme-name"));
     case QPlatformTheme::SystemIconFallbackThemeName:
         return QVariant(gtkSetting("gtk-fallback-icon-theme"));
+    case QPlatformTheme::MouseCursorTheme:
+        return QVariant(gtkSetting("gtk-cursor-theme-name"));
+    case QPlatformTheme::MouseCursorSize: {
+        int s = gtkSetting<gint>("gtk-cursor-theme-size");
+        if (s > 0)
+            return QVariant(QSize(s, s));
+        return QGnomeTheme::themeHint(hint);
+    }
     default:
         return QGnomeTheme::themeHint(hint);
     }
@@ -163,45 +163,28 @@ QString QGtk3Theme::gtkFontName() const
     return QGnomeTheme::gtkFontName();
 }
 
-QPlatformTheme::Appearance QGtk3Theme::appearance() const
+void QGtk3Theme::requestColorScheme(Qt::ColorScheme scheme)
 {
-    /*
-        https://docs.gtk.org/gtk3/running.html
+    if (m_requestedColorScheme == scheme)
+        return;
+    qCDebug(lcQGtk3Interface) << scheme << "has been requested. Theme supports color scheme:"
+                              << m_storage->colorScheme();
+    m_requestedColorScheme = scheme;
+    m_storage->handleThemeChange();
+}
 
-        It's possible to set a theme variant after the theme name when using GTK_THEME:
-
-            GTK_THEME=Adwaita:dark
-
-        Some themes also have "-dark" as part of their name.
-
-        We test this environment variable first because the documentation says
-        it's mainly used for easy debugging, so it should be possible to use it
-        to override any other settings.
-    */
-    QString themeName = qEnvironmentVariable("GTK_THEME");
-    const QRegularExpression darkRegex(QStringLiteral("[:-]dark"), QRegularExpression::CaseInsensitiveOption);
-    if (!themeName.isEmpty())
-        return darkRegex.match(themeName).hasMatch() ? Appearance::Dark : Appearance::Light;
-
-    /*
-        https://docs.gtk.org/gtk3/property.Settings.gtk-application-prefer-dark-theme.html
-
-        This setting controls which theme is used when the theme specified by
-        gtk-theme-name provides both light and dark variants. We can save a
-        regex check by testing this property first.
-    */
-    const auto preferDark = gtkSetting<gboolean>("gtk-application-prefer-dark-theme");
-    if (preferDark)
-        return Appearance::Dark;
-
-    /*
-        https://docs.gtk.org/gtk3/property.Settings.gtk-theme-name.html
-    */
-    themeName = gtkSetting("gtk-theme-name");
-    if (!themeName.isEmpty())
-        return darkRegex.match(themeName).hasMatch() ? Appearance::Dark : Appearance::Light;
-
-    return Appearance::Unknown;
+Qt::ColorScheme QGtk3Theme::colorScheme() const
+{
+    Q_ASSERT(m_storage);
+#ifdef QT_DEBUG
+    if (m_requestedColorScheme != Qt::ColorScheme::Unknown
+        && m_requestedColorScheme != m_storage->colorScheme()) {
+        qCDebug(lcQGtk3Interface) << "Requested color scheme" << m_requestedColorScheme
+                                  << "differs from theme color scheme" << m_storage->colorScheme();
+    }
+#endif
+    return m_requestedColorScheme == Qt::ColorScheme::Unknown ? m_storage->colorScheme()
+                                                              : m_requestedColorScheme;
 }
 
 bool QGtk3Theme::usePlatformNativeDialog(DialogType type) const
@@ -234,16 +217,6 @@ QPlatformDialogHelper *QGtk3Theme::createPlatformDialogHelper(DialogType type) c
     }
 }
 
-QPlatformMenu* QGtk3Theme::createPlatformMenu() const
-{
-    return new QGtk3Menu;
-}
-
-QPlatformMenuItem* QGtk3Theme::createPlatformMenuItem() const
-{
-    return new QGtk3MenuItem;
-}
-
 bool QGtk3Theme::useNativeFileDialog()
 {
     /* Require GTK3 >= 3.15.5 to avoid running into this bug:
@@ -257,4 +230,41 @@ bool QGtk3Theme::useNativeFileDialog()
     return gtk_check_version(3, 15, 5) == nullptr;
 }
 
+const QPalette *QGtk3Theme::palette(Palette type) const
+{
+    Q_ASSERT(m_storage);
+#ifdef QT_DEBUG
+    if (m_requestedColorScheme != Qt::ColorScheme::Unknown
+        && m_requestedColorScheme != m_storage->colorScheme()) {
+        qCDebug(lcQGtk3Interface) << "Current KDE theme doesn't support reuqested color scheme"
+                                  << m_requestedColorScheme << "Falling back to fusion palette.";
+        return QPlatformTheme::palette(type);
+    }
+#endif
+
+    return (m_requestedColorScheme != Qt::ColorScheme::Unknown
+            && m_requestedColorScheme != m_storage->colorScheme())
+               ? QPlatformTheme::palette(type) : m_storage->palette(type);
+}
+
+QPixmap QGtk3Theme::standardPixmap(StandardPixmap sp, const QSizeF &size) const
+{
+    Q_ASSERT(m_storage);
+    return m_storage->standardPixmap(sp, size);
+}
+
+const QFont *QGtk3Theme::font(Font type) const
+{
+    Q_ASSERT(m_storage);
+    return m_storage->font(type);
+}
+
+QIcon QGtk3Theme::fileIcon(const QFileInfo &fileInfo,
+                           QPlatformTheme::IconOptions iconOptions) const
+{
+    Q_UNUSED(iconOptions);
+    Q_ASSERT(m_storage);
+    return m_storage->fileIcon(fileInfo);
+}
+
 QT_END_NAMESPACE
