@@ -6,6 +6,7 @@
 
 #include <private/qtguiglobal_p.h>
 #include <private/qgenericunixthemes_p.h>
+#include "qgtk3storage_p.h"
 
 QT_BEGIN_NAMESPACE
 
@@ -17,17 +18,23 @@ public:
     virtual QVariant themeHint(ThemeHint hint) const override;
     virtual QString gtkFontName() const override;
 
-    Appearance appearance() const override;
+    Qt::ColorScheme colorScheme() const override;
+    void requestColorScheme(Qt::ColorScheme scheme) override;
 
     bool usePlatformNativeDialog(DialogType type) const override;
     QPlatformDialogHelper *createPlatformDialogHelper(DialogType type) const override;
 
-    QPlatformMenu* createPlatformMenu() const override;
-    QPlatformMenuItem* createPlatformMenuItem() const override;
+    const QPalette *palette(Palette type = SystemPalette) const override;
+    const QFont *font(Font type = SystemFont) const override;
+    QPixmap standardPixmap(StandardPixmap sp, const QSizeF &size) const override;
+    QIcon fileIcon(const QFileInfo &fileInfo,
+                           QPlatformTheme::IconOptions iconOptions = { }) const override;
 
     static const char *name;
 private:
+    Qt::ColorScheme m_requestedColorScheme = Qt::ColorScheme::Unknown;
     static bool useNativeFileDialog();
+    std::unique_ptr<QGtk3Storage> m_storage;
 };
 
 QT_END_NAMESPACE
