@@ -1,11 +1,8 @@
 // Copyright (C) 2018 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:critical reason:cryptography
 
-#ifndef NOMINMAX
-#define NOMINMAX
-#endif // NOMINMAX
-
-#include <QtNetwork/private/qnativesocketengine_p.h>
+#include <QtNetwork/private/qnativesocketengine_p_p.h>
 
 #include "qsslsocket_openssl_symbols_p.h"
 #include "qdtls_openssl_p.h"
@@ -182,7 +179,7 @@ extern "C" int q_generate_cookie_callback(SSL *ssl, unsigned char *dst,
 
     QMessageAuthenticationCode hmac(dtls->hashAlgorithm, dtls->secret);
     hmac.addData(peerData);
-    const QByteArray cookie = hmac.result();
+    const QByteArrayView cookie = hmac.resultView();
     Q_ASSERT(cookie.size() >= 0);
     // DTLS1_COOKIE_LENGTH is erroneously 256 bytes long, must be 255 - RFC 6347, 4.2.1.
     *cookieLength = qMin(DTLS1_COOKIE_LENGTH - 1, cookie.size());
@@ -205,7 +202,7 @@ extern "C" int q_verify_cookie_callback(SSL *ssl, const unsigned char *cookie,
         return 0;
 
     return newCookieLength == cookieLength
-           && !std::memcmp(cookie, newCookie, cookieLength);
+           && !q_CRYPTO_memcmp(cookie, newCookie, size_t(cookieLength));
 }
 
 extern "C" int q_X509DtlsCallback(int ok, X509_STORE_CTX *ctx)
@@ -357,7 +354,7 @@ extern "C" long q_dgram_ctrl(BIO *bio, int cmd, long num, void *ptr)
     //    command.
 
     if (!bio) {
-        qDebug(lcTlsBackend, "invalid 'bio' parameter (nullptr)");
+        qCDebug(lcTlsBackend, "invalid 'bio' parameter (nullptr)");
         return -1;
     }
 
@@ -795,8 +792,8 @@ QByteArray QDtlsClientVerifierOpenSSL::verifiedHello() const
 
 void QDtlsPrivateOpenSSL::TimeoutHandler::start(int hintMs)
 {
-    Q_ASSERT(timerId == -1);
-    timerId = startTimer(hintMs > 0 ? hintMs : timeoutMs, Qt::PreciseTimer);
+    Q_ASSERT(!timer.isActive());
+    timer.start(hintMs > 0 ? hintMs : timeoutMs, Qt::PreciseTimer, this);
 }
 
 void QDtlsPrivateOpenSSL::TimeoutHandler::doubleTimeout()
@@ -809,19 +806,15 @@ void QDtlsPrivateOpenSSL::TimeoutHandler::doubleTimeout()
 
 void QDtlsPrivateOpenSSL::TimeoutHandler::stop()
 {
-    if (timerId != -1) {
-        killTimer(timerId);
-        timerId = -1;
-    }
+    timer.stop();
 }
 
 void QDtlsPrivateOpenSSL::TimeoutHandler::timerEvent(QTimerEvent *event)
 {
     Q_UNUSED(event);
-    Q_ASSERT(timerId != -1);
+    Q_ASSERT(timer.isActive());
 
-    killTimer(timerId);
-    timerId = -1;
+    timer.stop();
 
     Q_ASSERT(dtlsConnection);
     dtlsConnection->reportTimeout();
