@@ -1,5 +1,6 @@
 // Copyright (C) 2021 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:critical reason:cryptography
 
 #include "qsslsocket_openssl_symbols_p.h"
 #include "qx509_openssl_p.h"
@@ -92,7 +93,7 @@ QSslCertificate findCertificateToFetch(const QList<QSslError> &tlsErrors, bool c
     if (checkAIA) {
         const auto extensions = certToFetch.extensions();
         for (const auto &ext : extensions) {
-            if (ext.oid() == QStringLiteral("1.3.6.1.5.5.7.1.1")) // See RFC 4325
+            if (ext.oid() == u"1.3.6.1.5.5.7.1.1") // See RFC 4325
                 return certToFetch;
         }
         //The only reason we check this extensions is because an application set trusted
@@ -490,7 +491,7 @@ void TlsCryptographOpenSSL::init(QSslSocket *qObj, QSslSocketPrivate *dObj)
     handshakeInterrupted = false;
 
     fetchAuthorityInformation = false;
-    caToFetch = QSslCertificate{};
+    caToFetch.reset();
 }
 
 void TlsCryptographOpenSSL::checkSettingSslContext(std::shared_ptr<QSslContext> tlsContext)
@@ -749,7 +750,7 @@ void TlsCryptographOpenSSL::enableHandshakeContinuation()
 void TlsCryptographOpenSSL::cancelCAFetch()
 {
     fetchAuthorityInformation = false;
-    caToFetch = QSslCertificate{};
+    caToFetch.reset();
 }
 
 void TlsCryptographOpenSSL::continueHandshake()
@@ -1803,7 +1804,7 @@ void TlsCryptographOpenSSL::caRootLoaded(QSslCertificate cert, QSslCertificate t
     Q_ASSERT(q);
 
     //Done, fetched already:
-    caToFetch = QSslCertificate{};
+    caToFetch.reset();
 
     if (fetchAuthorityInformation) {
         if (!q->sslConfiguration().caCertificates().contains(trustedRoot))
