@@ -1,5 +1,6 @@
 // Copyright (C) 2021 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qsslsocket_openssl_symbols_p.h"
 #include "qtlsbackend_openssl_p.h"
@@ -17,7 +18,7 @@
 #include <QtNetwork/qssl.h>
 
 #include <QtCore/qdir.h>
-#include <QtCore/qdiriterator.h>
+#include <QtCore/qdirlisting.h>
 #include <QtCore/qlist.h>
 #include <QtCore/qmutex.h>
 #include <QtCore/qscopeguard.h>
@@ -31,9 +32,13 @@ QT_BEGIN_NAMESPACE
 
 using namespace Qt::StringLiterals;
 
-Q_LOGGING_CATEGORY(lcTlsBackend, "qt.tlsbackend.ossl");
+#if defined(Q_OS_WIN) || defined(Q_OS_MACOS)
+constexpr auto DefaultWarningLevel = QtCriticalMsg;
+#else
+constexpr auto DefaultWarningLevel = QtDebugMsg;
+#endif
 
-Q_GLOBAL_STATIC(QRecursiveMutex, qt_opensslInitMutex)
+Q_LOGGING_CATEGORY(lcTlsBackend, "qt.tlsbackend.ossl", DefaultWarningLevel);
 
 static void q_loadCiphersForConnection(SSL *connection, QList<QSslCipher> &ciphers,
                                        QList<QSslCipher> &defaultCiphers)
@@ -59,8 +64,6 @@ static void q_loadCiphersForConnection(SSL *connection, QList<QSslCipher> &ciphe
     }
 }
 
-bool QTlsBackendOpenSSL::s_libraryLoaded = false;
-bool QTlsBackendOpenSSL::s_loadedCiphersAndCerts = false;
 int QTlsBackendOpenSSL::s_indexForSSLExtraData = -1;
 
 QString QTlsBackendOpenSSL::getErrorsFromOpenSsl()
@@ -92,12 +95,10 @@ void QTlsBackendOpenSSL::clearErrorQueue()
 
 bool QTlsBackendOpenSSL::ensureLibraryLoaded()
 {
-    if (!q_resolveOpenSslSymbols())
-        return false;
+    static bool libraryLoaded = []() {
+        if (!q_resolveOpenSslSymbols())
+            return false;
 
-    const QMutexLocker locker(qt_opensslInitMutex());
-
-    if (!s_libraryLoaded) {
         // Initialize OpenSSL.
         if (q_OPENSSL_init_ssl(0, nullptr) != 1)
             return false;
@@ -119,10 +120,10 @@ bool QTlsBackendOpenSSL::ensureLibraryLoaded()
             return false;
         }
 
-        s_libraryLoaded = true;
-    }
+        return true;
+    }();
 
-    return true;
+    return libraryLoaded;
 }
 
 QString QTlsBackendOpenSSL::backendName() const
@@ -175,11 +176,24 @@ void QTlsBackendOpenSSL::ensureInitialized() const
 
 void QTlsBackendOpenSSL::ensureCiphersAndCertsLoaded() const
 {
-    const QMutexLocker locker(qt_opensslInitMutex());
+    Q_CONSTINIT static bool initializationStarted = false;
+    Q_CONSTINIT static QAtomicInt initialized = Q_BASIC_ATOMIC_INITIALIZER(0);
+    Q_CONSTINIT static QRecursiveMutex initMutex;
 
-    if (s_loadedCiphersAndCerts)
+    if (initialized.loadAcquire())
         return;
-    s_loadedCiphersAndCerts = true;
+
+    const QMutexLocker locker(&initMutex);
+
+    if (initializationStarted || initialized.loadAcquire())
+        return;
+
+    // Indicate that the initialization has already started in the current
+    // thread in case of recursive calls. The atomic variable cannot be used
+    // for this because it is checked without holding the init mutex.
+    initializationStarted = true;
+
+    auto guard = qScopeGuard([] { initialized.storeRelease(1); });
 
     resetDefaultCiphers();
     resetDefaultEllipticCurves();
@@ -191,11 +205,12 @@ void QTlsBackendOpenSSL::ensureCiphersAndCertsLoaded() const
 #elif defined(Q_OS_UNIX) && !defined(Q_OS_DARWIN)
     // check whether we can enable on-demand root-cert loading (i.e. check whether the sym links are there)
     const QList<QByteArray> dirs = QSslSocketPrivate::unixRootCertDirectories();
-    QStringList symLinkFilter;
-    symLinkFilter << "[0-9a-f][0-9a-f][0-9a-f][0-9a-f][0-9a-f][0-9a-f][0-9a-f][0-9a-f].[0-9]"_L1;
+    const QStringList symLinkFilter{
+        u"[0-9a-f][0-9a-f][0-9a-f][0-9a-f][0-9a-f][0-9a-f][0-9a-f][0-9a-f].[0-9]"_s};
     for (const auto &dir : dirs) {
-        QDirIterator iterator(QLatin1StringView(dir), symLinkFilter, QDir::Files);
-        if (iterator.hasNext()) {
+        QDirListing dirList(QString::fromLatin1(dir), symLinkFilter,
+                            QDirListing::IteratorFlag::FilesOnly);
+        if (dirList.cbegin() != dirList.cend()) { // Not empty
             QSslSocketPrivate::setRootCertOnDemandLoadingSupported(true);
             break;
         }
@@ -350,7 +365,9 @@ QList<QSslCertificate> systemCaCertificates()
     QList<QSslCertificate> systemCerts;
 #if defined(Q_OS_WIN)
     HCERTSTORE hSystemStore;
-    hSystemStore = CertOpenSystemStoreW(0, L"ROOT");
+    hSystemStore =
+            CertOpenStore(CERT_STORE_PROV_SYSTEM, 0, 0,
+                          CERT_STORE_READONLY_FLAG | CERT_SYSTEM_STORE_CURRENT_USER, L"ROOT");
     if (hSystemStore) {
         PCCERT_CONTEXT pc = nullptr;
         while (1) {
@@ -375,14 +392,22 @@ QList<QSslCertificate> systemCaCertificates()
             QStringLiteral("/etc/pki/tls/certs/ca-bundle.crt"), // Fedora, Mandriva
             QStringLiteral("/usr/local/share/certs/ca-root-nss.crt") // FreeBSD's ca_root_nss
         };
-        QDir currentDir;
-        currentDir.setNameFilters(QStringList{QStringLiteral("*.pem"), QStringLiteral("*.crt")});
-        for (const auto &directory : directories) {
-            currentDir.setPath(QLatin1StringView(directory));
-            QDirIterator it(currentDir);
-            while (it.hasNext()) {
+
+        static const size_t extLen = strlen(".pem"); // or strlen(".crt")
+        auto hasMatchingExtension = [](const QString &fileName) {
+            if (size_t(fileName.size()) < extLen + 1)
+                return false;
+            auto ext = QStringView{fileName}.last(extLen);
+            return ext == ".pem"_L1 || ext == ".crt"_L1;
+        };
+
+        using F = QDirListing::IteratorFlag;
+        constexpr auto flags = F::FilesOnly | F::ResolveSymlinks; // Files and symlinks to files
+        for (const QByteArray &directory : directories) {
+            for (const auto &dirEntry : QDirListing(QFile::decodeName(directory), flags)) {
                 // use canonical path here to not load the same certificate twice if symlinked
-                certFiles.insert(it.nextFileInfo().canonicalFilePath());
+                if (hasMatchingExtension(dirEntry.fileName()))
+                    certFiles.insert(dirEntry.canonicalFilePath());
             }
         }
         for (const QString& file : std::as_const(certFiles))
