@@ -1,5 +1,6 @@
 // Copyright (C) 2014 Jeremy Lainé <jeremy.laine@m4x.org>
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:critical reason:data-parser
 
 
 #include "qasn1element_p.h"
@@ -343,9 +344,9 @@ QString QAsn1Element::toString() const
     if (mType == PrintableStringType || mType == TeletexStringType
         || mType == Rfc822NameType || mType == DnsNameType
         || mType == UniformResourceIdentifierType)
-        return QString::fromLatin1(mValue, mValue.size());
+        return QString::fromLatin1(mValue);
     if (mType == Utf8StringType)
-        return QString::fromUtf8(mValue, mValue.size());
+        return QString::fromUtf8(mValue);
 
     return QString();
 }
