@@ -83,7 +83,7 @@ Print dialog class declarations
                              Layout in qprintpropertieswidget.ui
 */
 
-static void initResources()
+static void _q_pdu_initResources()
 {
     Q_INIT_RESOURCE(qprintdialog);
 }
@@ -322,7 +322,7 @@ void QPrintPropertiesDialog::reject()
 
 void QPrintPropertiesDialog::accept()
 {
-#if QT_CONFIG(cups)
+#if QT_CONFIG(cups) && QT_CONFIG(messagebox)
     if (widget.pageSetup->hasPpdConflict()) {
         widget.tabs->setCurrentWidget(widget.tabPage);
         const QMessageBox::StandardButton answer = QMessageBox::warning(this, tr("Page Setup Conflicts"),
@@ -584,7 +584,7 @@ QPrintDialogPrivate::QPrintDialogPrivate()
     : top(nullptr), bottom(nullptr), buttons(nullptr), collapseButton(nullptr),
       explicitDuplexMode(QPrint::DuplexAuto)
 {
-    initResources();
+    _q_pdu_initResources();
 }
 
 QPrintDialogPrivate::~QPrintDialogPrivate()
@@ -785,10 +785,8 @@ void QPrintDialogPrivate::setupPrinter()
 #if QT_CONFIG(cups)
     if (options.pagesRadioButton->isChecked()) {
         const QPageRanges ranges = QPageRanges::fromString(options.pagesLineEdit->text());
-        if (!ranges.isEmpty()) {
-            p->setPrintRange(QPrinter::PageRange);
-            p->setPageRanges(ranges);
-        }
+        p->setPrintRange(QPrinter::AllPages);
+        p->setPageRanges(QPageRanges());
 
         // server-side page filtering
         QCUPSSupport::setPageRange(p, ranges.toString());
@@ -990,14 +988,17 @@ int QPrintDialog::exec()
 void QPrintDialog::accept()
 {
     Q_D(QPrintDialog);
-#if QT_CONFIG(cups)
-    if (d->options.pagesRadioButton->isChecked() && printer()->pageRanges().isEmpty()) {
-        QMessageBox::critical(this, tr("Invalid Pages Definition"),
-                              tr("%1 does not follow the correct syntax. Please use ',' to separate "
-                              "ranges and pages, '-' to define ranges and make sure ranges do "
-                              "not intersect with each other.").arg(d->options.pagesLineEdit->text()),
-                              QMessageBox::Ok, QMessageBox::Ok);
-        return;
+#if QT_CONFIG(cups) && QT_CONFIG(messagebox)
+    if (d->options.pagesRadioButton->isChecked()) {
+        const QString rangesText = d->options.pagesLineEdit->text();
+        if (rangesText.isEmpty() || QPageRanges::fromString(rangesText).isEmpty()) {
+            QMessageBox::critical(this, tr("Invalid Pages Definition"),
+                                  tr("%1 does not follow the correct syntax. Please use ',' to separate "
+                                     "ranges and pages, '-' to define ranges and make sure ranges do "
+                                     "not intersect with each other.").arg(rangesText),
+                                  QMessageBox::Ok, QMessageBox::Ok);
+            return;
+        }
     }
     if (d->top->d->m_duplexPpdOption && d->top->d->m_duplexPpdOption->conflicted) {
         const QMessageBox::StandardButton answer = QMessageBox::warning(this, tr("Duplex Settings Conflicts"),
@@ -1328,9 +1329,9 @@ QUnixPrintWidget::QUnixPrintWidget(QPrinter *printer, QWidget *parent)
             cur = home;
         else if (!cur.endsWith(u'/'))
             cur += u'/';
-        if (QGuiApplication::platformName() == QStringLiteral("xcb")) {
+        if (QGuiApplication::platformName() == "xcb"_L1) {
             if (printer->docName().isEmpty()) {
-                cur += QStringLiteral("print.pdf");
+                cur += "print.pdf"_L1;
             } else {
 #if QT_CONFIG(regularexpression)
                 const QRegularExpression re(QStringLiteral("(.*)\\.\\S+"));
@@ -1340,7 +1341,7 @@ QUnixPrintWidget::QUnixPrintWidget(QPrinter *printer, QWidget *parent)
                 else
 #endif
                     cur += printer->docName();
-                cur += QStringLiteral(".pdf");
+                cur += ".pdf"_L1;
             }
         } // xcb
 
