@@ -7,8 +7,8 @@
 #include "qsqlerror.h"
 #include "qsqlfield.h"
 #include "qsqlindex.h"
-#include "private/qobject_p.h"
 #include "private/qsqldriver_p.h"
+#include "private/qtools_p.h"
 
 #include <limits.h>
 
@@ -138,7 +138,9 @@ bool QSqlDriver::isOpenError() const
     \enum QSqlDriver::DriverFeature
 
     This enum contains a list of features a driver might support. Use
-    hasFeature() to query whether a feature is supported or not.
+    hasFeature() to query whether a feature is supported or not. Some features
+    depend on the database server so they can only properly determined after
+    the database connection is successfully opened with QSqlDatabase::open().
 
     \value Transactions  Whether the driver supports SQL transactions.
     \value QuerySize  Whether the database is capable of reporting the size
@@ -215,6 +217,7 @@ bool QSqlDriver::isOpenError() const
     \value SQLite
     \value Interbase
     \value DB2
+    \value [since 6.6] MimerSQL
 */
 
 /*!
@@ -403,7 +406,6 @@ bool QSqlDriver::isIdentifierEscaped(const QString &identifier, IdentifierType t
     Reimplement this function if you want to provide your own implementation in your
     QSqlDriver subclass,
 
-    \since 4.5
     \sa isIdentifierEscaped()
  */
 QString QSqlDriver::stripDelimiters(const QString &identifier, IdentifierType type) const
@@ -446,12 +448,11 @@ QString QSqlDriver::sqlStatement(StatementType type, const QString &tableName,
 {
     const auto tableNameString = tableName.isEmpty() ? QString()
                                     : prepareIdentifier(tableName, QSqlDriver::TableName, this);
-    int i;
     QString s;
     s.reserve(128);
     switch (type) {
     case SelectStatement:
-        for (i = 0; i < rec.count(); ++i) {
+        for (qsizetype i = 0; i < rec.count(); ++i) {
             if (rec.isGenerated(i))
                 s.append(prepareIdentifier(rec.fieldName(i), QSqlDriver::FieldName, this)).append(", "_L1);
         }
@@ -464,7 +465,7 @@ QString QSqlDriver::sqlStatement(StatementType type, const QString &tableName,
     {
         const QString tableNamePrefix = tableNameString.isEmpty()
                                             ? QString() : tableNameString + u'.';
-        for (int i = 0; i < rec.count(); ++i) {
+        for (qsizetype i = 0; i < rec.count(); ++i) {
             if (!rec.isGenerated(i))
                 continue;
             s.append(s.isEmpty() ? "WHERE "_L1 : " AND "_L1);
@@ -481,7 +482,7 @@ QString QSqlDriver::sqlStatement(StatementType type, const QString &tableName,
     }
     case UpdateStatement:
         s = s + "UPDATE "_L1 + tableNameString + " SET "_L1;
-        for (i = 0; i < rec.count(); ++i) {
+        for (qsizetype i = 0; i < rec.count(); ++i) {
             if (!rec.isGenerated(i))
                 continue;
             s.append(prepareIdentifier(rec.fieldName(i), QSqlDriver::FieldName, this)).append(u'=');
@@ -502,7 +503,7 @@ QString QSqlDriver::sqlStatement(StatementType type, const QString &tableName,
     case InsertStatement: {
         s = s + "INSERT INTO "_L1 + tableNameString + " ("_L1;
         QString vals;
-        for (i = 0; i < rec.count(); ++i) {
+        for (qsizetype i = 0; i < rec.count(); ++i) {
             if (!rec.isGenerated(i))
                 continue;
             s.append(prepareIdentifier(rec.fieldName(i), QSqlDriver::FieldName, this)).append(", "_L1);
@@ -613,15 +614,15 @@ QString QSqlDriver::formatValue(const QSqlField &field, bool trimStrings) const
             break;
         case QMetaType::QByteArray : {
             if (hasFeature(BLOB)) {
-                QByteArray ba = field.value().toByteArray();
-                QString res;
-                static const char hexchars[] = "0123456789abcdef";
-                for (int i = 0; i < ba.size(); ++i) {
-                    uchar s = (uchar) ba[i];
-                    res += QLatin1Char(hexchars[s >> 4]);
-                    res += QLatin1Char(hexchars[s & 0x0f]);
+                const QByteArray ba = field.value().toByteArray();
+                r.reserve((ba.size() + 1) * 2);
+                r += u'\'';
+                for (const char c : ba) {
+                    const uchar s = uchar(c);
+                    r += QLatin1Char(QtMiscUtils::toHexLower(s >> 4));
+                    r += QLatin1Char(QtMiscUtils::toHexLower(s & 0x0f));
                 }
-                r = u'\'' + res +  u'\'';
+                r += u'\'';
                 break;
             }
         }
@@ -680,7 +681,6 @@ QVariant QSqlDriver::handle() const
     Reimplement this function if you want to provide event notification support in your
     own QSqlDriver subclass,
 
-    \since 4.4
     \sa unsubscribeFromNotification(), subscribedToNotifications(), QSqlDriver::hasFeature()
 */
 bool QSqlDriver::subscribeToNotification(const QString &name)
@@ -704,7 +704,6 @@ bool QSqlDriver::subscribeToNotification(const QString &name)
     Reimplement this function if you want to provide event notification support in your
     own QSqlDriver subclass,
 
-    \since 4.4
     \sa subscribeToNotification(), subscribedToNotifications()
 */
 bool QSqlDriver::unsubscribeFromNotification(const QString &name)
@@ -719,7 +718,6 @@ bool QSqlDriver::unsubscribeFromNotification(const QString &name)
     Reimplement this function if you want to provide event notification support in your
     own QSqlDriver subclass,
 
-    \since 4.4
     \sa subscribeToNotification(), unsubscribeFromNotification()
 */
 QStringList QSqlDriver::subscribedToNotifications() const
@@ -728,16 +726,7 @@ QStringList QSqlDriver::subscribedToNotifications() const
 }
 
 /*!
-    \since 4.6
-
-    Sets the default numerical precision policy used by queries created
-    by this driver to \a precisionPolicy.
-
-    Note: Setting the default precision policy to \a precisionPolicy
-    doesn't affect any currently active queries.
-
-    \sa QSql::NumericalPrecisionPolicy, numericalPrecisionPolicy(),
-    QSqlQuery::setNumericalPrecisionPolicy(), QSqlQuery::numericalPrecisionPolicy()
+    Sets \l numericalPrecisionPolicy to \a precisionPolicy.
 */
 void QSqlDriver::setNumericalPrecisionPolicy(QSql::NumericalPrecisionPolicy precisionPolicy)
 {
@@ -746,12 +735,17 @@ void QSqlDriver::setNumericalPrecisionPolicy(QSql::NumericalPrecisionPolicy prec
 }
 
 /*!
-    \since 4.6
+    \property QSqlDriver::numericalPrecisionPolicy
+    \since 6.8
 
-    Returns the current default precision policy for the database connection.
+    This property holds the precision policy for the database connection.
+    \note Setting the precision policy doesn't affect any currently active queries.
 
-    \sa QSql::NumericalPrecisionPolicy, setNumericalPrecisionPolicy(),
-    QSqlQuery::numericalPrecisionPolicy(), QSqlQuery::setNumericalPrecisionPolicy()
+    \sa QSql::NumericalPrecisionPolicy, QSqlQuery::numericalPrecisionPolicy,
+    QSqlDatabase::numericalPrecisionPolicy
+*/
+/*!
+    Returns the \l numericalPrecisionPolicy.
 */
 QSql::NumericalPrecisionPolicy QSqlDriver::numericalPrecisionPolicy() const
 {
@@ -807,6 +801,18 @@ int QSqlDriver::maximumIdentifierLength(QSqlDriver::IdentifierType type) const
     return INT_MAX;
 }
 
+/*!
+    \since 6.9
+
+    Returns the database connection name the driver was created by with
+    QSqlDatabase::addDatabase()
+*/
+QString QSqlDriver::connectionName() const
+{
+    Q_D(const QSqlDriver);
+    return d->connectionName;
+}
+
 QT_END_NAMESPACE
 
 #include "moc_qsqldriver.cpp"
