@@ -9,15 +9,11 @@
 #include <QtTest/private/qbenchmark_p.h>
 #include <QtTest/private/qtestlog_p.h>
 
-#ifdef min // windows.h without NOMINMAX is included by the benchmark headers.
-#  undef min
-#endif
-#ifdef max
-#  undef max
-#endif
-
+#include <QtCore/qelapsedtimer.h>
 #include <QtCore/qlibraryinfo.h>
 
+#include <cstdio>
+
 #include <string.h>
 
 QT_BEGIN_NAMESPACE
@@ -28,7 +24,7 @@ QT_BEGIN_NAMESPACE
     QJUnitTestLogger implements logging in a JUnit-compatible XML format.
 
     The \l{JUnit XML} format was originally developed for Java testing.
-    It is supported by \l{Squish Test Center}.
+    It is supported by \l{Test Center}.
 */
 // QTBUG-95424 links to further useful documentation.
 
@@ -45,7 +41,7 @@ QJUnitTestLogger::~QJUnitTestLogger()
 
 // We track test timing per test case, so we
 // need to maintain our own elapsed timer.
-static QElapsedTimer elapsedTestcaseTime;
+Q_CONSTINIT static QElapsedTimer elapsedTestcaseTime;
 static qreal elapsedTestCaseSeconds()
 {
     return elapsedTestcaseTime.nsecsElapsed() / 1e9;
@@ -100,16 +96,16 @@ void QJUnitTestLogger::stopLogging()
 {
     char buf[10];
 
-    qsnprintf(buf, sizeof(buf), "%i", testCounter);
+    std::snprintf(buf, sizeof(buf), "%i", testCounter);
     currentTestSuite->addAttribute(QTest::AI_Tests, buf);
 
-    qsnprintf(buf, sizeof(buf), "%i", failureCounter);
+    std::snprintf(buf, sizeof(buf), "%i", failureCounter);
     currentTestSuite->addAttribute(QTest::AI_Failures, buf);
 
-    qsnprintf(buf, sizeof(buf), "%i", errorCounter);
+    std::snprintf(buf, sizeof(buf), "%i", errorCounter);
     currentTestSuite->addAttribute(QTest::AI_Errors, buf);
 
-    qsnprintf(buf, sizeof(buf), "%i", QTestLog::skipCount());
+    std::snprintf(buf, sizeof(buf), "%i", QTestLog::skipCount());
     currentTestSuite->addAttribute(QTest::AI_Skipped, buf);
 
     currentTestSuite->addAttribute(QTest::AI_Time,
@@ -134,20 +130,23 @@ void QJUnitTestLogger::enterTestFunction(const char *function)
 
 void QJUnitTestLogger::enterTestCase(const char *name)
 {
-    currentTestCase = new QTestElement(QTest::LET_TestCase);
-    currentTestCase->addAttribute(QTest::AI_Name, name);
-    currentTestCase->addAttribute(QTest::AI_Classname, QTestResult::currentTestObjectName());
-    listOfTestcases.push_back(currentTestCase);
+    {
+        QMutexLocker locker(&mutex);
+        currentTestCase = new QTestElement(QTest::LET_TestCase);
+        currentTestCase->addAttribute(QTest::AI_Name, name);
+        currentTestCase->addAttribute(QTest::AI_Classname, QTestResult::currentTestObjectName());
+        listOfTestcases.push_back(currentTestCase);
 
-    Q_ASSERT(!systemOutputElement && !systemErrorElement);
-    systemOutputElement = new QTestElement(QTest::LET_SystemOutput);
-    systemErrorElement = new QTestElement(QTest::LET_SystemError);
+        Q_ASSERT(!systemOutputElement && !systemErrorElement);
+        systemOutputElement = new QTestElement(QTest::LET_SystemOutput);
+        systemErrorElement = new QTestElement(QTest::LET_SystemError);
+    }
 
     // The element will be deleted when the suite is deleted
 
     ++testCounter;
 
-    elapsedTestcaseTime.restart();
+    elapsedTestcaseTime.start();
 }
 
 void QJUnitTestLogger::enterTestData(QTestData *)
@@ -163,7 +162,7 @@ void QJUnitTestLogger::enterTestData(QTestData *)
             currentTestCase->attribute(QTest::AI_Name));
         name->setPair(QTest::AI_Name, testIdentifier.data());
         lastTestFunction = QTestResult::currentTestFunction();
-        elapsedTestcaseTime.restart();
+        elapsedTestcaseTime.start();
     } else {
         // Create new test cases for remaining test data
         leaveTestCase();
@@ -178,6 +177,7 @@ void QJUnitTestLogger::leaveTestFunction()
 
 void QJUnitTestLogger::leaveTestCase()
 {
+    QMutexLocker locker(&mutex);
     currentTestCase->addAttribute(QTest::AI_Time,
         toSecondsFormat(elapsedTestCaseSeconds() * 1000).constData());
 
@@ -261,6 +261,7 @@ void QJUnitTestLogger::addMessage(MessageTypes type, const QString &message, con
     Q_UNUSED(file);
     Q_UNUSED(line);
 
+    QMutexLocker locker(&mutex);
     if (type == QFatal) {
         addFailure(QTest::LET_Error, "qfatal", message);
         return;
