@@ -12,6 +12,10 @@
 #include <QtTest/qtestassert.h>
 #include <QtTest/qtesteventloop.h>
 
+#include <climits>
+#include <cwchar>
+#include <QtCore/q26numeric.h>
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -39,7 +43,7 @@ namespace QTest
         }();
 
         if (failed && fatalFailure)
-            qTerminate();
+            std::terminate();
         Internal::failed = failed;
     }
 
@@ -149,13 +153,6 @@ void QTestResult::finishedCurrentTestData()
         addFailure("QEXPECT_FAIL was called without any subsequent verification statements");
 
     clearExpectFail();
-
-    if (!QTest::hasFailed() && QTestLog::unhandledIgnoreMessages()) {
-        QTestLog::printUnhandledIgnoreMessages();
-        addFailure("Not all expected messages were received");
-    }
-    QTestLog::clearIgnoreMessages();
-    QTestLog::clearFailOnWarnings();
 }
 
 /*!
@@ -175,6 +172,11 @@ void QTestResult::finishedCurrentTestData()
 */
 void QTestResult::finishedCurrentTestDataCleanup()
 {
+    if (!QTest::hasFailed() && QTestLog::unhandledIgnoreMessages()) {
+        QTestLog::printUnhandledIgnoreMessages();
+        addFailure("Not all expected messages were received");
+    }
+
     // If the current test hasn't failed or been skipped, then it passes.
     if (!QTest::hasFailed() && !QTest::skipCurrentTest) {
         if (QTest::blacklistCurrentTest)
@@ -289,23 +291,29 @@ void QTestResult::fail(const char *msg, const char *file, int line)
     checkStatement(false, msg, file, line);
 }
 
+// QPalette's << operator produces 1363 characters. A comparison failure
+// involving two palettes can therefore require 2726 characters, not including
+// the other output produced by QTest. Users might also have their own types
+// with large amounts of output, so use a sufficiently high value here.
+static constexpr size_t maxMsgLen = 4096;
+
 bool QTestResult::verify(bool statement, const char *statementStr,
                          const char *description, const char *file, int line)
 {
     QTEST_ASSERT(statementStr);
 
-    char msg[1024];
+    Q_DECL_UNINITIALIZED char msg[maxMsgLen];
     msg[0] = '\0';
 
     if (QTestLog::verboseLevel() >= 2) {
-        qsnprintf(msg, 1024, "QVERIFY(%s)", statementStr);
+        std::snprintf(msg, maxMsgLen, "QVERIFY(%s)", statementStr);
         QTestLog::info(msg, file, line);
     }
 
     if (statement == !!QTest::expectFailMode) {
-        qsnprintf(msg, 1024,
-                  statement ? "'%s' returned TRUE unexpectedly. (%s)" : "'%s' returned FALSE. (%s)",
-                  statementStr, description ? description : "");
+        std::snprintf(msg, maxMsgLen,
+                      statement ? "'%s' returned TRUE unexpectedly. (%s)" : "'%s' returned FALSE. (%s)",
+                      statementStr, description ? description : "");
     }
 
     return checkStatement(statement, msg, file, line);
@@ -313,42 +321,94 @@ bool QTestResult::verify(bool statement, const char *statementStr,
 
 static const char *leftArgNameForOp(QTest::ComparisonOperation op)
 {
-    return op == QTest::ComparisonOperation::CustomCompare ? "Actual   " : "Left   ";
+    switch (op) {
+    case QTest::ComparisonOperation::CustomCompare:
+        return "Actual   ";
+    case QTest::ComparisonOperation::ThreeWayCompare:
+        return "Left     ";
+    case QTest::ComparisonOperation::Equal:
+    case QTest::ComparisonOperation::NotEqual:
+    case QTest::ComparisonOperation::LessThan:
+    case QTest::ComparisonOperation::LessThanOrEqual:
+    case QTest::ComparisonOperation::GreaterThan:
+    case QTest::ComparisonOperation::GreaterThanOrEqual:
+        return "Computed ";
+    }
+    Q_UNREACHABLE_RETURN("");
 }
 
 static const char *rightArgNameForOp(QTest::ComparisonOperation op)
 {
-    return op == QTest::ComparisonOperation::CustomCompare ? "Expected " : "Right  ";
+    switch (op) {
+    case QTest::ComparisonOperation::CustomCompare:
+        return "Expected ";
+    case QTest::ComparisonOperation::ThreeWayCompare:
+        return "Right    ";
+    case QTest::ComparisonOperation::Equal:
+    case QTest::ComparisonOperation::NotEqual:
+    case QTest::ComparisonOperation::LessThan:
+    case QTest::ComparisonOperation::LessThanOrEqual:
+    case QTest::ComparisonOperation::GreaterThan:
+    case QTest::ComparisonOperation::GreaterThanOrEqual:
+        return "Baseline ";
+    }
+    Q_UNREACHABLE_RETURN("");
+}
+
+static int approx_wide_len(const char *s)
+{
+    std::mbstate_t state = {};
+    // QNX might stop at max when dst == nullptr, so pass INT_MAX,
+    // being the largest value this function will return:
+    auto r = std::mbsrtowcs(nullptr, &s, INT_MAX, &state);
+    if (r == size_t(-1)) // encoding error, fall back to strlen()
+        r = strlen(s); // `s` was not advanced since `dst == nullptr`
+    return q26::saturate_cast<int>(r);
 }
 
 // Overload to format failures for "const char *" - no need to strdup().
+static Q_DECL_COLD_FUNCTION
 void formatFailMessage(char *msg, size_t maxMsgLen,
                        const char *failureMsg,
                        const char *val1, const char *val2,
                        const char *actual, const char *expected,
                        QTest::ComparisonOperation op)
 {
-    size_t len1 = mbstowcs(nullptr, actual, maxMsgLen);    // Last parameter is not ignored on QNX
-    size_t len2 = mbstowcs(nullptr, expected, maxMsgLen);  // (result is never larger than this).
-    const int written = qsnprintf(msg, maxMsgLen, "%s\n", failureMsg);
+    const auto len1 = approx_wide_len(actual);
+    const auto len2 = approx_wide_len(expected);
+    const int written = std::snprintf(msg, maxMsgLen, "%s\n", failureMsg);
     msg += written;
     maxMsgLen -= written;
 
+    const auto protect = [](const char *s) { return s ? s : "<null>"; };
+
     if (val1 || val2) {
-        qsnprintf(msg, maxMsgLen, "   %s(%s)%*s %s\n   %s(%s)%*s %s",
-                    leftArgNameForOp(op), actual, qMax(len1, len2) - len1 + 1, ":",
-                    val1 ? val1 : "<null>",
-                    rightArgNameForOp(op), expected, qMax(len1, len2) - len2 + 1, ":",
-                    val2 ? val2 : "<null>");
+        std::snprintf(msg, maxMsgLen, "   %s(%s)%*s %s\n   %s(%s)%*s %s",
+                      leftArgNameForOp(op), actual, qMax(len1, len2) - len1 + 1, ":",
+                      protect(val1),
+                      rightArgNameForOp(op), expected, qMax(len1, len2) - len2 + 1, ":",
+                      protect(val2));
     } else {
         // only print variable names if neither value can be represented as a string
-        qsnprintf(msg, maxMsgLen, "   %s: %s\n   %s: %s",
-                    leftArgNameForOp(op), actual, rightArgNameForOp(op), expected);
+        std::snprintf(msg, maxMsgLen, "   %s: %s\n   %s: %s",
+                      leftArgNameForOp(op), actual, rightArgNameForOp(op), expected);
     }
 }
 
+const char *
+QTest::Internal::formatPropertyTestHelperFailure(char *msg, size_t maxMsgLen,
+                                                 const char *actual, const char *expected,
+                                                 const char *actualExpr, const char *expectedExpr)
+{
+    formatFailMessage(msg, maxMsgLen, "Comparison failed!",
+                      actual, expected, actualExpr, expectedExpr,
+                      QTest::ComparisonOperation::CustomCompare);
+    return msg;
+}
+
 // Format failures using the toString() template
 template <class Actual, class Expected>
+static Q_DECL_COLD_FUNCTION
 void formatFailMessage(char *msg, size_t maxMsgLen,
                        const char *failureMsg,
                        const Actual &val1, const Expected &val2,
@@ -371,15 +431,14 @@ static bool compareHelper(bool success, const char *failureMsg,
                           const char *file, int line,
                           bool hasValues = true)
 {
-    const size_t maxMsgLen = 1024;
-    char msg[maxMsgLen];
+    Q_DECL_UNINITIALIZED char msg[maxMsgLen];
     msg[0] = '\0';
 
     QTEST_ASSERT(expected);
     QTEST_ASSERT(actual);
 
     if (QTestLog::verboseLevel() >= 2) {
-        qsnprintf(msg, maxMsgLen, "QCOMPARE(%s, %s)", actual, expected);
+        std::snprintf(msg, maxMsgLen, "QCOMPARE(%s, %s)", actual, expected);
         QTestLog::info(msg, file, line);
     }
 
@@ -388,15 +447,15 @@ static bool compareHelper(bool success, const char *failureMsg,
 
     if (success) {
         if (QTest::expectFailMode) {
-            qsnprintf(msg, maxMsgLen,
-                      "QCOMPARE(%s, %s) returned TRUE unexpectedly.", actual, expected);
+            std::snprintf(msg, maxMsgLen,
+                          "QCOMPARE(%s, %s) returned TRUE unexpectedly.", actual, expected);
         }
         return checkStatement(success, msg, file, line);
     }
 
 
     if (!hasValues) {
-        qsnprintf(msg, maxMsgLen, "%s", failureMsg);
+        std::snprintf(msg, maxMsgLen, "%s", failureMsg);
         return checkStatement(success, msg, file, line);
     }
 
@@ -414,7 +473,7 @@ static bool compareHelper(bool success, const char *failureMsg,
                           const char *file, int line)
 {
     const size_t maxMsgLen = 1024;
-    char msg[maxMsgLen];
+    Q_DECL_UNINITIALIZED char msg[maxMsgLen];
     msg[0] = '\0';
 
     QTEST_ASSERT(expected);
@@ -423,14 +482,14 @@ static bool compareHelper(bool success, const char *failureMsg,
     QTEST_ASSERT(success || failureMsg);
 
     if (QTestLog::verboseLevel() >= 2) {
-        qsnprintf(msg, maxMsgLen, "QCOMPARE(%s, %s)", actual, expected);
+        std::snprintf(msg, maxMsgLen, "QCOMPARE(%s, %s)", actual, expected);
         QTestLog::info(msg, file, line);
     }
 
     if (success) {
         if (QTest::expectFailMode) {
-            qsnprintf(msg, maxMsgLen, "QCOMPARE(%s, %s) returned TRUE unexpectedly.",
-                      actual, expected);
+            std::snprintf(msg, maxMsgLen, "QCOMPARE(%s, %s) returned TRUE unexpectedly.",
+                          actual, expected);
         }
         return checkStatement(success, msg, file, line);
     }
@@ -534,7 +593,8 @@ bool QTestResult::compare(bool success, const char *failureMsg,
 void QTestResult::addFailure(const char *message, const char *file, int line)
 {
     clearExpectFail();
-    QTestEventLoop::instance().exitLoop();
+    if (qApp && QThread::currentThread() == qApp->thread())
+        QTestEventLoop::instance().exitLoop();
 
     if (QTest::blacklistCurrentTest)
         QTestLog::addBFail(message, file, line);
@@ -598,9 +658,10 @@ static const char *macroNameForOp(QTest::ComparisonOperation op)
         return "QCOMPARE_GT";
     case ComparisonOperation::GreaterThanOrEqual:
         return "QCOMPARE_GE";
+    case ComparisonOperation::ThreeWayCompare:
+        return "QCOMPARE_3WAY";
     }
-    Q_UNREACHABLE();
-    return "";
+    Q_UNREACHABLE_RETURN("");
 }
 
 static const char *failureMessageForOp(QTest::ComparisonOperation op)
@@ -609,51 +670,52 @@ static const char *failureMessageForOp(QTest::ComparisonOperation op)
     switch (op) {
     case ComparisonOperation::CustomCompare:
         return "Compared values are not the same"; /* not used */
+    case ComparisonOperation::ThreeWayCompare:
+        return "The result of operator<=>() is not what was expected";
     case ComparisonOperation::Equal:
-        return "Left value is expected to be equal to right value, but is not";
+        return "The computed value is expected to be equal to the baseline, but is not";
     case ComparisonOperation::NotEqual:
-        return "Left value is expected to be different from right value, but is not";
+        return "The computed value is expected to be different from the baseline, but is not";
     case ComparisonOperation::LessThan:
-        return "Left value is expected to be less than right value, but is not";
+        return "The computed value is expected to be less than the baseline, but is not";
     case ComparisonOperation::LessThanOrEqual:
-        return "Left value is expected to be less than or equal to right value, but is not";
+        return "The computed value is expected to be less than or equal to the baseline, but is not";
     case ComparisonOperation::GreaterThan:
-        return "Left value is expected to be greater than right value, but is not";
+        return "The computed value is expected to be greater than the baseline, but is not";
     case ComparisonOperation::GreaterThanOrEqual:
-        return "Left value is expected to be greater than or equal to right value, but is not";
+        return "The computed value is expected to be greater than or equal to the baseline, but is not";
     }
-    Q_UNREACHABLE();
-    return "";
+    Q_UNREACHABLE_RETURN("");
 }
 
-bool QTestResult::reportResult(bool success, qxp::function_ref<const char *()> lhs,
-                               qxp::function_ref<const char *()> rhs,
+bool QTestResult::reportResult(bool success, const void *lhs, const void *rhs,
+                               const char *(*lhsFormatter)(const void*),
+                               const char *(*rhsFormatter)(const void*),
                                const char *lhsExpr, const char *rhsExpr,
                                QTest::ComparisonOperation op, const char *file, int line,
                                const char *failureMessage)
 {
-    const size_t maxMsgLen = 1024;
-    char msg[maxMsgLen];
+    Q_DECL_UNINITIALIZED char msg[maxMsgLen];
     msg[0] = '\0';
 
     QTEST_ASSERT(lhsExpr);
     QTEST_ASSERT(rhsExpr);
 
     if (QTestLog::verboseLevel() >= 2) {
-        qsnprintf(msg, maxMsgLen, "%s(%s, %s)", macroNameForOp(op), lhsExpr, rhsExpr);
+        std::snprintf(msg, maxMsgLen, "%s(%s, %s)", macroNameForOp(op), lhsExpr, rhsExpr);
         QTestLog::info(msg, file, line);
     }
 
     if (success) {
         if (QTest::expectFailMode) {
-            qsnprintf(msg, maxMsgLen, "%s(%s, %s) returned TRUE unexpectedly.",
-                      macroNameForOp(op), lhsExpr, rhsExpr);
+            std::snprintf(msg, maxMsgLen, "%s(%s, %s) returned TRUE unexpectedly.",
+                          macroNameForOp(op), lhsExpr, rhsExpr);
         }
         return checkStatement(success, msg, file, line);
     }
 
-    const std::unique_ptr<const char[]> lhsPtr{ lhs() };
-    const std::unique_ptr<const char[]> rhsPtr{ rhs() };
+    const std::unique_ptr<const char[]> lhsPtr{ lhsFormatter(lhs) };
+    const std::unique_ptr<const char[]> rhsPtr{ rhsFormatter(rhs) };
 
     if (!failureMessage)
         failureMessage = failureMessageForOp(op);
@@ -664,4 +726,62 @@ bool QTestResult::reportResult(bool success, qxp::function_ref<const char *()> l
     return checkStatement(success, msg, file, line);
 }
 
+bool QTestResult::report3WayResult(bool success,
+                                   const char *failureMessage,
+                                   const void *lhs, const void *rhs,
+                                   const char *(*lhsFormatter)(const void*),
+                                   const char *(*rhsFormatter)(const void*),
+                                   const char *lhsExpression, const char *rhsExpression,
+                                   const char *(*actualOrderFormatter)(const void *),
+                                   const char *(*expectedOrderFormatter)(const void *),
+                                   const void *actualOrder, const void *expectedOrder,
+                                   const char *expectedExpression,
+                                   const char *file, int line)
+{
+    char msg[maxMsgLen];
+    msg[0] = '\0';
+
+    QTEST_ASSERT(lhsExpression);
+    QTEST_ASSERT(rhsExpression);
+    QTEST_ASSERT(expectedExpression);
+    const char *macroName = macroNameForOp(QTest::ComparisonOperation::ThreeWayCompare);
+    const std::string actualExpression = std::string(lhsExpression) + " <=> " + rhsExpression;
+
+    if (QTestLog::verboseLevel() >= 2) {
+        std::snprintf(msg, maxMsgLen, "%s(%s, %s, %s)",
+                      macroName, lhsExpression, rhsExpression, expectedExpression);
+        QTestLog::info(msg, file, line);
+    }
+
+    if (success) {
+        if (QTest::expectFailMode) {
+            std::snprintf(msg, maxMsgLen, "%s(%s, %s, %s) returned TRUE unexpectedly.",
+                          macroName, lhsExpression, rhsExpression, expectedExpression);
+        }
+        return checkStatement(success, msg, file, line);
+    }
+    const std::unique_ptr<const char[]> lhsStr{lhsFormatter(lhs)};
+    const std::unique_ptr<const char[]> rhsStr{rhsFormatter(rhs)};
+
+    const std::unique_ptr<const char[]> actual{actualOrderFormatter(actualOrder)};
+    const std::unique_ptr<const char[]> expected{expectedOrderFormatter(expectedOrder)};
+
+    if (!failureMessage)
+        failureMessage = failureMessageForOp(QTest::ComparisonOperation::ThreeWayCompare);
+
+    // Left and Right compared parameters of QCOMPARE_3WAY
+    formatFailMessage(msg, maxMsgLen, failureMessage,
+                      lhsStr.get(), rhsStr.get(),
+                      lhsExpression, rhsExpression,
+                      QTest::ComparisonOperation::ThreeWayCompare);
+
+    // Actual and Expected results of comparison
+    formatFailMessage(msg + strlen(msg), maxMsgLen - strlen(msg), "",
+                      actual.get(), expected.get(),
+                      actualExpression.c_str(), expectedExpression,
+                      QTest::ComparisonOperation::CustomCompare);
+
+    return checkStatement(success, msg, file, line);
+}
+
 QT_END_NAMESPACE
