@@ -2,31 +2,23 @@
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "utils.h"
-#include "elfreader.h"
 
 #include <QtCore/QString>
 #include <QtCore/QDebug>
 #include <QtCore/QDir>
 #include <QtCore/QFile>
 #include <QtCore/QFileInfo>
+#include <QtCore/QProcess>
 #include <QtCore/QTemporaryFile>
 #include <QtCore/QScopedPointer>
 #include <QtCore/QScopedArrayPointer>
 #include <QtCore/QStandardPaths>
 #if defined(Q_OS_WIN)
 #  include <QtCore/qt_windows.h>
+#  include <QtCore/private/qsystemerror_p.h>
 #  include <shlwapi.h>
 #  include <delayimp.h>
-#else // Q_OS_WIN
-#  include <sys/wait.h>
-#  include <sys/types.h>
-#  include <sys/stat.h>
-#  include <unistd.h>
-#  include <stdlib.h>
-#  include <string.h>
-#  include <errno.h>
-#  include <fcntl.h>
-#endif  // !Q_OS_WIN
+#endif  // Q_OS_WIN
 
 QT_BEGIN_NAMESPACE
 
@@ -61,7 +53,7 @@ bool createSymbolicLink(const QFileInfo &source, const QString &target, QString
     return true;
 }
 
-bool createDirectory(const QString &directory, QString *errorMessage)
+bool createDirectory(const QString &directory, QString *errorMessage, bool dryRun)
 {
     const QFileInfo fi(directory);
     if (fi.isDir())
@@ -73,11 +65,13 @@ bool createDirectory(const QString &directory, QString *errorMessage)
     }
     if (optVerboseLevel)
         std::wcout << "Creating " << QDir::toNativeSeparators(directory) << "...\n";
-    QDir dir;
-    if (!dir.mkpath(directory)) {
-        *errorMessage = QString::fromLatin1("Could not create directory %1.").
-                        arg(QDir::toNativeSeparators(directory));
-        return false;
+    if (!dryRun) {
+        QDir dir;
+        if (!dir.mkpath(directory)) {
+            *errorMessage = QString::fromLatin1("Could not create directory %1.")
+                                    .arg(QDir::toNativeSeparators(directory));
+            return false;
+        }
     }
     return true;
 }
@@ -92,7 +86,7 @@ QStringList findSharedLibraries(const QDir &directory, Platform platform,
         nameFilter += u'*';
     if (debugMatchMode == MatchDebug && platformHasDebugSuffix(platform))
         nameFilter += u'd';
-    nameFilter += sharedLibrarySuffix(platform);
+    nameFilter += sharedLibrarySuffix();
     QStringList result;
     QString errorMessage;
     const QFileInfoList &dlls = directory.entryInfoList(QStringList(nameFilter), QDir::Files);
@@ -115,24 +109,6 @@ QStringList findSharedLibraries(const QDir &directory, Platform platform,
     return result;
 }
 
-#ifdef Q_OS_WIN
-QString winErrorMessage(unsigned long error)
-{
-    QString rc = QString::fromLatin1("#%1: ").arg(error);
-    char16_t *lpMsgBuf;
-
-    const DWORD len = FormatMessage(
-            FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
-            NULL, error, 0, reinterpret_cast<LPTSTR>(&lpMsgBuf), 0, NULL);
-    if (len) {
-        rc = QString::fromUtf16(lpMsgBuf, int(len));
-        LocalFree(lpMsgBuf);
-    } else {
-        rc += QString::fromLatin1("<unknown error>");
-    }
-    return rc;
-}
-
 // Case-Normalize file name via GetShortPathNameW()/GetLongPathNameW()
 QString normalizeFileName(const QString &name)
 {
@@ -146,47 +122,6 @@ QString normalizeFileName(const QString &name)
     return QDir::fromNativeSeparators(QString::fromWCharArray(result));
 }
 
-// Find a tool binary in the Windows SDK 8
-QString findSdkTool(const QString &tool)
-{
-    QStringList paths = QString::fromLocal8Bit(qgetenv("PATH")).split(u';');
-    const QByteArray sdkDir = qgetenv("WindowsSdkDir");
-    if (!sdkDir.isEmpty())
-        paths.prepend(QDir::cleanPath(QString::fromLocal8Bit(sdkDir)) + "/Tools/x64"_L1);
-    return QStandardPaths::findExecutable(tool, paths);
-}
-
-// runProcess helper: Create a temporary file for stdout/stderr redirection.
-static HANDLE createInheritableTemporaryFile()
-{
-    wchar_t path[MAX_PATH];
-    if (!GetTempPath(MAX_PATH, path))
-        return INVALID_HANDLE_VALUE;
-    wchar_t name[MAX_PATH];
-    if (!GetTempFileName(path, L"temp", 0, name)) // Creates file.
-        return INVALID_HANDLE_VALUE;
-    SECURITY_ATTRIBUTES securityAttributes;
-    ZeroMemory(&securityAttributes, sizeof(securityAttributes));
-    securityAttributes.nLength = sizeof(securityAttributes);
-    securityAttributes.bInheritHandle = TRUE;
-    return CreateFile(name, GENERIC_READ | GENERIC_WRITE,
-                      FILE_SHARE_READ | FILE_SHARE_WRITE, &securityAttributes,
-                      TRUNCATE_EXISTING,
-                      FILE_ATTRIBUTE_TEMPORARY | FILE_FLAG_DELETE_ON_CLOSE, NULL);
-}
-
-// runProcess helper: Rewind and read out a temporary file for stdout/stderr.
-static inline void readTemporaryProcessFile(HANDLE handle, QByteArray *result)
-{
-    if (SetFilePointer(handle, 0, 0, FILE_BEGIN) == 0xFFFFFFFF)
-        return;
-    char buf[1024];
-    DWORD bytesRead;
-    while (ReadFile(handle, buf, sizeof(buf), &bytesRead, NULL) && bytesRead)
-        result->append(buf, int(bytesRead));
-    CloseHandle(handle);
-}
-
 static inline void appendToCommandLine(const QString &argument, QString *commandLine)
 {
     const bool needsQuote = argument.contains(u' ');
@@ -199,8 +134,6 @@ static inline void appendToCommandLine(const QString &argument, QString *command
         commandLine->append(u'"');
 }
 
-// runProcess: Run a command line process (replacement for QProcess which
-// does not exist in the bootstrap library).
 bool runProcess(const QString &binary, const QStringList &args,
                 const QString &workingDirectory,
                 unsigned long *exitCode, QByteArray *stdOut, QByteArray *stdErr,
@@ -209,209 +142,35 @@ bool runProcess(const QString &binary, const QStringList &args,
     if (exitCode)
         *exitCode = 0;
 
-    STARTUPINFO si;
-    ZeroMemory(&si, sizeof(si));
-    si.cb = sizeof(si);
+    QProcess process;
+    process.setProgram(binary);
+    process.setArguments(args);
+    process.setWorkingDirectory(workingDirectory);
 
-    STARTUPINFO myInfo;
-    GetStartupInfo(&myInfo);
-    si.hStdInput = myInfo.hStdInput;
-    si.hStdOutput = myInfo.hStdOutput;
-    si.hStdError = myInfo.hStdError;
-
-    PROCESS_INFORMATION pi;
-    ZeroMemory(&pi, sizeof(PROCESS_INFORMATION));
-    const QChar backSlash = u'\\';
-    QString nativeWorkingDir = QDir::toNativeSeparators(workingDirectory.isEmpty() ?  QDir::currentPath() : workingDirectory);
-    if (!nativeWorkingDir.endsWith(backSlash))
-        nativeWorkingDir += backSlash;
-
-    if (stdOut) {
-        si.hStdOutput = createInheritableTemporaryFile();
-        if (si.hStdOutput == INVALID_HANDLE_VALUE) {
-            if (errorMessage)
-                *errorMessage = QStringLiteral("Error creating stdout temporary file");
-            return false;
-        }
-        si.dwFlags |= STARTF_USESTDHANDLES;
-    }
-
-    if (stdErr) {
-        si.hStdError = createInheritableTemporaryFile();
-        if (si.hStdError == INVALID_HANDLE_VALUE) {
-            if (errorMessage)
-                *errorMessage = QStringLiteral("Error creating stderr temporary file");
-            return false;
-        }
-        si.dwFlags |= STARTF_USESTDHANDLES;
-    }
-
-    // Create a copy of the command line which CreateProcessW can modify.
-    QString commandLine;
-    appendToCommandLine(binary, &commandLine);
-    for (const QString &a : args)
-        appendToCommandLine(a, &commandLine);
-    if (optVerboseLevel > 1)
+    // Output the command if requested.
+    if (optVerboseLevel > 1) {
+        QString commandLine;
+        appendToCommandLine(binary, &commandLine);
+        for (const QString &a : args)
+            appendToCommandLine(a, &commandLine);
         std::wcout << "Running: " << commandLine << '\n';
+    }
 
-    QScopedArrayPointer<wchar_t> commandLineW(new wchar_t[commandLine.size() + 1]);
-    commandLine.toWCharArray(commandLineW.data());
-    commandLineW[commandLine.size()] = 0;
-    if (!CreateProcessW(0, commandLineW.data(), 0, 0, /* InheritHandles */ TRUE, 0, 0,
-                        reinterpret_cast<LPCWSTR>(nativeWorkingDir.utf16()), &si, &pi)) {
-        if (stdOut)
-            CloseHandle(si.hStdOutput);
-        if (stdErr)
-            CloseHandle(si.hStdError);
+    process.start();
+    if (!process.waitForStarted() || !process.waitForFinished()) {
         if (errorMessage)
-            *errorMessage = QStringLiteral("CreateProcessW failed: ") + winErrorMessage(GetLastError());
+            *errorMessage = process.errorString();
         return false;
     }
 
-    WaitForSingleObject(pi.hProcess, INFINITE);
-    CloseHandle(pi.hThread);
-    if (exitCode)
-        GetExitCodeProcess(pi.hProcess, exitCode);
-    CloseHandle(pi.hProcess);
-
     if (stdOut)
-        readTemporaryProcessFile(si.hStdOutput, stdOut);
+        *stdOut = process.readAllStandardOutput();
     if (stdErr)
-        readTemporaryProcessFile(si.hStdError, stdErr);
+        *stdErr = process.readAllStandardError();
+
     return true;
 }
 
-#else // Q_OS_WIN
-
-static inline char *encodeFileName(const QString &f)
-{
-    const QByteArray encoded = QFile::encodeName(f);
-    char *result = new char[encoded.size() + 1];
-    strcpy(result, encoded.constData());
-    return result;
-}
-
-static inline char *tempFilePattern()
-{
-    QString path = QDir::tempPath();
-    if (!path.endsWith(u'/'))
-        path += u'/';
-    path += QStringLiteral("tmpXXXXXX");
-    return encodeFileName(path);
-}
-
-static inline QByteArray readOutRedirectFile(int fd)
-{
-    enum { bufSize = 256 };
-
-    QByteArray result;
-    if (!lseek(fd, 0, 0)) {
-        char buf[bufSize];
-        while (true) {
-            const ssize_t rs = read(fd, buf, bufSize);
-            if (rs <= 0)
-                break;
-            result.append(buf, int(rs));
-        }
-    }
-    close(fd);
-    return result;
-}
-
-// runProcess: Run a command line process (replacement for QProcess which
-// does not exist in the bootstrap library).
-bool runProcess(const QString &binary, const QStringList &args,
-                const QString &workingDirectory,
-                unsigned long *exitCode, QByteArray *stdOut, QByteArray *stdErr,
-                QString *errorMessage)
-{
-    QScopedArrayPointer<char> stdOutFileName;
-    QScopedArrayPointer<char> stdErrFileName;
-
-    int stdOutFile = 0;
-    if (stdOut) {
-        stdOutFileName.reset(tempFilePattern());
-        stdOutFile = mkstemp(stdOutFileName.data());
-        if (stdOutFile < 0) {
-            *errorMessage = QStringLiteral("mkstemp() failed: ") + QString::fromLocal8Bit(strerror(errno));
-            return false;
-        }
-    }
-
-    int stdErrFile = 0;
-    if (stdErr) {
-        stdErrFileName.reset(tempFilePattern());
-        stdErrFile = mkstemp(stdErrFileName.data());
-        if (stdErrFile < 0) {
-            *errorMessage = QStringLiteral("mkstemp() failed: ") + QString::fromLocal8Bit(strerror(errno));
-            return false;
-        }
-    }
-
-    const pid_t pID = fork();
-
-    if (pID < 0) {
-        *errorMessage = QStringLiteral("Fork failed: ") + QString::fromLocal8Bit(strerror(errno));
-        return false;
-    }
-
-    if (!pID) { // Child
-        if (stdOut) {
-            dup2(stdOutFile, STDOUT_FILENO);
-            close(stdOutFile);
-        }
-        if (stdErr) {
-            dup2(stdErrFile, STDERR_FILENO);
-            close(stdErrFile);
-        }
-
-        if (!workingDirectory.isEmpty() && !QDir::setCurrent(workingDirectory)) {
-            std::wcerr << "Failed to change working directory to " << workingDirectory << ".\n";
-            ::_exit(-1);
-        }
-
-        char **argv  = new char *[args.size() + 2]; // Create argv.
-        char **ap = argv;
-        *ap++ = encodeFileName(binary);
-        for (const QString &a : std::as_const(args))
-            *ap++ = encodeFileName(a);
-        *ap = 0;
-
-        execvp(argv[0], argv);
-        ::_exit(-1);
-    }
-
-    int status;
-    pid_t waitResult;
-
-    do {
-        waitResult = waitpid(pID, &status, 0);
-    } while (waitResult == -1 && errno == EINTR);
-
-    if (stdOut) {
-        *stdOut = readOutRedirectFile(stdOutFile);
-        unlink(stdOutFileName.data());
-    }
-    if (stdErr) {
-        *stdErr = readOutRedirectFile(stdErrFile);
-        unlink(stdErrFileName.data());
-    }
-
-    if (waitResult < 0) {
-        *errorMessage = QStringLiteral("Wait failed: ") + QString::fromLocal8Bit(strerror(errno));
-        return false;
-    }
-    if (!WIFEXITED(status)) {
-        *errorMessage = binary + QStringLiteral(" did not exit cleanly.");
-        return false;
-    }
-    if (exitCode)
-        *exitCode = WEXITSTATUS(status);
-    return true;
-}
-
-#endif // !Q_OS_WIN
-
 // Find a file in the path using ShellAPI. This can be used to locate DLLs which
 // QStandardPaths cannot do.
 QString findInPath(const QString &file)
@@ -435,14 +194,18 @@ const char *qmakeInfixKey = "QT_INFIX";
 QMap<QString, QString> queryQtPaths(const QString &qtpathsBinary, QString *errorMessage)
 {
     const QString binary = !qtpathsBinary.isEmpty() ? qtpathsBinary : QStringLiteral("qtpaths");
+    const QString colonSpace = QStringLiteral(": ");
     QByteArray stdOut;
     QByteArray stdErr;
     unsigned long exitCode = 0;
-    if (!runProcess(binary, QStringList(QStringLiteral("-query")), QString(), &exitCode, &stdOut, &stdErr, errorMessage))
+    if (!runProcess(binary, QStringList(QStringLiteral("-query")), QString(), &exitCode, &stdOut,
+                    &stdErr, errorMessage)) {
+        *errorMessage = QStringLiteral("Error running binary ") + binary + colonSpace + *errorMessage;
         return QMap<QString, QString>();
+    }
     if (exitCode) {
         *errorMessage = binary + QStringLiteral(" returns ") + QString::number(exitCode)
-            + QStringLiteral(": ") + QString::fromLocal8Bit(stdErr);
+            + colonSpace + QString::fromLocal8Bit(stdErr);
         return QMap<QString, QString>();
     }
     const QString output = QString::fromLocal8Bit(stdOut).trimmed().remove(u'\r');
@@ -462,10 +225,9 @@ QMap<QString, QString> queryQtPaths(const QString &qtpathsBinary, QString *error
     }
     QFile qconfigPriFile(result.value(QStringLiteral("QT_HOST_DATA")) + QStringLiteral("/mkspecs/qconfig.pri"));
     if (qconfigPriFile.open(QIODevice::ReadOnly | QIODevice::Text)) {
-        while (true) {
-            const QByteArray line = qconfigPriFile.readLine();
-            if (line.isEmpty())
-                break;
+        QByteArray lineArray;
+        while (qconfigPriFile.readLineInto(&lineArray)) {
+            QByteArrayView line = QByteArrayView(lineArray);
             if (line.startsWith("QT_LIBINFIX")) {
                 const int pos = line.indexOf('=');
                 if (pos >= 0) {
@@ -478,7 +240,7 @@ QMap<QString, QString> queryQtPaths(const QString &qtpathsBinary, QString *error
         }
     } else {
         std::wcerr << "Warning: Unable to read " << QDir::toNativeSeparators(qconfigPriFile.fileName())
-            << ": " << qconfigPriFile.errorString()<< '\n';
+            << colonSpace << qconfigPriFile.errorString()<< '\n';
     }
     return result;
 }
@@ -564,37 +326,6 @@ bool updateFile(const QString &sourceFileName, const QStringList &nameFilters,
     return true;
 }
 
-bool readElfExecutable(const QString &elfExecutableFileName, QString *errorMessage,
-                       QStringList *dependentLibraries, unsigned *wordSize,
-                       bool *isDebug)
-{
-    ElfReader elfReader(elfExecutableFileName);
-    const ElfData data = elfReader.readHeaders();
-    if (data.sectionHeaders.isEmpty()) {
-        *errorMessage = QStringLiteral("Unable to read ELF binary \"")
-            + QDir::toNativeSeparators(elfExecutableFileName) + QStringLiteral("\": ")
-            + elfReader.errorString();
-            return false;
-    }
-    if (wordSize)
-        *wordSize = data.elfclass == Elf_ELFCLASS64 ? 64 : 32;
-    if (dependentLibraries) {
-        dependentLibraries->clear();
-        const QList<QByteArray> libs = elfReader.dependencies();
-        if (libs.isEmpty()) {
-            *errorMessage = QStringLiteral("Unable to read dependenices of ELF binary \"")
-                + QDir::toNativeSeparators(elfExecutableFileName) + QStringLiteral("\": ")
-                + elfReader.errorString();
-                return false;
-        }
-        for (const QByteArray &l : libs)
-            dependentLibraries->push_back(QString::fromLocal8Bit(l));
-    }
-    if (isDebug)
-        *isDebug = data.symbolsType != UnknownSymbols && data.symbolsType != NoSymbols;
-    return true;
-}
-
 #ifdef Q_OS_WIN
 
 static inline QString stringFromRvaPtr(const void *rvaPtr)
@@ -716,13 +447,23 @@ static inline MsvcDebugRuntimeResult checkMsvcDebugRuntime(const QStringList &de
         qsizetype pos = 0;
         if (lib.startsWith("MSVCR"_L1, Qt::CaseInsensitive)
             || lib.startsWith("MSVCP"_L1, Qt::CaseInsensitive)
-            || lib.startsWith("VCRUNTIME"_L1, Qt::CaseInsensitive)) {
+            || lib.startsWith("VCRUNTIME"_L1, Qt::CaseInsensitive)
+            || lib.startsWith("VCCORLIB"_L1, Qt::CaseInsensitive)
+            || lib.startsWith("CONCRT"_L1, Qt::CaseInsensitive)
+            || lib.startsWith("UCRTBASE"_L1, Qt::CaseInsensitive)) {
             qsizetype lastDotPos = lib.lastIndexOf(u'.');
             pos = -1 == lastDotPos ? 0 : lastDotPos - 1;
         }
 
-        if (pos > 0 && lib.contains("_app"_L1, Qt::CaseInsensitive))
-            pos -= 4;
+        if (pos > 0) {
+            const auto removeExtraSuffix = [&lib, &pos](const QString &suffix) -> void {
+                if (lib.contains(suffix, Qt::CaseInsensitive))
+                    pos -= suffix.size();
+            };
+            removeExtraSuffix("_app"_L1);
+            removeExtraSuffix("_atomic_wait"_L1);
+            removeExtraSuffix("_codecvt_ids"_L1);
+        }
 
         if (pos)
             return lib.at(pos).toLower() == u'd' ? MsvcDebugRuntime : MsvcReleaseRuntime;
@@ -730,29 +471,44 @@ static inline MsvcDebugRuntimeResult checkMsvcDebugRuntime(const QStringList &de
     return NoMsvcRuntime;
 }
 
+template <class ImageNtHeader>
+inline QStringList determineDependentLibs(const ImageNtHeader *nth, const void *fileMemory,
+                                           QString *errorMessage)
+{
+    return readImportSections(nth, fileMemory, errorMessage);
+}
+
+template <class ImageNtHeader>
+inline bool determineDebug(const ImageNtHeader *nth, const void *fileMemory,
+                           QStringList *dependentLibrariesIn, QString *errorMessage)
+{
+    if (nth->FileHeader.Characteristics & IMAGE_FILE_DEBUG_STRIPPED)
+        return false;
+
+    const QStringList dependentLibraries = dependentLibrariesIn != nullptr ?
+                *dependentLibrariesIn :
+                determineDependentLibs(nth, fileMemory, errorMessage);
+
+    const bool hasDebugEntry = nth->OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_DEBUG].Size;
+    // When an MSVC debug entry is present, check whether the debug runtime
+    // is actually used to detect -release / -force-debug-info builds.
+    const MsvcDebugRuntimeResult msvcrt = checkMsvcDebugRuntime(dependentLibraries);
+    if (msvcrt == NoMsvcRuntime)
+        return hasDebugEntry;
+    else
+        return hasDebugEntry && msvcrt == MsvcDebugRuntime;
+}
+
 template <class ImageNtHeader>
 inline void determineDebugAndDependentLibs(const ImageNtHeader *nth, const void *fileMemory,
-                                           bool isMinGW,
                                            QStringList *dependentLibrariesIn,
                                            bool *isDebugIn, QString *errorMessage)
 {
-    const bool hasDebugEntry = nth->OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_DEBUG].Size;
-    QStringList dependentLibraries;
-    if (dependentLibrariesIn || (isDebugIn != nullptr && hasDebugEntry && !isMinGW))
-        dependentLibraries = readImportSections(nth, fileMemory, errorMessage);
-
     if (dependentLibrariesIn)
-        *dependentLibrariesIn = dependentLibraries;
-    if (isDebugIn != nullptr) {
-        if (isMinGW) {
-            // Use logic that's used e.g. in objdump / pfd library
-            *isDebugIn = !(nth->FileHeader.Characteristics & IMAGE_FILE_DEBUG_STRIPPED);
-        } else {
-            // When an MSVC debug entry is present, check whether the debug runtime
-            // is actually used to detect -release / -force-debug-info builds.
-            *isDebugIn = hasDebugEntry && checkMsvcDebugRuntime(dependentLibraries) != MsvcReleaseRuntime;
-        }
-    }
+        *dependentLibrariesIn = determineDependentLibs(nth, fileMemory, errorMessage);
+
+    if (isDebugIn)
+        *isDebugIn = determineDebug(nth, fileMemory, dependentLibrariesIn, errorMessage);
 }
 
 // Read a PE executable and determine dependent libraries, word size
@@ -778,19 +534,22 @@ bool readPeExecutable(const QString &peExecutableFileName, QString *errorMessage
         hFile = CreateFile(reinterpret_cast<const WCHAR*>(peExecutableFileName.utf16()), GENERIC_READ, FILE_SHARE_READ, NULL,
                              OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
         if (hFile == INVALID_HANDLE_VALUE || hFile == NULL) {
-            *errorMessage = QString::fromLatin1("Cannot open '%1': %2").arg(peExecutableFileName, winErrorMessage(GetLastError()));
+            *errorMessage = QString::fromLatin1("Cannot open '%1': %2")
+                .arg(peExecutableFileName, QSystemError::windowsString());
             break;
         }
 
         hFileMap = CreateFileMapping(hFile, NULL, PAGE_READONLY, 0, 0, NULL);
         if (hFileMap == NULL) {
-            *errorMessage = QString::fromLatin1("Cannot create file mapping of '%1': %2").arg(peExecutableFileName, winErrorMessage(GetLastError()));
+            *errorMessage = QString::fromLatin1("Cannot create file mapping of '%1': %2")
+                .arg(peExecutableFileName, QSystemError::windowsString());
             break;
         }
 
         fileMemory = MapViewOfFile(hFileMap, FILE_MAP_READ, 0, 0, 0);
         if (!fileMemory) {
-            *errorMessage = QString::fromLatin1("Cannot map '%1': %2").arg(peExecutableFileName, winErrorMessage(GetLastError()));
+            *errorMessage = QString::fromLatin1("Cannot map '%1': %2")
+                .arg(peExecutableFileName, QSystemError::windowsString());
             break;
         }
 
@@ -803,10 +562,10 @@ bool readPeExecutable(const QString &peExecutableFileName, QString *errorMessage
             *wordSizeIn = wordSize;
         if (wordSize == 32) {
             determineDebugAndDependentLibs(reinterpret_cast<const IMAGE_NT_HEADERS32 *>(ntHeaders),
-                                           fileMemory, isMinGW, dependentLibrariesIn, isDebugIn, errorMessage);
+                                           fileMemory, dependentLibrariesIn, isDebugIn, errorMessage);
         } else {
             determineDebugAndDependentLibs(reinterpret_cast<const IMAGE_NT_HEADERS64 *>(ntHeaders),
-                                           fileMemory, isMinGW, dependentLibrariesIn, isDebugIn, errorMessage);
+                                           fileMemory, dependentLibrariesIn, isDebugIn, errorMessage);
         }
 
         if (machineArchIn)
@@ -889,6 +648,53 @@ QString findD3dCompiler(Platform platform, const QString &qtBinDir, unsigned wor
     return QString();
 }
 
+QStringList findDxc(Platform platform, const QString &qtBinDir, unsigned wordSize)
+{
+    QStringList results;
+    const QString kitDir = QString::fromLocal8Bit(qgetenv("WindowsSdkDir"));
+    const QString suffix = QLatin1StringView(windowsSharedLibrarySuffix);
+    for (QString prefix : { QStringLiteral("dxcompiler"), QStringLiteral("dxil") }) {
+        QString name = prefix + suffix;
+        if (!kitDir.isEmpty()) {
+            QString redistDirPath = QDir::cleanPath(kitDir) + QStringLiteral("/Redist/D3D/");
+            if (platform.testFlag(ArmBased)) {
+                redistDirPath += wordSize == 32 ? QStringLiteral("arm") : QStringLiteral("arm64");
+            } else {
+                redistDirPath += wordSize == 32 ? QStringLiteral("x86") : QStringLiteral("x64");
+            }
+            QDir redistDir(redistDirPath);
+            if (redistDir.exists()) {
+                const QFileInfoList files = redistDir.entryInfoList(QStringList(prefix + u'*' + suffix), QDir::Files);
+                if (!files.isEmpty()) {
+                    results.append(files.front().absoluteFilePath());
+                    continue;
+                }
+            }
+        }
+        // Check the bin directory of the Qt SDK (in case it is shadowed by the
+        // Windows system directory in PATH).
+        const QFileInfo fi(qtBinDir + u'/' + name);
+        if (fi.isFile()) {
+            results.append(fi.absoluteFilePath());
+            continue;
+        }
+        // Try to find it in the PATH (e.g. the Vulkan SDK ships these, even if Windows itself doesn't).
+        if (platform.testFlag(IntelBased)) {
+            QString errorMessage;
+            unsigned detectedWordSize;
+            const QString dll = findInPath(name);
+            if (!dll.isEmpty()
+                && readPeExecutable(dll, &errorMessage, 0, &detectedWordSize, 0)
+                && detectedWordSize == wordSize)
+            {
+                results.append(dll);
+                continue;
+            }
+        }
+    }
+    return results;
+}
+
 #else // Q_OS_WIN
 
 bool readPeExecutable(const QString &, QString *errorMessage,
@@ -903,6 +709,11 @@ QString findD3dCompiler(Platform, const QString &, unsigned)
     return QString();
 }
 
+QStringList findDxc(Platform, const QString &, unsigned)
+{
+    return QStringList();
+}
+
 #endif // !Q_OS_WIN
 
 // Search for "qt_prfxpath=xxxx" in \a path, and replace it with "qt_prfxpath=."
