@@ -17,7 +17,6 @@
 #include <qdebug.h>
 
 #include <private/qwidgetresizehandler_p.h>
-#include <private/qstylesheetstyle_p.h>
 #include <qpa/qplatformtheme.h>
 
 #include <private/qhighdpiscaling_p.h>
@@ -854,7 +853,7 @@ void QDockWidgetPrivate::endDrag(EndDragMode mode)
                     // Reparent, if the drag was out of a dock widget group window
                     if (mode == EndDragMode::LocationChange) {
                         if (auto *groupWindow = qobject_cast<QDockWidgetGroupWindow *>(q->parentWidget()))
-                            groupWindow->reparent(q);
+                            groupWindow->reparentToMainWindow(q);
                     }
                 }
                 q->activateWindow();
@@ -1173,17 +1172,19 @@ void QDockWidgetPrivate::unplug(const QRect &rect)
     QDockWidgetLayout *dwLayout = qobject_cast<QDockWidgetLayout*>(layout);
     if (dwLayout->nativeWindowDeco(true))
         r.adjust(0, dwLayout->titleHeight(), 0, 0);
-    setWindowState(true, true, r);
+    setWindowState({WindowState::Floating, WindowState::Unplug}, r);
 }
 
 void QDockWidgetPrivate::plug(const QRect &rect)
 {
-    setWindowState(false, false, rect);
+    setWindowState(WindowStates(), rect);
 }
 
-void QDockWidgetPrivate::setWindowState(bool floating, bool unplug, const QRect &rect)
+void QDockWidgetPrivate::setWindowState(WindowStates states, const QRect &rect)
 {
     Q_Q(QDockWidget);
+    const bool floating = states.testFlag(WindowState::Floating);
+    bool unplug = states.testFlag(WindowState::Unplug);
 
     if (!floating && parent) {
         QMainWindowLayout *mwlayout = qt_mainwindow_layout_from_dock(q);
@@ -1357,7 +1358,15 @@ QDockWidget::QDockWidget(const QString &title, QWidget *parent, Qt::WindowFlags
     Destroys the dock widget.
 */
 QDockWidget::~QDockWidget()
-{ }
+{
+    Q_D(QDockWidget);
+    d->inDestructor = true;
+    // Do all the unregistering while we're still a QDockWidget. Otherwise, it
+    // would be ~QObject() which does that and then QDockAreaLayout::takeAt(),
+    // acting on QEvent::ChildRemoved, will try to access our QWidget-ness when
+    // replacing us with a QPlaceHolderItem, causing UB:
+    setParent(nullptr);
+}
 
 /*!
     Returns the widget for the dock widget. This function returns zero
@@ -1415,10 +1424,12 @@ void QDockWidget::setFeatures(QDockWidget::DockWidgetFeatures features)
     update();
     if (closableChanged && layout->nativeWindowDeco()) {
         QDockWidgetGroupWindow *floatingTab = qobject_cast<QDockWidgetGroupWindow *>(parent());
-        if (floatingTab && !isFloating())
+        if (floatingTab && !isFloating()) {
             floatingTab->adjustFlags();
-        else
-            d->setWindowState(true /*floating*/, true /*unplug*/);  //this ensures the native decoration is drawn
+        } else {
+            d->setWindowState({QDockWidgetPrivate::WindowState::Floating,
+                               QDockWidgetPrivate::WindowState::Unplug});
+        }
     }
 }
 
@@ -1485,7 +1496,9 @@ void QDockWidgetPrivate::setFloating(bool floating)
         }
     }
 
-    setWindowState(floating, false, floating ? r : QRect());
+    WindowStates states;
+    states.setFlag(WindowState::Floating, floating);
+    setWindowState(states, floating ? r : QRect());
 
     if (floating && r.isNull()) {
         if (q->x() < 0 || q->y() < 0) //may happen if we have been hidden
@@ -1634,8 +1647,12 @@ bool QDockWidget::event(QEvent *event)
     case QEvent::Hide:
         if (layout != nullptr)
             layout->keepSize(this);
-        d->toggleViewAction->setChecked(false);
-        emit visibilityChanged(false);
+        // If we are in the destructor, don't emit any signals, as those might
+        // be handled by a slot that requires this dock widget to still be alive.
+        if (!d->inDestructor) {
+            d->toggleViewAction->setChecked(false);
+            emit visibilityChanged(false);
+        }
         break;
     case QEvent::Show: {
         d->toggleViewAction->setChecked(true);
@@ -1791,6 +1808,7 @@ QAction * QDockWidget::toggleViewAction() const
     dock \a area, or is moved to a different location in its current
     dock area. This happens when the dock widget is moved
     programmatically or is dragged to a new location by the user.
+    \sa dockLocation(), setDockLocation()
 */
 
 /*!
@@ -1843,10 +1861,56 @@ void QDockWidget::setTitleBarWidget(QWidget *widget)
     d->updateButtons();
     if (isWindow()) {
         //this ensures the native decoration is drawn
-        d->setWindowState(true /*floating*/, true /*unplug*/);
+        d->setWindowState({QDockWidgetPrivate::WindowState::Floating,
+                           QDockWidgetPrivate::WindowState::Unplug});
     }
 }
 
+/*!
+    \since 6.9
+
+    Assigns this dock widget to \a area. If docked at another dock location, it
+    will move to \a area. If floating or part of floating tabs, the next call
+    of setFloating(false) will dock it at \a area.
+
+    \note setDockLocation(Qt::NoDockLocation) is equivalent to setFloating(true).
+
+    \sa dockLocation(), dockLocationChanged()
+ */
+void QDockWidget::setDockLocation(Qt::DockWidgetArea area)
+{
+    if (area == Qt::NoDockWidgetArea && !isFloating()) {
+        setFloating(true);
+        return;
+    }
+
+    auto *mainWindow = const_cast<QMainWindow *>(mainwindow_from_dock(this));
+    Q_ASSERT(mainWindow);
+    mainWindow->addDockWidget(area, this);
+}
+
+/*!
+    \property QDockWidget::dockLocation
+    \since 6.9
+
+    \brief the current dock location, or Qt::NoDockLocation if this dock widget
+    is floating or has no mainwindow parent.
+ */
+Qt::DockWidgetArea QDockWidget::dockLocation() const
+{
+    // QDockWidgetPrivate::setWindowState() emits NoDockWidgetArea if
+    // the dock widget becomes floating.
+    // QMainWindowLayout::dockWidgetArea() always returns the area where
+    // the dock widget's item_list is kept.
+    if (isFloating())
+        return Qt::NoDockWidgetArea;
+
+    auto *mainWindow = mainwindow_from_dock(this);
+    Q_ASSERT(mainWindow);
+    // FIXME in Qt 7: Make dockWidgetArea take a const QDockWidget* argument
+    return mainWindow->dockWidgetArea(const_cast<QDockWidget *>(this));
+}
+
 /*!
     \since 4.3
     Returns the custom title bar widget set on the QDockWidget, or
