@@ -647,9 +647,7 @@ QMdiAreaPrivate::QMdiAreaPrivate()
       indexToNextWindow(-1),
       indexToPreviousWindow(-1),
       indexToHighlighted(-1),
-      indexToLastActiveTab(-1),
-      resizeTimerId(-1),
-      tabToPreviousTimerId(-1)
+      indexToLastActiveTab(-1)
 {
 }
 
@@ -990,7 +988,7 @@ void QMdiAreaPrivate::activateHighlightedWindow()
         return;
 
     Q_ASSERT(indexToHighlighted < childWindows.size());
-    if (tabToPreviousTimerId != -1)
+    if (tabToPreviousTimer.isActive())
         activateWindow(nextVisibleSubWindow(-1, QMdiArea::ActivationHistoryOrder));
     else
         activateWindow(childWindows.at(indexToHighlighted));
@@ -1491,7 +1489,7 @@ void QMdiAreaPrivate::highlightNextSubWindow(int increaseFactor)
 
     // Only highlight if we're not switching back to the previously active window (Ctrl-Tab once).
 #if QT_CONFIG(rubberband)
-    if (tabToPreviousTimerId == -1)
+    if (!tabToPreviousTimer.isActive())
         showRubberBandFor(highlight);
 #endif
 
@@ -2334,13 +2332,11 @@ void QMdiArea::resizeEvent(QResizeEvent *resizeEvent)
 void QMdiArea::timerEvent(QTimerEvent *timerEvent)
 {
     Q_D(QMdiArea);
-    if (timerEvent->timerId() == d->resizeTimerId) {
-        killTimer(d->resizeTimerId);
-        d->resizeTimerId = -1;
+    if (timerEvent->id() == d->resizeTimer.id()) {
+        d->resizeTimer.stop();
         d->arrangeMinimizedSubWindows();
-    } else if (timerEvent->timerId() == d->tabToPreviousTimerId) {
-        killTimer(d->tabToPreviousTimerId);
-        d->tabToPreviousTimerId = -1;
+    } else if (timerEvent->id() == d->tabToPreviousTimer.id()) {
+        d->tabToPreviousTimer.stop();
         if (d->indexToHighlighted < 0)
             return;
 #if QT_CONFIG(rubberband)
