@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtWidgets module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qwidgetlinecontrol_p.h"
 
@@ -49,7 +13,7 @@
 #endif
 #include <qpa/qplatformtheme.h>
 #include <qstylehints.h>
-#ifndef QT_NO_ACCESSIBILITY
+#if QT_CONFIG(accessibility)
 #include "qaccessible.h"
 #endif
 
@@ -61,6 +25,8 @@
 
 #include "qvalidator.h"
 
+using namespace std::chrono_literals;
+
 QT_BEGIN_NAMESPACE
 
 
@@ -98,7 +64,7 @@ void QWidgetLineControl::updateDisplayText(bool forceUpdate)
 
     if (m_echoMode == QLineEdit::Password) {
         str.fill(m_passwordCharacter);
-        if (m_passwordEchoTimer != 0 && m_cursor > 0 && m_cursor <= m_text.length()) {
+        if (m_passwordEchoTimer.isActive() && m_cursor > 0 && m_cursor <= m_text.size()) {
             int cursor = m_cursor - 1;
             QChar uc = m_text.at(cursor);
             str[cursor] = uc;
@@ -118,11 +84,10 @@ void QWidgetLineControl::updateDisplayText(bool forceUpdate)
     // drawing boxes when using fonts that don't have glyphs for such
     // characters)
     QChar* uc = str.data();
-    for (int i = 0; i < (int)str.length(); ++i) {
+    for (int i = 0; i < (int)str.size(); ++i) {
         if ((uc[i].unicode() < 0x20 && uc[i].unicode() != 0x09)
             || uc[i] == QChar::LineSeparator
-            || uc[i] == QChar::ParagraphSeparator
-            || uc[i] == QChar::ObjectReplacementCharacter)
+            || uc[i] == QChar::ParagraphSeparator)
             uc[i] = QChar(0x0020);
     }
 
@@ -278,7 +243,7 @@ void QWidgetLineControl::clear()
 {
     int priorState = m_undoState;
     m_selstart = 0;
-    m_selend = m_text.length();
+    m_selend = m_text.size();
     removeSelectedText();
     separate();
     finishChange(priorState, /*update*/false, /*edited*/false);
@@ -322,7 +287,7 @@ void QWidgetLineControl::setSelection(int start, int length)
         if (start == m_selstart && start + length == m_selend && m_cursor == m_selend)
             return;
         m_selstart = start;
-        m_selend = qMin(start + length, (int)m_text.length());
+        m_selend = qMin(start + length, (int)m_text.size());
         m_cursor = m_selend;
     } else if (length < 0){
         if (start == m_selend && start + length == m_selstart && m_cursor == m_selstart)
@@ -365,7 +330,7 @@ void QWidgetLineControl::init(const QString &txt)
     m_textLayout.setCacheEnabled(true);
     m_text = txt;
     updateDisplayText();
-    m_cursor = m_text.length();
+    m_cursor = m_text.size();
     if (const QPlatformTheme *theme = QGuiApplicationPrivate::platformTheme()) {
         m_keyboardScheme = theme->themeHint(QPlatformTheme::KeyboardScheme).toInt();
         m_passwordMaskDelay = theme->themeHint(QPlatformTheme::PasswordMaskDelay).toInt();
@@ -527,14 +492,14 @@ void QWidgetLineControl::processInputMethodEvent(QInputMethodEvent *event)
         if (echoMode() == QLineEdit::PasswordEchoOnEdit && !passwordEchoEditing()) {
             updatePasswordEchoEditing(true);
             m_selstart = 0;
-            m_selend = m_text.length();
+            m_selend = m_text.size();
         }
         removeSelectedText();
     }
 
     int c = m_cursor; // cursor position after insertion of commit string
     if (event->replacementStart() <= 0)
-        c += event->commitString().length() - qMin(-event->replacementStart(), event->replacementLength());
+        c += event->commitString().size() - qMin(-event->replacementStart(), event->replacementLength());
 
     m_cursor += event->replacementStart();
     if (m_cursor < 0)
@@ -550,15 +515,15 @@ void QWidgetLineControl::processInputMethodEvent(QInputMethodEvent *event)
         internalInsert(event->commitString());
         cursorPositionChanged = true;
     } else {
-        m_cursor = qBound(0, c, m_text.length());
+        m_cursor = qBound(0, c, m_text.size());
     }
 
     for (int i = 0; i < event->attributes().size(); ++i) {
         const QInputMethodEvent::Attribute &a = event->attributes().at(i);
         if (a.type == QInputMethodEvent::Selection) {
-            m_cursor = qBound(0, a.start + a.length, m_text.length());
+            m_cursor = qBound(0, a.start + a.length, m_text.size());
             if (a.length) {
-                m_selstart = qMax(0, qMin(a.start, m_text.length()));
+                m_selstart = qMax(0, qMin(a.start, m_text.size()));
                 m_selend = m_cursor;
                 if (m_selend < m_selstart) {
                     qSwap(m_selstart, m_selend);
@@ -590,7 +555,7 @@ void QWidgetLineControl::processInputMethodEvent(QInputMethodEvent *event)
     }
 #endif //QT_NO_IM
     const int oldPreeditCursor = m_preeditCursor;
-    m_preeditCursor = event->preeditString().length();
+    m_preeditCursor = event->preeditString().size();
     m_hideCursor = false;
     QList<QTextLayout::FormatRange> formats;
     formats.reserve(event->attributes().size());
@@ -731,7 +696,7 @@ bool QWidgetLineControl::finishChange(int validateFromState, bool update, bool e
         }
 #endif
         if (validateFromState >= 0 && wasValidInput && !m_validInput) {
-            if (m_transactions.count())
+            if (m_transactions.size())
                 return false;
             internalUndo(validateFromState);
             m_history.erase(m_history.begin() + m_undoState, m_history.end());
@@ -755,7 +720,7 @@ bool QWidgetLineControl::finishChange(int validateFromState, bool update, bool e
         emit selectionChanged();
     }
     if (m_cursor == m_lastCursorPos)
-        updateMicroFocus();
+        emit updateMicroFocus();
     emitCursorPositionChanged();
     return true;
 }
@@ -773,7 +738,7 @@ void QWidgetLineControl::internalSetText(const QString &txt, int pos, bool edite
     QString oldText = m_text;
     if (m_maskData) {
         m_text = maskString(0, txt, true);
-        m_text += clearString(m_text.length(), m_maxLength - m_text.length());
+        m_text += clearString(m_text.size(), m_maxLength - m_text.size());
         if (edited && oldText == m_text)
             emit inputRejected();
     } else {
@@ -781,11 +746,11 @@ void QWidgetLineControl::internalSetText(const QString &txt, int pos, bool edite
     }
     m_history.clear();
     m_modifiedState =  m_undoState = 0;
-    m_cursor = (pos < 0 || pos > m_text.length()) ? m_text.length() : pos;
+    m_cursor = (pos < 0 || pos > m_text.size()) ? m_text.size() : pos;
     m_textDirty = (oldText != m_text);
     const bool changed = finishChange(-1, true, edited);
 
-#ifndef QT_NO_ACCESSIBILITY
+#if QT_CONFIG(accessibility)
     if (changed) {
         if (oldText.isEmpty()) {
             QAccessibleTextInsertEvent event(accessibleObject(), 0, txt);
@@ -838,8 +803,8 @@ void QWidgetLineControl::addCommand(const Command &cmd)
 void QWidgetLineControl::internalInsert(const QString &s)
 {
     if (m_echoMode == QLineEdit::Password) {
-        if (m_passwordEchoTimer != 0)
-            killTimer(m_passwordEchoTimer);
+        if (m_passwordEchoTimer.isActive())
+            m_passwordEchoTimer.stop();
         int delay = m_passwordMaskDelay;
 #ifdef QT_BUILD_INTERNAL
         if (m_passwordMaskDelayOverride >= 0)
@@ -847,7 +812,7 @@ void QWidgetLineControl::internalInsert(const QString &s)
 #endif
 
         if (delay > 0)
-            m_passwordEchoTimer = startTimer(delay);
+            m_passwordEchoTimer.start(delay * 1ms, this);
     }
     if (hasSelectedText())
         addCommand(Command(SetSelection, m_cursor, u'\0', m_selstart, m_selend));
@@ -855,35 +820,35 @@ void QWidgetLineControl::internalInsert(const QString &s)
         QString ms = maskString(m_cursor, s);
         if (ms.isEmpty() && !s.isEmpty())
             emit inputRejected();
-#ifndef QT_NO_ACCESSIBILITY
+#if QT_CONFIG(accessibility)
         QAccessibleTextInsertEvent insertEvent(accessibleObject(), m_cursor, ms);
         QAccessible::updateAccessibility(&insertEvent);
 #endif
-        for (int i = 0; i < (int) ms.length(); ++i) {
+        for (int i = 0; i < (int) ms.size(); ++i) {
             addCommand (Command(DeleteSelection, m_cursor + i, m_text.at(m_cursor + i), -1, -1));
             addCommand(Command(Insert, m_cursor + i, ms.at(i), -1, -1));
         }
-        m_text.replace(m_cursor, ms.length(), ms);
-        m_cursor += ms.length();
+        m_text.replace(m_cursor, ms.size(), ms);
+        m_cursor += ms.size();
         m_cursor = nextMaskBlank(m_cursor);
         m_textDirty = true;
-#ifndef QT_NO_ACCESSIBILITY
+#if QT_CONFIG(accessibility)
         QAccessibleTextCursorEvent event(accessibleObject(), m_cursor);
         QAccessible::updateAccessibility(&event);
 #endif
     } else {
-        int remaining = m_maxLength - m_text.length();
+        int remaining = m_maxLength - m_text.size();
         if (remaining != 0) {
-#ifndef QT_NO_ACCESSIBILITY
+#if QT_CONFIG(accessibility)
             QAccessibleTextInsertEvent insertEvent(accessibleObject(), m_cursor, s);
             QAccessible::updateAccessibility(&insertEvent);
 #endif
             m_text.insert(m_cursor, s.left(remaining));
-            for (int i = 0; i < (int) s.left(remaining).length(); ++i)
+            for (int i = 0; i < (int) s.left(remaining).size(); ++i)
                addCommand(Command(Insert, m_cursor++, s.at(i), -1, -1));
             m_textDirty = true;
         }
-        if (s.length() > remaining)
+        if (s.size() > remaining)
             emit inputRejected();
     }
 }
@@ -901,13 +866,13 @@ void QWidgetLineControl::internalInsert(const QString &s)
 */
 void QWidgetLineControl::internalDelete(bool wasBackspace)
 {
-    if (m_cursor < (int) m_text.length()) {
+    if (m_cursor < (int) m_text.size()) {
         cancelPasswordEchoTimer();
         if (hasSelectedText())
             addCommand(Command(SetSelection, m_cursor, u'\0', m_selstart, m_selend));
         addCommand(Command((CommandType)((m_maskData ? 2 : 0) + (wasBackspace ? Remove : Delete)),
                    m_cursor, m_text.at(m_cursor), -1, -1));
-#ifndef QT_NO_ACCESSIBILITY
+#if QT_CONFIG(accessibility)
         QAccessibleTextRemoveEvent event(accessibleObject(), m_cursor, m_text.at(m_cursor));
         QAccessible::updateAccessibility(&event);
 #endif
@@ -932,7 +897,7 @@ void QWidgetLineControl::internalDelete(bool wasBackspace)
 */
 void QWidgetLineControl::removeSelectedText()
 {
-    if (m_selstart < m_selend && m_selend <= (int) m_text.length()) {
+    if (m_selstart < m_selend && m_selend <= (int) m_text.size()) {
         cancelPasswordEchoTimer();
         separate();
         int i ;
@@ -948,7 +913,7 @@ void QWidgetLineControl::removeSelectedText()
             for (i = m_selend-1; i >= m_selstart; --i)
                 addCommand (Command(RemoveSelection, i, m_text.at(i), -1, -1));
         }
-#ifndef QT_NO_ACCESSIBILITY
+#if QT_CONFIG(accessibility)
         QAccessibleTextRemoveEvent event(accessibleObject(), m_selstart, m_text.mid(m_selstart, m_selend - m_selstart));
         QAccessible::updateAccessibility(&event);
 #endif
@@ -974,7 +939,7 @@ void QWidgetLineControl::removeSelectedText()
 */
 void QWidgetLineControl::parseInputMask(const QString &maskFields)
 {
-    int delimiter = maskFields.indexOf(QLatin1Char(';'));
+    qsizetype delimiter = maskFields.indexOf(u';');
     if (maskFields.isEmpty() || delimiter == 0) {
         if (m_maskData) {
             m_maskData.reset();
@@ -985,17 +950,17 @@ void QWidgetLineControl::parseInputMask(const QString &maskFields)
     }
 
     if (delimiter == -1) {
-        m_blank = QLatin1Char(' ');
+        m_blank = u' ';
         m_inputMask = maskFields;
     } else {
         m_inputMask = maskFields.left(delimiter);
-        m_blank = (delimiter + 1 < maskFields.length()) ? maskFields[delimiter + 1] : QLatin1Char(' ');
+        m_blank = (delimiter + 1 < maskFields.size()) ? maskFields[delimiter + 1] : u' ';
     }
 
     // calculate m_maxLength / m_maskData length
     m_maxLength = 0;
     bool escaped = false;
-    for (int i=0; i<m_inputMask.length(); i++) {
+    for (int i=0; i<m_inputMask.size(); i++) {
         const auto c = m_inputMask.at(i);
         if (escaped) {
            ++m_maxLength;
@@ -1008,10 +973,8 @@ void QWidgetLineControl::parseInputMask(const QString &maskFields)
            continue;
         }
 
-        if (c != QLatin1Char('\\') && c != QLatin1Char('!') &&
-             c != QLatin1Char('<') && c != QLatin1Char('>') &&
-             c != QLatin1Char('{') && c != QLatin1Char('}') &&
-             c != QLatin1Char('[') && c != QLatin1Char(']'))
+        if (c != u'\\' && c != u'!' && c != u'<' && c != u'>' &&
+             c != u'{' && c != u'}' && c != u'[' && c != u']')
             m_maxLength++;
     }
 
@@ -1021,7 +984,7 @@ void QWidgetLineControl::parseInputMask(const QString &maskFields)
     bool s;
     bool escape = false;
     int index = 0;
-    for (int i = 0; i < m_inputMask.length(); i++) {
+    for (int i = 0; i < m_inputMask.size(); i++) {
         const auto c = m_inputMask.at(i);
         if (escape) {
             s = true;
@@ -1030,13 +993,13 @@ void QWidgetLineControl::parseInputMask(const QString &maskFields)
             m_maskData[index].caseMode = m;
             index++;
             escape = false;
-        } else if (c == QLatin1Char('<')) {
-                m = MaskInputData::Lower;
-        } else if (c == QLatin1Char('>')) {
+        } else if (c == u'<') {
+            m = MaskInputData::Lower;
+        } else if (c == u'>') {
             m = MaskInputData::Upper;
-        } else if (c == QLatin1Char('!')) {
+        } else if (c == u'!') {
             m = MaskInputData::NoCaseMode;
-        } else if (c != QLatin1Char('{') && c != QLatin1Char('}') && c != QLatin1Char('[') && c != QLatin1Char(']')) {
+        } else if (c != u'{' && c != u'}' && c != u'[' && c != u']') {
             switch (c.unicode()) {
             case 'A':
             case 'a':
@@ -1124,23 +1087,23 @@ bool QWidgetLineControl::isValidInput(QChar key, QChar mask) const
             return true;
         break;
     case '#':
-        if (key.isNumber() || key == QLatin1Char('+') || key == QLatin1Char('-') || key == m_blank)
+        if (key.isNumber() || key == u'+' || key == u'-' || key == m_blank)
             return true;
         break;
     case 'B':
-        if (key == QLatin1Char('0') || key == QLatin1Char('1'))
+        if (key == u'0' || key == u'1')
             return true;
         break;
     case 'b':
-        if (key == QLatin1Char('0') || key == QLatin1Char('1') || key == m_blank)
+        if (key == u'0' || key == u'1' || key == m_blank)
             return true;
         break;
     case 'H':
-        if (key.isNumber() || (key >= QLatin1Char('a') && key <= QLatin1Char('f')) || (key >= QLatin1Char('A') && key <= QLatin1Char('F')))
+        if (key.isNumber() || (key >= u'a' && key <= u'f') || (key >= u'A' && key <= u'F'))
             return true;
         break;
     case 'h':
-        if (key.isNumber() || (key >= QLatin1Char('a') && key <= QLatin1Char('f')) || (key >= QLatin1Char('A') && key <= QLatin1Char('F')) || key == m_blank)
+        if (key.isNumber() || (key >= u'a' && key <= u'f') || (key >= u'A' && key <= u'F') || key == m_blank)
             return true;
         break;
     default:
@@ -1170,7 +1133,7 @@ bool QWidgetLineControl::hasAcceptableInput(const QString &str) const
     if (!m_maskData)
         return true;
 
-    if (str.length() != m_maxLength)
+    if (str.size() != m_maxLength)
         return false;
 
     for (int i=0; i < m_maxLength; ++i) {
@@ -1205,47 +1168,47 @@ QString QWidgetLineControl::maskString(int pos, const QString &str, bool clear)
     QString s = QString::fromLatin1("");
     int i = pos;
     while (i < m_maxLength) {
-        if (strIndex < str.length()) {
+        if (strIndex < str.size()) {
             if (m_maskData[i].separator) {
                 s += m_maskData[i].maskChar;
-                if (str[(int)strIndex] == m_maskData[i].maskChar)
+                if (str[strIndex] == m_maskData[i].maskChar)
                     strIndex++;
                 ++i;
             } else {
-                if (isValidInput(str[(int)strIndex], m_maskData[i].maskChar)) {
+                if (isValidInput(str[strIndex], m_maskData[i].maskChar)) {
                     switch (m_maskData[i].caseMode) {
                     case MaskInputData::Upper:
-                        s += str[(int)strIndex].toUpper();
+                        s += str[strIndex].toUpper();
                         break;
                     case MaskInputData::Lower:
-                        s += str[(int)strIndex].toLower();
+                        s += str[strIndex].toLower();
                         break;
                     default:
-                        s += str[(int)strIndex];
+                        s += str[strIndex];
                     }
                     ++i;
                 } else {
                     // search for separator first
-                    int n = findInMask(i, true, true, str[(int)strIndex]);
+                    int n = findInMask(i, true, true, str[strIndex]);
                     if (n != -1) {
-                        if (str.length() != 1 || i == 0 || (i > 0 && (!m_maskData[i-1].separator || m_maskData[i-1].maskChar != str[(int)strIndex]))) {
+                        if (str.size() != 1 || i == 0 || (i > 0 && (!m_maskData[i-1].separator || m_maskData[i-1].maskChar != str[strIndex]))) {
                             s += QStringView{fill}.mid(i, n - i + 1);
                             i = n + 1; // update i to find + 1
                         }
                     } else {
                         // search for valid m_blank if not
-                        n = findInMask(i, true, false, str[(int)strIndex]);
+                        n = findInMask(i, true, false, str[strIndex]);
                         if (n != -1) {
                             s += QStringView{fill}.mid(i, n - i);
                             switch (m_maskData[n].caseMode) {
                             case MaskInputData::Upper:
-                                s += str[(int)strIndex].toUpper();
+                                s += str[strIndex].toUpper();
                                 break;
                             case MaskInputData::Lower:
-                                s += str[(int)strIndex].toLower();
+                                s += str[strIndex].toLower();
                                 break;
                             default:
-                                s += str[(int)strIndex];
+                                s += str[strIndex];
                             }
                             i = n + 1; // updates i to find + 1
                         }
@@ -1296,7 +1259,7 @@ QString QWidgetLineControl::stripString(const QString &str) const
         return str;
 
     QString s;
-    int end = qMin(m_maxLength, (int)str.length());
+    int end = qMin(m_maxLength, (int)str.size());
     for (int i = 0; i < end; ++i)
         if (m_maskData[i].separator)
             s += m_maskData[i].maskChar;
@@ -1434,8 +1397,8 @@ void QWidgetLineControl::emitCursorPositionChanged()
     if (m_cursor != m_lastCursorPos) {
         const int oldLast = m_lastCursorPos;
         m_lastCursorPos = m_cursor;
-        cursorPositionChanged(oldLast, m_cursor);
-#ifndef QT_NO_ACCESSIBILITY
+        emit cursorPositionChanged(oldLast, m_cursor);
+#if QT_CONFIG(accessibility)
         // otherwise we send a selection update which includes the cursor
         if (!hasSelectedText()) {
             QAccessibleTextCursorEvent event(accessibleObject(), m_cursor);
@@ -1483,7 +1446,7 @@ void QWidgetLineControl::complete(int key)
             return;
         int n = 0;
         if (key == Qt::Key_Up || key == Qt::Key_Down) {
-            if (textAfterSelection().length())
+            if (textAfterSelection().size())
                 return;
             QString prefix = hasSelectedText() ? textBeforeSelection()
                 : text;
@@ -1539,15 +1502,12 @@ void QWidgetLineControl::setBlinkingCursorEnabled(bool enable)
 
 void QWidgetLineControl::updateCursorBlinking()
 {
-    if (m_blinkTimer) {
-        killTimer(m_blinkTimer);
-        m_blinkTimer = 0;
-    }
+    m_blinkTimer.stop();
 
     if (m_blinkEnabled && !m_readOnly) {
-        int flashTime = QGuiApplication::styleHints()->cursorFlashTime();
-        if (flashTime >= 2)
-            m_blinkTimer = startTimer(flashTime / 2);
+        const auto flashTime = QGuiApplication::styleHints()->cursorFlashTime() * 1ms;
+        if (flashTime >= 2ms)
+            m_blinkTimer.start(flashTime / 2, this);
     }
 
     m_blinkStatus = 1;
@@ -1557,31 +1517,28 @@ void QWidgetLineControl::updateCursorBlinking()
 // This is still used by QDeclarativeTextInput in the qtquick1 repo
 void QWidgetLineControl::resetCursorBlinkTimer()
 {
-    if (!m_blinkEnabled || m_blinkTimer == 0)
+    if (!m_blinkEnabled || !m_blinkTimer.isActive())
         return;
-    killTimer(m_blinkTimer);
-    m_blinkTimer = 0;
-    int flashTime = QGuiApplication::styleHints()->cursorFlashTime();
-    if (flashTime >= 2)
-        m_blinkTimer = startTimer(flashTime / 2);
+    m_blinkTimer.stop();
+    const auto flashTime = QGuiApplication::styleHints()->cursorFlashTime() * 1ms;
+    if (flashTime >= 2ms)
+        m_blinkTimer.start(flashTime / 2, this);
     m_blinkStatus = 1;
 }
 
 void QWidgetLineControl::timerEvent(QTimerEvent *event)
 {
-    if (event->timerId() == m_blinkTimer) {
+    const auto eventId = event->id();
+    if (eventId == m_blinkTimer.id()) {
         m_blinkStatus = !m_blinkStatus;
         emit updateNeeded(inputMask().isEmpty() ? cursorRect() : QRect());
-    } else if (event->timerId() == m_deleteAllTimer) {
-        killTimer(m_deleteAllTimer);
-        m_deleteAllTimer = 0;
+    } else if (eventId == m_deleteAllTimer.id()) {
+        m_deleteAllTimer.stop();
         clear();
-    } else if (event->timerId() == m_tripleClickTimer) {
-        killTimer(m_tripleClickTimer);
-        m_tripleClickTimer = 0;
-    } else if (event->timerId() == m_passwordEchoTimer) {
-        killTimer(m_passwordEchoTimer);
-        m_passwordEchoTimer = 0;
+    } else if (eventId == m_tripleClickTimer.id()) {
+        m_tripleClickTimer.stop();
+    } else if (eventId == m_passwordEchoTimer.id()) {
+        m_passwordEchoTimer.stop();
         updateDisplayText();
     }
 }
@@ -1677,6 +1634,7 @@ void QWidgetLineControl::processKeyEvent(QKeyEvent* event)
                     setText(m_completer->currentCompletion());
                     inlineCompletionAccepted = true;
                 }
+                break;
             default:
                 break; // normal key processing
             }
@@ -1848,13 +1806,18 @@ void QWidgetLineControl::processKeyEvent(QKeyEvent* event)
     }
     else if (event == QKeySequence::DeleteEndOfWord) {
         if (!isReadOnly()) {
-            cursorWordForward(true);
-            del();
+            if (!hasSelectedText())
+                cursorWordForward(true);
+
+            if (hasSelectedText())
+                del();
         }
     }
     else if (event == QKeySequence::DeleteStartOfWord) {
         if (!isReadOnly()) {
-            cursorWordBackward(true);
+            if (!hasSelectedText())
+                cursorWordBackward(true);
+
             if (hasSelectedText())
                 del();
         }
@@ -1928,8 +1891,8 @@ void QWidgetLineControl::processKeyEvent(QKeyEvent* event)
                             updatePasswordEchoEditing(false);
 
                         emit editFocusChange(false);
-                    } else if (!m_deleteAllTimer) {
-                        m_deleteAllTimer = startTimer(750);
+                    } else if (!m_deleteAllTimer.isActive()) {
+                        m_deleteAllTimer.start(750ms, this);
                     }
                 } else {
                     unknown = true;
