@@ -6,9 +6,12 @@
 #include <QPropertyAnimation>
 #include <QSignalSpy>
 
+#include <QtCore/qbasictimer.h>
 #include <QtCore/qparallelanimationgroup.h>
 #include <QtCore/qscopeguard.h>
 
+using namespace std::chrono_literals;
+
 Q_DECLARE_METATYPE(QAbstractAnimation::State)
 
 class tst_QParallelAnimationGroup : public QObject
@@ -107,7 +110,7 @@ class UncontrolledAnimation : public QPropertyAnimation
     Q_OBJECT
 public:
     UncontrolledAnimation(QObject *target, const QByteArray &propertyName, QObject *parent = nullptr)
-        : QPropertyAnimation(target, propertyName, parent), id(0)
+        : QPropertyAnimation(target, propertyName, parent)
     {
         setDuration(250);
         setEndValue(0);
@@ -118,22 +121,21 @@ public:
 protected:
     void timerEvent(QTimerEvent *event) override
     {
-        if (event->timerId() == id)
+        if (event->id() == timer.id())
             stop();
     }
 
     void updateRunning(bool running)
     {
         if (running) {
-            id = startTimer(500);
+            timer.start(500ms, this);
         } else {
-            killTimer(id);
-            id = 0;
+            timer.stop();
         }
     }
 
 private:
-    int id;
+    QBasicTimer timer;
 };
 
 void tst_QParallelAnimationGroup::setCurrentTime()
