@@ -11,6 +11,11 @@
 #include <QtTest/QTest>
 #include <QList>
 #include <QMap>
+#include <QScopeGuard>
+
+#ifdef Q_OS_UNIX
+#  include <signal.h>
+#endif
 
 class tst_qmessagehandler : public QObject
 {
@@ -38,6 +43,10 @@ private slots:
     void qMessagePattern();
     void setMessagePattern();
 
+    void fatalWarnings_data();
+    void fatalWarnings();
+    void fatalWarningsRaceCondition();
+
     void formatLogMessage_data();
     void formatLogMessage();
 
@@ -838,7 +847,7 @@ void tst_qmessagehandler::qMessagePattern()
     QSKIP("This test requires QProcess support");
 #else
 #ifdef Q_OS_ANDROID
-    QSKIP("This test crashes on Android");
+    QSKIP("This test is disabled on Android");
 #endif
     QFETCH(QString, pattern);
     QFETCH(bool, valid);
@@ -884,7 +893,7 @@ void tst_qmessagehandler::setMessagePattern()
     QSKIP("This test requires QProcess support");
 #else
 #ifdef Q_OS_ANDROID
-    QSKIP("This test crashes on Android");
+    QSKIP("This test is disabled on Android");
 #endif
 
     //
@@ -917,6 +926,104 @@ void tst_qmessagehandler::setMessagePattern()
 #endif // QT_CONFIG(process)
 }
 
+void tst_qmessagehandler::fatalWarnings_data()
+{
+    QTest::addColumn<QString>("varName");
+    QTest::addColumn<QString>("varValue");
+    QTest::addColumn<QByteArray>("presentOutput");
+    QTest::addColumn<QByteArray>("absentOutput");
+
+    QTest::newRow("QT_FATAL_WARNINGS=1")
+            << "QT_FATAL_WARNINGS" << "1"
+            << QByteArray("[warning] qWarning") << QByteArray("[critical] qCritical");
+    QTest::newRow("QT_FATAL_CRITICALS=1")
+            << "QT_FATAL_CRITICALS" << "1"
+            << QByteArray("[critical] qCritical") << QByteArray("[warning] qDebug with category");
+    QTest::newRow("QT_FATAL_WARNINGS=2")
+            << "QT_FATAL_WARNINGS" << "2"
+            << QByteArray("[warning] qDebug with category") << QByteArray("[debug] qDebug2");
+
+#if !QT_CONFIG(process)
+    QSKIP("This test requires QProcess support");
+#endif
+}
+
+void tst_qmessagehandler::fatalWarnings()
+{
+#ifdef Q_OS_ANDROID
+    QSKIP("This test is disabled on Android");
+#endif
+#if QT_CONFIG(process)
+    QFETCH(QString, varName);
+    QFETCH(QString, varValue);
+    QFETCH(QByteArray, presentOutput);
+    QFETCH(QByteArray, absentOutput);
+
+    QProcess process;
+    const QString appExe(backtraceHelperPath());
+
+    //
+    // test QT_FATAL_WARNINGS / QT_FATAL_CRITICALS
+    //
+    QProcessEnvironment environment = m_baseEnvironment;
+    environment.insert(varName, varValue);
+    process.setProcessEnvironment(environment);
+
+    process.start(appExe, {}, QIODevice::Text | QIODevice::ReadWrite);
+    QVERIFY2(process.waitForStarted(), qPrintable(
+        QString::fromLatin1("Could not start %1: %2").arg(appExe, process.errorString())));
+    process.waitForFinished();
+
+    QCOMPARE(process.exitStatus(), QProcess::CrashExit);
+#  ifdef Q_OS_UNIX
+    QCOMPARE(process.exitCode(), SIGABRT);
+#  elif defined(Q_CC_MSVC)
+    // __fastfail produces STATUS_STACK_BUFFER_OVERRUN
+    QCOMPARE(process.exitCode(), int(0xc0000409));
+#  else
+    // RaiseFastFail produces STATUS_FAIL_FAST_EXCEPTION
+    QCOMPARE(process.exitCode(), int(0xc0000602));
+#  endif
+
+    QList<QByteArray> lines = process.readAllStandardError().split('\n');
+    auto outputOnError = qScopeGuard([&lines] {
+        qDebug("Output was:\n");
+        for (const QByteArray &line : lines)
+            qDebug() << line;
+    });
+
+    QVERIFY2(lines.contains(presentOutput), presentOutput);
+    QVERIFY2(!lines.contains(absentOutput), absentOutput);
+    outputOnError.dismiss();
+#endif // QT_CONFIG(process)
+}
+
+void tst_qmessagehandler::fatalWarningsRaceCondition()
+{
+#if !QT_CONFIG(process)
+    QSKIP("This test requires QProcess support");
+#elif !defined(Q_OS_UNIX)
+    QSKIP("This test only works on Unix systems (need a SIGABRT handler)");
+#elif defined(Q_OS_ANDROID)
+    QSKIP("No helper for this");
+#else
+    QProcess process;
+    const QString appExe(QCoreApplication::applicationDirPath() + "/qlogging_race_helper");
+
+    QProcessEnvironment environment = m_baseEnvironment;
+    environment.insert("QT_FATAL_WARNINGS", "1");
+    process.setProcessEnvironment(environment);
+
+    process.start(appExe, {}, QIODevice::Text | QIODevice::ReadWrite);
+    QVERIFY2(process.waitForStarted(), qPrintable(
+        QString::fromLatin1("Could not start %1: %2").arg(appExe, process.errorString())));
+    process.waitForFinished();
+
+    QCOMPARE(process.exitStatus(), QProcess::CrashExit);
+    QCOMPARE(process.exitCode(), SIGABRT);
+#endif
+}
+
 Q_DECLARE_METATYPE(QtMsgType)
 
 void tst_qmessagehandler::formatLogMessage_data()
@@ -980,8 +1087,8 @@ void tst_qmessagehandler::formatLogMessage()
 QString tst_qmessagehandler::backtraceHelperPath()
 {
 #ifdef Q_OS_ANDROID
-    QString appExe(QCoreApplication::applicationDirPath()
-                   + QLatin1String("/lib" BACKTRACE_HELPER_NAME ".so"));
+    qFatal("Launching the helper (which does exist) produces 'No such file or directory'.");
+    QString appExe;
 #else
     QString appExe(QCoreApplication::applicationDirPath()
                    + QLatin1String("/" BACKTRACE_HELPER_NAME));
