@@ -1,5 +1,5 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QTest>
 
@@ -9,13 +9,16 @@
 #include <qeventloop.h>
 #include <private/qeventloop_p.h>
 #if defined(Q_OS_UNIX)
-  #include <private/qeventdispatcher_unix_p.h>
+  #if !defined(Q_OS_WASM)
+    #include <private/qeventdispatcher_unix_p.h>
+  #endif
   #include <QtCore/private/qcore_unix_p.h>
   #if defined(HAVE_GLIB)
     #include <private/qeventdispatcher_glib_p.h>
   #endif
 #endif
 #include <qmutex.h>
+#include <QtCore/qscopeguard.h>
 #include <qthread.h>
 #include <qtimer.h>
 #include <qwaitcondition.h>
@@ -23,6 +26,12 @@
 #include <QTcpSocket>
 #include <QSignalSpy>
 
+#include <atomic>
+#include <optional>
+#include <thread>
+
+using namespace std::chrono_literals;
+
 class EventLoopExiter : public QObject
 {
     Q_OBJECT
@@ -150,6 +159,7 @@ private slots:
 #endif
     void processEventsExcludeTimers();
     void deliverInDefinedOrder();
+    void canUseQThreadQuitToExitEventLoopInStdThread();
 
     // keep this test last:
     void nestedLoops();
@@ -414,8 +424,8 @@ public slots:
         dataSent = serverSocket->waitForBytesWritten(-1);
 
         if (dataSent) {
-            pollfd pfd = qt_make_pollfd(socket->socketDescriptor(), POLLIN);
-            dataReadable = (1 == qt_safe_poll(&pfd, 1, nullptr));
+            pollfd pfd = qt_make_pollfd(int(socket->socketDescriptor()), POLLIN);
+            dataReadable = (1 == qt_safe_poll(&pfd, 1, QDeadlineTimer::Forever));
         }
 
         if (!dataReadable) {
@@ -473,34 +483,34 @@ void tst_QEventLoop::processEventsExcludeSocket()
 class TimerReceiver : public QObject
 {
 public:
-    int gotTimerEvent;
+    Qt::TimerId gotTimerEvent;
 
     TimerReceiver()
-        : QObject(), gotTimerEvent(-1)
+        : QObject(), gotTimerEvent(Qt::TimerId::Invalid)
     { }
 
     void timerEvent(QTimerEvent *event) override
     {
-        gotTimerEvent = event->timerId();
+        gotTimerEvent = event->id();
     }
 };
 
 void tst_QEventLoop::processEventsExcludeTimers()
 {
     TimerReceiver timerReceiver;
-    int timerId = timerReceiver.startTimer(0);
+    Qt::TimerId timerId = Qt::TimerId{timerReceiver.startTimer(0ns)};
 
     QEventLoop eventLoop;
 
     // normal process events will send timers
     eventLoop.processEvents();
     QCOMPARE(timerReceiver.gotTimerEvent, timerId);
-    timerReceiver.gotTimerEvent = -1;
+    timerReceiver.gotTimerEvent = Qt::TimerId::Invalid;
 
     // but not if we exclude timers
     eventLoop.processEvents(QEventLoop::X11ExcludeTimers);
 
-#if defined(Q_OS_UNIX)
+#if defined(Q_OS_UNIX) && !defined(Q_OS_WASM)
     QAbstractEventDispatcher *eventDispatcher = QCoreApplication::eventDispatcher();
     if (!qobject_cast<QEventDispatcherUNIX *>(eventDispatcher)
   #if defined(HAVE_GLIB)
@@ -510,13 +520,12 @@ void tst_QEventLoop::processEventsExcludeTimers()
 #endif
         QEXPECT_FAIL("", "X11ExcludeTimers only supported in the UNIX/Glib dispatchers", Continue);
 
-    QCOMPARE(timerReceiver.gotTimerEvent, -1);
-    timerReceiver.gotTimerEvent = -1;
+    QCOMPARE(timerReceiver.gotTimerEvent, Qt::TimerId::Invalid);
 
     // resume timer processing
     eventLoop.processEvents();
     QCOMPARE(timerReceiver.gotTimerEvent, timerId);
-    timerReceiver.gotTimerEvent = -1;
+    timerReceiver.gotTimerEvent = Qt::TimerId::Invalid;
 }
 
 namespace DeliverInDefinedOrder {
@@ -588,6 +597,46 @@ void tst_QEventLoop::deliverInDefinedOrder()
 
 }
 
+void tst_QEventLoop::canUseQThreadQuitToExitEventLoopInStdThread()
+{
+#ifdef Q_CC_MINGW
+    QSKIP("Disabled for MINGW, due to a runtime bug (QTBUG-131892).");
+#endif
+    std::atomic<QThread*> adopted = nullptr;
+    std::atomic<bool> called = false;
+    std::atomic<bool> timedOut = false;
+
+    auto t = std::thread([&] {
+            adopted.store(QThread::currentThread());
+#ifdef __cpp_lib_atomic_wait
+            adopted.notify_one();
+#endif
+            QEventLoop loop;
+            // fallback in case the invokeMethod() below fails
+            QTimer::singleShot(1s, &loop, [&] { timedOut.store(true); loop.quit(); });
+            loop.exec();
+        });
+
+    QObject obj;
+    std::optional joiner = qScopeGuard([&] { t.join(); }); // CTAD
+#ifdef __cpp_lib_atomic_wait
+    adopted.wait(nullptr); // no timed version exists :(
+    QVERIFY(adopted.load());
+#else
+    QVERIFY(QTest::qWaitFor([&] { return adopted.load(); }));
+#endif
+    QVERIFY(obj.moveToThread(adopted.load()));
+    QCOMPARE(obj.thread(), adopted.load());
+    // The lambda will only be executed when `adopted` has an event loop running:
+    QVERIFY(QMetaObject::invokeMethod(&obj, [&] {
+                                                called.store(true);
+                                                QThread::currentThread()->quit();
+                                            }, Qt::QueuedConnection));
+    joiner.reset(); // joins
+    QVERIFY(!timedOut.load());
+    QVERIFY(called.load());
+}
+
 class JobObject : public QObject
 {
     Q_OBJECT
