@@ -23,6 +23,7 @@
 #include <QSignalSpy>
 #include <QtTest/private/qpropertytesthelper_p.h>
 
+#include <QtCore/private/qsingleshottimer_p.h>
 #include <qtimer.h>
 #include <qthread.h>
 #include <qelapsedtimer.h>
@@ -82,8 +83,13 @@ private slots:
     void singleShotToFunctors();
     void singleShot_chrono();
     void singleShot_static();
+    void singleShotDestructionBeforeEventDispatcher();
     void crossThreadSingleShotToFunctor_data();
     void crossThreadSingleShotToFunctor();
+#ifdef QT_BUILD_INTERNAL
+    void crossThreadSingleShotDestruction_data();
+    void crossThreadSingleShotDestruction();
+#endif
     void timerOrder();
     void timerOrder_data();
     void timerOrderBackgroundThread();
@@ -458,10 +464,10 @@ void tst_QTimer::basic_chrono()
 
 void tst_QTimer::livelock_data()
 {
-    QTest::addColumn<int>("interval");
-    QTest::newRow("zero timer") << 0;
-    QTest::newRow("non-zero timer") << 1;
-    QTest::newRow("longer than sleep") << 20;
+    QTest::addColumn<std::chrono::milliseconds>("interval");
+    QTest::newRow("zero timer") << 0ms;
+    QTest::newRow("non-zero timer") << 1ms;
+    QTest::newRow("longer than sleep") << 20ms;
 }
 
 /*!
@@ -473,14 +479,14 @@ void tst_QTimer::livelock_data()
 class LiveLockTester : public QObject
 {
 public:
-    LiveLockTester(int i)
+    LiveLockTester(std::chrono::milliseconds i)
         : interval(i),
           timeoutsForFirst(0), timeoutsForExtra(0), timeoutsForSecond(0),
           postEventAtRightTime(false)
     {
-        firstTimerId = startTimer(interval);
-        extraTimerId = startTimer(interval + 80);
-        secondTimerId = -1; // started later
+        firstTimerId = Qt::TimerId{startTimer(interval)};
+        extraTimerId = Qt::TimerId{startTimer(interval + 80ms)};
+        secondTimerId = Qt::TimerId::Invalid; // started later
     }
 
     bool event(QEvent *e) override
@@ -496,28 +502,28 @@ public:
 
     void timerEvent(QTimerEvent *te) override
     {
-        if (te->timerId() == firstTimerId) {
+        if (te->id() == firstTimerId) {
             if (++timeoutsForFirst == 1) {
                 killTimer(extraTimerId);
-                extraTimerId = -1;
+                extraTimerId = Qt::TimerId::Invalid;
                 QCoreApplication::postEvent(this, new QEvent(static_cast<QEvent::Type>(4002)));
-                secondTimerId = startTimer(interval);
+                secondTimerId = Qt::TimerId{startTimer(interval)};
             }
-        } else if (te->timerId() == secondTimerId) {
+        } else if (te->id() == secondTimerId) {
             ++timeoutsForSecond;
-        } else if (te->timerId() == extraTimerId) {
+        } else if (te->id() == extraTimerId) {
             ++timeoutsForExtra;
         }
 
         // sleep for 2ms
         QTest::qSleep(2);
-        killTimer(te->timerId());
+        killTimer(te->id());
     }
 
-    const int interval;
-    int firstTimerId;
-    int secondTimerId;
-    int extraTimerId;
+    const std::chrono::milliseconds interval;
+    Qt::TimerId firstTimerId;
+    Qt::TimerId secondTimerId;
+    Qt::TimerId extraTimerId;
     int timeoutsForFirst;
     int timeoutsForExtra;
     int timeoutsForSecond;
@@ -533,7 +539,7 @@ void tst_QTimer::livelock()
       events (since new posted events are not sent until the next
       iteration of the eventloop either).
     */
-    QFETCH(int, interval);
+    QFETCH(std::chrono::milliseconds, interval);
     LiveLockTester tester(interval);
     QTest::qWait(180); // we have to use wait here, since we're testing timers with a non-zero timeout
     QTRY_COMPARE(tester.timeoutsForFirst, 1);
@@ -1138,7 +1144,7 @@ DontBlockEvents::DontBlockEvents()
 
 void DontBlockEvents::timerEvent(QTimerEvent* event)
 {
-    if (event->timerId() == m_timer.timerId()) {
+    if (event->id() == m_timer.id()) {
         QMetaObject::invokeMethod(this, "paintEvent", Qt::QueuedConnection);
         m_timer.start(0, this);
         count++;
@@ -1186,6 +1192,22 @@ void tst_QTimer::postedEventsShouldNotStarveTimers()
     QTRY_VERIFY_WITH_TIMEOUT(timeoutSpy.size() > 5, 100);
 }
 
+void tst_QTimer::singleShotDestructionBeforeEventDispatcher()
+{
+    // This makes sure the QSingleShotTimer doesn't cause a crash when the
+    // event dispatcher is deleted. As of the time of this test's writing, the
+    // QSST was parented to the dispatcher, so if it hadn't yet expired, it
+    // would be deleted by the QObject destructor, which is too late to
+    // unregister the timer.
+
+    auto thr = QThread::create([] {
+        QObject o;
+        QTimer::singleShot(300s, &o, [] {});
+    });
+    thr->start();
+    thr->wait();
+}
+
 struct DummyFunctor {
     static QThread *callThread;
     void operator()() {
@@ -1236,6 +1258,91 @@ void tst_QTimer::crossThreadSingleShotToFunctor()
     QCOMPARE(DummyFunctor::callThread, &t);
 }
 
+#ifdef QT_BUILD_INTERNAL
+class SingleShotReceiver : public QObject
+{
+    Q_OBJECT
+public:
+    SingleShotReceiver() = default;
+
+    bool timeout = false;
+
+public slots:
+    void timerFired() {
+        timeout = true;
+        thread()->quit();
+    }
+};
+
+void tst_QTimer::crossThreadSingleShotDestruction_data()
+{
+    QTest::addColumn<std::chrono::milliseconds>("timeout");
+    QTest::addColumn<std::chrono::milliseconds>("threadWaitTime");
+
+    // No point in testing zero timeouts, we don't create a QSingleShotTimer
+    // for those anyway.
+    QTest::addRow("1ms") << 1ms << 10ms;
+    QTest::addRow("1s") << 1000ms << 10ms;
+}
+
+void tst_QTimer::crossThreadSingleShotDestruction()
+{
+    QFETCH(std::chrono::milliseconds, timeout);
+    QFETCH(std::chrono::milliseconds, threadWaitTime);
+
+    static bool deadTimerDestroyed;
+    class SingleShotTimer : public QSingleShotTimer
+    {
+    public:
+        using QSingleShotTimer::QSingleShotTimer;
+        ~SingleShotTimer() override { deadTimerDestroyed = true; }
+    };
+
+    {
+        QThread t;
+        std::unique_ptr<SingleShotReceiver> receiver(new SingleShotReceiver());
+        receiver->moveToThread(&t);
+
+        // create the timer before the thread is running. Otherwise, the
+        // assignment to QPointer might happen after the timer already fired and
+        // destroyed itself, leaving QPointer dangling.
+        QPointer<QSingleShotTimer> timer = new QSingleShotTimer(timeout, Qt::CoarseTimer,
+                                                                receiver.get(), SLOT(timerFired()));
+        t.start();
+
+        const bool threadQuit = t.wait(threadWaitTime);
+
+        // If we waited long enough, then the timer should have fired, quit
+        // the thread, and self-destructed; otherwise the wait will have timed
+        // out and the timer is still alive.
+        QCOMPARE(threadQuit, receiver->timeout);
+        QCOMPARE(threadQuit, !timer);
+        deadTimerDestroyed = threadQuit;
+
+        if (!threadQuit) {
+            t.quit();
+
+            // now that the thread is definitely dead, start a new timer that is
+            // never even going to get started. It should still be destroyed, as
+            // it will be (temporarily) owned by the event that we post to start
+            // the timer in the target thread.
+            // Note: we can't use QPointer here, as the thread is already
+            // running.
+            (void)new SingleShotTimer(timeout, Qt::CoarseTimer, receiver.get(), SLOT(timerFired()));
+            QVERIFY(t.wait());
+            // the timer should never have fired
+            QVERIFY(!receiver->timeout);
+        }
+
+        // the timer objects should be destroyed by now in all cases
+        QVERIFY(!timer);
+    }
+
+    return; // Events posted to a thread after event loop exit are leaking.
+    QVERIFY(deadTimerDestroyed);
+}
+#endif
+
 void tst_QTimer::callOnTimeout()
 {
     QTimer timer;
@@ -1542,6 +1649,10 @@ void tst_QTimer::initMain()
 void tst_QTimer::cleanupTestCase()
 {
     delete s_staticSingleShotUser;
+
+    // Same as singleShotDestructionBeforeEventDispatcher() above, but for the
+    // main thread.
+    QTimer::singleShot(300s, this, [] {});
 }
 
 void tst_QTimer::singleShot_static()
