@@ -1497,9 +1497,18 @@ template <typename ByteArray> void tst_QByteArrayApiSymmetry::toFloat() const
     QCOMPARE(ByteArray().toFloat(&ok), 0.0f);
     QVERIFY(!ok);
 
+    ok = true;
     QCOMPARE(ByteArray("").toFloat(&ok), 0.0f);
     QVERIFY(!ok);
 
+    ok = true;
+    QCOMPARE(ByteArray(" ").toFloat(&ok), 0.0f);
+    QVERIFY(!ok);
+
+    ok = true;
+    QCOMPARE(ByteArray("  ").toFloat(&ok), 0.0f);
+    QVERIFY(!ok);
+
     // NB: floats < 1e-6 are zero as far as QCOMPARE() is concerned !
     const char data[] = "0.0000931322574615478515625";
     const float expectedValue = 9.31322574615478515625e-5f;
@@ -1523,6 +1532,8 @@ void tst_QByteArrayApiSymmetry::toDouble_data() const
 
     QTest::newRow("null") << QByteArray() << 0.0 << false;
     QTest::newRow("empty") << QByteArray("") << 0.0 << false;
+    QTest::newRow("space-only") << QByteArray(" ") << 0.0 << false;
+    QTest::newRow("spaces-only") << QByteArray("  ") << 0.0 << false;
 
     QTest::newRow("decimal") << QByteArray("1.2345") << 1.2345 << true;
     QTest::newRow("exponent lowercase") << QByteArray("1.2345e+01") << 12.345 << true;
