@@ -1,10 +1,12 @@
 // Copyright (C) 2018 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com, author Sérgio Martins <sergio.martins@kdab.com>
 // Copyright (C) 2020 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QTest>
 #include <QtCore/QScopeGuard>
 
+#include <optional>
+
 /*!
  \class tst_QScopeGuard
  \internal
@@ -20,6 +22,7 @@ private Q_SLOTS:
     void construction();
     void constructionFromLvalue();
     void constructionFromRvalue();
+    void optionalGuard();
     void leavingScope();
     void exceptions();
 };
@@ -117,6 +120,24 @@ void tst_QScopeGuard::constructionFromRvalue()
     QCOMPARE(Callable::moved, 1);
 }
 
+void tst_QScopeGuard::optionalGuard()
+{
+    int i = 0;
+    auto lambda = [&] { ++i; };
+    std::optional sg = false ? std::optional{qScopeGuard(lambda)} : std::nullopt;
+    QVERIFY(!sg);
+    QCOMPARE(i, 0);
+    sg.emplace(qScopeGuard(lambda));
+    QVERIFY(sg);
+    sg->dismiss();
+    sg.reset();
+    QCOMPARE(i, 0);
+    sg.emplace(qScopeGuard(lambda));
+    QCOMPARE(i, 0);
+    sg.reset();
+    QCOMPARE(i, 1);
+}
+
 void tst_QScopeGuard::leavingScope()
 {
     auto cleanup = qScopeGuard([] { s_globalState++; QCOMPARE(s_globalState, 3); });
