@@ -1,5 +1,5 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 
 #include <QTest>
@@ -473,6 +473,7 @@ void tst_QKeySequence::toStringFromKeycode_data()
     QTest::newRow("A") << QKeySequence(Qt::Key_A) << "A";
     QTest::newRow("-1") << QKeySequence(-1) << "";
     QTest::newRow("Unknown") << QKeySequence(Qt::Key_unknown) << "";
+    QTest::newRow("Ctrl+Unknown") << QKeySequence(Qt::ControlModifier | Qt::Key_unknown) << "";
     QTest::newRow("Ctrl+Num+Ins") << QKeySequence(Qt::ControlModifier | Qt::KeypadModifier | Qt::Key_Insert) << "Ctrl+Num+Ins";
     QTest::newRow("Ctrl+Num+Del") << QKeySequence(Qt::ControlModifier | Qt::KeypadModifier | Qt::Key_Delete) << "Ctrl+Num+Del";
     QTest::newRow("Ctrl+Alt+Num+Del") << QKeySequence(Qt::ControlModifier | Qt::AltModifier | Qt::KeypadModifier | Qt::Key_Delete) << "Ctrl+Alt+Num+Del";
@@ -532,6 +533,20 @@ void tst_QKeySequence::parseString_data()
     QTest::newRow("Meta+A") << "Meta+a" <<  QKeySequence(Qt::META | Qt::Key_A);
     QTest::newRow("mEtA+A") << "mEtA+a" <<  QKeySequence(Qt::META | Qt::Key_A);
     QTest::newRow("Ctrl++") << "Ctrl++" << QKeySequence(Qt::CTRL | Qt::Key_Plus);
+    QTest::newRow("+") << "+" << QKeySequence(Qt::Key_Plus);
+
+    // Tolerance for spaces
+    QTest::newRow("Ctrl_+_Del") << "Ctrl + Del" << QKeySequence(Qt::CTRL | Qt::Key_Delete);
+    QTest::newRow("Ctrl+Del_") << "Ctrl+Del " << QKeySequence(Qt::CTRL | Qt::Key_Delete);
+    QTest::newRow("Ctrl_+_Del_") << "Ctrl + Del " << QKeySequence(Qt::CTRL | Qt::Key_Delete);
+    QTest::newRow("space") << " " << QKeySequence(Qt::Key_Space);
+    QTest::newRow("Ctrl+space") << "Ctrl+ " << QKeySequence(Qt::CTRL | Qt::Key_Space);
+    QTest::newRow("Ctrl_++") << "Ctrl ++" << QKeySequence(Qt::CTRL | Qt::Key_Plus);
+    QTest::newRow("Ctrl_+_+") << "Ctrl + +" << QKeySequence(Qt::CTRL | Qt::Key_Plus);
+    QTest::newRow("Ctrl_+_+_") << "Ctrl + + " << QKeySequence(Qt::CTRL | Qt::Key_Plus);
+    QTest::newRow("+_") << "+ " << QKeySequence(Qt::Key_Plus);
+    QTest::newRow("_+") << " +" << QKeySequence(Qt::Key_Plus);
+    QTest::newRow("_+_") << " + " << QKeySequence(Qt::Key_Plus);
 
     // Invalid modifiers
     QTest::newRow("Win+A") << "Win+a" <<  QKeySequence(Qt::Key_unknown);
@@ -546,7 +561,9 @@ void tst_QKeySequence::parseString_data()
     QTest::newRow("4+3=2") << "4+3=2" <<  QKeySequence(Qt::Key_unknown);
     QTest::newRow("Alabama") << "Alabama" << QKeySequence(Qt::Key_unknown);
     QTest::newRow("Simon+G") << "Simon+G" << QKeySequence(Qt::Key_unknown);
-    QTest::newRow("Shift+++2") << "Shift+++2" <<  QKeySequence(Qt::Key_unknown);
+    QTest::newRow("Shift+++2") << "Shift+++2" << QKeySequence(Qt::Key_unknown);
+    QTest::newRow("Ctrl+D_el") << "Ctrl+D el" << QKeySequence(Qt::Key_unknown);
+    QTest::newRow("Ct_rl+D_el") << "Ct rl+D el" << QKeySequence(Qt::Key_unknown);
 
     // Wrong order
     QTest::newRow("A+Meta") << "a+Meta" <<  QKeySequence(Qt::Key_unknown);
@@ -558,11 +575,8 @@ void tst_QKeySequence::parseString_data()
     //QTest::newRow("Ctrl") << "Ctrl" << QKeySequence(Qt::CTRL);
     //QTest::newRow("Shift") << "Shift" << QKeySequence(Qt::SHIFT);
 
-    // Only Keys
-    QTest::newRow("a") << "a" << QKeySequence(Qt::Key_A);
-    QTest::newRow("A") << "A" << QKeySequence(Qt::Key_A);
-
     // Incomplete
+    QTest::newRow("Ctrl+") << "Ctrl+" << QKeySequence(Qt::Key_unknown);
     QTest::newRow("Meta+Shift+") << "Meta+Shift+" << QKeySequence(Qt::Key_unknown);
 }
 
