@@ -1,5 +1,5 @@
 // Copyright (C) 2020 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 
 #include <QTest>
@@ -73,7 +73,7 @@ private slots:
     void toCmykNonDestructive();
 
     void toHsl_data();
-    void toHsl();;
+    void toHsl();
     void toHslNonDestructive();
 
     void convertTo();
