@@ -129,6 +129,7 @@ private slots:
     void min_maximumWidth();
     void negativeLineWidth();
     void embeddedImageLineHeight();
+    void unmatchedShapedSubstring();
     void maximumLayoutWidthInWrappedLayout();
 
 private:
@@ -2811,6 +2812,31 @@ void tst_QTextLayout::embeddedImageLineHeight()
     }
 }
 
+void tst_QTextLayout::unmatchedShapedSubstring()
+{
+    QString s;
+    s += QChar(9977);
+    s += QChar(65039);
+    s += QChar(8205);
+    s += QChar(9794);
+    s += QChar(65039);
+
+    QTextLayout lout;
+
+    QTextOption opt;
+    opt.setFlags(QTextOption::DisableEmojiParsing);
+    lout.setTextOption(opt);
+
+    // Note: Shaping of this string would previously assert on some platforms
+    lout.setText(s);
+    lout.beginLayout();
+    lout.createLine();
+    lout.endLayout();
+
+    QList<QGlyphRun> glyphRuns = lout.glyphRuns();
+    QVERIFY(glyphRuns.size() > 0);
+}
+
 void tst_QTextLayout::maximumLayoutWidthInWrappedLayout()
 {
     QString s = QString::fromUtf8("Lorem ipsum dolor sit amet, consectetur adipiscing elit.\n"
