@@ -1,10 +1,11 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // Copyright (C) 2016 Intel Corporation.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 
 #include <QTest>
 #include <QSignalSpy>
+#include <QtTest/private/qpropertytesthelper_p.h>
 #if QT_CONFIG(process)
 #include <QProcess>
 #endif
@@ -123,6 +124,7 @@ private slots:
     void verifyListenWithDescriptor_data();
 
     void serverBindingsAndProperties();
+    void socketBindings();
 
 protected slots:
     void socketClosedSlot();
@@ -300,8 +302,8 @@ void tst_QLocalSocket::listen_data()
     QTest::addColumn<bool>("canListen");
     QTest::addColumn<bool>("close");
     QTest::newRow("null") << QString() << false << false;
-    QTest::newRow("tst_localsocket") << "tst_localsocket" << true << true;
-    QTest::newRow("tst_localsocket") << "tst_localsocket" << true << false;
+    QTest::newRow("tst_localsocket,close") << "tst_localsocket" << true << true;
+    QTest::newRow("tst_localsocket,no-close") << "tst_localsocket" << true << false;
 }
 
 // start a server that listens, but don't connect a socket, make sure everything is in order
@@ -1216,7 +1218,7 @@ private:
 void tst_QLocalSocket::processConnection()
 {
 #if !QT_CONFIG(process)
-    QSKIP("No qprocess support", SkipAll);
+    QSKIP("No qprocess support");
 #else
 
 #ifdef Q_OS_WIN
@@ -1662,6 +1664,9 @@ void tst_QLocalSocket::asyncDisconnectNotify()
 void tst_QLocalSocket::verifySocketOptions_data()
 {
 #ifdef Q_OS_LINUX
+    if (::geteuid() == 0)
+        QSKIP("Running this test as root doesn't make sense");
+
     QTest::addColumn<QString>("service");
     QTest::addColumn<QLocalServer::SocketOption>("opts");
     QTest::addColumn<QFile::Permissions>("perms");
@@ -1707,7 +1712,7 @@ void tst_QLocalSocket::verifySocketOptions()
 
 void tst_QLocalSocket::verifyListenWithDescriptor()
 {
-#ifdef Q_OS_UNIX
+#if defined(Q_OS_UNIX) && !defined(Q_OS_VXWORKS)
     QFETCH(QString, path);
     QFETCH(bool, abstract);
     QFETCH(bool, bound);
@@ -1807,14 +1812,18 @@ void tst_QLocalSocket::serverBindingsAndProperties()
 {
     CrashSafeLocalServer server;
 
-    QProperty<QLocalServer::SocketOptions> sockOpts;
-    server.bindableSocketOptions().setBinding(Qt::makePropertyBinding(sockOpts));
-    sockOpts = QLocalServer::GroupAccessOption | QLocalServer::UserAccessOption;
-    QCOMPARE(server.socketOptions(), sockOpts.value());
+    QTestPrivate::testReadWritePropertyBasics(
+            server, QLocalServer::SocketOptions{QLocalServer::GroupAccessOption},
+            QLocalServer::SocketOptions{QLocalServer::OtherAccessOption}, "socketOptions");
+}
 
-    sockOpts.setBinding(server.bindableSocketOptions().makeBinding());
-    server.setSocketOptions(QLocalServer::OtherAccessOption);
-    QCOMPARE(sockOpts.value(), QLocalServer::OtherAccessOption);
+void tst_QLocalSocket::socketBindings()
+{
+    QLocalSocket socket;
+
+    QTestPrivate::testReadWritePropertyBasics(
+            socket, QLocalSocket::SocketOptions{QLocalSocket::AbstractNamespaceOption},
+            QLocalSocket::SocketOptions{QLocalSocket::NoOptions}, "socketOptions");
 }
 
 QTEST_MAIN(tst_QLocalSocket)
