@@ -1,30 +1,18 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
+#include <QtNetwork/qtcpserver.h>
+#include <QtNetwork/qtcpsocket.h>
+#include <QtNetwork/qhostaddress.h>
+
+#include <QtCore/qcoreapplication.h>
 
-#include <QtCore>
-#include <QtNetwork>
-#if defined(Q_OS_WIN) && defined(Q_CC_MSVC)
-#  include <crtdbg.h>
-#endif
 #ifdef Q_OS_UNIX
-#  include <sys/resource.h>
 #  include <unistd.h>
 #endif
 
 int main(int argc, char *argv[])
 {
-#if defined(Q_OS_WIN) && defined(Q_CC_MSVC)
-    // Windows: Suppress crash notification dialog.
-    _CrtSetReportMode(_CRT_ERROR, _CRTDBG_MODE_DEBUG);
-#elif defined(RLIMIT_CORE)
-    // Unix: set our core dump limit to zero to request no dialogs.
-    if (struct rlimit rlim; getrlimit(RLIMIT_CORE, &rlim) == 0) {
-        rlim.rlim_cur = 0;
-        setrlimit(RLIMIT_CORE, &rlim);
-    }
-#endif
-
     QCoreApplication app(argc, argv);
     if (argc < 1) {
         fprintf(stderr, "Need a port number\n");
@@ -39,7 +27,7 @@ int main(int argc, char *argv[])
             // let's see if we can find the process that would be holding this
             // still open
 #ifdef Q_OS_LINUX
-            static const char *ss_args[] = {
+            static const char *const ss_args[] = {
                 "ss", "-nap", "sport", "=", argv[1], nullptr
             };
             dup2(STDERR_FILENO, STDOUT_FILENO);
