@@ -1,10 +1,11 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QApplication>
 #include <QtWidgets>
 #include <QTest>
 #include <QtCore/qcoreapplication.h>
+#include <QtCore/qregularexpression.h>
 
 // some versions of CALayer.h use 'slots' as an identifier
 #define QT_NO_KEYWORDS
@@ -62,13 +63,16 @@ QDebug operator<<(QDebug dbg, AXErrorTag err)
 @interface TestAXObject : NSObject
 {
     AXUIElementRef reference;
+    bool axError;
 }
     @property (readonly) NSString *role;
+    @property (readonly) NSString *roleDescription;
     @property (readonly) NSString *title;
     @property (readonly) NSString *description;
     @property (readonly) NSString *value;
     @property (readonly) CGRect rect;
     @property (readonly) NSArray *actions;
+    @property (readonly) bool valid;
 @end
 
 @implementation TestAXObject
@@ -77,11 +81,18 @@ QDebug operator<<(QDebug dbg, AXErrorTag err)
 
     if ((self = [super init])) {
         reference = ref;
+        if (!reference) {
+            qCritical() << "Null reference!";
+            axError = true;
+        } else {
+            axError = false;
+        }
     }
     return self;
 }
 
 - (AXUIElementRef) ref { return reference; }
+- (bool)errorOccurred { return axError; }
 - (void) print {
     NSLog(@"Accessible Object role: '%@', title: '%@', description: '%@', value: '%@', rect: '%@'", self.role, self.title, self.description, self.value, NSStringFromRect(NSRectFromCGRect(self.rect)));
     NSLog(@"    Children: %ld", [[self childList] count]);
@@ -101,14 +112,63 @@ QDebug operator<<(QDebug dbg, AXErrorTag err)
 - (NSArray*) childList
 {
     NSArray *list;
-    AXUIElementCopyAttributeValues(
-                reference,
-                kAXChildrenAttribute,
-                0, 100, /*min, max*/
-                (CFArrayRef *) &list);
+    AXError err;
+
+    if (kAXErrorSuccess != (err = AXUIElementCopyAttributeValues(reference, kAXChildrenAttribute,
+                                                                 0, 100, /*min, max*/
+                                                                 (CFArrayRef *) &list))) {
+        axError = true;
+        qDebug() << "AXUIElementCopyAttributeValue(kAXChildrenAttribute) returned error = "
+                 << AXErrorTag(err) << "with reference" << reference;
+    }
     return list;
 }
 
+- (NSArray *)tableRows
+{
+    NSArray *arr;
+    AXError err;
+
+    if (kAXErrorSuccess != (err = AXUIElementCopyAttributeValues(reference, kAXRowsAttribute,
+                                                                    0, 100, /*min, max*/
+                                                                    (CFArrayRef *) &arr))) {
+        axError = true;
+        qDebug() << "AXUIElementCopyAttributeValue(kAXRowsAttribute) returned error = "
+                 << AXErrorTag(err) << "with reference" << reference;
+    }
+    return arr;
+}
+
+- (NSArray *)tableColumns
+{
+    NSArray *arr;
+    AXError err;
+
+    if (kAXErrorSuccess != (err = AXUIElementCopyAttributeValues(reference, kAXColumnsAttribute,
+                                                                    0, 100, /*min, max*/
+                                                                    (CFArrayRef *) &arr))) {
+        axError = true;
+        qDebug() << "AXUIElementCopyAttributeValue(kAXColumnsAttribute) returned error = "
+                 << AXErrorTag(err) << "with reference" << reference;
+    }
+    return arr;
+}
+
+- (NSArray *)tabs
+{
+    NSArray *arr;
+    AXError err;
+
+    if (kAXErrorSuccess != (err = AXUIElementCopyAttributeValues(reference, kAXTabsAttribute,
+                                                                    0, 100, /*min, max*/
+                                                                    (CFArrayRef *) &arr))) {
+        axError = true;
+        qDebug() << "AXUIElementCopyAttributeValue(kAXTabsAttribute) returned error = "
+                 << AXErrorTag(err) << "with reference" << reference;
+    }
+    return arr;
+}
+
 - (AXUIElementRef) findDirectChildByRole: (CFStringRef) role
 {
     TestAXObject *result = nil;
@@ -203,8 +263,8 @@ QDebug operator<<(QDebug dbg, AXErrorTag err)
     CFTypeRef value = NULL;
     AXError err;
 
-    if (kAXErrorSuccess != (err = AXUIElementCopyAttributeValue(reference, attribute, &value)))
-    {
+    if (kAXErrorSuccess != (err = AXUIElementCopyAttributeValue(reference, attribute, &value))) {
+        axError = true;
         qDebug() << "AXUIElementCopyAttributeValue(" << QString::fromCFString(attribute) << ") returned error = " << AXErrorTag(err);
     }
     return value;
@@ -250,8 +310,8 @@ QDebug operator<<(QDebug dbg, AXErrorTag err)
     CFTypeRef value = NULL;
     AXError err;
 
-    if (kAXErrorSuccess != (err = AXUIElementCopyParameterizedAttributeValue(reference, attribute, parameter, &value)))
-    {
+    if (kAXErrorSuccess != (err = AXUIElementCopyParameterizedAttributeValue(reference, attribute, parameter, &value))) {
+        axError = true;
         CFStringRef description = CFCopyDescription(parameter);
         qDebug() << "AXUIElementCopyParameterizedAttributeValue(" << QString::fromCFString(attribute) << ", parameter=" << QString::fromCFString(description) << ") returned error = " << AXErrorTag(err);
         CFRelease(description);
@@ -289,8 +349,8 @@ QDebug operator<<(QDebug dbg, AXErrorTag err)
     AXError err;
     CFArrayRef actions;
 
-    if (kAXErrorSuccess != (err = AXUIElementCopyActionNames(reference, &actions)))
-    {
+    if (kAXErrorSuccess != (err = AXUIElementCopyActionNames(reference, &actions))) {
+        axError = true;
         qDebug() << "AXUIElementCopyActionNames(...) returned error = " << AXErrorTag(err);
     }
 
@@ -301,13 +361,15 @@ QDebug operator<<(QDebug dbg, AXErrorTag err)
 {
     AXError err;
 
-    if (kAXErrorSuccess != (err = AXUIElementPerformAction(reference, action)))
-    {
+    if (kAXErrorSuccess != (err = AXUIElementPerformAction(reference, action))) {
+        axError = true;
         qDebug() << "AXUIElementPerformAction("  << QString::fromCFString(action) << ") returned error = " << AXErrorTag(err);
     }
 }
 
+- (bool)                valid { return reference != nil; }
 - (NSString*)           role { return [self _stringAttributeValue:kAXRoleAttribute]; }
+- (NSString*)           roleDescription { return [self _stringAttributeValue:kAXRoleDescriptionAttribute]; }
 - (NSString*)           title { return [self _stringAttributeValue:kAXTitleAttribute]; }
 - (NSString*)           description { return [self _stringAttributeValue:kAXDescriptionAttribute]; }
 - (NSString*)           value { return [self _stringAttributeValue:kAXValueAttribute]; }
@@ -320,6 +382,7 @@ QDebug operator<<(QDebug dbg, AXErrorTag err)
     return rect;
 }
 - (AXUIElementRef)      parent { return (AXUIElementRef)[self _attributeValue:kAXParentAttribute]; }
+- (AXUIElementRef)      window { return (AXUIElementRef)[self _attributeValue:kAXWindowAttribute]; }
 - (BOOL)                focused { return [self _boolAttributeValue:kAXFocusedAttribute]; }
 - (NSInteger)           numberOfCharacters { return [self _numberAttributeValue:kAXNumberOfCharactersAttribute]; }
 - (NSString*)           selectedText { return [self _stringAttributeValue:kAXSelectedTextAttribute]; }
@@ -384,6 +447,10 @@ private Q_SLOTS:
     void hierarchyTest();
     void notificationsTest();
     void checkBoxTest();
+    void tableViewTest();
+    void treeViewTest();
+    void tabBarTest();
+    void windowTest();
 
 private:
     AccessibleTestWindow *m_window;
@@ -393,7 +460,7 @@ private:
 void tst_QAccessibilityMac::init()
 {
     m_window = new AccessibleTestWindow();
-    m_window->setWindowTitle("Test window");
+    m_window->setWindowTitle(QString("Test window - %1").arg(QTest::currentTestFunction()));
     m_window->show();
     m_window->resize(400, 400);
 
@@ -434,6 +501,7 @@ void tst_QAccessibilityMac::singleWidgetTest()
     delete le;
     QCoreApplication::processEvents();
     TestAXObject *lineEditInvalid = [[TestAXObject alloc] initWithAXUIElementRef: lineEditRef];
+    QTest::ignoreMessage(QtDebugMsg, QRegularExpression("kAXErrorInvalidUIElement"));
     QVERIFY([[lineEditInvalid value] length] == 0);
 }
 
@@ -458,7 +526,7 @@ void tst_QAccessibilityMac::lineEditTest()
     // height of window includes title bar
     QVERIFY([window rect].size.height >= 400);
 
-    QVERIFY([window.title isEqualToString:@"Test window"]);
+    QVERIFY([window.title isEqualToString:@"Test window - lineEditTest"]);
 
     // children of window:
     AXUIElementRef lineEditElement = [window findDirectChildByRole: kAXTextFieldRole];
@@ -641,5 +709,261 @@ void tst_QAccessibilityMac::checkBoxTest()
     QVERIFY([cb valueNumber] == 2);
 }
 
+void tst_QAccessibilityMac::tableViewTest()
+{
+    QTableWidget *tw = new QTableWidget(3, 2, m_window);
+    struct Person
+    {
+        const char *name;
+        const char *address;
+    };
+    const Person contents[] = { { "Socrates", "Greece" },
+                                { "Confucius", "China" },
+                                { "Kant", "Preussia" }
+                              };
+    for (int i = 0; i < int(sizeof(contents) / sizeof(Person)); ++i) {
+        Person p = contents[i];
+        QTableWidgetItem *name = new QTableWidgetItem(QString::fromLatin1(p.name));
+        tw->setItem(i, 0, name);
+        QTableWidgetItem *address = new QTableWidgetItem(QString::fromLatin1(p.address));
+        tw->setItem(i, 1, address);
+    }
+    m_window->addWidget(tw);
+    QVERIFY(QTest::qWaitForWindowExposed(m_window));
+    QCoreApplication::processEvents();
+
+    TestAXObject *appObject = [TestAXObject getApplicationAXObject];
+    QVERIFY(appObject);
+
+    NSArray *windowList = [appObject windowList];
+    // one window
+    QVERIFY([windowList count] == 1);
+    AXUIElementRef windowRef = (AXUIElementRef)[windowList objectAtIndex:0];
+    QVERIFY(windowRef != nil);
+    TestAXObject *window = [[TestAXObject alloc] initWithAXUIElementRef:windowRef];
+
+    // children of window:
+    AXUIElementRef tableView = [window findDirectChildByRole:kAXTableRole];
+    QVERIFY(tableView != nil);
+
+    TestAXObject *tv = [[TestAXObject alloc] initWithAXUIElementRef:tableView];
+
+    // here start actual tableview tests
+    // Should have 2 columns
+    const unsigned int columnCount = 2;
+    NSArray *columnArray = [tv tableColumns];
+    QCOMPARE([columnArray count], columnCount);
+
+    // should have 3 rows
+    const unsigned int rowCount = 3;
+    NSArray *rowArray = [tv tableRows];
+    QCOMPARE([rowArray count], rowCount);
+
+    // The individual cells are children of the rows
+    TestAXObject *row = [[TestAXObject alloc] initWithAXUIElementRef:(AXUIElementRef)rowArray[0]];
+    TestAXObject *cell = [[TestAXObject alloc] initWithAXUIElementRef:(AXUIElementRef)[row childList][0]];
+    QVERIFY([cell.title isEqualToString:@"Socrates"]);
+    row = [[TestAXObject alloc] initWithAXUIElementRef:(AXUIElementRef)rowArray[2]];
+    cell = [[TestAXObject alloc] initWithAXUIElementRef:(AXUIElementRef)[row childList][1]];
+    QVERIFY([cell.title isEqualToString:@"Preussia"]);
+
+    // both rows and columns are direct children of the table
+    NSArray *childList = [tv childList];
+    QCOMPARE([childList count], columnCount + rowCount);
+    for (id child in childList) {
+        TestAXObject *childObject = [[TestAXObject alloc] initWithAXUIElementRef:(AXUIElementRef)child];
+        QVERIFY([childObject.role isEqualToString:NSAccessibilityRowRole] ||
+               [childObject.role isEqualToString:NSAccessibilityColumnRole]);
+    }
+}
+
+void tst_QAccessibilityMac::treeViewTest()
+{
+    QTreeWidget *tw = new QTreeWidget;
+    tw->setColumnCount(2);
+    QTreeWidgetItem *root = new QTreeWidgetItem(tw, {"/", "0"});
+    root->setExpanded(false);
+    QTreeWidgetItem *users = new QTreeWidgetItem(root,{ "Users", "1"});
+    (void)new QTreeWidgetItem(root, {"Applications", "2"});
+    QTreeWidgetItem *lastChild = new QTreeWidgetItem(root, {"Libraries", "3"});
+
+    m_window->addWidget(tw);
+    QVERIFY(QTest::qWaitForWindowExposed(m_window));
+    QCoreApplication::processEvents();
+
+    TestAXObject *appObject = [TestAXObject getApplicationAXObject];
+    QVERIFY(appObject);
+
+    NSArray *windowList = [appObject windowList];
+    // one window
+    QVERIFY([windowList count] == 1);
+    AXUIElementRef windowRef = (AXUIElementRef)[windowList objectAtIndex:0];
+    QVERIFY(windowRef != nil);
+    TestAXObject *window = [[TestAXObject alloc] initWithAXUIElementRef:windowRef];
+    QVERIFY(window.valid);
+
+    // children of window
+
+    auto accessibleTreeView = [window]() -> TestAXObject * {
+        AXUIElementRef treeView = [window findDirectChildByRole:kAXOutlineRole];
+        Q_ASSERT(treeView != nil);
+        TestAXObject *tv = [[TestAXObject alloc] initWithAXUIElementRef:treeView];
+        return tv;
+    };
+
+    TestAXObject *tv = accessibleTreeView();
+    QVERIFY(tv.valid);
+
+    // here start actual treeview tests. NSAccessibilityOutline is a specialization
+    // of NSAccessibilityTable, and we represent trees as tables.
+    const auto cellText = [&tv](int rowIndex, int columnIndex) -> QString {
+        NSArray *rowArray = [tv tableRows];
+        Q_ASSERT(rowArray.count > uint(rowIndex));
+        TestAXObject *row = [[TestAXObject alloc] initWithAXUIElementRef:(AXUIElementRef)rowArray[rowIndex]];
+        Q_ASSERT(row && row.valid);
+        TestAXObject *cell = [[TestAXObject alloc] initWithAXUIElementRef:(AXUIElementRef)row.childList[columnIndex]];
+        Q_ASSERT(cell && cell.valid);
+        QString result = QString::fromNSString(cell.title);
+        [rowArray release];
+        return result;
+    };
+
+    // Should have 2 columns
+    const unsigned int columnCount = 2;
+    {
+        NSArray *columnArray = tv.tableColumns;
+        QCOMPARE(columnArray.count, columnCount);
+        [columnArray release];
+    }
+
+    // Should have 1 row for now - as long as the root item is not expanded.
+    // That row should have 2 columns.
+    {
+        NSArray *rowArray = tv.tableRows;
+        QCOMPARE(rowArray.count, 1u);
+        {
+            TestAXObject *row = [[TestAXObject alloc] initWithAXUIElementRef:(AXUIElementRef)rowArray[0]];
+            QCOMPARE(row.childList.count, columnCount);
+            [row release];
+        }
+        [rowArray release];
+    }
+    QCOMPARE(cellText(0, 0), root->text(0));
+
+    root->setExpanded(true);
+    // Now that the root node is expanded we should now see all rows.
+    // Each row should have the same number of columns as the tree view.
+    {
+        NSArray *rowArray = tv.tableRows;
+        QCOMPARE(rowArray.count, root->childCount() + 1u);
+
+        for (ulong r = 0; r < rowArray.count; ++r) {
+            QVERIFY(rowArray[r] != nil);
+            TestAXObject *row = [[TestAXObject alloc] initWithAXUIElementRef:(AXUIElementRef)rowArray[r]];
+            const uint childCount = row.childList.count;
+            QCOMPARE(childCount, columnCount);
+            [row release];
+        }
+        [rowArray release];
+    }
+
+    // this should not trigger any assert
+    tw->setCurrentItem(lastChild);
+
+    // cache will have been cleared, reinitialize
+    [tv release];
+    tv = accessibleTreeView();
+    QVERIFY(tv.valid);
+
+    QCOMPARE(cellText(0, 0), root->text(0));
+    QCOMPARE(cellText(1, 0), users->text(0));
+    QCOMPARE(cellText(1, 1), users->text(1));
+
+    [appObject release];
+    [window release];
+}
+
+void tst_QAccessibilityMac::tabBarTest()
+{
+    QTabBar *tbar = new QTabBar;
+    static const unsigned int nTabs = 20;
+    for (unsigned int i = 0; i < nTabs; ++i)
+        tbar->addTab(QString::number(i));
+    tbar->setUsesScrollButtons(true);
+
+    m_window->addWidget(tbar);
+    QVERIFY(QTest::qWaitForWindowExposed(m_window));
+    QCoreApplication::processEvents();
+
+    TestAXObject *appObject = [TestAXObject getApplicationAXObject];
+    QVERIFY(appObject);
+
+    NSArray *windowList = [appObject windowList];
+    // one window
+    QVERIFY([windowList count] == 1);
+    AXUIElementRef windowRef = (AXUIElementRef)[windowList objectAtIndex:0];
+    QVERIFY(windowRef != nil);
+    TestAXObject *window = [[TestAXObject alloc] initWithAXUIElementRef:windowRef];
+    QVERIFY(window.valid);
+
+    // children of window
+    AXUIElementRef axTarBar = [window findDirectChildByRole:kAXTabGroupRole];
+    QVERIFY(axTarBar != nil);
+
+    TestAXObject *tb = [[TestAXObject alloc] initWithAXUIElementRef:axTarBar];
+    QVERIFY(tb.valid);
+
+    [appObject release];
+    [window release];
+
+    NSArray *tbChildList = [tb childList];
+    // +2 because of the scroll buttons
+    QCOMPARE([tbChildList count], nTabs + 2);
+
+    NSArray *tbTabsList = [tb tabs];
+    QCOMPARE([tbTabsList count], nTabs);
+
+    for (unsigned int i = 0; i < nTabs; ++i) {
+        AXUIElementRef axTab = (AXUIElementRef)[tbTabsList objectAtIndex:i];
+        QVERIFY(axTab != nil);
+
+        TestAXObject *tab = [[TestAXObject alloc] initWithAXUIElementRef:axTab];
+        QVERIFY(tab.valid);
+        QCOMPARE(QString::fromNSString(tab.role), QString::fromCFString(kAXRadioButtonRole));
+        QCOMPARE(QString::fromNSString(tab.title), QString::number(i));
+        QCOMPARE(QString::fromNSString(tab.roleDescription), "tab");
+    }
+}
+
+void tst_QAccessibilityMac::windowTest()
+{
+    QTextEdit *textEdit = new QTextEdit;
+    m_window->addWidget(textEdit);
+    QVERIFY(QTest::qWaitForWindowExposed(m_window));
+    QCoreApplication::processEvents();
+
+    TestAXObject *appObject = [TestAXObject getApplicationAXObject];
+    QVERIFY(appObject);
+
+    NSArray *windowList = [appObject windowList];
+    // one window
+    QVERIFY([windowList count] == 1);
+    AXUIElementRef windowRef = (AXUIElementRef)[windowList objectAtIndex:0];
+    QVERIFY(windowRef != nil);
+    TestAXObject *window = [[TestAXObject alloc] initWithAXUIElementRef:windowRef];
+    QVERIFY(window.valid);
+
+    AXUIElementRef axTextEdit = [window findDirectChildByRole:kAXTextAreaRole];
+    QVERIFY(axTextEdit != nil);
+
+    [appObject release];
+    [window release];
+
+    TestAXObject *edit = [[TestAXObject alloc] initWithAXUIElementRef:axTextEdit];
+    QVERIFY(edit.valid);
+    AXUIElementRef axWindow = edit.window;
+    QVERIFY(axWindow);
+}
+
 QTEST_MAIN(tst_QAccessibilityMac)
 #include "tst_qaccessibilitymac.moc"
