@@ -2,6 +2,7 @@
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QTest>
+#include <QtTest/private/qtesthelpers_p.h>
 #include <QtGui/QWindow>
 #include <QtGui/QCursor>
 #include <QtGui/private/qguiapplication_p.h>
@@ -256,8 +257,9 @@ void tst_Mouse::doubleClick()
 {
     MouseWindow w;
     w.show();
-    w.setGeometry(100, 100, 200, 200);
+    w.resize(200, 200);
     QVERIFY(QTest::qWaitForWindowActive(&w));
+    QVERIFY(QTestPrivate::ensurePositionTopLeft(&w));
 
     // click
     QPoint point(10, 10);
@@ -283,13 +285,15 @@ void tst_Mouse::doubleClick()
     ts = w.lastTimeStamp;
     QTest::mouseClick(&w, Qt::LeftButton, {}, point, 10);
     QCOMPARE_GE(w.lastTimeStamp, ts + 500); // because the last release had a default delay
-    QTest::mouseClick(&w, Qt::LeftButton, {}, point);
+    ts = w.lastTimeStamp;
+    QTest::mouseClick(&w, Qt::LeftButton, {}, point, 10); // 10 ms before press, 10 ms before release
     QCOMPARE(w.doubleClickCount, 2);
+    QCOMPARE(w.lastTimeStamp, ts + 20);
 
     // use the mouseDClick function to generate another double-click
     ts = w.lastTimeStamp;
     QTest::mouseDClick(&w, Qt::LeftButton, {}, point);
-    QCOMPARE_GE(w.lastTimeStamp, ts + 500); // because the last release had a default delay
+    QCOMPARE(w.lastTimeStamp, ts + 4); // 1 ms before each press and release
     QCOMPARE(w.doubleClickCount, 3);
 
     // use the mouseClick function with default delay to avoid double-click
