@@ -1,5 +1,5 @@
 // Copyright (C) 2011 Olivier Goffart.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #ifndef CXX11_ENUMS_H
 #define CXX11_ENUMS_H
@@ -47,4 +47,20 @@ public:
     Q_FLAGS(ClassFlags)
 };
 
+class CXX11Enums3 : public QObject
+{
+    Q_OBJECT
+public:
+    enum class EnumClass { A0, A1, A2, A3 };
+    enum TypedEnum : char { B0, B1 , B2, B3 };
+    enum class TypedEnumClass : char { C0, C1, C2, C3 };
+    enum NormalEnum { D2 = 2, D3, D0 =0 , D1 };
+    enum class ClassFlag { F0 = 1, F1 = 2, F2 = 4, F3 = 8 };
+
+    Q_ENUM(EnumClass)
+    Q_ENUM(TypedEnum)
+    Q_ENUM(TypedEnumClass)
+    Q_ENUM(NormalEnum)
+};
+
 #endif // CXX11_ENUMS_H
