@@ -468,8 +468,10 @@ void tst_QFileDialog2::settingsCompatibility()
 {
     static const QByteArray ba32 = QByteArrayLiteral("\x00\x00\x00\xFF\x00\x00\x00\x00\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x01\xF7\x00\x00\x00\x04\x01\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00""d\xFF\xFF\xFF\xFF\x00\x00\x00\x81\x00\x00\x00\x00\x00\x00\x00\x04\x00\x00\x01\t\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00>\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00""B\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00n\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x03\xE8\x00\xFF\xFF\xFF\xFF\x00\x00\x00\x00");
     static const QByteArray ba64 = QByteArrayLiteral("\x00\x00\x00\xFF\x00\x00\x00\x00\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x01\xF7\x00\x00\x00\x04\x01\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00""d\xFF\xFF\xFF\xFF\x00\x00\x00\x81\x00\x00\x00\x00\x00\x00\x00\x04\x00\x00\x01\t\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00>\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00""B\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00n\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x03\xE8\x00\xFF\xFF\xFF\xFF\x00\x00\x00\x00");
+    QByteArray impHeaderData;
     QFETCH(QString, qtVersion);
     QFETCH(QDataStream::Version, dsVersion);
+
     // Create a header view, convert template to target format and store it in settings
     {
         QSettings settings(QSettings::UserScope, "QtProject");
@@ -477,18 +479,38 @@ void tst_QFileDialog2::settingsCompatibility()
         settings.setValue("sidebarWidth", 93); // random value
         settings.setValue("shortcuts", QStringList({settings.fileName(), "/tmp"}));
         settings.setValue("qtVersion", qtVersion);
-        settings.setValue("treeViewHeader", dsVersion < QDataStream::Qt_6_0 ? ba32 : ba64);
+        impHeaderData = dsVersion < QDataStream::Qt_6_0 ? ba32 : ba64;
+        settings.setValue("treeViewHeader", impHeaderData);
         settings.endGroup();
     }
     // Create a file dialog, read settings write them back
     {
         QFileDialog fd;
     }
-    // Read back settings and compare byte array
+
+    // This test has been changed a bit since a settings readback and compare likely isn't very useful.
+    // It would either require that we could centrally (and/or decentrally) can set the saveState and
+    // restoreState versions (which currently isn't the case) or it will block
+    // any (also backward compatible) extension of saveState and restoreState (which isn't acceptable).
+    // So we check that we can indeed import the old data and that a transient saveResult will result
+    // in the same data.
+    // That we are decent forward compatible from newer to older versions is difficult to test.
     QSettings settings(QSettings::UserScope, "QtProject");
     settings.beginGroup("FileDialog");
     const QByteArray savedState = settings.value("treeViewHeader").toByteArray();
-    QCOMPARE(savedState, ba32);
+
+    // Check that we can restore data from an old version.
+    QHeaderView header1(Qt::Horizontal);
+    QVERIFY(header1.restoreState(impHeaderData));
+
+    // Check that we can restore the now saved data.
+    QHeaderView header2(Qt::Horizontal);
+    QVERIFY(header2.restoreState(savedState));
+
+    // Check that these states provide the same result
+    QByteArray h2data = header2.saveState();
+    QCOMPARE(header1.saveState(), h2data);
+    QCOMPARE(h2data, savedState);
 }
 #endif
 
