@@ -1,11 +1,10 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 
 #include <QTest>
 #include <QMessageBox>
 #include <QDebug>
-#include <QPair>
 #include <QSet>
 #include <QList>
 #include <QPointer>
@@ -23,7 +22,11 @@ class tst_QMessageBox : public QObject
     Q_OBJECT
 
 private slots:
+    void initTestCase_data();
+    void init();
+
     void sanityTest();
+    void baseClassSetVisible();
     void defaultButton();
     void escapeButton();
     void clickedButton();
@@ -34,6 +37,9 @@ private slots:
     void detailsButtonText();
     void expandDetailsWithoutMoving();
 
+    void optionsEmptyByDefault();
+    void changeNativeOption();
+
 #ifndef Q_OS_MAC
     void shortcut();
 #endif
@@ -51,6 +57,14 @@ private slots:
     void acceptedRejectedSignals();
     void acceptedRejectedSignals_data();
 
+    void overrideDone_data();
+    void overrideDone();
+
+    void hideNativeByDestruction();
+
+    void explicitDoneAfterButtonClicked();
+    void legacyApiReturnValue();
+
     void cleanup();
 };
 
@@ -129,6 +143,60 @@ void ExecCloseHelper::timerEvent(QTimerEvent *te)
     }
 }
 
+void tst_QMessageBox::initTestCase_data()
+{
+    QTest::addColumn<bool>("useNativeDialog");
+    QTest::newRow("widget") << false;
+    if (const QPlatformTheme *theme = QGuiApplicationPrivate::platformTheme()) {
+        if (theme->usePlatformNativeDialog(QPlatformTheme::MessageDialog))
+            QTest::newRow("native") << true;
+    }
+}
+
+void tst_QMessageBox::init()
+{
+    QFETCH_GLOBAL(bool, useNativeDialog);
+    qApp->setAttribute(Qt::AA_DontUseNativeDialogs, !useNativeDialog);
+}
+
+class OverridingMessageBox : public QMessageBox
+{
+public:
+    void done(int result) override {
+        doneResult = result;
+        QMessageBox::done(result);
+    }
+    std::optional<int> doneResult;
+};
+
+void tst_QMessageBox::overrideDone_data()
+{
+    QTest::addColumn<QMessageBox::StandardButton>("button");
+    QTest::addColumn<int>("closeAction");
+    QTest::addColumn<int>("result");
+
+    QTest::newRow("close") << QMessageBox::Help << int(ExecCloseHelper::CloseWindow) << 0;
+    QTest::newRow("yes") << QMessageBox::Yes << int(Qt::Key_Enter) << int(QMessageBox::Yes);
+    QTest::newRow("no") << QMessageBox::No << int(Qt::Key_Enter) << int(QMessageBox::No);
+}
+
+void tst_QMessageBox::overrideDone()
+{
+    QFETCH(QMessageBox::StandardButton, button);
+    QFETCH(int, closeAction);
+    QFETCH(int, result);
+
+    OverridingMessageBox messageBox;
+    messageBox.addButton(button);
+    messageBox.setDefaultButton(button);
+    ExecCloseHelper closeHelper;
+    closeHelper.start(closeAction, &messageBox);
+    messageBox.exec();
+    QVERIFY(messageBox.doneResult.has_value());
+    QCOMPARE(*messageBox.doneResult, result);
+
+}
+
 void tst_QMessageBox::cleanup()
 {
     QTRY_VERIFY(QApplication::topLevelWidgets().isEmpty()); // OS X requires TRY
@@ -155,6 +223,15 @@ void tst_QMessageBox::sanityTest()
     msgBox.exec();
 }
 
+void tst_QMessageBox::baseClassSetVisible()
+{
+    QMessageBox msgBox;
+    msgBox.setText("Hello World");
+    msgBox.QDialog::setVisible(true);
+    QCOMPARE(msgBox.isVisible(), true);
+    msgBox.close();
+}
+
 void tst_QMessageBox::button()
 {
     QMessageBox msgBox;
@@ -363,7 +440,7 @@ void tst_QMessageBox::shortcut()
     msgBox.addButton("&Maybe", QMessageBox::YesRole);
     ExecCloseHelper closeHelper;
     closeHelper.start(Qt::Key_M, &msgBox);
-    QCOMPARE(msgBox.exec(), 2);
+    QCOMPARE(msgBox.exec(), 4);
 }
 #endif
 
@@ -398,72 +475,61 @@ void tst_QMessageBox::staticSourceCompat()
     // source compat tests for < 4.2
 QT_WARNING_PUSH
 QT_WARNING_DISABLE_DEPRECATED
+#define COMPARE(real, exp) do {\
+    const auto pressed = static_cast<QMessageBox::StandardButton>(real);\
+    const auto expected = static_cast<QMessageBox::StandardButton>(exp);\
+    if (!QTest::qCompare(pressed, expected, #real, #exp, __FILE__, __LINE__)) \
+        return; } while (false)
+
     ExecCloseHelper closeHelper;
     closeHelper.start(Qt::Key_Enter);
     ret = QMessageBox::information(nullptr, "title", "text", QMessageBox::Yes, QMessageBox::No);
-    int expectedButton = int(QMessageBox::Yes);
-    if (const QPlatformTheme *theme = QGuiApplicationPrivate::platformTheme()) {
-        const int dialogButtonBoxLayout = theme->themeHint(QPlatformTheme::DialogButtonBoxLayout).toInt();
-        if (dialogButtonBoxLayout == QDialogButtonBox::MacLayout
-            || dialogButtonBoxLayout == QDialogButtonBox::GnomeLayout)
-            expectedButton = int(QMessageBox::No);
-    }
-    QCOMPARE(ret, expectedButton);
+    COMPARE(ret, QMessageBox::No);
     QVERIFY(closeHelper.done());
 
     closeHelper.start(Qt::Key_Enter);
     ret = QMessageBox::information(nullptr, "title", "text", QMessageBox::Yes | QMessageBox::Default, QMessageBox::No);
-    QCOMPARE(ret, int(QMessageBox::Yes));
+    COMPARE(ret, int(QMessageBox::Yes));
     QVERIFY(closeHelper.done());
 
+#if QT_DEPRECATED_SINCE(6, 2)
+    // The overloads below are valid only before 6.2
     closeHelper.start(Qt::Key_Enter);
     ret = QMessageBox::information(nullptr, "title", "text", QMessageBox::Yes, QMessageBox::No | QMessageBox::Default);
-    QCOMPARE(ret, int(QMessageBox::No));
+    COMPARE(ret, int(QMessageBox::No));
     QVERIFY(closeHelper.done());
 
     closeHelper.start(Qt::Key_Enter);
     ret = QMessageBox::information(nullptr, "title", "text", QMessageBox::Yes | QMessageBox::Default, QMessageBox::No | QMessageBox::Escape);
-    QCOMPARE(ret, int(QMessageBox::Yes));
+    COMPARE(ret, int(QMessageBox::Yes));
     QVERIFY(closeHelper.done());
 
     closeHelper.start(Qt::Key_Enter);
     ret = QMessageBox::information(nullptr, "title", "text", QMessageBox::Yes | QMessageBox::Escape, QMessageBox::No | QMessageBox::Default);
-    QCOMPARE(ret, int(QMessageBox::No));
+    COMPARE(ret, int(QMessageBox::No));
     QVERIFY(closeHelper.done());
 
     // the button text versions
     closeHelper.start(Qt::Key_Enter);
     ret = QMessageBox::information(nullptr, "title", "text", "Yes", "No", QString(), 1);
-    QCOMPARE(ret, 1);
+    COMPARE(ret, 1);
     QVERIFY(closeHelper.done());
-
-    if (0) { // don't run these tests since the dialog won't close!
-        closeHelper.start(Qt::Key_Escape);
-        ret = QMessageBox::information(nullptr, "title", "text", "Yes", "No", QString(), 1);
-        QCOMPARE(ret, -1);
-        QVERIFY(closeHelper.done());
-
-        closeHelper.start(Qt::Key_Escape);
-        ret = QMessageBox::information(nullptr, "title", "text", "Yes", "No", QString(), 0, 1);
-        QCOMPARE(ret, 1);
-        QVERIFY(closeHelper.done());
-    }
+#endif // QT_DEPRECATED_SINCE(6, 2)
+#undef COMPARE
 QT_WARNING_POP
 }
 
 void tst_QMessageBox::instanceSourceCompat()
 {
-QT_WARNING_PUSH
-QT_WARNING_DISABLE_DEPRECATED
-     QMessageBox mb("Application name here",
-                    "Saving the file will overwrite the original file on the disk.\n"
-                    "Do you really want to save?",
-                    QMessageBox::Information,
-                    QMessageBox::Yes | QMessageBox::Default,
-                    QMessageBox::No,
-                    QMessageBox::Cancel | QMessageBox::Escape);
-    mb.setButtonText(QMessageBox::Yes, "Save");
-    mb.setButtonText(QMessageBox::No, "Discard");
+    QMessageBox mb(QMessageBox::Information,
+                   "Application name here",
+                   "Saving the file will overwrite the original file on the disk.\n"
+                   "Do you really want to save?",
+                   QMessageBox::Yes | QMessageBox::No | QMessageBox::Cancel);
+    mb.setDefaultButton(QMessageBox::Yes);
+    mb.setEscapeButton(QMessageBox::Cancel);
+    mb.button(QMessageBox::Yes)->setText("Save");
+    mb.button(QMessageBox::No)->setText("Discard");
     mb.addButton("&Revert", QMessageBox::RejectRole);
     mb.addButton("&Zoo", QMessageBox::ActionRole);
 
@@ -475,15 +541,18 @@ QT_WARNING_DISABLE_DEPRECATED
 #ifndef Q_OS_MAC
     // mnemonics are not used on OS X
     closeHelper.start(QKeyCombination(Qt::ALT | Qt::Key_R).toCombined(), &mb);
-    QCOMPARE(mb.exec(), 0);
+    QCOMPARE(mb.exec(), 2);
     closeHelper.start(QKeyCombination(Qt::ALT | Qt::Key_Z).toCombined(), &mb);
-    QCOMPARE(mb.exec(), 1);
+    QCOMPARE(mb.exec(), 3);
 #endif
-QT_WARNING_POP
 }
 
 void tst_QMessageBox::detailsText()
 {
+    QFETCH_GLOBAL(bool, useNativeDialog);
+    if (useNativeDialog)
+         QSKIP("Native dialogs do not propagate expose events");
+
     QMessageBox box;
     QString text("This is the details text.");
     box.setDetailedText(text);
@@ -497,6 +566,10 @@ void tst_QMessageBox::detailsText()
 
 void tst_QMessageBox::detailsButtonText()
 {
+    QFETCH_GLOBAL(bool, useNativeDialog);
+    if (useNativeDialog)
+         QSKIP("Native dialogs do not propagate expose events");
+
     QMessageBox box;
     box.setDetailedText("bla");
     box.open();
@@ -518,6 +591,10 @@ void tst_QMessageBox::detailsButtonText()
 
 void tst_QMessageBox::expandDetailsWithoutMoving() // QTBUG-32473
 {
+    QFETCH_GLOBAL(bool, useNativeDialog);
+    if (useNativeDialog)
+         QSKIP("Native dialogs do not propagate expose events");
+
     tst_ResizingMessageBox box;
     box.setDetailedText("bla");
     box.show();
@@ -543,6 +620,20 @@ void tst_QMessageBox::expandDetailsWithoutMoving() // QTBUG-32473
     QCOMPARE(box.geometry().topLeft(), geom.topLeft());
 }
 
+void tst_QMessageBox::optionsEmptyByDefault()
+{
+    QMessageBox b;
+    QCOMPARE(b.options(), QMessageBox::Options());
+    QVERIFY(!b.testOption(QMessageBox::Option::DontUseNativeDialog));
+}
+
+void tst_QMessageBox::changeNativeOption()
+{
+    QMessageBox b;
+    b.setOption(QMessageBox::Option::DontUseNativeDialog);
+    QCOMPARE(b.options(), QMessageBox::Options(QMessageBox::Option::DontUseNativeDialog));
+}
+
 void tst_QMessageBox::incorrectDefaultButton()
 {
     ExecCloseHelper closeHelper;
@@ -557,6 +648,7 @@ void tst_QMessageBox::incorrectDefaultButton()
     QMessageBox::question(nullptr, "", "I've been hit!",QFlag(QMessageBox::Ok | QMessageBox::Cancel),QMessageBox::Save);
     QVERIFY(closeHelper.done());
 
+#if QT_DEPRECATED_SINCE(6, 2)
     closeHelper.start(Qt::Key_Escape);
     QTest::ignoreMessage(QtWarningMsg, "QDialogButtonBox::createButton: Invalid ButtonRole, button not added");
     QTest::ignoreMessage(QtWarningMsg, "QDialogButtonBox::createButton: Invalid ButtonRole, button not added");
@@ -566,6 +658,7 @@ QT_WARNING_DISABLE_DEPRECATED
     QMessageBox::question(nullptr, "", "I've been hit!",QMessageBox::Ok | QMessageBox::Cancel,QMessageBox::Save | QMessageBox::Cancel,QMessageBox::Ok);
 QT_WARNING_POP
     QVERIFY(closeHelper.done());
+#endif
 }
 
 void tst_QMessageBox::updateSize()
@@ -615,6 +708,10 @@ Q_DECLARE_METATYPE(RoleSet);
 
 void tst_QMessageBox::acceptedRejectedSignals()
 {
+    QFETCH_GLOBAL(bool, useNativeDialog);
+    if (useNativeDialog)
+         QSKIP("Native dialogs do not propagate expose events");
+
     QMessageBox messageBox(QMessageBox::Information, "Test window", "Test text");
 
     QFETCH(ButtonsCreator, buttonsCreator);
@@ -694,5 +791,94 @@ void tst_QMessageBox::acceptedRejectedSignals_data()
     addCustomButtonsData();
 }
 
+void tst_QMessageBox::hideNativeByDestruction()
+{
+    QWidget window;
+    QWidget *child = new QWidget(&window);
+    QPointer<QMessageBox> dialog = new QMessageBox(child);
+    // Make it application modal so that we don't end up with a sheet on macOS
+    dialog->setWindowModality(Qt::ApplicationModal);
+    window.show();
+    QVERIFY(QTest::qWaitForWindowExposed(&window));
+    dialog->open();
+
+    // We test that the dialog opens and closes by watching the activation of the
+    // transient parent window. If it doesn't deactivate, then we have to skip.
+    const auto windowActive = [&window]{ return window.isActiveWindow(); };
+    const auto windowInactive = [&window]{ return !window.isActiveWindow(); };
+    if (!QTest::qWaitFor(windowInactive, 2000))
+        QSKIP("Dialog didn't activate");
+
+    // This should destroy the dialog and close the native window
+    child->deleteLater();
+    QTRY_VERIFY(!dialog);
+    // If the native window is still open, then the transient parent can't become
+    // active
+    window.activateWindow();
+    QVERIFY(QTest::qWaitFor(windowActive));
+}
+
+void tst_QMessageBox::explicitDoneAfterButtonClicked()
+{
+    QMessageBox msgBox;
+    auto *standardButton = msgBox.addButton(QMessageBox::Ok);
+    auto *customButton = msgBox.addButton("Custom", QMessageBox::RejectRole);
+
+    QSignalSpy acceptedSpy(&msgBox, &QDialog::accepted);
+    QSignalSpy rejectedSpy(&msgBox, &QDialog::rejected);
+
+    msgBox.setDefaultButton(standardButton);
+    ExecCloseHelper closeHelper;
+    closeHelper.start(Qt::Key_Enter, &msgBox);
+    msgBox.exec();
+    QCOMPARE(msgBox.clickedButton(), standardButton);
+    QCOMPARE(msgBox.result(), QMessageBox::Ok);
+    QCOMPARE(acceptedSpy.size(), 1);
+    QCOMPARE(rejectedSpy.size(), 0);
+
+    msgBox.accept();
+    QCOMPARE(msgBox.result(), QDialog::Accepted);
+    QCOMPARE(acceptedSpy.size(), 2);
+    QCOMPARE(rejectedSpy.size(), 0);
+    msgBox.reject();
+    QCOMPARE(msgBox.result(), QDialog::Rejected);
+    QCOMPARE(acceptedSpy.size(), 2);
+    QCOMPARE(rejectedSpy.size(), 1);
+
+    msgBox.setDefaultButton(customButton);
+    closeHelper.start(Qt::Key_Enter, &msgBox);
+    msgBox.exec();
+    QCOMPARE(msgBox.clickedButton(), customButton);
+    QVERIFY(msgBox.result() != QDialog::Accepted);
+    QVERIFY(msgBox.result() != QDialog::Rejected);
+    QCOMPARE(acceptedSpy.size(), 2);
+    QCOMPARE(rejectedSpy.size(), 2);
+
+    msgBox.accept();
+    QCOMPARE(msgBox.result(), QDialog::Accepted);
+    QCOMPARE(acceptedSpy.size(), 3);
+    QCOMPARE(rejectedSpy.size(), 2);
+    msgBox.reject();
+    QCOMPARE(msgBox.result(), QDialog::Rejected);
+    QCOMPARE(acceptedSpy.size(), 3);
+    QCOMPARE(rejectedSpy.size(), 3);
+}
+
+void tst_QMessageBox::legacyApiReturnValue()
+{
+    ExecCloseHelper closeHelper;
+    for (int i = 0; i < 3; ++i) {
+        closeHelper.start(Qt::Key_Enter);
+QT_WARNING_PUSH
+QT_WARNING_DISABLE_DEPRECATED
+        QCOMPARE(QMessageBox::warning(nullptr, "Title", "Text",
+            "Button 0", "Button 1", "Button 2", i), i);
+        closeHelper.start(Qt::Key_Escape);
+        QCOMPARE(QMessageBox::warning(nullptr, "Title", "Text",
+            "Button 0", "Button 1", "Button 2", 0, i), i);
+QT_WARNING_POP
+    }
+}
+
 QTEST_MAIN(tst_QMessageBox)
 #include "tst_qmessagebox.moc"
