@@ -1,10 +1,11 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include "../../../shared/highdpi.h"
 
 #include <QTest>
 #include <QTimeZone>
+#include <QDateTime>
 #include <QTimer>
 #include <QTestEventLoop>
 #include <QSignalSpy>
@@ -34,6 +35,7 @@
 #include <qscreen.h>
 
 #include <QtWidgets/private/qabstractitemdelegate_p.h>
+#include <QtWidgets/private/qapplication_p.h>
 
 Q_DECLARE_METATYPE(QAbstractItemDelegate::EndEditHint)
 
@@ -745,7 +747,6 @@ void tst_QItemDelegate::dateTimeEditor()
     widget.setItem(0, 2, item3);
     widget.show();
     QVERIFY(QTest::qWaitForWindowExposed(&widget));
-    QApplication::setActiveWindow(&widget);
 
     widget.editItem(item1);
 
@@ -761,7 +762,6 @@ void tst_QItemDelegate::dateTimeEditor()
     timeEditor->setTime(time.addSecs(60));
 
     widget.clearFocus();
-    qApp->setActiveWindow(&widget);
     widget.setFocus();
     widget.editItem(item2);
 
@@ -1025,7 +1025,6 @@ void tst_QItemDelegate::decoration()
     TestItemDelegate delegate;
     table.setItemDelegate(&delegate);
     table.show();
-    QApplication::setActiveWindow(&table);
     QVERIFY(QTest::qWaitForWindowActive(&table));
 
     QVariant value;
@@ -1280,7 +1279,6 @@ void tst_QItemDelegate::enterKey()
     QListView view;
     view.setModel(&model);
     view.show();
-    QApplication::setActiveWindow(&view);
     view.setFocus();
     QVERIFY(QTest::qWaitForWindowActive(&view));
 
@@ -1340,7 +1338,6 @@ void tst_QItemDelegate::task257859_finalizeEdit()
     QListView view;
     view.setModel(&model);
     view.show();
-    QApplication::setActiveWindow(&view);
     view.setFocus();
     QVERIFY(QTest::qWaitForWindowActive(&view));
 
@@ -1402,7 +1399,7 @@ void tst_QItemDelegate::comboBox()
     widget.setItem(0, 0, item1);
     widget.show();
     QVERIFY(QTest::qWaitForWindowExposed(&widget));
-    QApplication::setActiveWindow(&widget);
+    QApplicationPrivate::setActiveWindow(&widget);
 
     widget.editItem(item1);
 
@@ -1467,7 +1464,6 @@ void tst_QItemDelegate::testLineEditValidation()
     view.setItemDelegate(&delegate);
     view.show();
     view.setFocus();
-    QApplication::setActiveWindow(&view);
     QVERIFY(QTest::qWaitForWindowActive(&view));
 
     QPointer<QLineEdit> editor;
@@ -1591,12 +1587,12 @@ void tst_QItemDelegate::dateTextForRole_data()
     QDate date(2013, 12, 11);
     QTime time(10, 9, 8, 765);
     // Ensure we exercise every time-spec variant:
-    QTest::newRow("local") << QDateTime(date, time, Qt::LocalTime);
-    QTest::newRow("UTC") << QDateTime(date, time, Qt::UTC);
-#if QT_CONFIG(timezone)
+    QTest::newRow("local") << QDateTime(date, time);
+    QTest::newRow("UTC") << QDateTime(date, time, QTimeZone::UTC);
+#  if QT_CONFIG(timezone)
     QTest::newRow("zone") << QDateTime(date, time, QTimeZone("Europe/Dublin"));
-#endif
-    QTest::newRow("offset") << QDateTime(date, time, Qt::OffsetFromUTC, 36000);
+#  endif
+    QTest::newRow("offset") << QDateTime(date, time, QTimeZone::fromSecondsAheadOfUtc(36000));
 #endif
 }
 
@@ -1626,7 +1622,8 @@ void tst_QItemDelegate::reuseEditor()
         using QItemDelegate::QItemDelegate;
         ~ReusingDelegate()
         {
-            cached->deleteLater();
+            if (cached)
+                cached->deleteLater();
         }
 
         QWidget* createEditor(QWidget* parent,
