@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2019 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2019 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QtWidgets/QApplication>
 #include <QtWidgets/QCheckBox>
@@ -45,6 +20,8 @@
 #include <QtWidgets/QSplitter>
 #include <QtWidgets/QStyle>
 #include <QtWidgets/QStyleFactory>
+#include <QtWidgets/QTabBar>
+#include <QtWidgets/QTabWidget>
 #include <QtWidgets/QTableWidget>
 #include <QtWidgets/QToolButton>
 #include <QtWidgets/QToolTip>
@@ -63,6 +40,8 @@
 #include <private/qstylesheetstyle_p.h>
 #include <private/qhighdpiscaling_p.h>
 #include <QtTest/private/qtesthelpers_p.h>
+#include <qpa/qplatformtheme.h>
+#include <QtWidgets/private/qapplication_p.h>
 
 using namespace QTestPrivate;
 
@@ -77,6 +56,7 @@ private slots:
     void cleanup();
     void repolish();
     void repolish_without_crashing();
+    void repolish_children();
     void numinstances();
     void widgetsBeforeAppStyleSheet();
     void widgetsAfterAppStyleSheet();
@@ -116,6 +96,7 @@ private slots:
     void proxyStyle();
     void dialogButtonBox();
     void emptyStyleSheet();
+    void toolTip_data();
     void toolTip();
     void embeddedFonts();
     void opaquePaintEvent_data();
@@ -128,6 +109,7 @@ private slots:
     void QTBUG36933_brokenPseudoClassLookup();
     void styleSheetChangeBeforePolish();
     void placeholderColor();
+    void accent();
     void enumPropertySelector_data();
     void enumPropertySelector();
     //at the end because it mess with the style.
@@ -135,6 +117,7 @@ private slots:
     void appStyle();
     void QTBUG11658_cachecrash();
     void styleSheetTargetAttribute();
+    void tabWidgetDocumentModeCrash();
     void unpolish();
 
     void highdpiImages_data();
@@ -142,6 +125,10 @@ private slots:
 
     void iconSizes_data();
     void iconSizes();
+    void inheritWidgetPalette_data();
+    void inheritWidgetPalette();
+
+    void resetFont();
 
 private:
     static QColor COLOR(const QWidget &w)
@@ -443,6 +430,19 @@ void tst_QStyleSheetStyle::repolish_without_crashing()
     QCOMPARE(COLOR(*label), QColor(Qt::red));
 }
 
+void tst_QStyleSheetStyle::repolish_children()
+{
+    QWidget parent;
+    parent.setStyleSheet("QPushButton { color: red; background: white }");
+    QPushButton p2(&parent);
+    // a layout would call show, triggering a polish of the child while
+    // the parent on which the style sheet is set remains unpolished
+    p2.show();
+    QCOMPARE(BACKGROUND(p2), Qt::white);
+    parent.setStyleSheet("QPushButton { color: red; background: red }");
+    QCOMPARE(BACKGROUND(p2), Qt::red);
+}
+
 void tst_QStyleSheetStyle::widgetStyle()
 {
     qApp->setStyleSheet(QString());
@@ -970,7 +970,7 @@ void tst_QStyleSheetStyle::focusColors()
     // ten pixels of the right color requires quite a many characters, as the
     // majority of the pixels will have slightly different colors due to the
     // anti-aliasing effect.
-#if !defined(Q_OS_WIN32) && !(defined(Q_OS_LINUX) && defined(Q_CC_GNU) && !defined(Q_CC_INTEL))
+#if !defined(Q_OS_WIN32) && !(defined(Q_OS_LINUX) && defined(Q_CC_GNU))
     QSKIP("This is a fragile test which fails on many esoteric platforms because of focus problems"
           " (for example, QTBUG-33959)."
           "That doesn't mean that the feature doesn't work in practice.");
@@ -985,7 +985,6 @@ void tst_QStyleSheetStyle::focusColors()
     centerOnScreen(&frame);
     frame.show();
 
-    QApplication::setActiveWindow(&frame);
     QVERIFY(QTest::qWaitForWindowActive(&frame));
 
     for (QWidget *widget : frame.widgets()) {
@@ -1031,7 +1030,6 @@ void tst_QStyleSheetStyle::hoverColors()
     QCursor::setPos(frame.geometry().topLeft() - QPoint(100, 0));
     frame.show();
 
-    QApplication::setActiveWindow(&frame);
     QVERIFY(QTest::qWaitForWindowActive(&frame));
 
     QWindow *frameWindow = frame.windowHandle();
@@ -1664,19 +1662,32 @@ private:
     const QString m_oldStyleName;
 };
 
+void tst_QStyleSheetStyle::toolTip_data()
+{
+    QTest::addColumn<QString>("style");
+
+    QTest::newRow("fusion") << QString("Fusion");
+#ifdef Q_OS_WINDOWS
+    QTest::newRow("windowsvista") << QString("WindowsVista");
+#endif
+}
+
 void tst_QStyleSheetStyle::toolTip()
 {
+    QFETCH(QString, style);
+
     if (QGuiApplication::platformName().startsWith(QLatin1String("wayland"), Qt::CaseInsensitive))
         QSKIP("Wayland: This fails. Figure out why.");
 
-    qApp->setStyleSheet(QString());
     QWidget w;
     w.resize(m_testSize);
     w.setWindowTitle(QTest::currentTestFunction());
+
     // Use "Fusion" to prevent the Vista style from clobbering the tooltip palette in polish().
-    QStyle *fusionStyle = QStyleFactory::create(QLatin1String("Fusion"));
-    QVERIFY(fusionStyle);
-    ApplicationStyleSetter as(fusionStyle);
+    QStyle *appStyle = QStyleFactory::create(style);
+    QVERIFY(appStyle);
+    ApplicationStyleSetter as(appStyle);
+
     QHBoxLayout layout(&w);
     w.setLayout(&layout);
 
@@ -1704,37 +1715,65 @@ void tst_QStyleSheetStyle::toolTip()
     wid4->setToolTip("this is wid4");
     wid4->setObjectName("wid4");
 
+    QWidget *wid5 = new QPushButton("wid5", &w);
+    layout.addWidget(wid5);
+    wid5->setStyleSheet("QToolTip { background: #ff0; color: #f00 }");
+    wid5->setToolTip("this is wid5");
+    wid5->setObjectName("wid5");
+
     centerOnScreen(&w);
     w.show();
-    QApplication::setActiveWindow(&w);
     QVERIFY(QTest::qWaitForWindowActive(&w));
 
-    const QColor normalToolTip = QToolTip::palette().color(QPalette::Inactive, QPalette::ToolTipBase);
+    QColor normalToolTipBgColor = QToolTip::palette().color(QPalette::Inactive, QPalette::ToolTipBase);
+
+#ifdef Q_OS_MACOS
+    // macOS uses tool tip text color set in label palette
+    const QPalette *labelPalette = QGuiApplicationPrivate::platformTheme()->palette(QPlatformTheme::LabelPalette);
+    QColor normalToolTipFgColor = labelPalette->color(QPalette::Inactive, QPalette::ToolTipText);
+#else
+    QColor normalToolTipFgColor = QToolTip::palette().color(QPalette::Inactive, QPalette::ToolTipText);
+#endif
+
     // Tooltip on the widget without stylesheet, then to other widget,
     // including one without stylesheet (the tooltip will be reused,
     // but its color must change)
-    const QWidgetList widgets{wid4, wid1, wid2, wid3, wid4};
-    const QList<QColor> colors { normalToolTip, QColor("#ae2"), QColor("#f81"), QColor("#0b8"),
-                                 normalToolTip };
+    const QWidgetList widgets{wid4, wid1, wid2, wid3, wid4, wid5};
+    const QList<QColor> bgcolors { normalToolTipBgColor, QColor("#ae2"), QColor("#f81"),
+                                QColor("#0b8"), normalToolTipBgColor, QColor("#ff0")};
+    const QList<QColor> fgcolors { normalToolTipFgColor, normalToolTipFgColor, normalToolTipFgColor,
+                                normalToolTipFgColor, normalToolTipFgColor, QColor("#f00")};
 
     QWidgetList topLevels;
-    for (int i = 0; i < widgets.count() ; ++i) {
+    for (int i = 0; i < widgets.size() ; ++i) {
         QWidget *wid = widgets.at(i);
-        QColor col = colors.at(i);
+        QColor bgColor = bgcolors.at(i);
+        QColor fgColor = fgcolors.at(i);
 
         QToolTip::showText( QPoint(0,0) , "This is " + wid->objectName(), wid);
 
         topLevels = QApplication::topLevelWidgets();
         QWidget *tooltip = nullptr;
-        for (QWidget *widget : qAsConst(topLevels)) {
+        for (QWidget *widget : std::as_const(topLevels)) {
             if (widget->inherits("QTipLabel")) {
                 tooltip = widget;
                 break;
             }
         }
+
         QVERIFY(tooltip);
         QTRY_VERIFY(tooltip->isVisible()); // Wait until Roll-Effect is finished (Windows Vista)
-        QCOMPARE(tooltip->palette().color(tooltip->backgroundRole()), col);
+
+#ifdef Q_OS_WINDOWS
+        // If tooltip palette contains empty resolve mask, validate with inherited palette
+        if (!tooltip->palette().resolveMask()) {
+            bgColor = w.palette().color(tooltip->backgroundRole());
+            fgColor = w.palette().color(tooltip->foregroundRole());
+        }
+#endif
+
+        QCOMPARE(tooltip->palette().color(tooltip->backgroundRole()), bgColor);
+        QCOMPARE(tooltip->palette().color(tooltip->foregroundRole()), fgColor);
     }
 
     QToolTip::showText( QPoint(0,0) , "This is " + wid3->objectName(), wid3);
@@ -1742,7 +1781,7 @@ void tst_QStyleSheetStyle::toolTip()
     delete wid3; //should not crash;
     QTest::qWait(10);
     topLevels = QApplication::topLevelWidgets();
-    for (QWidget *widget : qAsConst(topLevels))
+    for (QWidget *widget : std::as_const(topLevels))
         widget->update(); //should not crash either
 }
 
@@ -1852,7 +1891,6 @@ void tst_QStyleSheetStyle::complexWidgetFocus()
 
     centerOnScreen(&frame);
     frame.show();
-    QApplication::setActiveWindow(&frame);
     QVERIFY(QTest::qWaitForWindowActive(&frame));
     for (QWidget *widget : widgets) {
         widget->setFocus();
@@ -1941,7 +1979,6 @@ void tst_QStyleSheetStyle::task232085_spinBoxLineEditBg()
 
     centerOnScreen(&frame);
     frame.show();
-    QApplication::setActiveWindow(&frame);
     spinbox->setFocus();
     QVERIFY(QTest::qWaitForWindowActive(&frame));
 
@@ -2077,7 +2114,6 @@ void tst_QStyleSheetStyle::QTBUG36933_brokenPseudoClassLookup()
     QVERIFY(QTest::qWaitForWindowExposed(&widget));
 
     widget.activateWindow();
-    QApplication::setActiveWindow(&widget);
     QVERIFY(QTest::qWaitForWindowActive(&widget));
 
     QHeaderView *verticalHeader = widget.verticalHeader();
@@ -2298,6 +2334,16 @@ void tst_QStyleSheetStyle::styleSheetTargetAttribute()
     QCOMPARE(pb.testAttribute(Qt::WA_StyleSheetTarget), false);
 }
 
+void tst_QStyleSheetStyle::tabWidgetDocumentModeCrash()
+{
+    // Don't crash when matching accessing documentMode of QTabWidget while styling QTabBar.
+    qApp->setStyleSheet("QTabWidget[documentMode=true] QTabBar { font-size: 40pt; }");
+
+    QTabWidget tabWidget;
+    tabWidget.setDocumentMode(true);
+    QCOMPARE(FONTSIZE(*tabWidget.tabBar()), 40);
+}
+
 void tst_QStyleSheetStyle::unpolish()
 {
     QWidget w;
@@ -2350,11 +2396,32 @@ void tst_QStyleSheetStyle::placeholderColor()
     QLineEdit le2;
     le2.setEnabled(false);
     le1.ensurePolished();
-    QCOMPARE(le1.palette().placeholderText(), red);
+    QColor phColor = le1.palette().placeholderText().color();
+    QCOMPARE(phColor.rgb(), red.rgb());
+    QVERIFY(phColor.alpha() < red.alpha());
+
     le2.ensurePolished();
-    QCOMPARE(le2.palette().placeholderText(), red);
+    phColor = le2.palette().placeholderText().color();
+    QCOMPARE(phColor.rgb(), red.rgb());
+    QVERIFY(phColor.alpha() < red.alpha());
+
     le2.setEnabled(true);
-    QCOMPARE(le2.palette().placeholderText(), red);
+    phColor = le2.palette().placeholderText().color();
+    QCOMPARE(phColor.rgb(), red.rgb());
+    QVERIFY(phColor.alpha() < red.alpha());
+
+    const char *phSpec = "#aabbccdd";
+    le1.setStyleSheet(QString("QLineEdit { placeholder-text-color: %1; }").arg(phSpec));
+    QCOMPARE(le1.palette().placeholderText().color(), QColor(phSpec));
+}
+
+void tst_QStyleSheetStyle::accent()
+{
+    QLineEdit lineEdit;
+    const QColor universe(42, 42, 42);
+    lineEdit.setStyleSheet(QString("QLineEdit { accent-color: %1; }").arg(universe.name()));
+    lineEdit.ensurePolished();
+    QCOMPARE(lineEdit.palette().accent().color(), universe);
 }
 
 void tst_QStyleSheetStyle::enumPropertySelector_data()
@@ -2451,6 +2518,53 @@ void tst_QStyleSheetStyle::iconSizes()
     QCOMPARE(button.iconSize(), iconSize);
 }
 
+void tst_QStyleSheetStyle::inheritWidgetPalette_data()
+{
+    QTest::addColumn<const QString>("styleSheet");
+    QTest::addColumn<const QColor>("phColorPalette");
+
+    QTest::addRow("blueAndGreen") << "QLineEdit {color: rgb(0,0,255);}" << QColor(Qt::green);
+    QTest::addRow("emptyStyleSheet") << QString() << QColor(Qt::green);
+
+}
+
+void tst_QStyleSheetStyle::inheritWidgetPalette()
+{
+    QFETCH(const QString, styleSheet);
+    QFETCH(const QColor, phColorPalette);
+
+    QLineEdit edit;
+    QPalette palette = edit.palette();
+    palette.setBrush(QPalette::PlaceholderText, phColorPalette);
+    edit.setPalette(palette);
+    edit.setStyleSheet(styleSheet);
+    const QColor phColor = edit.palette().placeholderText().color();
+
+    QCOMPARE(phColor, phColorPalette);
+}
+
+void tst_QStyleSheetStyle::resetFont()
+{
+    QDoubleSpinBox sb;
+    sb.setStyleSheet(R"(QDoubleSpinBox[changed="true"] {font: italic;})");
+
+    auto checkFont = [&sb](bool isItalic) {
+        sb.setProperty("changed", isItalic);
+        sb.style()->polish(&sb);
+        const auto children = sb.findChildren<QWidget *>();
+        for (const auto *w : children) {
+            auto diagnostics = qScopeGuard([&] {
+                qWarning() << "Failure with" << w << "should be italic:" << isItalic;
+            });
+            QCOMPARE(w->font().italic(), isItalic);
+            diagnostics.dismiss();
+        }
+    };
+    checkFont(false);
+    checkFont(true);
+    checkFont(false);
+}
+
 QTEST_MAIN(tst_QStyleSheetStyle)
 #include "tst_qstylesheetstyle.moc"
 
