@@ -1,5 +1,5 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 
 #include <QTest>
@@ -19,6 +19,8 @@
 #include <qpa/qplatformintegration.h>
 #include <qpa/qplatformtheme.h>
 
+#include <QtWidgets/private/qapplication_p.h>
+
 class tst_QAbstractButton : public QObject
 {
     Q_OBJECT
@@ -478,7 +480,6 @@ void tst_QAbstractButton::setShortcut()
 
     QKeySequence seq( Qt::Key_A );
     testWidget->setShortcut( seq );
-    QApplication::setActiveWindow(testWidget);
     testWidget->activateWindow();
     // must be active to get shortcuts
     QVERIFY(QTest::qWaitForWindowActive(testWidget));
@@ -639,7 +640,7 @@ void tst_QAbstractButton::keyNavigation()
     }
 
     widget.show();
-    qApp->setActiveWindow(&widget);
+    QApplicationPrivate::setActiveWindow(&widget);
     widget.activateWindow();
     QVERIFY(QTest::qWaitForWindowActive(&widget));
 
