@@ -8,10 +8,12 @@
 #include <QPushButton>
 #include <QMainWindow>
 #include <QMenuBar>
+#include <QPlainTextEdit>
 #include <QToolBar>
 #include <QToolButton>
 #include <QStatusBar>
 #include <QListWidget>
+#include <QVBoxLayout>
 #include <QWidgetAction>
 #include <QScreen>
 #include <QSpinBox>
@@ -96,6 +98,8 @@ private slots:
     void QTBUG_89082_actionTipsHide();
     void QTBUG8122_widgetActionCrashOnClose();
     void widgetActionTriggerClosesMenu();
+    void widgetActionContextMenu();
+
     void transientParent();
 
     void QTBUG_10735_crashWithDialog();
@@ -1590,6 +1594,36 @@ void tst_QMenu::widgetActionTriggerClosesMenu()
     QCOMPARE(actionTriggered, &widgetAction);
 }
 
+void tst_QMenu::widgetActionContextMenu() // QTBUG-134757
+{
+    QPushButton openButton("open");
+    QMenu *menu = new QMenu(&openButton);
+    QVBoxLayout *layout = new QVBoxLayout;
+    QWidgetAction widgetAction(menu);
+    QWidget menuWidget(menu);
+    QPlainTextEdit edit;
+    openButton.setMenu(menu);
+    menuWidget.setLayout(layout);
+    widgetAction.setDefaultWidget(&menuWidget);
+    menu->addAction(&widgetAction);
+    layout->addWidget(&edit);
+
+    openButton.show();
+    QVERIFY(QTest::qWaitForWindowExposed(&openButton));
+
+    // Click the QPushButton to open its menu
+    QTest::mouseClick(&openButton, Qt::LeftButton);
+    QVERIFY(QTest::qWaitForWindowExposed(&menuWidget));
+    QWindow *popupWindow = edit.window()->windowHandle();
+    QVERIFY(popupWindow);
+    QCOMPARE(QApplication::activePopupWidget(), menu);
+
+    // Right-click the QPlainTextEdit to open its context menu
+    QTest::mouseClick(popupWindow, Qt::RightButton);
+    QVERIFY(qobject_cast<QMenu *>(QApplication::activePopupWidget()));
+    QCOMPARE_NE(QApplication::activePopupWidget(), menu);
+}
+
 void tst_QMenu::transientParent()
 {
     QMainWindow window;
@@ -1597,12 +1631,12 @@ void tst_QMenu::transientParent()
     window.menuBar()->setNativeMenuBar(false);
     centerOnScreen(&window);
 
-    QMenu *fileMenu = new QMenu("&File");
-    QAction *exitAct = new QAction("Exit");
+    QMenu *fileMenu = new QMenu("&File", &window);
+    QAction *exitAct = new QAction("Exit", &window);
     fileMenu->addAction(exitAct);
 
-    QMenu *editMenu = new QMenu("&Edit");
-    QAction *undoAct = new QAction("Undo");
+    QMenu *editMenu = new QMenu("&Edit", &window);
+    QAction *undoAct = new QAction("Undo", &window);
     editMenu->addAction(undoAct);
 
     QMenuBar *menuBar = new QMenuBar;
@@ -1610,31 +1644,64 @@ void tst_QMenu::transientParent()
     menuBar->addMenu(editMenu);
     window.setMenuBar(menuBar);
 
-    // On Mac, we need to create native key events to test menu
-    // action activation, so skip this part of the test.
-#if QT_CONFIG(shortcut) && !defined(Q_OS_DARWIN)
+    QMenu *bookmarksMenu = new QMenu(&window);
+    bookmarksMenu->addAction("This is KDE!");
+
+    QMenu *contextMenu = new QMenu(&window);
+    QAction *bookmarksAction = contextMenu->addAction("&Bookmarks");
+    bookmarksAction->setMenu(bookmarksMenu);
+
     window.show();
     QVERIFY(QTest::qWaitForWindowActive(&window));
     QWindow *topLevel = window.windowHandle();
     QVERIFY(topLevel);
 
+    // Show the standalone bookmarks menu. It should be a child of the main window.
+    bookmarksMenu->popup(window.geometry().center());
+    QTRY_VERIFY(QTest::qWaitForWindowExposed(bookmarksMenu));
+    QVERIFY(bookmarksMenu->windowHandle());
+    QCOMPARE(bookmarksMenu->windowHandle()->transientParent(), topLevel);
+    bookmarksMenu->close();
+
+    // Show the bookmarks menu attached to the context menu. Even though the bookmarks menu is
+    // a child of the main window, its transient parent will be the context menu.
+    contextMenu->popup(window.geometry().center());
+    QTRY_VERIFY(QTest::qWaitForWindowExposed(contextMenu));
+    QVERIFY(contextMenu->windowHandle());
+    QCOMPARE(contextMenu->windowHandle()->transientParent(), topLevel);
+
+    contextMenu->setActiveAction(bookmarksAction);
+    QTRY_VERIFY(QTest::qWaitForWindowExposed(bookmarksMenu));
+    QVERIFY(bookmarksMenu->windowHandle());
+    QCOMPARE(bookmarksMenu->windowHandle()->transientParent(), contextMenu->windowHandle());
+    contextMenu->close();
+
+    // Show the standalone bookmarks menu. Its transient parent will be the main window again.
+    bookmarksMenu->popup(window.geometry().center());
+    QTRY_VERIFY(QTest::qWaitForWindowExposed(bookmarksMenu));
+    QVERIFY(bookmarksMenu->windowHandle());
+    QCOMPARE(bookmarksMenu->windowHandle()->transientParent(), topLevel);
+    bookmarksMenu->close();
+
+    // On Mac, we need to create native key events to test menu
+    // action activation, so skip this part of the test.
+#if QT_CONFIG(shortcut) && !defined(Q_OS_DARWIN)
     window.setFocus();
     QVERIFY(QTest::qWaitForWindowFocused(&window));
     QVERIFY(window.hasFocus());
 
     QTest::keyPress(&window, Qt::Key_F, Qt::AltModifier);
     QTRY_VERIFY(QTest::qWaitForWindowExposed(fileMenu));
-    if (fileMenu->isWindow() && fileMenu->window() && fileMenu->window()->windowHandle())
-        QVERIFY(fileMenu->window()->windowHandle()->transientParent());
+    QVERIFY(fileMenu->windowHandle());
+    QCOMPARE(fileMenu->windowHandle()->transientParent(), topLevel);
     QTest::keyRelease(fileMenu, Qt::Key_F, Qt::AltModifier);
 
     QTest::keyPress(fileMenu, Qt::Key_E, Qt::AltModifier);
     QTRY_VERIFY(QTest::qWaitForWindowExposed(editMenu));
-    if (editMenu->isWindow() && editMenu->window() && editMenu->window()->windowHandle())
-        QVERIFY(editMenu->window()->windowHandle()->transientParent());
+    QVERIFY(editMenu->windowHandle());
+    QCOMPARE(editMenu->windowHandle()->transientParent(), topLevel);
     QTest::keyRelease(editMenu, Qt::Key_E, Qt::AltModifier);
 #endif // QT_CONFIG(shortcut) && !Q_OS_DARWIN
-
 }
 
 class MyMenu : public QMenu
@@ -1982,11 +2049,13 @@ void tst_QMenu::QTBUG_61039_menu_shortcuts()
     if (!QGuiApplicationPrivate::platformIntegration()->hasCapability(QPlatformIntegration::WindowActivation))
         QSKIP("Window activation is not supported");
 
-    QAction *actionKamen = new QAction("Action Kamen");
+    QObject reaper;
+
+    QAction *actionKamen = new QAction("Action Kamen", &reaper);
 #if QT_CONFIG(shortcut)
     actionKamen->setShortcut(QKeySequence(QLatin1String("K")));
 #endif
-    QAction *actionJoe = new QAction("Action Joe");
+    QAction *actionJoe = new QAction("Action Joe", &reaper);
 #if QT_CONFIG(shortcut)
     actionJoe->setShortcut(QKeySequence(QLatin1String("Ctrl+J")));
 #endif
@@ -2177,6 +2246,13 @@ void tst_QMenu::invisibleActions()
     contextMenu.popup(globalPos);
     QVERIFY(!contextMenu.isVisible());
     QVERIFY(!contextMenu.exec());
+
+    // a QMenu might not have any (visible) actions, but still have contents
+    QPushButton *buttonInMenu = new QPushButton(&contextMenu);
+    buttonInMenu->show();
+
+    contextMenu.popup(globalPos);
+    QVERIFY(contextMenu.isVisible());
 }
 
 #if QT_CONFIG(shortcut) && !defined(Q_OS_DARWIN)
