@@ -1,5 +1,5 @@
 // Copyright (C) 2019 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 
 #include <QTest>
@@ -12,12 +12,13 @@
 #include <qtextbrowser.h>
 #include <qtextobject.h>
 
+#include <QtWidgets/private/qapplication_p.h>
+
 class TestBrowser : public QTextBrowser
 {
 public:
     inline TestBrowser() {
         show();
-        QApplication::setActiveWindow(this);
         activateWindow();
         setFocus();
         QVERIFY(QTest::qWaitForWindowActive(this));
