@@ -69,8 +69,6 @@ int main(int argc, char *argv[])
     QSurfaceFormat format = transparentChildWindow->format();
     format.setAlphaBufferSize(8);
     transparentChildWindow->setFormat(format);
-    // FIXME: Windows requires this, even for child windows
-    transparentChildWindow->setFlag(Qt::FramelessWindowHint);
     transparentChildWindow->setParent(&window);
     transparentChildWindow->setGeometry(350, 50, 100, 100);
     transparentChildWindow->showNormal();
