@@ -1,5 +1,5 @@
 // Copyright (C) 2016 Intel Corporation
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QtCore/QCoreApplication>
 
@@ -15,7 +15,7 @@ int main(int argc, char *argv[])
     QStringList args = a.arguments();
     args.takeFirst();   // skip application name
 
-    foreach (const QString &path, args) {
+    for (const QString &path : std::as_const(args)) {
         QStorageInfo info(path);
         if (!info.isValid()) {
             // no error string...
